Package: extraDistr
Type: Package
Title: Additional Univariate and Multivariate Distributions
Version: 1.10.0.4
Date: 2026-05-18
Authors@R: c(person(given  = "Tymoteusz",
                    family = "Wolodzko",
                    role   = "aut",
                    email  = "twolodzko+extradistr@gmail.com"),
             person(given  = "Sigbert",
                    family = "Klinke",
                    role   = "cre",
                    email  = "sigbert@wiwi.hu-berlin.de"),
             person(given  = "Thomas",
                    family = "Farrar",
                    role   = "ctb",
                    email  = "FarrarT@cput.ac.za")
            )
Description: Density, distribution function, quantile function
    and random generation for a number of univariate
    and multivariate distributions. This package implements the
    following distributions: Bernoulli, beta-binomial, beta-negative
    binomial, beta prime, Bhattacharjee, Birnbaum-Saunders,
    bivariate normal, bivariate Poisson, categorical, Dirichlet,
    Dirichlet-multinomial, discrete gamma, discrete Laplace,
    discrete normal, discrete uniform, discrete Weibull, Frechet,
    gamma-Poisson, generalized extreme value, Gompertz,
    generalized Pareto, Gumbel, half-Cauchy, half-normal, half-t,
    Huber density, inverse chi-squared, inverse-gamma, Kumaraswamy,
    Laplace, location-scale t, logarithmic, Lomax, multivariate
    hypergeometric, multinomial, negative hypergeometric,
    non-standard beta, normal mixture, Poisson mixture, Pareto,
    power, reparametrized beta, Rayleigh, shifted Gompertz, Skellam,
    slash, triangular, truncated binomial, truncated normal,
    truncated Poisson, Tukey lambda, Wald, zero-inflated binomial,
    zero-inflated negative binomial, zero-inflated Poisson.
License: GPL-2
URL: https://github.com/sigbertklinke/extraDistr
Encoding: UTF-8
Depends: R (>= 3.1.0)
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp
Suggests: testthat, LaplacesDemon, VGAM, evd, skellam, triangle, actuar
NeedsCompilation: yes
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Packaged: 2026-05-18 08:48:53 UTC; sigbert
Author: Tymoteusz Wolodzko [aut],
  Sigbert Klinke [cre],
  Thomas Farrar [ctb]
Maintainer: Sigbert Klinke <sigbert@wiwi.hu-berlin.de>
Repository: CRAN
Date/Publication: 2026-05-18 10:30:07 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-18 14:22:59 UTC; unix
Archs: extraDistr.so.dSYM

# Generated by roxygen2: do not edit by hand

export(dbbinom)
export(dbern)
export(dbetapr)
export(dbhatt)
export(dbnbinom)
export(dbvnorm)
export(dbvpois)
export(dcat)
export(ddgamma)
export(ddirichlet)
export(ddirmnom)
export(ddlaplace)
export(ddnorm)
export(ddunif)
export(ddweibull)
export(dfatigue)
export(dfrechet)
export(dgev)
export(dgompertz)
export(dgpd)
export(dgpois)
export(dgumbel)
export(dhcauchy)
export(dhnorm)
export(dht)
export(dhuber)
export(dinvchisq)
export(dinvgamma)
export(dkumar)
export(dlaplace)
export(dlgser)
export(dlomax)
export(dlst)
export(dmixnorm)
export(dmixpois)
export(dmnom)
export(dmvhyper)
export(dnhyper)
export(dnsbeta)
export(dpareto)
export(dpower)
export(dprop)
export(drayleigh)
export(dsgomp)
export(dskellam)
export(dslash)
export(dtbinom)
export(dtnorm)
export(dtpois)
export(dtriang)
export(dwald)
export(dzib)
export(dzinb)
export(dzip)
export(pbbinom)
export(pbern)
export(pbetapr)
export(pbhatt)
export(pbnbinom)
export(pcat)
export(pdgamma)
export(pdlaplace)
export(pdnorm)
export(pdunif)
export(pdweibull)
export(pfatigue)
export(pfrechet)
export(pgev)
export(pgompertz)
export(pgpd)
export(pgpois)
export(pgumbel)
export(phcauchy)
export(phnorm)
export(pht)
export(phuber)
export(pinvchisq)
export(pinvgamma)
export(pkumar)
export(plaplace)
export(plgser)
export(plomax)
export(plst)
export(pmixnorm)
export(pmixpois)
export(pnhyper)
export(pnsbeta)
export(ppareto)
export(ppower)
export(pprop)
export(prayleigh)
export(psgomp)
export(pslash)
export(ptbinom)
export(ptnorm)
export(ptpois)
export(ptriang)
export(pwald)
export(pzib)
export(pzinb)
export(pzip)
export(qbern)
export(qbetapr)
export(qcat)
export(qdunif)
export(qdweibull)
export(qfatigue)
export(qfrechet)
export(qgev)
export(qgompertz)
export(qgpd)
export(qgumbel)
export(qhcauchy)
export(qhnorm)
export(qht)
export(qhuber)
export(qinvchisq)
export(qinvgamma)
export(qkumar)
export(qlaplace)
export(qlgser)
export(qlomax)
export(qlst)
export(qnhyper)
export(qnsbeta)
export(qpareto)
export(qpower)
export(qprop)
export(qrayleigh)
export(qtbinom)
export(qtlambda)
export(qtnorm)
export(qtpois)
export(qtriang)
export(qzib)
export(qzinb)
export(qzip)
export(rbbinom)
export(rbern)
export(rbetapr)
export(rbhatt)
export(rbnbinom)
export(rbvnorm)
export(rbvpois)
export(rcat)
export(rcatlp)
export(rdgamma)
export(rdirichlet)
export(rdirmnom)
export(rdlaplace)
export(rdnorm)
export(rdunif)
export(rdweibull)
export(rfatigue)
export(rfrechet)
export(rgev)
export(rgompertz)
export(rgpd)
export(rgpois)
export(rgumbel)
export(rhcauchy)
export(rhnorm)
export(rht)
export(rhuber)
export(rinvchisq)
export(rinvgamma)
export(rkumar)
export(rlaplace)
export(rlgser)
export(rlomax)
export(rlst)
export(rmixnorm)
export(rmixpois)
export(rmnom)
export(rmvhyper)
export(rnhyper)
export(rnsbeta)
export(rpareto)
export(rpower)
export(rprop)
export(rrayleigh)
export(rsgomp)
export(rsign)
export(rskellam)
export(rslash)
export(rtbinom)
export(rtlambda)
export(rtnorm)
export(rtpois)
export(rtriang)
export(rwald)
export(rzib)
export(rzinb)
export(rzip)
importFrom(Rcpp,sourceCpp)
importFrom(stats,pgamma)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qgamma)
importFrom(stats,qnorm)
importFrom(stats,rchisq)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
useDynLib(extraDistr, .registration = TRUE)

#' Beta-binomial distribution
#'
#' Probability mass function and random generation
#' for the beta-binomial distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param alpha,beta      non-negative parameters of the beta distribution.
#' @param size            number of trials (zero or more).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @details
#' 
#' If \eqn{p \sim \mathrm{Beta}(\alpha, \beta)}{p ~ Beta(\alpha, \beta)} and
#' \eqn{X \sim \mathrm{Binomial}(n, p)}{X ~ Binomial(n, p)}, then 
#' \eqn{X \sim \mathrm{BetaBinomial}(n, \alpha, \beta)}{X ~ BetaBinomial(n, \alpha, \beta)}.
#' 
#' Probability mass function
#' \deqn{
#' f(x) = {n \choose x} \frac{\mathrm{B}(x+\alpha, n-x+\beta)}{\mathrm{B}(\alpha, \beta)}
#' }{
#' f(x) = choose(n, x) * B(x+\alpha, n-x+\beta) / B(\alpha, \beta)
#' }
#'
#' Cumulative distribution function is calculated using recursive algorithm that employs the fact that
#' \eqn{\Gamma(x) = (x - 1)!}, and
#' \eqn{
#' \mathrm{B}(x, y) = \frac{\Gamma(x)\Gamma(y)}{\Gamma(x+y)}
#' }{
#' B(x, y) = (\Gamma(x)\Gamma(y))/\Gamma(x+y)
#' }, and that
#' \eqn{
#' {n \choose k} = \prod_{i=1}^k \frac{n+1-i}{i}
#' }{
#' choose(n,k) = prod((n+1-(1:k))/(1:k))
#' }. This enables re-writing probability mass function as
#' 
#' \deqn{
#' f(x) = \left( \prod_{i=1}^x \frac{n+1-i}{i} \right) \frac{\frac{(\alpha+x-1)!\,(\beta+n-x-1)!}{(\alpha+\beta+n-1)!}}{\mathrm{B}(\alpha,\beta)}
#' }{
#' f(x) = prod((n+1-(1:x))/(1:x)) * (((\alpha+x-1)!*(\beta+n-x-1)!)/((\alpha+\beta+n+1)!)) / B(\alpha, \beta)
#' }
#' 
#' what makes recursive updating from \eqn{x} to \eqn{x+1} easy using the properties of factorials
#' 
#' \deqn{
#' f(x+1) = \left( \prod_{i=1}^x \frac{n+1-i}{i} \right) \frac{n+1-x+1}{x+1} \frac{\frac{(\alpha+x-1)! \,(\alpha+x)\,(\beta+n-x-1)! \, (\beta+n-x)^{-1}}{(\alpha+\beta+n-1)!\,(\alpha+\beta+n)}}{\mathrm{B}(\alpha,\beta)}
#' }{
#' f(x+1) = prod((n+1-(1:x))/(1:x)) * ((n+1-x+1)/(x+1)) * (((\alpha+x-1)!*(\alpha+x)*(\beta+n-x-1)!/(\beta+n-x))/((\alpha+\beta+n+1)!*(\alpha+\beta+n))) / B(\alpha, \beta)
#' }
#' 
#' and let's us efficiently calculate cumulative distribution function as a sum of probability mass functions
#' 
#' \deqn{F(x) = \sum_{k=0}^x f(k)}{F(x) = f(0)+...+f(x)}
#' 
#'
#' @seealso \code{\link[stats]{Beta}}, \code{\link[stats]{Binomial}}
#' 
#' @examples 
#' 
#' x <- rbbinom(1e5, 1000, 5, 13)
#' xx <- 0:1000
#' hist(x, 100, freq = FALSE)
#' lines(xx-0.5, dbbinom(xx, 1000, 5, 13), col = "red")
#' hist(pbbinom(x, 1000, 5, 13))
#' xx <- seq(0, 1000, by = 0.1)
#' plot(ecdf(x))
#' lines(xx, pbbinom(xx, 1000, 5, 13), col = "red", lwd = 2)
#'
#' @name BetaBinom
#' @aliases BetaBinom
#' @aliases dbbinom
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dbbinom <- function(x, size, alpha = 1, beta = 1, log = FALSE) {
  cpp_dbbinom(x, size, alpha, beta, log[1L])
}


#' @rdname BetaBinom
#' @export

pbbinom <- function(q, size, alpha = 1, beta = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pbbinom(q, size, alpha, beta, lower.tail[1L], log.p[1L])
}


#' @rdname BetaBinom
#' @export

rbbinom <- function(n, size, alpha = 1, beta = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rbbinom(n, size, alpha, beta)
}

test_that("Testing PDFs & PMFs against infinite values", {

  expect_true(!is.nan(dbetapr(Inf, 1, 1, 1)) && is.finite(dbetapr(Inf, 1, 1, 1)))
  expect_true(!is.nan(dbbinom(Inf, 10, 1, 1)) && is.finite(dbbinom(Inf, 10, 1, 1)))
  expect_true(!is.nan(dbnbinom(Inf, 10, 1, 1)) && is.finite(dbnbinom(Inf, 10, 1, 1)))
  expect_true(!is.nan(dbhatt(Inf)) && is.finite(dbhatt(Inf)))
  expect_true(!is.nan(dfatigue(Inf, 1)) && is.finite(dfatigue(Inf, 1)))
  expect_true(!is.nan(dcat(Inf, c(0.5, 0.5))) && is.finite(dcat(Inf, c(0.5, 0.5))))
  expect_true(!is.nan(ddlaplace(Inf, 0, 0.5)) && is.finite(ddlaplace(Inf, 0, 0.5)))
  expect_true(!is.nan(ddnorm(Inf)) && is.finite(ddnorm(Inf)))
  expect_true(!is.nan(ddgamma(Inf, 9, 1) && is.finite(ddgamma(Inf, 9, 1))))
  expect_true(!is.nan(ddweibull(Inf, 0.5, 1)) && is.finite(ddweibull(Inf, 0.5, 1)))  
  expect_true(!is.nan(dfrechet(Inf)) && is.finite(dfrechet(Inf)))
  expect_true(!is.nan(dgpois(Inf, 1, 1)) && is.finite(dgpois(Inf, 1, 1)))
  expect_true(!is.nan(dgev(Inf, 1, 1, 1)) && is.finite(dgev(Inf, 1, 1, 1)))
  expect_true(!is.nan(dgompertz(Inf, 1, 1)) && is.finite(dgompertz(Inf, 1, 1)))
  expect_true(!is.nan(dgpd(Inf, 1, 1, 1)) && is.finite(dgpd(Inf, 1, 1, 1)))
  expect_true(!is.nan(dgumbel(Inf)) && is.finite(dgumbel(Inf)))
  expect_true(!is.nan(dhuber(Inf)) && is.finite(dhuber(Inf)))
  expect_true(!is.nan(dhcauchy(Inf, 1)) && is.finite(dhcauchy(Inf, 1)))
  expect_true(!is.nan(dhnorm(Inf, 1)) && is.finite(dhnorm(Inf, 1)))
  expect_true(!is.nan(dht(Inf, 5, 1)) && is.finite(dht(Inf, 5, 1)))
  expect_true(!is.nan(dinvgamma(Inf, 1, 1)) && is.finite(dinvgamma(Inf, 1, 1)))
  expect_true(!is.nan(dlaplace(Inf)) && is.finite(dlaplace(Inf)))
  expect_true(!is.nan(dlgser(Inf, 0.5)) && is.finite(dlgser(Inf, 0.5)))
  expect_true(!is.nan(dlomax(Inf, 1, 1)) && is.finite(dlomax(Inf, 1, 1)))
  expect_true(!is.nan(dmixnorm(Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))) && is.finite(dmixnorm(Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(dmixpois(Inf, c(1,2,3), c(1/3,1/3,1/3))) && is.finite(dmixpois(Inf, c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(dnhyper(Inf, 60, 35, 15)) && is.finite(dnhyper(Inf, 60, 35, 15)))
  expect_true(!is.nan(dlst(Inf, df = 2)) && is.finite(dlst(Inf, df = 2)))
  expect_true(!is.nan(dpareto(Inf)) && is.finite(dpareto(Inf)))
  expect_true(!is.nan(dpower(Inf, 1, 1)) && is.finite(dpower(Inf, 1, 1)))
  expect_true(!is.nan(dprop(Inf, 10, 0.5)) && is.finite(dprop(Inf, 10, 0.5)))
  expect_true(!is.nan(drayleigh(Inf)) && is.finite(drayleigh(Inf)))
  expect_true(!is.nan(dsgomp(Inf, 0.4, 1)) && is.finite(dsgomp(Inf, 0.4, 1)))
  expect_true(!is.nan(dslash(Inf)) && is.finite(dslash(Inf)))
  expect_true(!is.nan(dtpois(Inf, lambda = 5)) && is.finite(dtpois(Inf, lambda = 5)))
  expect_true(!is.nan(dtpois(Inf, lambda = 5, a = 6)) && is.finite(dtpois(Inf, lambda = 5, a = 6)))
  expect_true(!is.nan(dwald(Inf, 1, 1)) && is.finite(dwald(Inf, 1, 1)))
  expect_true(!is.nan(dzip(Inf, 1, 0.5)) && is.finite(dzip(Inf, 1, 0.5)))
  expect_true(!is.nan(dzib(Inf, 1, 1, 0.5)) && is.finite(dzinb(Inf, 1, 1, 0.5)))
  expect_true(!is.nan(dzinb(Inf, 1, 1, 0.5)) && is.finite(dzinb(Inf, 1, 1, 0.5)))

})


test_that("Testing CDFs against infinite values", {
  
  expect_true(!is.nan(pbetapr(Inf, 1, 1, 1)) && is.finite(pbetapr(Inf, 1, 1, 1)))
  expect_true(!is.nan(pbbinom(Inf, 10, 1, 1)) && is.finite(pbbinom(Inf, 10, 1, 1)))
  expect_true(!is.nan(pbnbinom(Inf, 10, 1, 1)) && is.finite(pbnbinom(Inf, 10, 1, 1)))
  expect_true(!is.nan(pbhatt(Inf)) && is.finite(pbhatt(Inf)))
  expect_true(!is.nan(pfatigue(Inf, 1)) && is.finite(pfatigue(Inf, 1)))
  expect_true(!is.nan(pcat(Inf, c(0.5, 0.5))) && is.finite(pcat(Inf, c(0.5, 0.5))))
  expect_true(!is.nan(pdlaplace(Inf, 0, 0.5)) && is.finite(pdlaplace(Inf, 0, 0.5)))
  expect_true(!is.nan(pdnorm(Inf)) && is.finite(pdnorm(Inf)))
  expect_true(!is.nan(pdgamma(Inf, 9, 1) && is.finite(pdgamma(Inf, 9, 1))))
  expect_true(!is.nan(pdweibull(Inf, 0.5, 1)) && is.finite(pdweibull(Inf, 0.5, 1)))  
  expect_true(!is.nan(pfrechet(Inf)) && is.finite(pfrechet(Inf)))
  expect_true(!is.nan(pgpois(Inf, 1, 1)) && is.finite(pgpois(Inf, 1, 1)))
  expect_true(!is.nan(pgev(Inf, 1, 1, 1)) && is.finite(pgev(Inf, 1, 1, 1)))
  expect_true(!is.nan(pgompertz(Inf, 1, 1)) && is.finite(pgompertz(Inf, 1, 1)))
  expect_true(!is.nan(pgpd(Inf, 1, 1, 1)) && is.finite(pgpd(Inf, 1, 1, 1)))
  expect_true(!is.nan(pgumbel(Inf)) && is.finite(pgumbel(Inf)))
  expect_true(!is.nan(phuber(Inf)) && is.finite(phuber(Inf)))
  expect_true(!is.nan(phcauchy(Inf, 1)) && is.finite(phcauchy(Inf, 1)))
  expect_true(!is.nan(phnorm(Inf, 1)) && is.finite(phnorm(Inf, 1)))
  expect_true(!is.nan(pht(Inf, 5, 1)) && is.finite(pht(Inf, 5, 1)))
  expect_true(!is.nan(pinvgamma(Inf, 1, 1)) && is.finite(pinvgamma(Inf, 1, 1)))
  expect_true(!is.nan(plaplace(Inf)) && is.finite(plaplace(Inf)))
  expect_true(!is.nan(plgser(Inf, 0.5)) && is.finite(plgser(Inf, 0.5)))
  expect_true(!is.nan(plomax(Inf, 1, 1)) && is.finite(plomax(Inf, 1, 1)))
  expect_true(!is.nan(pmixnorm(Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))) && is.finite(pmixnorm(Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(pmixpois(Inf, c(1,2,3), c(1/3,1/3,1/3))) && is.finite(pmixpois(Inf, c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(pnhyper(Inf, 60, 35, 15)) && is.finite(pnhyper(Inf, 60, 35, 15)))
  expect_true(!is.nan(plst(Inf, df = 2)) && is.finite(plst(Inf, df = 2)))
  expect_true(!is.nan(ppareto(Inf)) && is.finite(ppareto(Inf)))
  expect_true(!is.nan(ppower(Inf, 1, 1)) && is.finite(ppower(Inf, 1, 1)))
  expect_true(!is.nan(pprop(Inf, 10, 0.5)) && is.finite(pprop(Inf, 10, 0.5)))
  expect_true(!is.nan(prayleigh(Inf)) && is.finite(prayleigh(Inf)))
  expect_true(!is.nan(psgomp(Inf, 0.4, 1)) && is.finite(psgomp(Inf, 0.4, 1)))
  expect_true(!is.nan(pslash(Inf)) && is.finite(pslash(Inf)))
  expect_true(!is.nan(ptpois(Inf, lambda = 5)) && is.finite(ptpois(Inf, lambda = 5)))
  expect_true(!is.nan(ptpois(Inf, lambda = 5, a = 6)) && is.finite(ptpois(Inf, lambda = 5, a = 6)))
  expect_true(!is.nan(pwald(Inf, 1, 1)) && is.finite(pwald(Inf, 1, 1)))
  expect_true(!is.nan(pzip(Inf, 1, 0.5)) && is.finite(pzip(Inf, 1, 0.5)))
  expect_true(!is.nan(pzib(Inf, 1, 1, 0.5)) && is.finite(pzinb(Inf, 1, 1, 0.5)))
  expect_true(!is.nan(pzinb(Inf, 1, 1, 0.5)) && is.finite(pzinb(Inf, 1, 1, 0.5)))
  
})




test_that("Testing PDFs & PMFs against negatively infinite values", {
  
  expect_true(!is.nan(dbetapr(-Inf, 1, 1, 1)) && is.finite(dbetapr(-Inf, 1, 1, 1)))
  expect_true(!is.nan(dbbinom(-Inf, 10, 1, 1)) && is.finite(dbbinom(-Inf, 10, 1, 1)))
  expect_true(!is.nan(dbnbinom(-Inf, 10, 1, 1)) && is.finite(dbnbinom(-Inf, 10, 1, 1)))
  expect_true(!is.nan(dbhatt(-Inf)) && is.finite(dbhatt(-Inf)))
  expect_true(!is.nan(dfatigue(-Inf, 1)) && is.finite(dfatigue(-Inf, 1)))
  expect_true(!is.nan(dcat(-Inf, c(0.5, 0.5))) && is.finite(dcat(-Inf, c(0.5, 0.5))))
  expect_true(!is.nan(ddlaplace(-Inf, 0, 0.5)) && is.finite(ddlaplace(-Inf, 0, 0.5)))
  expect_true(!is.nan(ddnorm(-Inf)) && is.finite(ddnorm(-Inf)))
  expect_true(!is.nan(ddgamma(-Inf, 9, 1) && is.finite(ddgamma(-Inf, 9, 1))))
  expect_true(!is.nan(ddweibull(-Inf, 0.5, 1)) && is.finite(ddweibull(-Inf, 0.5, 1)))  
  expect_true(!is.nan(dfrechet(-Inf)) && is.finite(dfrechet(-Inf)))
  expect_true(!is.nan(dgpois(-Inf, 1, 1)) && is.finite(dgpois(-Inf, 1, 1)))
  expect_true(!is.nan(dgev(-Inf, 1, 1, 1)) && is.finite(dgev(-Inf, 1, 1, 1)))
  expect_true(!is.nan(dgompertz(-Inf, 1, 1)) && is.finite(dgompertz(-Inf, 1, 1)))
  expect_true(!is.nan(dgpd(-Inf, 1, 1, 1)) && is.finite(dgpd(-Inf, 1, 1, 1)))
  expect_true(!is.nan(dgumbel(-Inf)) && is.finite(dgumbel(-Inf)))
  expect_true(!is.nan(dhuber(-Inf)) && is.finite(dhuber(-Inf)))
  expect_true(!is.nan(dhcauchy(-Inf, 1)) && is.finite(dhcauchy(-Inf, 1)))
  expect_true(!is.nan(dhnorm(-Inf, 1)) && is.finite(dhnorm(-Inf, 1)))
  expect_true(!is.nan(dht(-Inf, 5, 1)) && is.finite(dht(-Inf, 5, 1)))
  expect_true(!is.nan(dinvgamma(-Inf, 1, 1)) && is.finite(dinvgamma(-Inf, 1, 1)))
  expect_true(!is.nan(dlaplace(-Inf)) && is.finite(dlaplace(-Inf)))
  expect_true(!is.nan(dlgser(-Inf, 0.5)) && is.finite(dlgser(-Inf, 0.5)))
  expect_true(!is.nan(dlomax(-Inf, 1, 1)) && is.finite(dlomax(-Inf, 1, 1)))
  expect_true(!is.nan(dmixnorm(-Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))) && is.finite(dmixnorm(-Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(dmixpois(-Inf, c(1,2,3), c(1/3,1/3,1/3))) && is.finite(dmixpois(-Inf, c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(dnhyper(-Inf, 60, 35, 15)) && is.finite(dnhyper(-Inf, 60, 35, 15)))
  expect_true(!is.nan(dlst(-Inf, df = 2)) && is.finite(dlst(-Inf, df = 2)))
  expect_true(!is.nan(dpareto(-Inf)) && is.finite(dpareto(-Inf)))
  expect_true(!is.nan(dpower(-Inf, 1, 1)) && is.finite(dpower(-Inf, 1, 1)))
  expect_true(!is.nan(dprop(-Inf, 10, 0.5)) && is.finite(dprop(-Inf, 10, 0.5)))
  expect_true(!is.nan(drayleigh(-Inf)) && is.finite(drayleigh(-Inf)))
  expect_true(!is.nan(dsgomp(-Inf, 0.4, 1)) && is.finite(dsgomp(-Inf, 0.4, 1)))
  expect_true(!is.nan(dslash(-Inf)) && is.finite(dslash(-Inf)))
  expect_true(!is.nan(dtpois(-Inf, lambda = 5)) && is.finite(dtpois(-Inf, lambda = 5)))
  expect_true(!is.nan(dtpois(-Inf, lambda = 5, a = 6)) && is.finite(dtpois(-Inf, lambda = 5, a = 6)))
  expect_true(!is.nan(dwald(-Inf, 1, 1)) && is.finite(dwald(-Inf, 1, 1)))
  expect_true(!is.nan(dzip(-Inf, 1, 0.5)) && is.finite(dzip(-Inf, 1, 0.5)))
  expect_true(!is.nan(dzib(-Inf, 1, 1, 0.5)) && is.finite(dzinb(-Inf, 1, 1, 0.5)))
  expect_true(!is.nan(dzinb(-Inf, 1, 1, 0.5)) && is.finite(dzinb(-Inf, 1, 1, 0.5)))
  
})


test_that("Testing CDFs against negatively infinite values", {
  
  expect_true(!is.nan(pbetapr(-Inf, 1, 1, 1)) && is.finite(pbetapr(-Inf, 1, 1, 1)))
  expect_true(!is.nan(pbbinom(-Inf, 10, 1, 1)) && is.finite(pbbinom(-Inf, 10, 1, 1)))
  expect_true(!is.nan(pbnbinom(-Inf, 10, 1, 1)) && is.finite(pbnbinom(-Inf, 10, 1, 1)))
  expect_true(!is.nan(pbhatt(-Inf)) && is.finite(pbhatt(-Inf)))
  expect_true(!is.nan(pfatigue(-Inf, 1)) && is.finite(pfatigue(-Inf, 1)))
  expect_true(!is.nan(pcat(-Inf, c(0.5, 0.5))) && is.finite(pcat(-Inf, c(0.5, 0.5))))
  expect_true(!is.nan(pdlaplace(-Inf, 0, 0.5)) && is.finite(pdlaplace(-Inf, 0, 0.5)))
  expect_true(!is.nan(pdnorm(-Inf)) && is.finite(pdnorm(-Inf)))
  expect_true(!is.nan(pdgamma(-Inf, 9, 1) && is.finite(pdgamma(-Inf, 9, 1))))
  expect_true(!is.nan(pdweibull(-Inf, 0.5, 1)) && is.finite(pdweibull(-Inf, 0.5, 1)))  
  expect_true(!is.nan(pfrechet(-Inf)) && is.finite(pfrechet(-Inf)))
  expect_true(!is.nan(pgpois(-Inf, 1, 1)) && is.finite(pgpois(-Inf, 1, 1)))
  expect_true(!is.nan(pgev(-Inf, 1, 1, 1)) && is.finite(pgev(-Inf, 1, 1, 1)))
  expect_true(!is.nan(pgompertz(-Inf, 1, 1)) && is.finite(pgompertz(-Inf, 1, 1)))
  expect_true(!is.nan(pgpd(-Inf, 1, 1, 1)) && is.finite(pgpd(-Inf, 1, 1, 1)))
  expect_true(!is.nan(pgumbel(-Inf)) && is.finite(pgumbel(-Inf)))
  expect_true(!is.nan(phuber(-Inf)) && is.finite(phuber(-Inf)))
  expect_true(!is.nan(phcauchy(-Inf, 1)) && is.finite(phcauchy(-Inf, 1)))
  expect_true(!is.nan(phnorm(-Inf, 1)) && is.finite(phnorm(-Inf, 1)))
  expect_true(!is.nan(pht(-Inf, 5, 1)) && is.finite(pht(-Inf, 5, 1)))
  expect_true(!is.nan(pinvgamma(-Inf, 1, 1)) && is.finite(pinvgamma(-Inf, 1, 1)))
  expect_true(!is.nan(plaplace(-Inf)) && is.finite(plaplace(-Inf)))
  expect_true(!is.nan(plgser(-Inf, 0.5)) && is.finite(plgser(-Inf, 0.5)))
  expect_true(!is.nan(plomax(-Inf, 1, 1)) && is.finite(plomax(-Inf, 1, 1)))
  expect_true(!is.nan(pmixnorm(-Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))) && is.finite(pmixnorm(-Inf, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(pmixpois(-Inf, c(1,2,3), c(1/3,1/3,1/3))) && is.finite(pmixpois(-Inf, c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(!is.nan(pnhyper(-Inf, 60, 35, 15)) && is.finite(pnhyper(-Inf, 60, 35, 15)))
  expect_true(!is.nan(plst(-Inf, df = 2)) && is.finite(plst(-Inf, df = 2)))
  expect_true(!is.nan(ppareto(-Inf)) && is.finite(ppareto(-Inf)))
  expect_true(!is.nan(ppower(-Inf, 1, 1)) && is.finite(ppower(-Inf, 1, 1)))
  expect_true(!is.nan(pprop(-Inf, 10, 0.5)) && is.finite(pprop(-Inf, 10, 0.5)))
  expect_true(!is.nan(prayleigh(-Inf)) && is.finite(prayleigh(-Inf)))
  expect_true(!is.nan(psgomp(-Inf, 0.4, 1)) && is.finite(psgomp(-Inf, 0.4, 1)))
  expect_true(!is.nan(pslash(-Inf)) && is.finite(pslash(-Inf)))
  expect_true(!is.nan(ptpois(-Inf, lambda = 5)) && is.finite(ptpois(-Inf, lambda = 5)))
  expect_true(!is.nan(ptpois(-Inf, lambda = 5, a = 6)) && is.finite(ptpois(-Inf, lambda = 5, a = 6)))
  expect_true(!is.nan(pwald(-Inf, 1, 1)) && is.finite(pwald(-Inf, 1, 1)))
  expect_true(!is.nan(pzip(-Inf, 1, 0.5)) && is.finite(pzip(-Inf, 1, 0.5)))
  expect_true(!is.nan(pzib(-Inf, 1, 1, 0.5)) && is.finite(pzinb(-Inf, 1, 1, 0.5)))
  expect_true(!is.nan(pzinb(-Inf, 1, 1, 0.5)) && is.finite(pzinb(-Inf, 1, 1, 0.5)))
  
})

test_that("Discrete probabilities sum to unity", {

  expect_equal(sum(ddunif(-2:10, -2, 10)), 1)
  
  expect_equal(sum(dbbinom(0:10, 10, 1, 1)), 1)
  expect_equal(sum(dbbinom(0:10, 10, 1, 10)), 1)
  expect_equal(sum(dbbinom(0:10, 10, 10, 1)), 1)
  expect_equal(sum(dbbinom(0:10, 10, 10, 10)), 1)
  
  expect_equal(sum(dbnbinom(0:1e5, 10, 1, 1)), 1, tolerance = 1e-3)
  expect_equal(sum(dbnbinom(0:1e5, 10, 1, 10)), 1, tolerance = 1e-3)
  expect_equal(sum(dbnbinom(0:1e5, 10, 10, 1)), 1, tolerance = 1e-3)
  expect_equal(sum(dbnbinom(0:1e5, 10, 10, 10)), 1, tolerance = 1e-3)
  
  expect_equal(sum(dbvpois(expand.grid(0:100, 0:100), a = 10, b = 10, c = 10)), 1, tolerance = 1e-3)
  
  divBySum <- function(x) x/sum(x, na.rm = TRUE)
  
  expect_equal(sum(dcat(1:10, divBySum(runif(10)))), 1)
  
  expect_equal(sum(ddlaplace(-50:50, 0, 0.5)), 1)
  expect_equal(sum(ddlaplace(-50:50, 0, 0.1)), 1)
  expect_equal(sum(ddlaplace(-50:50, 0, 0.7)), 1)
  
  expect_equal(sum(ddnorm(-50:50, sd = 5)), 1)
  expect_equal(sum(ddnorm(-50:50, sd = 7)), 1)
  
  
  expect_equal(sum(dnhyper(0:100, 60, 35, 15)), 1)
  
  expect_equal(sum(dgpois(0:100, 10, 1)), 1)
  
  expect_equal(sum(dmnom(expand.grid(0:30, 0:30, 0:30), 30, c(0.2, 0.1, 0.7))), 1)
  expect_equal(sum(ddirmnom(expand.grid(0:30, 0:30, 0:30), 30, c(2, 3, 8))), 1)
  expect_equal(sum(dmvhyper(expand.grid(0:30, 0:30, 0:30), c(20, 30, 28), 30)), 1)
  
  expect_equal(sum(dskellam(-100:100, 10, 20)), 1)
  
  expect_equal(sum(dtpois(0:100, 30, a = -Inf, b = Inf)), 1)
  expect_equal(sum(dtpois(0:100, 30, a = 0, b = Inf)), 1)
  expect_equal(sum(dtpois(0:100, 30, a = 25, b = Inf)), 1)
  expect_equal(sum(dtpois(0:100, 30, a = -Inf, b = 32)), 1)
  expect_equal(sum(dtpois(0:100, 30, a = 25, b = 32)), 1)
  
  expect_equal(sum(dtbinom(0:100, 100, 0.29, a = -Inf, b = Inf)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.29, a = 0, b = Inf)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.29, a = 25, b = Inf)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.29, a = -Inf, b = 32)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.29, a = 25, b = 32)), 1)
  
  expect_equal(sum(dtbinom(0:100, 100, 0.5, a = -Inf, b = Inf)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.5, a = 0, b = Inf)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.5, a = 25, b = Inf)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.5, a = -Inf, b = 32)), 1)
  expect_equal(sum(dtbinom(0:100, 100, 0.5, a = 25, b = 32)), 1)
  
  expect_equal(sum(dlgser(0:100, 0.6)), 1)
  expect_equal(sum(dlgser(0:100, 0.1)), 1)
  expect_equal(sum(dlgser(0:100, 0.8)), 1)
  
  
  expect_equal(sum(dmixpois(0:100, c(10, 20, 5), c(0.2, 0.5, 0.3))), 1)
  
  expect_equal(sum(dzib(0:1000, 30, 0.33, 0.4)), 1)
  expect_equal(sum(dzinb(0:1000, 30, 0.33, 0.4)), 1)
  expect_equal(sum(dzip(0:1000, 30, 0.4)), 1)
  
})

test_that("Fix for #16 works", {

  test_dat <- data.frame(
      q = c(1, 10, 10, 100),
      size = c(10, 100, 1000, 1000)
    )

  # This failed in #16
  testthat::expect_visible(with(
    test_dat,
    pbbinom(q, size, 1, 1)
  ))

  testthat::expect_visible(with(
    test_dat,
    pbnbinom(q, size, 1, 1)
  ))

})

test_that("Zero probabilities for non-integers", {
  
  expect_warning(expect_equal(0, ddlaplace(0.5, 0, 0.5)))
  expect_warning(expect_equal(0, ddnorm(0.5)))
  expect_warning(expect_equal(0, ddgamma(0.5, 9, 1)))
  expect_warning(expect_equal(0, ddweibull(0.5, 0.5, 1)))
  expect_warning(expect_equal(0, ddunif(0.5, 0, 5)))
  expect_warning(expect_equal(0, dcat(0.5, c(0.5, 0.5))))
  expect_warning(expect_equal(0, dmnom(c(0.5, 1), 5, c(0.5, 0.5))))
  expect_warning(expect_equal(0, dnhyper(0.5, 60, 35, 15)))
  expect_warning(expect_equal(0, dbern(0.5)))
  expect_warning(expect_equal(0, dbbinom(0.5, 1, 1, 1)))
  expect_warning(expect_equal(0, dbnbinom(0.5, 1, 1, 1)))
  expect_warning(expect_equal(0, dgpois(0.5, 1, 1)))
  expect_warning(expect_equal(0, dlgser(0.5, 0.5)))
  expect_warning(expect_equal(0, dmvhyper(c(1, 1, 0.5), c(2, 2, 2), 3)))
  expect_warning(expect_equal(0, dskellam(0.5, 1, 1)))
  expect_warning(expect_equal(0, dtpois(0.5, lambda = 25, a = 0)))
  expect_warning(expect_equal(0, dtbinom(0.5, 100, 0.56, a = 0)))
  expect_warning(expect_equal(0, dzip(0.5, 1, 0.5)))
  expect_warning(expect_equal(0, dzib(0.5, 1, 0.5, 0.5)))
  expect_warning(expect_equal(0, dzinb(0.5, 1, 0.5, 0.5)))
  expect_warning(expect_equal(0, dmixpois(0.5, c(1,2,3), c(1/3,1/3,1/3))))
  
})

test_that("cdf vs cumsum(pdf)", {
  
  xx <- seq(-500, 500, by = 1)
  epsilon <- 1e-4 # sqrt(.Machine$double.eps)
  
  expect_equal(cumsum(ddlaplace(xx, 0, 0.5)), pdlaplace(xx, 0, 0.5), tolerance = epsilon)
  expect_equal(cumsum(ddnorm(xx, 0, 15)), pdnorm(xx, 0, 15), tolerance = epsilon)
  
  xx <- seq(0, 200, by = 1)
  expect_equal(cumsum(ddweibull(xx, .32, 1)), pdweibull(xx, .32, 1), tolerance = epsilon)
  expect_equal(cumsum(ddunif(xx, 1, 199)), pdunif(xx, 1, 199), tolerance = epsilon)
  expect_equal(cumsum(ddgamma(xx, 9, 1)), pdgamma(xx, 9, 1), tolerance = epsilon)
  
  p <- rdirichlet(1, rep(1, 100))
  expect_equal(cumsum(dcat(xx, p)), pcat(xx, p), tolerance = epsilon)
  expect_equal(cumsum(dnhyper(xx, 60, 35, 15)), pnhyper(xx, 60, 35, 15), tolerance = epsilon)
  
  dnhyper2 <- function(x, n, m, r) ifelse(x<r | x>=(n+r), 0, choose(x-1, r-1)*choose(m+n-x, m-r)/choose(m+n, n))
  expect_equal(dnhyper2(xx, 60, 35, 15), dnhyper(xx, 60, 35, 15), tolerance = epsilon)
  expect_equal(cumsum(dnhyper2(xx, 60, 35, 15)), pnhyper(xx, 60, 35, 15), tolerance = epsilon)
  
  expect_equal(cumsum(dtbinom(xx, 200, 0.5, a = 100)), ptbinom(xx, 200, 0.5, a = 100), tolerance = epsilon)
  expect_equal(cumsum(dtbinom(xx, 200, 0.5, b = 100)), ptbinom(xx, 200, 0.5, b = 100), tolerance = epsilon)
  
  expect_equal(cumsum(dbbinom(xx, 200, 5, 13)), pbbinom(xx, 200, 5, 13), tolerance = epsilon)
  expect_equal(cumsum(dbnbinom(xx, 70, 5, 13)), pbnbinom(xx, 70, 5, 13), tolerance = epsilon)
  expect_equal(cumsum(dgpois(xx, 500, 16)), pgpois(xx, 500, 16), tolerance = epsilon)
  
  expect_equal(cumsum(dlgser(xx, 0.9)), plgser(xx, 0.9), tolerance = epsilon)
  expect_equal(cumsum(dtpois(xx, 100, a = 80, b = 150)), ptpois(xx, 100, a = 80, b = 150), tolerance = epsilon)
  expect_equal(cumsum(dtbinom(xx, 100, 0.5, a = 80, b = 150)), ptbinom(xx, 100, 0.5, a = 80, b = 150), tolerance = epsilon)
  
  expect_equal(cumsum(dzip(xx, 70, 0.5)), pzip(xx, 70, 0.5), tolerance = epsilon)
  expect_equal(cumsum(dzib(xx, 200, 0.5, 0.5)), pzib(xx, 200, 0.5, 0.5), tolerance = epsilon)
  expect_equal(cumsum(dzinb(xx, 70, 0.5, 0.5)), pzinb(xx, 70, 0.5, 0.5), tolerance = epsilon)
  expect_equal(cumsum(dmixpois(xx, c(40,50,70), c(1/3,1/3,1/3))), pmixpois(xx, c(40,50,70), c(1/3,1/3,1/3)), tolerance = epsilon)
  
})

test_that("Check if log-probabilities are logs of probabilities (PMF's and PDF's)", {
  
  x <- c(-Inf, -100, -10, -5, -1, -0.5, 0, 0.5, 1, 5, 10, 100, Inf)
  
  expect_equal(suppressWarnings(dbbinom(x, 1, 1, 1, log = TRUE)),
               log(suppressWarnings(dbbinom(x, 1, 1, 1))))
  expect_equal(suppressWarnings(dbern(x, 0.5, log = TRUE)),
               log(suppressWarnings(dbern(x, 0.5))))
  expect_equal(suppressWarnings(dbetapr(x, 1, 1, 1, log = TRUE)),
               log(suppressWarnings(dbetapr(x, 1, 1, 1))))
  expect_equal(dbhatt(x, sigma = 1, log = TRUE),
               log(dbhatt(x, sigma = 1)))
  expect_equal(suppressWarnings(dbnbinom(x, 1, 1, 1, log = TRUE)),
               log(suppressWarnings(dbnbinom(x, 1, 1, 1))))
  expect_equal(dbvnorm(x, x, sd1 = 1, log = TRUE),
               log(dbvnorm(x, x, sd1 = 1)))
  expect_equal(suppressWarnings(dbvpois(x, x, 1, 1, 1, log = TRUE)),
               log(suppressWarnings(dbvpois(x, x, 1, 1, 1))))
  expect_equal(suppressWarnings(dcat(x, c(0.5, 0.5), log = TRUE)),
               log(suppressWarnings(dcat(x, c(0.5, 0.5)))))
  expect_equal(ddirichlet(c(0.5, 0.5), c(1, 0.5), log = TRUE),
               log(ddirichlet(c(0.5, 0.5), c(1, 0.5))))
  expect_equal(suppressWarnings(ddlaplace(x, 0, scale = 0.5, log = TRUE)),
               log(suppressWarnings(ddlaplace(x, 0, scale = 0.5))))
  expect_equal(suppressWarnings(ddnorm(x, sd = 1, log = TRUE)),
               log(suppressWarnings(ddnorm(x, sd = 1))))
  expect_equal(suppressWarnings(ddgamma(x, 9, 1, log = TRUE)),
               log(suppressWarnings(ddgamma(x, 9, 1))))
  expect_equal(suppressWarnings(ddunif(x, min = 10, max = 100, log = TRUE)),
               log(suppressWarnings(ddunif(x, min = 10, max = 100))))
  expect_equal(suppressWarnings(ddweibull(x, 0.5, 1, log = TRUE)),
               log(suppressWarnings(ddweibull(x, 0.5, 1))))
  expect_equal(dfatigue(x, 1, 1, log = TRUE),
               log(dfatigue(x, 1, 1)))
  expect_equal(dfrechet(x, lambda = 1, log = TRUE),
               log(dfrechet(x, lambda = 1)))
  expect_equal(dgev(x, 1, 1, 1, log = TRUE),
               log(dgev(x, 1, 1, 1)))
  # expect_equal(dgompertz(x, 1, 1, log = TRUE),
  #              log(dgompertz(x, 1, 1)))
  expect_equal(dgpd(x, 1, 1, 1, log = TRUE),
               log(dgpd(x, 1, 1, 1)))
  expect_equal(suppressWarnings(dgpois(x, 1, 1, log = TRUE)),
               log(suppressWarnings(dgpois(x, 1, 1))))
  # expect_equal(dgumbel(x, sigma = 1, log = TRUE),
  #              log(dgumbel(x, sigma = 1)))
  expect_equal(dhcauchy(x, 1, log = TRUE),
               log(dhcauchy(x, 1)))
  # expect_equal(dhnorm(x, 1, log = TRUE),
  #              log(dhnorm(x, 1)))  # numerical precission
  expect_equal(dht(x, 5, 1, log = TRUE),
               log(dht(x, 5, 1)))
  expect_equal(dhuber(x, 0, 1, 1, log = TRUE),
               log(dhuber(x, 0, 1, 1)))
  expect_equal(dinvgamma(x, 1, 1, log = TRUE),
               log(dinvgamma(x, 1, 1)))
  expect_equal(dinvchisq(x, 1, 1, log = TRUE),
               log(dinvchisq(x, 1, 1)))
  expect_equal(dkumar(x, 1, 1, log = TRUE),
               log(dkumar(x, 1, 1)))
  expect_equal(dlaplace(x, 0, 1, log = TRUE),
               log(dlaplace(x, 0, 1)))
  expect_equal(suppressWarnings(dlgser(x, 0.5, log = TRUE)),
               log(suppressWarnings(dlgser(x, 0.5))))
  expect_equal(dlomax(x, 1, 1, log = TRUE),
               log(dlomax(x, 1, 1)))
  expect_equal(dmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3), log = TRUE),
               log(dmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_equal(suppressWarnings(dmixpois(x, c(1,2,3), c(1/3,1/3,1/3), log = TRUE)),
               log(suppressWarnings(dmixpois(x, c(1,2,3), c(1/3,1/3,1/3)))))
  expect_equal(suppressWarnings(dnhyper(x, 60, 35, 15, log = TRUE)),
               log(suppressWarnings(dnhyper(x, 60, 35, 15))))
  expect_equal(ddirmnom(c(1, 1, 1), 2, c(1, 1, 1), log = TRUE),
               log(ddirmnom(c(1, 1, 1), 2, c(1, 1, 1))))
  expect_equal(dmnom(c(1, 1, 1), 2, c(1/3, 1/3, 1/3), log = TRUE),
               log(dmnom(c(1, 1, 1), 2, c(1/3, 1/3, 1/3))))
  expect_equal(dmvhyper(c(1, 2, 2), c(2,3,4), 5, log = TRUE),
               log(dmvhyper(c(1, 2, 2), c(2,3,4), 5)))
  expect_equal(dnsbeta(x, 1, 1, -2, 2, log = TRUE),
               log(dnsbeta(x, 1, 1, -2, 2)))
  expect_equal(dlst(x, 2, 0, 1, log = TRUE),
               log(dlst(x, 2, 0, 1)))
  expect_equal(dpareto(x, 1, 1, log = TRUE),
               log(dpareto(x, 1, 1)))
  expect_equal(dprop(x, 10, 0.5, log = TRUE),
               log(dprop(x, 10, 0.5)))
  # expect_equal(drayleigh(x, 1, log = TRUE),
  #              log(drayleigh(x, 1)))
  expect_equal(suppressWarnings(dskellam(x, 1, 1, log = TRUE)),
               log(suppressWarnings(dskellam(x, 1, 1))))
  expect_equal(dsgomp(x, 0.4, 1, log = TRUE),
               log(dsgomp(x, 0.4, 1)))
  expect_equal(dslash(x, sigma = 1, log = TRUE),
               log(dslash(x, sigma = 1)))
  expect_equal(dtnorm(x, 0, 1, 1, 2, log = TRUE), 
               log(dtnorm(x, 0, 1, 1, 2)))
  expect_equal(suppressWarnings(dtpois(x, lambda = 25, a = 0, log = TRUE)),
               log(suppressWarnings(dtpois(x, lambda = 25, a = 0))))
  expect_equal(suppressWarnings(dtbinom(x, 100, 0.67, a = 60, b = 70, log = TRUE)),
               log(suppressWarnings(dtbinom(x, 100, 0.67, a = 60, b = 70))))
  expect_equal(dtriang(x, 1, 2, 1.5, log = TRUE),
               log(dtriang(x, 1, 2, 1.5)))
  expect_equal(dwald(x, 1, 1, log = TRUE),
               log(dwald(x, 1, 1)))
  expect_equal(suppressWarnings(dzip(x, 1, 0.5, log = TRUE)),
               log(suppressWarnings(dzip(x, 1, 0.5))))
  expect_equal(suppressWarnings(dzib(x, 1, 0.5, 0.5, log = TRUE)),
               log(suppressWarnings(dzib(x, 1, 0.5, 0.5))))
  expect_equal(suppressWarnings(dzinb(x, 1, 0.5, 0.5, log = TRUE)),
               log(suppressWarnings(dzinb(x, 1, 0.5, 0.5))))
  
})


test_that("Check if log-probabilities are logs of probabilities (CDF's)", {
  
  x <- c(-Inf, -100, -10, -5, -1, -0.5, 0, 0.5, 1, 5, 10, 100, Inf)
  
  expect_equal(suppressWarnings(pbbinom(x, 1, 1, 1, log.p = TRUE)),
               log(suppressWarnings(pbbinom(x, 1, 1, 1))))
  expect_equal(suppressWarnings(pbern(x, 0.5, log.p = TRUE)),
               log(suppressWarnings(pbern(x, 0.5))))
  expect_equal(suppressWarnings(pbetapr(x, 1, 1, 1, log.p = TRUE)),
               log(suppressWarnings(pbetapr(x, 1, 1, 1))))
  expect_equal(pbhatt(x, sigma = 1, log.p = TRUE),
               log(pbhatt(x, sigma = 1)))
  expect_equal(suppressWarnings(pbnbinom(x, 1, 1, 1, log.p = TRUE)),
               log(suppressWarnings(pbnbinom(x, 1, 1, 1))))
  expect_equal(suppressWarnings(pcat(x, c(0.5, 0.5), log.p = TRUE)),
               log(suppressWarnings(pcat(x, c(0.5, 0.5)))))
  expect_equal(suppressWarnings(pdlaplace(x, 0, scale = 0.5, log.p = TRUE)),
               log(suppressWarnings(pdlaplace(x, 0, scale = 0.5))))
  # expect_equal(suppressWarnings(pdnorm(x, sd = 1, log.p = TRUE)),
  #              log(suppressWarnings(pdnorm(x, sd = 1))))
  expect_equal(suppressWarnings(pdgamma(x, 9, 1, log.p = TRUE)),
               log(suppressWarnings(pdgamma(x, 9, 1))))
  expect_equal(suppressWarnings(pdunif(x, min = 10, max = 100, log.p = TRUE)),
               log(suppressWarnings(pdunif(x, min = 10, max = 100))))
  expect_equal(suppressWarnings(pdweibull(x, 0.5, 1, log.p = TRUE)),
               log(suppressWarnings(pdweibull(x, 0.5, 1))))
  expect_equal(pfatigue(x, 1, 1, log.p = TRUE),
               log(pfatigue(x, 1, 1)))
  expect_equal(pfrechet(x, lambda = 1, log.p = TRUE),
               log(pfrechet(x, lambda = 1)))
  expect_equal(pgev(x, 1, 1, 1, log.p = TRUE),
               log(pgev(x, 1, 1, 1)))
  # expect_equal(pgompertz(x, 1, 1, log.p = TRUE),
  #              log(pgompertz(x, 1, 1)))
  expect_equal(pgpd(x, 1, 1, 1, log.p = TRUE),
               log(pgpd(x, 1, 1, 1)))
  expect_equal(suppressWarnings(pgpois(x, 1, 1, log.p = TRUE)),
               log(suppressWarnings(pgpois(x, 1, 1))))
  # expect_equal(pgumbel(x, sigma = 1, log.p = TRUE),
  #              log(pgumbel(x, sigma = 1)))
  expect_equal(phcauchy(x, 1, log.p = TRUE),
               log(phcauchy(x, 1)))
  expect_equal(phnorm(x, 1, log.p = TRUE),
               log(phnorm(x, 1)))
  expect_equal(pht(x, 5, 1, log.p = TRUE),
               log(pht(x, 5, 1)))
  expect_equal(phuber(x, 0, 1, 1, log.p = TRUE),
               log(phuber(x, 0, 1, 1)))
  expect_equal(pinvgamma(x, 1, 1, log.p = TRUE),
               log(pinvgamma(x, 1, 1)))
  expect_equal(pinvchisq(x, 1, 1, log.p = TRUE),
               log(pinvchisq(x, 1, 1)))
  expect_equal(pkumar(x, 1, 1, log.p = TRUE),
               log(pkumar(x, 1, 1)))
  expect_equal(plaplace(x, 0, 1, log.p = TRUE),
               log(plaplace(x, 0, 1)))
  expect_equal(suppressWarnings(plgser(x, 0.5, log.p = TRUE)),
               log(suppressWarnings(plgser(x, 0.5))))
  expect_equal(plomax(x, 1, 1, log.p = TRUE),
               log(plomax(x, 1, 1)))
  expect_equal(pmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3), log.p = TRUE),
               log(pmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_equal(suppressWarnings(pmixpois(x, c(1,2,3), c(1/3,1/3,1/3), log.p = TRUE)),
               log(suppressWarnings(pmixpois(x, c(1,2,3), c(1/3,1/3,1/3)))))
  expect_equal(suppressWarnings(pnhyper(x, 60, 35, 15, log.p = TRUE)),
               log(suppressWarnings(pnhyper(x, 60, 35, 15))))
  expect_equal(pnsbeta(x, 1, 1, -2, 2, log.p = TRUE),
               log(pnsbeta(x, 1, 1, -2, 2)))
  expect_equal(plst(x, 2, 0, 1, log.p = TRUE),
               log(plst(x, 2, 0, 1)))
  expect_equal(ppareto(x, 1, 1, log.p = TRUE),
               log(ppareto(x, 1, 1)))
  expect_equal(pprop(x, 10, 0.5, log.p = TRUE),
               log(pprop(x, 10, 0.5)))
  # expect_equal(prayleigh(x, 1, log.p = TRUE),
  #              log(prayleigh(x, 1)))
  expect_equal(psgomp(x, 0.4, 1, log.p = TRUE),
               log(psgomp(x, 0.4, 1)))
  expect_equal(pslash(x, sigma = 1, log.p = TRUE),
               log(pslash(x, sigma = 1)))
  expect_equal(ptnorm(x, 0, 1, 1, 2, log.p = TRUE), 
               log(ptnorm(x, 0, 1, 1, 2)))
  expect_equal(suppressWarnings(ptpois(x, lambda = 25, a = 0, log.p = TRUE)),
               log(suppressWarnings(ptpois(x, lambda = 25, a = 0))))
  expect_equal(suppressWarnings(ptbinom(x, 100, 0.67, a = 60, b = 70, log.p = TRUE)),
               log(suppressWarnings(ptbinom(x, 100, 0.67, a = 60, b = 70))))
  expect_equal(ptriang(x, 1, 2, 1.5, log.p = TRUE),
               log(ptriang(x, 1, 2, 1.5)))
  expect_equal(pwald(x, 1, 1, log.p = TRUE),
               log(pwald(x, 1, 1)))
  expect_equal(suppressWarnings(pzip(x, 1, 0.5, log.p = TRUE)),
               log(suppressWarnings(pzip(x, 1, 0.5))))
  expect_equal(suppressWarnings(pzib(x, 1, 0.5, 0.5, log.p = TRUE)),
               log(suppressWarnings(pzib(x, 1, 0.5, 0.5))))
  expect_equal(suppressWarnings(pzinb(x, 1, 0.5, 0.5, log.p = TRUE)),
               log(suppressWarnings(pzinb(x, 1, 0.5, 0.5))))
  
})

test_that("All probabilities/densities >= 0", {

  x <- c(-Inf, -100, -10, -5, -1, -0.5, 0, 0.5, 1, 5, 10, 100, Inf)
  
  expect_true(suppressWarnings(all(dbbinom(x, 1, 1, 1) >= 0)))
  expect_true(suppressWarnings(all(dbern(x, 0.5) >= 0)))
  expect_true(suppressWarnings(all(dbetapr(x, 1, 1, 1) >= 0)))
  expect_true(all(dbhatt(x, sigma = 1) >= 0))
  expect_true(suppressWarnings(all(dbnbinom(x, 1, 1, 1) >= 0)))
  expect_true(all(dbvnorm(x, x, sd1 = 1) >= 0))
  expect_true(suppressWarnings(all(dbvpois(x, x, 1, 1, 1) >= 0)))
  expect_true(suppressWarnings(all(dcat(x, c(0.5, 0.5)) >= 0)))
  expect_true(all(ddirichlet(c(0.5, 0.5), c(1, 0.5)) >= 0))
  expect_true(suppressWarnings(all(ddlaplace(x, 0, scale = 0.5) >= 0)))
  expect_true(suppressWarnings(all(ddnorm(x, sd = 1) >= 0)))
  expect_true(suppressWarnings(all(ddgamma(x, 9, 1) >= 0)))
  expect_true(suppressWarnings(all(ddunif(x, min = 10, max = 100) >= 0)))
  expect_true(suppressWarnings(all(ddweibull(x, 0.5, 1) >= 0)))
  expect_true(all(dfatigue(x, 1, 1) >= 0))
  expect_true(all(dfrechet(x, lambda = 1) >= 0))
  expect_true(all(dgev(x, 1, 1, 1) >= 0))
  expect_true(all(dgompertz(x, 1, 1) >= 0))
  expect_true(all(dgpd(x, 1, 1, 1) >= 0))
  expect_true(suppressWarnings(all(dgpois(x, 1, 1) >= 0)))
  expect_true(all(dgumbel(x, sigma = 1) >= 0))
  expect_true(all(dhcauchy(x, 1) >= 0))
  expect_true(all(dhnorm(x, 1) >= 0))
  expect_true(all(dht(x, 5, 1) >= 0))
  expect_true(all(dhuber(x, 0, 1, 1) >= 0))
  expect_true(all(dinvgamma(x, 1, 1) >= 0))
  expect_true(all(dinvchisq(x, 1, 1) >= 0))
  expect_true(all(dkumar(x, 1, 1) >= 0))
  expect_true(all(dlaplace(x, 0, 1) >= 0))
  expect_true(suppressWarnings(all(dlgser(x, 0.5) >= 0)))
  expect_true(all(dlomax(x, 1, 1) >= 0))
  expect_true(all(dmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3)) >= 0))
  expect_true(suppressWarnings(all(dmixpois(x, c(1,2,3), c(1/3,1/3,1/3)) >= 0)))
  expect_true(suppressWarnings(all(dnhyper(x, 60, 35, 15) >= 0)))
  expect_true(all(ddirmnom(c(1, 1, 1), 2, c(1, 1, 1)) >= 0))
  expect_true(all(dmnom(c(1, 1, 1), 2, c(1/3, 1/3, 1/3)) >= 0))
  expect_true(all(dmvhyper(c(1, 2, 2), c(2,3,4), 5) >= 0))
  expect_true(all(dnsbeta(x, 1, 1, -2, 2) >= 0))
  expect_true(all(dlst(x, 2, 0, 1) >= 0))
  expect_true(all(dpareto(x, 1, 1) >= 0))
  expect_true(all(dprop(x, 10, 0.5) >= 0))
  expect_true(all(drayleigh(x, 1) >= 0))
  expect_true(suppressWarnings(all(dskellam(x, 1, 1) >= 0)))
  expect_true(all(dsgomp(x, 0.4, 1) >= 0))
  expect_true(all(dslash(x, sigma = 1) >= 0))
  expect_true(all(dtnorm(x, 0, 1, 1, 2) >= 0))
  expect_true(suppressWarnings(all(dtpois(x, lambda = 25, a = 0) >= 0)))
  expect_true(suppressWarnings(all(dtbinom(x, 100, 0.67, a = 60, b = 70) >= 0)))
  expect_true(all(dtriang(x, 1, 2, 1.5) >= 0))
  expect_true(all(dwald(x, 1, 1) >= 0))
  expect_true(suppressWarnings(all(dzip(x, 1, 0.5) >= 0)))
  expect_true(suppressWarnings(all(dzib(x, 1, 0.5, 0.5) >= 0)))
  expect_true(suppressWarnings(all(dzinb(x, 1, 0.5, 0.5) >= 0)))
  
})




test_that("All cumulative probabilities >= 0 and <= 1", {
  
  x <- c(-Inf, -100, -10, -5, -1, -0.5, 0, 0.5, 1, 5, 10, 100, Inf)
  
  expect_true(all(pbbinom(x, 1, 1, 1) >= 0 & pbbinom(x, 1, 1, 1) <= 1))
  expect_true(all(pbern(x, 0.5) >= 0 & pbern(x, 0.5) <= 1))
  expect_true(all(pbetapr(x, 1, 1, 1) >= 0 & pbetapr(x, 1, 1, 1) <= 1))
  expect_true(all(pbhatt(x, sigma = 1) >= 0 & pbhatt(x, sigma = 1) <= 1))
  expect_true(all(pbnbinom(x, 1, 1, 1) >= 0 & pbnbinom(x, 1, 1, 1) <= 1))
  # expect_true(all(pbvnorm(x, x, sd1 = 1) >= 0))
  # expect_true(all(pbvpois(x, x, 1, 1, 1) >= 0))
  expect_true(all(pcat(x, c(0.5, 0.5)) >= 0 & pcat(x, c(0.5, 0.5)) <= 1))
  # expect_true(all(pdirichlet(c(0.5, 0.5), c(1, 0.5)) >= 0))
  expect_true(all(pdlaplace(x, 0, scale = 0.5) >= 0 & pdlaplace(x, 0, scale = 0.5) <= 1))
  expect_true(all(pdnorm(x, sd = 50) >= 0 & pdnorm(x, sd = 50) <= 1))
  expect_true(all(pdgamma(x, 9, 1) >= 0 & pdgamma(x, 9, 1) <= 1))
  expect_true(all(pdunif(x, min = 10, max = 100) >= 0 & pdunif(x, min = 10, max = 100) <= 1))
  expect_true(all(pdweibull(x, 0.5, 1) >= 0 & pdweibull(x, 0.5, 1) <= 1))
  expect_true(all(pfatigue(x, 1, 1) >= 0 & pfatigue(x, 1, 1) <= 1))
  expect_true(all(pfrechet(x, lambda = 1) >= 0 & pfrechet(x, lambda = 1) <= 1))
  expect_true(all(pgev(x, 1, 1, 1) >= 0 & pgev(x, 1, 1, 1) <= 1))
  expect_true(all(pgompertz(x, 1, 1) >= 0 & pgompertz(x, 1, 1) <= 1))
  expect_true(all(pgpd(x, 1, 1, 1) >= 0 & pgpd(x, 1, 1, 1) <= 1))
  expect_true(all(pgpois(x, 1, 1) >= 0 & pgpois(x, 1, 1) <= 1))
  expect_true(all(pgumbel(x, sigma = 1) >= 0 & pgumbel(x, sigma = 1) <= 1))
  expect_true(all(phcauchy(x, 1) >= 0 & phcauchy(x, 1) <= 1))
  expect_true(all(phnorm(x, 1) >= 0 & phnorm(x, 1) <= 1))
  expect_true(all(pht(x, 5, 1) >= 0 & pht(x, 5, 1) <= 1))
  expect_true(all(phuber(x, 0, 1, 1) >= 0 & phuber(x, 0, 1, 1) <= 1))
  expect_true(all(pinvgamma(x, 1, 1) >= 0 & pinvgamma(x, 1, 1) <= 1))
  expect_true(all(pinvchisq(x, 1, 1) >= 0 & pinvchisq(x, 1, 1) <= 1))
  expect_true(all(pkumar(x, 1, 1) >= 0 & pkumar(x, 1, 1) <= 1))
  expect_true(all(plaplace(x, 0, 1) >= 0 & plaplace(x, 0, 1) <= 1))
  expect_true(all(plgser(x, 0.5) >= 0 & plgser(x, 0.5) <= 1))
  expect_true(all(plomax(x, 1, 1) >= 0 & plomax(x, 1, 1) <= 1))
  expect_true(all(pmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3)) >= 0 & pmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3)) <= 1))
  expect_true(all(pmixpois(x, c(1,2,3), c(1/3,1/3,1/3)) >= 0 & pmixpois(x, c(1,2,3), c(1/3,1/3,1/3)) <= 1))
  expect_true(all(pnhyper(x, 60, 35, 15) >= 0 & pnhyper(x, 60, 35, 15) <= 1))
  # expect_true(all(pdirmnom(c(1, 1, 1), 2, c(1, 1, 1)) >= 0))
  # expect_true(all(pmnom(c(1, 1, 1), 2, c(1/3, 1/3, 1/3)) >= 0))
  # expect_true(all(pmvhyper(c(1, 2, 2), c(2,3,4), 5) >= 0))
  expect_true(all(pnsbeta(x, 1, 1, -2, 2) >= 0 & pnsbeta(x, 1, 1, -2, 2) <= 1))
  expect_true(all(plst(x, 2, 0, 1) >= 0 & plst(x, 2, 0, 1) <= 1))
  expect_true(all(ppareto(x, 1, 1) >= 0 & ppareto(x, 1, 1) <= 1))
  expect_true(all(pprop(x, 10, 0.5) >= 0 & pprop(x, 10, 0.5) <= 1))
  expect_true(all(prayleigh(x, 1) >= 0 & prayleigh(x, 1) <= 1))
  # expect_true(all(pskellam(x, 1, 1) >= 0))
  expect_true(all(psgomp(x, 0.4, 1) >= 0 & psgomp(x, 0.4, 1) <= 1))
  expect_true(all(pslash(x, sigma = 1) >= 0 & pslash(x, sigma = 1) <= 1))
  expect_true(all(ptnorm(x, 0, 1, 1, 2) >= 0 & ptnorm(x, 0, 1, 1, 2) <= 1))
  expect_true(all(ptpois(x, lambda = 25, a = 0) >= 0 & ptpois(x, lambda = 25, a = 0) <= 1))
  expect_true(all(ptbinom(x, 100, 0.67, 60, 70) >= 0 & ptbinom(x, 100, 0.67, 60, 70) <= 1))
  expect_true(all(ptbinom(x, 100, 0.67, a = 60, b = 70) >= 0 & ptbinom(x, 100, 0.67, a = 60, b = 70) <= 1))
  expect_true(all(ptriang(x, 1, 2, 1.5) >= 0 & ptriang(x, 1, 2, 1.5) <= 1))
  expect_true(all(pwald(x, 1, 1) >= 0 & pwald(x, 1, 1) <= 1))
  expect_true(all(pzip(x, 1, 0.5) >= 0 & pzip(x, 1, 0.5) <= 1))
  expect_true(all(pzib(x, 1, 0.5, 0.5) >= 0 & pzib(x, 1, 0.5, 0.5) <= 1))
  expect_true(all(pzinb(x, 1, 0.5, 0.5) >= 0 & pzinb(x, 1, 0.5, 0.5) <= 1))
  
})

#' Power distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the power distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param alpha,beta      parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{\beta x^{\beta-1}}{\alpha^\beta}
#' }{
#' f(x) = (\beta*x^(\beta-1)) / (\alpha^\beta)
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \frac{x^\beta}{\alpha^\beta}
#' }{
#' F(x) = x^\beta / \alpha^\beta
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \alpha p^{1/\beta}
#' }{
#' F^-1(p) = \alpha * p^(1/\beta)
#' }
#' 
#' @examples 
#' 
#' x <- rpower(1e5, 5, 16)
#' hist(x, 100, freq = FALSE)
#' curve(dpower(x, 5, 16), 2, 6, col = "red", add = TRUE, n = 5000)
#' hist(ppower(x, 5, 16))
#' plot(ecdf(x))
#' curve(ppower(x, 5, 16), 2, 6, col = "red", lwd = 2, add = TRUE)
#'
#' @name PowerDist
#' @aliases PowerDist
#' @aliases dpower
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dpower <- function(x, alpha, beta, log = FALSE) {
  cpp_dpower(x, alpha, beta, log[1L])
}


#' @rdname PowerDist
#' @export

ppower <- function(q, alpha, beta, lower.tail = TRUE, log.p = FALSE) {
  cpp_ppower(q, alpha, beta, lower.tail[1L], log.p[1L])
}


#' @rdname PowerDist
#' @export

qpower <- function(p, alpha, beta, lower.tail = TRUE, log.p = FALSE) {
  cpp_qpower(p, alpha, beta, lower.tail[1L], log.p[1L])
}


#' @rdname PowerDist
#' @export

rpower <- function(n, alpha, beta) {
  if (length(n) > 1) n <- length(n)
  cpp_rpower(n, alpha, beta)
}

test_that("Zeros in quantile functions", {
  
  expect_true(!is.nan(qbetapr(0, 1, 1, 1)))
  expect_true(!is.nan(qfatigue(0, 1)))
  expect_true(!is.nan(qcat(0, c(0.5, 0.5))))
  expect_true(!is.nan(qdweibull(0, 0.5, 1)))  
  expect_true(!is.nan(qfrechet(0)))
  expect_true(!is.nan(qgev(0, 1, 1, 1)))
  expect_true(!is.nan(qgompertz(0, 1, 1)))
  expect_true(!is.nan(qgpd(0, 1, 1, 1)))
  expect_true(!is.nan(qgumbel(0)))
  expect_true(!is.nan(qhuber(0)))
  expect_true(!is.nan(qhcauchy(0, 1)))
  expect_true(!is.nan(qhnorm(0, 1)))
  expect_true(!is.nan(qht(0, 5, 1)))
  expect_true(!is.nan(qinvgamma(0, 1, 1)))
  expect_true(!is.nan(qlaplace(0)))
  expect_true(!is.nan(qlgser(0, 0.5)))
  expect_true(!is.nan(qlomax(0, 1, 1)))
  expect_true(!is.nan(qnhyper(0, 60, 35, 15)))
  expect_true(!is.nan(qlst(0, df = 2)))
  expect_true(!is.nan(qpareto(0)))
  expect_true(!is.nan(qpower(0, 1, 1)))
  expect_true(!is.nan(qprop(0, 10, 0.5)))
  expect_true(!is.nan(qrayleigh(0)))
  expect_true(!is.nan(qtlambda(0, 0.5)))
  expect_true(!is.nan(qtbinom(0, 100, 0.83, 76, 86)))
  expect_true(!is.nan(qzip(0, 1, 0.5)))
  expect_true(!is.nan(qzib(0, 1, 1, 0.5)))
  expect_true(!is.nan(qzinb(0, 1, 1, 0.5)))
  
  expect_true(!is.nan(qtpois(0, lambda = 5, a = 0)))
  expect_true(!is.nan(qtpois(0, lambda = 5, a = 6)))
  
  expect_true(!is.nan(pdgamma(0, 9, 1)))
  expect_true(!is.nan(pdnorm(0, 1, 2)))
  
})

test_that("Ones in quantile functions", {
  
  expect_true(!is.nan(qbetapr(1, 1, 1, 1)))
  expect_true(!is.nan(qfatigue(1, 1)))
  expect_true(!is.nan(qcat(1, c(0.5, 0.5))))
  expect_true(!is.nan(qdweibull(1, 0.5, 1)))  
  expect_true(!is.nan(qfrechet(1)))
  expect_true(!is.nan(qgev(1, 1, 1, 1)))
  expect_true(!is.nan(qgompertz(1, 1, 1)))
  expect_true(!is.nan(qgpd(1, 1, 1, 1)))
  expect_true(!is.nan(qgumbel(1)))
  expect_true(!is.nan(qhuber(1)))
  expect_true(!is.nan(qhcauchy(1, 1)))
  expect_true(!is.nan(qhnorm(1, 1)))
  expect_true(!is.nan(qht(1, 5, 1)))
  expect_true(!is.nan(qinvgamma(1, 1, 1)))
  expect_true(!is.nan(qlaplace(1)))
  expect_true(!is.nan(qlgser(1, 0.5)))
  expect_true(!is.nan(qlomax(1, 1, 1)))
  expect_true(!is.nan(qnhyper(1, 60, 35, 15)))
  expect_true(!is.nan(qlst(1, df = 2)))
  expect_true(!is.nan(qpareto(1)))
  expect_true(!is.nan(qpower(1, 1, 1)))
  expect_true(!is.nan(qprop(1, 10, 0.5)))
  expect_true(!is.nan(qrayleigh(1)))
  expect_true(!is.nan(qtlambda(1, 0.5)))
  expect_true(!is.nan(qtbinom(1, 100, 0.83, 76, 86)))
  expect_true(!is.nan(qzip(1, 1, 0.5)))
  expect_true(!is.nan(qzib(1, 1, 1, 0.5)))
  expect_true(!is.nan(qzinb(1, 1, 1, 0.5)))
  
  expect_true(!is.nan(qtpois(1, lambda = 5, a = 0)))
  expect_true(!is.nan(qtpois(1, lambda = 5, a = 6)))
  
})



test_that("Checking p = F(F^-1(p))", {
  
  pp <- seq(0, 1, by = 0.001)
  
  expect_equal(pp, pbetapr(qbetapr(pp, 1, 1, 1), 1, 1, 1))
  expect_equal(pp, pfatigue(qfatigue(pp, 1), 1))
  expect_equal(pp, pfrechet(qfrechet(pp)))
  expect_equal(pp, pgev(qgev(pp, 1, 1, 1), 1, 1, 1))
  expect_equal(pp, pgompertz(qgompertz(pp, 1, 1), 1, 1))
  expect_equal(pp, pgpd(qgpd(pp, 1, 1, 1), 1, 1, 1))
  expect_equal(pp, pgumbel(qgumbel(pp)))
  expect_equal(pp, phuber(qhuber(pp)))
  
  expect_equal(pp, phcauchy(qhcauchy(pp, 1), 1))
  expect_equal(pp, phnorm(qhnorm(pp, 1), 1))
  expect_equal(pp, pht(qht(pp, 5, 1), 5, 1))

  expect_equal(pp, pinvgamma(qinvgamma(pp, 1, 1), 1, 1))
  expect_equal(pp, plaplace(qlaplace(pp)))
  expect_equal(pp, plomax(qlomax(pp, 1, 1), 1, 1))
  expect_equal(pp, plst(qlst(pp, df = 2), df = 2))
  expect_equal(pp, ppareto(qpareto(pp)))
  expect_equal(pp, ppower(qpower(pp, 1, 1), 1, 1))
  expect_equal(pp, pprop(qprop(pp, 10, 0.5), 10, 0.5))
  expect_equal(pp, prayleigh(qrayleigh(pp)))

})

#' Half-normal distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the half-normal distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param sigma	          positive valued scale parameter.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @details
#' If \eqn{X} follows normal distribution centered at 0 and parametrized
#' by scale \eqn{\sigma}, then \eqn{|X|} follows half-normal distribution
#' parametrized by scale \eqn{\sigma}. Half-t distribution with \eqn{\nu=\infty}
#' degrees of freedom converges to half-normal distribution.
#' 
#' @references
#' Gelman, A. (2006). Prior distributions for variance parameters in hierarchical
#' models (comment on article by Browne and Draper).
#' Bayesian analysis, 1(3), 515-534.
#' 
#' @references 
#' Jacob, E. and Jayakumar, K. (2012).
#' On Half-Cauchy Distribution and Process.
#' International Journal of Statistika and Mathematika, 3(2), 77-81.
#' 
#' @seealso \code{\link{HalfT}}
#' 
#' @examples 
#' 
#' x <- rhnorm(1e5, 2)
#' hist(x, 100, freq = FALSE)
#' curve(dhnorm(x, 2), 0, 8, col = "red", add = TRUE)
#' hist(phnorm(x, 2))
#' plot(ecdf(x))
#' curve(phnorm(x, 2), 0, 8, col = "red", lwd = 2, add = TRUE)
#'
#' @name HalfNormal
#' @aliases HalfNormal
#' @aliases dhnorm
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dhnorm <- function(x, sigma = 1, log = FALSE) {
  cpp_dhnorm(x, sigma, log[1L])
}


#' @rdname HalfNormal
#' @export

phnorm <- function(q, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_phnorm(q, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname HalfNormal
#' @export

qhnorm <- function(p, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qhnorm(p, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname HalfNormal
#' @export

rhnorm <- function(n, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rhnorm(n, sigma)
}

test_that("Wrong parameter values in PDF and PMF functions", {
  
  expect_warning(expect_true(is.nan(dbbinom(1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(dbbinom(1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(dbbinom(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dbern(1, -1))))
  expect_warning(expect_true(is.nan(dbern(1, 2))))
  
  expect_warning(expect_true(is.nan(dbetapr(1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(dbetapr(1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(dbetapr(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dbhatt(1, sigma = -1))))
  expect_warning(expect_true(is.nan(dbhatt(1, a = -1))))
  
  expect_warning(expect_true(is.nan(dbnbinom(1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(dbnbinom(1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(dbnbinom(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dbvnorm(1, 1, sd1 = -1))))
  expect_warning(expect_true(is.nan(dbvnorm(1, 1, sd2 = -1))))
  expect_warning(expect_true(is.nan(dbvnorm(1, 1, cor = -2))))
  expect_warning(expect_true(is.nan(dbvnorm(1, 1, cor = 2))))
  
  expect_warning(expect_true(is.nan(dbvpois(1, 1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(dbvpois(1, 1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(dbvpois(1, 1, 1, 1, -1))))

  expect_warning(expect_true(is.nan(dcat(1, c(-1, 0.5)))))
  expect_warning(expect_true(is.nan(dcat(1, c(0.5, -1)))))
  
  expect_warning(expect_true(is.nan(ddirichlet(c(0.5, 0.5), c(-1, 0.5)))))
  expect_warning(expect_true(is.nan(ddirichlet(c(0.5, 0.5), c(0.5, -1)))))
  
  expect_warning(expect_true(is.nan(ddlaplace(1, 0, scale = -1))))
  expect_warning(expect_true(is.nan(ddlaplace(1, 0, scale = 2))))
  
  expect_warning(expect_true(is.nan(ddnorm(1, sd = -1))))
  
  expect_warning(expect_true(is.nan(ddgamma(1, -9, 1))))
  expect_warning(expect_true(is.nan(ddgamma(1, 9, -1))))
  
  expect_warning(expect_true(is.nan(ddunif(1, min = 10, max = 1))))
  expect_warning(expect_true(is.nan(ddunif(1, min = 0, max = Inf))))
  expect_warning(expect_true(is.nan(ddunif(1, min = -Inf, max = Inf))))
  expect_warning(expect_true(is.nan(ddunif(1, min = Inf, max = -Inf))))
  
  expect_warning(expect_true(is.nan(ddweibull(1, -1, 1)))) 
  expect_warning(expect_true(is.nan(ddweibull(1, 2, 1)))) 
  expect_warning(expect_true(is.nan(ddweibull(1, 0.5, -1))))
  
  expect_warning(expect_true(is.nan(dfatigue(1, -1, 1))))
  expect_warning(expect_true(is.nan(dfatigue(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dfrechet(1, lambda = -1))))
  expect_warning(expect_true(is.nan(dfrechet(1, sigma = -1))))
  
  expect_warning(expect_true(is.nan(dgev(1, 1, -1, 1))))
  
  expect_warning(expect_true(is.nan(dgompertz(1, -1, 1))))
  expect_warning(expect_true(is.nan(dgompertz(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dgpd(1, 1, -1, 1))))
  
  expect_warning(expect_true(is.nan(dgpois(1, -1, 1))))
  expect_warning(expect_true(is.nan(dgpois(1, 1, -1))))
  expect_warning(expect_true(is.nan(dgpois(1, 1, scale = 0))))
  
  expect_warning(expect_true(is.nan(dgumbel(1, sigma = -1))))
  
  expect_warning(expect_true(is.nan(dhcauchy(1, -1))))
  
  expect_warning(expect_true(is.nan(dhnorm(1, -1))))
  
  expect_warning(expect_true(is.nan(dht(1, 5, -1))))
  
  expect_warning(expect_true(is.nan(dhuber(1, 0, -1, 1))))
  expect_warning(expect_true(is.nan(dhuber(1, 0, 1, -1))))
  
  expect_warning(expect_true(is.nan(dinvgamma(1, -1, 1))))
  expect_warning(expect_true(is.nan(dinvgamma(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dinvchisq(1, -1, 1))))
  expect_warning(expect_true(is.nan(dinvchisq(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dkumar(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(dkumar(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(dlaplace(1, 0, -1))))
  
  expect_warning(expect_true(is.nan(dlgser(1, -1))))
  expect_warning(expect_true(is.nan(dlgser(1, 2))))
  
  expect_warning(expect_true(is.nan(dlomax(1, -1, 1))))
  expect_warning(expect_true(is.nan(dlomax(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dmixnorm(0, c(1,2,3), c(-1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixnorm(0, c(1,2,3), c(1,-2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixnorm(0, c(1,2,3), c(1,2,-3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixnorm(0, c(1,2,3), c(1,2,3), c(-1,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,-1,1/3)))))
  expect_warning(expect_true(is.nan(dmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,1/3,-1)))))
  
  expect_warning(expect_true(is.nan(dmixpois(0, c(-1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixpois(0, c(1,-2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixpois(0, c(1,2,-3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixpois(0, c(1,2,3), c(-1,1/3,1/3)))))
  expect_warning(expect_true(is.nan(dmixpois(0, c(1,2,3), c(1/3,-1,1/3)))))
  expect_warning(expect_true(is.nan(dmixpois(0, c(1,2,3), c(1/3,1/3,-1)))))
  
  expect_warning(expect_true(is.nan(dnhyper(1, 60.5, 35, 15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60, 35.5, 15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60, 35, 15.5))))
  expect_warning(expect_true(is.nan(dnhyper(1, -60, 35, 15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60, -35, 15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60, 35, -15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60, 35, 40))))
  
  expect_warning(expect_true(is.nan(dnhyper(1, 60.5, 35, 15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60.5, 35, 15))))
  expect_warning(expect_true(is.nan(dnhyper(1, 60.5, 35, 15))))
  expect_warning(expect_true(is.nan(ddirmnom(c(1, 1, 1), 1.5, c(1, 1, 1)))))
  expect_warning(expect_true(is.nan(ddirmnom(c(1, 1, 1), -3, c(1, 1, 1)))))
  expect_warning(expect_true(is.nan(ddirmnom(c(1, 1, 1), 3, c(-1, 1, 1)))))
  expect_warning(expect_true(is.nan(ddirmnom(c(1, 1, 1), 3, c(1, -1, 1)))))
  expect_warning(expect_true(is.nan(ddirmnom(c(1, 1, 1), 3, c(1, 1, -1)))))
  
  expect_warning(expect_true(is.nan(dmnom(c(1, 1, 1), 1.5, c(1/3, 1/3, 1/3)))))
  expect_warning(expect_true(is.nan(dmnom(c(1, 1, 1), 3, c(-1, 1/3, 1/3)))))
  expect_warning(expect_true(is.nan(dmnom(c(1, 1, 1), 3, c(1/3, -1, 1/3)))))
  expect_warning(expect_true(is.nan(dmnom(c(1, 1, 1), 3, c(1/3, 1/3, -1)))))

  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2.5,3,4), 5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,3.5,4), 5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,3,4.5), 5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,3,4), 5.5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(-2,3,4), 5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,-3,4), 5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,3,-4), 5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,3,4), -5))))
  expect_warning(expect_true(is.nan(dmvhyper(c(1, 2, 2), c(2,3,4), 85))))
  
  expect_warning(expect_true(is.nan(dnsbeta(0.5, -1, 1, -2, 2))))
  expect_warning(expect_true(is.nan(dnsbeta(0.5, 1, -1, -2, 2))))
  expect_warning(expect_true(is.nan(dnsbeta(0.5, 1, 1, 2, -2))))
  
  expect_warning(expect_true(is.nan(dlst(1, -2, 0, 1))))
  expect_warning(expect_true(is.nan(dlst(1, 2, 0, -1))))
  
  expect_warning(expect_true(is.nan(dpareto(0, -1, 1))))
  expect_warning(expect_true(is.nan(dpareto(0, 1, -1))))

  expect_warning(expect_true(is.nan(dprop(1, -10, 0.5))))
  expect_warning(expect_true(is.nan(dprop(1, 10, -1))))
  expect_warning(expect_true(is.nan(dprop(1, 10, 2))))
  
  expect_warning(expect_true(is.nan(drayleigh(0, -1))))
  
  expect_warning(expect_true(is.nan(dsgomp(1, -0.4, 1))))
  expect_warning(expect_true(is.nan(dsgomp(1, 0.4, -1))))
  
  expect_warning(expect_true(is.nan(dskellam(1, -1, 1))))
  expect_warning(expect_true(is.nan(dskellam(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dslash(1, sigma = -1))))
  
  expect_warning(expect_true(is.nan(dtnorm(1, 0, -1, -2, 2))))
  expect_warning(expect_true(is.nan(dtnorm(1, 0, 1, 2, -2))))
  expect_warning(expect_true(is.nan(dtnorm(1, 0, 1, 0, 0)))) 
  
  expect_warning(expect_true(is.nan(dtpois(1, lambda = -5, a = 0))))
  expect_warning(expect_true(is.nan(dtpois(1, lambda = -5, a = 6))))
  expect_warning(expect_true(is.nan(dtpois(1, lambda = -5, a = 6, b = 5))))

  expect_warning(expect_true(is.nan(dtriang(1, 0, 0, 0))))
  expect_warning(expect_true(is.nan(dtriang(1, 1, -1, 0))))
  expect_warning(expect_true(is.nan(dtriang(1, -1, 1, 2))))
  expect_warning(expect_true(is.nan(dtriang(1, -1, 1, -2))))
  
  expect_warning(expect_true(is.nan(dwald(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(dzip(1, -1, 0.5))))
  expect_warning(expect_true(is.nan(dzip(1, 1, -1))))
  expect_warning(expect_true(is.nan(dzip(1, 1, 2))))
  
  expect_warning(expect_true(is.nan(dzib(1, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.nan(dzib(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(dzib(1, 1, 0.5, 2))))
  expect_warning(expect_true(is.nan(dzib(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(dzib(1, 1, 0.5, 2))))
  
  expect_warning(expect_true(is.nan(dzinb(1, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.nan(dzinb(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(dzinb(1, 1, 0.5, 2))))
  expect_warning(expect_true(is.nan(dzinb(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(dzinb(1, 1, 0.5, 2))))
  
})





test_that("Wrong parameter values in CDF functions", {
  
  expect_warning(expect_true(is.nan(pbbinom(1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(pbbinom(1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(pbbinom(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pbbinom(1, c(-1, 1), c(1, 1), c(1, 1))[1])))
  expect_warning(expect_true(is.nan(pbbinom(1, c(1, 1), c(-1, 1), c(1, 1))[1])))
  expect_warning(expect_true(is.nan(pbbinom(1, c(1, 1), c(1, 1), c(-1, 1))[1])))
  
  expect_warning(expect_true(is.nan(pbbinom(1, c(1, -1), c(1, 1), c(1, 1))[2])))
  expect_warning(expect_true(is.nan(pbbinom(1, c(1, 1), c(1, -1), c(1, 1))[2])))
  expect_warning(expect_true(is.nan(pbbinom(1, c(1, 1), c(1, 1), c(1, -1))[2])))
  
  expect_warning(expect_true(is.nan(pbetapr(1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(pbetapr(1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(pbetapr(1, 1, 1, -1))))

  expect_warning(expect_true(is.nan(pbern(1, -1))))
  expect_warning(expect_true(is.nan(pbern(1, 2))))
  
  expect_warning(expect_true(is.nan(pbhatt(1, sigma = -1))))
  expect_warning(expect_true(is.nan(pbhatt(1, a = -1))))
  
  expect_warning(expect_true(is.nan(pbnbinom(1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(pbnbinom(1, 1, -1, 1))))
  expect_warning(expect_true(is.nan(pbnbinom(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pcat(1, c(-1, 0.5)))))
  expect_warning(expect_true(is.nan(pcat(1, c(0.5, -1)))))

  expect_warning(expect_true(is.nan(pdlaplace(1, 0, scale = -1))))
  expect_warning(expect_true(is.nan(pdlaplace(1, 0, scale = 2))))
  
  expect_warning(expect_true(is.nan(pdnorm(1, sd = -1))))
  
  expect_warning(expect_true(is.nan(pdgamma(1, -9, 1))))
  expect_warning(expect_true(is.nan(pdgamma(1, 9, -1))))
  
  expect_warning(expect_true(is.nan(pdunif(1, min = 10, max = 1))))
  expect_warning(expect_true(is.nan(pdunif(1, min = 0, max = Inf))))
  expect_warning(expect_true(is.nan(pdunif(1, min = -Inf, max = Inf))))
  expect_warning(expect_true(is.nan(pdunif(1, min = Inf, max = -Inf))))
  
  expect_warning(expect_true(is.nan(pdweibull(1, -1, 1)))) 
  expect_warning(expect_true(is.nan(pdweibull(1, 2, 1)))) 
  expect_warning(expect_true(is.nan(pdweibull(1, 0.5, -1))))
  
  expect_warning(expect_true(is.nan(pfatigue(1, -1, 1))))
  expect_warning(expect_true(is.nan(pfatigue(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pfrechet(1, lambda = -1))))
  expect_warning(expect_true(is.nan(pfrechet(1, sigma = -1))))
  
  expect_warning(expect_true(is.nan(pgev(1, 1, -1, 1))))
  
  expect_warning(expect_true(is.nan(pgompertz(1, -1, 1))))
  expect_warning(expect_true(is.nan(pgompertz(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pgpd(1, 1, -1, 1))))
  
  expect_warning(expect_true(is.nan(pgpois(1, -1, 1))))
  expect_warning(expect_true(is.nan(pgpois(1, 1, -1))))
  expect_warning(expect_true(is.nan(pgpois(1, 1, scale = 0))))
  
  expect_warning(expect_true(is.nan(pgumbel(1, sigma = -1))))
  
  expect_warning(expect_true(is.nan(phcauchy(1, -1))))
  
  expect_warning(expect_true(is.nan(phnorm(1, -1))))
  
  expect_warning(expect_true(is.nan(pht(1, 5, -1))))
  
  expect_warning(expect_true(is.nan(phuber(1, 0, -1, 1))))
  expect_warning(expect_true(is.nan(phuber(1, 0, 1, -1))))
  
  expect_warning(expect_true(is.nan(pinvgamma(1, -1, 1))))
  expect_warning(expect_true(is.nan(pinvgamma(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pinvchisq(1, -1, 1))))
  expect_warning(expect_true(is.nan(pinvchisq(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pkumar(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(pkumar(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(plaplace(1, 0, -1))))
  
  expect_warning(expect_true(is.nan(plgser(1, -1))))
  expect_warning(expect_true(is.nan(plgser(1, 2))))
  
  expect_warning(expect_true(is.nan(plomax(1, -1, 1))))
  expect_warning(expect_true(is.nan(plomax(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pmixnorm(0, c(1,2,3), c(-1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixnorm(0, c(1,2,3), c(1,-2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixnorm(0, c(1,2,3), c(1,2,-3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixnorm(0, c(1,2,3), c(1,2,3), c(-1,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,-1,1/3)))))
  expect_warning(expect_true(is.nan(pmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,1/3,-1)))))
  
  expect_warning(expect_true(is.nan(pmixpois(0, c(-1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixpois(0, c(1,-2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixpois(0, c(1,2,-3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixpois(0, c(1,2,3), c(-1,1/3,1/3)))))
  expect_warning(expect_true(is.nan(pmixpois(0, c(1,2,3), c(1/3,-1,1/3)))))
  expect_warning(expect_true(is.nan(pmixpois(0, c(1,2,3), c(1/3,1/3,-1)))))
  
  expect_warning(expect_true(is.nan(pnhyper(1, 60.5, 35, 15))))
  expect_warning(expect_true(is.nan(pnhyper(1, 60, 35.5, 15))))
  expect_warning(expect_true(is.nan(pnhyper(1, 60, 35, 15.5))))
  expect_warning(expect_true(is.nan(pnhyper(1, -60, 35, 15))))
  expect_warning(expect_true(is.nan(pnhyper(1, 60, -35, 15))))
  expect_warning(expect_true(is.nan(pnhyper(1, 60, 35, -15))))
  expect_warning(expect_true(is.nan(pnhyper(1, 60, 35, 40))))
  
  expect_warning(expect_true(is.nan(pnsbeta(0.5, -1, 1, -2, 2))))
  expect_warning(expect_true(is.nan(pnsbeta(0.5, 1, -1, -2, 2))))
  expect_warning(expect_true(is.nan(pnsbeta(0.5, 1, 1, 2, -2))))
  
  expect_warning(expect_true(is.nan(plst(1, -2, 0, 1))))
  expect_warning(expect_true(is.nan(plst(1, 2, 0, -1))))
  
  expect_warning(expect_true(is.nan(ppareto(0, -1, 1))))
  expect_warning(expect_true(is.nan(ppareto(0, 1, -1))))

  expect_warning(expect_true(is.nan(pprop(1, -10, 0.5))))
  expect_warning(expect_true(is.nan(pprop(1, 10, -1))))
  expect_warning(expect_true(is.nan(pprop(1, 10, 2))))
  
  expect_warning(expect_true(is.nan(prayleigh(0, -1))))
  
  expect_warning(expect_true(is.nan(psgomp(1, -0.4, 1))))
  expect_warning(expect_true(is.nan(psgomp(1, 0.4, -1))))

  expect_warning(expect_true(is.nan(pslash(1, sigma = -1))))
  
  expect_warning(expect_true(is.nan(ptnorm(1, 0, -1, -2, 2))))
  expect_warning(expect_true(is.nan(ptnorm(1, 0, 1, 2, -2))))
  expect_warning(expect_true(is.nan(ptnorm(1, 0, 1, 0, 0))))
  
  expect_warning(expect_true(is.nan(ptpois(1, lambda = -5, a = 0))))
  expect_warning(expect_true(is.nan(ptpois(1, lambda = -5, a = 6))))
  expect_warning(expect_true(is.nan(ptpois(1, lambda = -5, a = 6, b = 5))))

  expect_warning(expect_true(is.nan(ptriang(1, 0, 0, 0))))
  expect_warning(expect_true(is.nan(ptriang(1, 1, -1, 0))))
  expect_warning(expect_true(is.nan(ptriang(1, -1, 1, 2))))
  expect_warning(expect_true(is.nan(ptriang(1, -1, 1, -2))))
  
  expect_warning(expect_true(is.nan(pwald(1, 1, -1))))
  
  expect_warning(expect_true(is.nan(pzip(1, -1, 0.5))))
  expect_warning(expect_true(is.nan(pzip(1, 1, -1))))
  expect_warning(expect_true(is.nan(pzip(1, 1, 2))))
  
  expect_warning(expect_true(is.nan(pzib(1, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.nan(pzib(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(pzib(1, 1, 0.5, 2))))
  expect_warning(expect_true(is.nan(pzib(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(pzib(1, 1, 0.5, 2))))
  
  expect_warning(expect_true(is.nan(pzinb(1, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.nan(pzinb(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(pzinb(1, 1, 0.5, 2))))
  expect_warning(expect_true(is.nan(pzinb(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(pzinb(1, 1, 0.5, 2))))
  
})




test_that("Wrong parameter values in quantile functions", {

  expect_warning(expect_true(is.nan(qbetapr(0.5, -1, 1, 1))))
  expect_warning(expect_true(is.nan(qbetapr(0.5, 1, -1, 1))))
  expect_warning(expect_true(is.nan(qbetapr(0.5, 1, 1, -1))))
  
  expect_warning(expect_true(is.nan(qbern(0.5, -1))))
  expect_warning(expect_true(is.nan(qbern(0.5, 2))))

  expect_warning(expect_true(is.nan(qcat(0.5, c(-1, 0.5)))))
  expect_warning(expect_true(is.nan(qcat(0.5, c(0.5, -1)))))
  
  expect_warning(expect_true(is.nan(qdunif(0.5, min = 10, max = 1))))
  expect_warning(expect_true(is.nan(qdunif(0.5, min = 0, max = Inf))))
  expect_warning(expect_true(is.nan(qdunif(0.5, min = -Inf, max = Inf))))
  expect_warning(expect_true(is.nan(qdunif(0.5, min = Inf, max = -Inf))))
  
  expect_warning(expect_true(is.nan(qdweibull(0.5, -1, 1)))) 
  expect_warning(expect_true(is.nan(qdweibull(0.5, 2, 1)))) 
  expect_warning(expect_true(is.nan(qdweibull(0.5, 0.5, -1))))
  
  expect_warning(expect_true(is.nan(qfatigue(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qfatigue(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(qfrechet(0.5, lambda = -1))))
  expect_warning(expect_true(is.nan(qfrechet(0.5, sigma = -1))))
  
  expect_warning(expect_true(is.nan(qgev(0.5, 1, -1, 1))))
  
  expect_warning(expect_true(is.nan(qgompertz(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qgompertz(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(qgpd(0.5, 1, -1, 1))))

  expect_warning(expect_true(is.nan(qgumbel(0.5, sigma = -1))))
  
  expect_warning(expect_true(is.nan(qhcauchy(0.5, -1))))
  
  expect_warning(expect_true(is.nan(qhnorm(0.5, -1))))
  
  expect_warning(expect_true(is.nan(qht(0.5, 5, -1))))
  
  expect_warning(expect_true(is.nan(qhuber(0.5, 0, -1, 1))))
  expect_warning(expect_true(is.nan(qhuber(0.5, 0, 1, -1))))
  
  expect_warning(expect_true(is.nan(qinvgamma(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qinvgamma(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(qinvchisq(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qinvchisq(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(qkumar(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qkumar(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(qlaplace(0.5, 0, -1))))
  
  expect_warning(expect_true(is.nan(qlgser(0.5, -1))))
  expect_warning(expect_true(is.nan(qlgser(0.5, 2))))
  
  expect_warning(expect_true(is.nan(qlomax(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qlomax(0.5, 1, -1))))
  
  expect_warning(expect_true(is.nan(qnhyper(0.5, 60.5, 35, 15))))
  expect_warning(expect_true(is.nan(qnhyper(0.5, 60, 35.5, 15))))
  expect_warning(expect_true(is.nan(qnhyper(0.5, 60, 35, 15.5))))
  expect_warning(expect_true(is.nan(qnhyper(0.5, -60, 35, 15))))
  expect_warning(expect_true(is.nan(qnhyper(0.5, 60, -35, 15))))
  expect_warning(expect_true(is.nan(qnhyper(0.5, 60, 35, -15))))
  expect_warning(expect_true(is.nan(qnhyper(0.5, 60, 35, 40))))

  expect_warning(expect_true(is.nan(qnsbeta(0.5, -1, 1, -2, 2))))
  expect_warning(expect_true(is.nan(qnsbeta(0.5, 1, -1, -2, 2))))
  expect_warning(expect_true(is.nan(qnsbeta(0.5, 1, 1, 2, -2))))
  
  expect_warning(expect_true(is.nan(qlst(0.5, -2, 0, 1))))
  expect_warning(expect_true(is.nan(qlst(0.5, 2, 0, -1))))
  
  expect_warning(expect_true(is.nan(qpareto(0.5, -1, 1))))
  expect_warning(expect_true(is.nan(qpareto(0.5, 1, -1))))

  expect_warning(expect_true(is.nan(qprop(0.5, -10, 0.5))))
  expect_warning(expect_true(is.nan(qprop(0.5, 10, -1))))
  expect_warning(expect_true(is.nan(qprop(0.5, 10, 2))))
  
  expect_warning(expect_true(is.nan(qrayleigh(0, -1))))

  expect_warning(expect_true(is.nan(qtnorm(0.5, 0, -1, -2, 2))))
  expect_warning(expect_true(is.nan(qtnorm(0.5, 0, 1, 2, -2))))
  expect_warning(expect_true(is.nan(qtnorm(0.5, 0, 1, 0, 0))))
  
  expect_warning(expect_true(is.nan(qtpois(0.5, lambda = -5, a = 0))))
  expect_warning(expect_true(is.nan(qtpois(0.5, lambda = -5, a = 6))))
  expect_warning(expect_true(is.nan(qtpois(0.5, lambda = -5, a = 6, b = 5))))

  expect_warning(expect_true(is.nan(qtriang(0.5, 0, 0, 0))))
  expect_warning(expect_true(is.nan(qtriang(0.5, 1, -1, 0))))
  expect_warning(expect_true(is.nan(qtriang(0.5, -1, 1, 2))))
  expect_warning(expect_true(is.nan(qtriang(0.5, -1, 1, -2))))

  expect_warning(expect_true(is.nan(qzip(0.5, -1, 0.5))))
  expect_warning(expect_true(is.nan(qzip(0.5, 1, -1))))
  expect_warning(expect_true(is.nan(qzip(0.5, 1, 2))))
  
  expect_warning(expect_true(is.nan(qzib(0.5, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.nan(qzib(0.5, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(qzib(0.5, 1, 0.5, 2))))
  expect_warning(expect_true(is.nan(qzib(0.5, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(qzib(0.5, 1, 0.5, 2))))
  
  expect_warning(expect_true(is.nan(qzinb(0.5, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.nan(qzinb(0.5, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(qzinb(0.5, 1, 0.5, 2))))
  expect_warning(expect_true(is.nan(qzinb(0.5, 1, -1, 0.5))))
  expect_warning(expect_true(is.nan(qzinb(0.5, 1, 0.5, 2))))
  
})




test_that("Wrong parameter values in RNG functions", {
  
  expect_warning(expect_true(is.na(rbbinom(1, -1, 1, 1))))
  expect_warning(expect_true(is.na(rbbinom(1, 1, -1, 1))))
  expect_warning(expect_true(is.na(rbbinom(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.na(rbetapr(1, -1, 1, 1))))
  expect_warning(expect_true(is.na(rbetapr(1, 1, -1, 1))))
  expect_warning(expect_true(is.na(rbetapr(1, 1, 1, -1))))
  
  expect_warning(expect_true(is.na(rbern(1, -1))))
  expect_warning(expect_true(is.na(rbern(1, 2))))
  
  expect_warning(expect_true(is.na(rbhatt(1, sigma = -1))))
  expect_warning(expect_true(is.na(rbhatt(1, a = -1))))
  
  expect_warning(expect_true(all(is.na(rbnbinom(1, -1, 1, 1)))))
  expect_warning(expect_true(all(is.na(rbnbinom(1, 1, -1, 1)))))
  expect_warning(expect_true(all(is.na(rbnbinom(1, 1, 1, -1)))))
  
  expect_warning(expect_true(all(is.na(rbvnorm(1, sd1 = -1)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, sd2 = -1)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, cor = -2)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, cor = 2)))))
  
  expect_warning(expect_true(all(is.na(rbvpois(1, -1, 1, 1)))))
  expect_warning(expect_true(all(is.na(rbvpois(1, 1, -1, 1)))))
  expect_warning(expect_true(all(is.na(rbvpois(1, 1, 1, -1)))))
  
  expect_warning(expect_true(is.na(rcat(1, c(-1, 0.5)))))
  expect_warning(expect_true(is.na(rcat(1, c(0.5, -1)))))
  expect_warning(expect_true(is.na(rcat(2, matrix(c(-1, 0.5, 0.5, 0.5), byrow = T, ncol = 2))[1])))
  expect_warning(expect_true(is.na(rcat(2, matrix(c(0.5, -1, 0.5, 0.5), byrow = T, ncol = 2))[1])))
  expect_warning(expect_true(is.na(rcat(2, matrix(c(0.5, 0.5, -1, 0.5), byrow = T, ncol = 2))[2])))
  expect_warning(expect_true(is.na(rcat(2, matrix(c(0.5, 0.5, 0.5, -1), byrow = T, ncol = 2))[2])))
  
  expect_warning(expect_true(all(is.na(rdirichlet(1, c(-1, 0.5))))))
  expect_warning(expect_true(all(is.na(rdirichlet(1, c(0.5, -1))))))
  
  expect_warning(expect_true(is.na(rdnorm(1, sd = -1))))
  
  expect_warning(expect_true(is.nan(rdgamma(1, -9, 1))))
  expect_warning(expect_true(is.nan(rdgamma(1, 9, -1))))
  
  expect_warning(expect_true(is.na(rdunif(1, min = 10, max = 1))))
  expect_warning(expect_true(is.na(rdunif(1, min = 0, max = Inf))))
  expect_warning(expect_true(is.na(rdunif(1, min = -Inf, max = Inf))))
  expect_warning(expect_true(is.na(rdunif(1, min = Inf, max = -Inf))))
  
  expect_warning(expect_true(is.na(rdweibull(1, -1, 1)))) 
  expect_warning(expect_true(is.na(rdweibull(1, 2, 1)))) 
  expect_warning(expect_true(is.na(rdweibull(1, 0.5, -1))))
  
  expect_warning(expect_true(is.na(rfatigue(1, -1, 1))))
  expect_warning(expect_true(is.na(rfatigue(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rfrechet(1, lambda = -1))))
  expect_warning(expect_true(is.na(rfrechet(1, sigma = -1))))
  
  expect_warning(expect_true(is.na(rgev(1, 1, -1, 1))))
  
  expect_warning(expect_true(is.na(rgompertz(1, -1, 1))))
  expect_warning(expect_true(is.na(rgompertz(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rgpd(1, 1, -1, 1))))
  
  expect_warning(expect_true(is.na(rgpois(1, -1, 1))))
  expect_warning(expect_true(is.na(rgpois(1, 1, -1))))
  expect_warning(expect_true(is.na(rgpois(1, 1, scale = 0))))
  
  expect_warning(expect_true(is.na(rgumbel(1, sigma = -1))))
  
  expect_warning(expect_true(is.na(rhcauchy(1, -1))))
  
  expect_warning(expect_true(is.na(rhnorm(1, -1))))
  
  expect_warning(expect_true(is.na(rht(1, 5, -1))))
  
  expect_warning(expect_true(is.na(rhuber(1, 0, -1, 1))))
  expect_warning(expect_true(is.na(rhuber(1, 0, 1, -1))))
  
  expect_warning(expect_true(is.na(rinvgamma(1, -1, 1))))
  expect_warning(expect_true(is.na(rinvgamma(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rinvchisq(1, -1, 1))))
  expect_warning(expect_true(is.na(rinvchisq(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rkumar(1, -1, 1))))
  expect_warning(expect_true(is.na(rkumar(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rlaplace(1, 0, -1))))
  
  expect_warning(expect_true(is.na(rlgser(1, -1))))
  expect_warning(expect_true(is.na(rlgser(1, 2))))
  
  expect_warning(expect_true(is.na(rlomax(1, -1, 1))))
  expect_warning(expect_true(is.na(rlomax(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(-1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,-2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,-3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), c(-1,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), c(1/3,-1,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), c(1/3,1/3,-1)))))
  
  expect_warning(expect_true(is.na(rmixpois(1, c(-1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,-2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,-3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), c(-1,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), c(1/3,-1,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), c(1/3,1/3,-1)))))
  
  expect_warning(expect_true(is.na(rnhyper(1, 60.5, 35, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, 35.5, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, 35, 15.5))))
  expect_warning(expect_true(is.na(rnhyper(1, -60, 35, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, -35, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, 35, -15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, 35, 40))))
  
  expect_warning(expect_true(all(is.na(rdirmnom(1, 1.5, c(1, 1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, -3, c(1, 1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, c(-1, 1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, c(1, -1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, c(1, 1, -1))))))
  
  expect_warning(expect_true(all(is.na(rmnom(1, 3, c(-1, 1/3, 1/3))))))
  expect_warning(expect_true(all(is.na(rmnom(1, 3, c(1/3, -1, 1/3))))))
  expect_warning(expect_true(all(is.na(rmnom(1, 3, c(1/3, 1/3, -1))))))
  
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,3,4), 99)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(-2,3,4), 99)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,-3,4), 99)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,3,-4), 99)))))
  
  expect_warning(expect_true(is.na(rnsbeta(1, -1, 1, -2, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, -1, -2, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, 1, 2, -2))))
  
  expect_warning(expect_true(is.na(rlst(1, -2, 0, 1))))
  expect_warning(expect_true(is.na(rlst(1, 2, 0, -1))))
  
  expect_warning(expect_true(is.na(rpareto(1, -1, 1))))
  expect_warning(expect_true(is.na(rpareto(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rprop(1, -10, 0.5))))
  expect_warning(expect_true(is.na(rprop(1, 10, -1))))
  expect_warning(expect_true(is.na(rprop(1, 10, 2))))
  
  expect_warning(expect_true(is.na(rrayleigh(1, -1))))
  
  expect_warning(expect_true(is.na(rsgomp(1, -0.4, 1))))
  expect_warning(expect_true(is.na(rsgomp(1, 0.4, -1))))
  
  expect_warning(expect_true(is.na(rskellam(1, -1, 1))))
  expect_warning(expect_true(is.na(rskellam(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rslash(1, sigma = -1))))
  
  expect_warning(expect_true(is.na(rtnorm(1, 0, -1, -2, 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, 1, 2, -2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, 1, 0, 0))))
  
  expect_warning(expect_true(is.na(rtpois(1, lambda = -5, a = 0))))
  expect_warning(expect_true(is.na(rtpois(1, lambda = -5, a = 6))))
  expect_warning(expect_true(is.na(rtpois(1, lambda = -5, a = 6, b = 5))))

  expect_warning(expect_true(is.na(rtriang(1, 0, 0, 0))))
  expect_warning(expect_true(is.na(rtriang(1, 1, -1, 0))))
  expect_warning(expect_true(is.na(rtriang(1, -1, 1, 2))))
  expect_warning(expect_true(is.na(rtriang(1, -1, 1, -2))))
  
  expect_warning(expect_true(is.na(rwald(1, 1, -1))))
  
  expect_warning(expect_true(is.na(rzip(1, -1, 0.5))))
  expect_warning(expect_true(is.na(rzip(1, 1, -1))))
  expect_warning(expect_true(is.na(rzip(1, 1, 2))))
  
  expect_warning(expect_true(is.na(rzib(1, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, 0.5, 2))))
  expect_warning(expect_true(is.na(rzib(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, 0.5, 2))))
  
  expect_warning(expect_true(is.na(rzinb(1, -1, 0.5, 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, 0.5, 2))))
  expect_warning(expect_true(is.na(rzinb(1, 1, -1, 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, 0.5, 2))))
  
})

#' Lomax distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Lomax distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param lambda,kappa    positive valued parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{\lambda \kappa}{(1+\lambda x)^{\kappa+1}}
#' }{
#' f(x) = \lambda*\kappa / (1+\lambda*x)^(\kappa+1)
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = 1-(1+\lambda x)^{-\kappa}
#' }{
#' F(x) = 1-(1+\lambda*x)^-\kappa
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \frac{(1-p)^{-1/\kappa} -1}{\lambda}
#' }{
#' F^-1(p) = ((1-p)^(-1/\kappa)-1) / \lambda
#' }
#' 
#' @examples 
#' 
#' x <- rlomax(1e5, 5, 16)
#' hist(x, 100, freq = FALSE)
#' curve(dlomax(x, 5, 16), 0, 1, col = "red", add = TRUE, n = 5000)
#' hist(plomax(x, 5, 16))
#' plot(ecdf(x))
#' curve(plomax(x, 5, 16), 0, 1, col = "red", lwd = 2, add = TRUE)
#'
#' @name Lomax
#' @aliases Lomax
#' @aliases dlomax
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dlomax <- function(x, lambda, kappa, log = FALSE) {
  cpp_dlomax(x, lambda, kappa, log[1L])
}


#' @rdname Lomax
#' @export

plomax <- function(q, lambda, kappa, lower.tail = TRUE, log.p = FALSE) {
  cpp_plomax(q, lambda, kappa, lower.tail[1L], log.p[1L])
}


#' @rdname Lomax
#' @export

qlomax <- function(p, lambda, kappa, lower.tail = TRUE, log.p = FALSE) {
  cpp_qlomax(p, lambda, kappa, lower.tail[1L], log.p[1L])
}


#' @rdname Lomax
#' @export

rlomax <- function(n, lambda, kappa) {
  if (length(n) > 1) n <- length(n)
  cpp_rlomax(n, lambda, kappa)
}

#' Negative hypergeometric distribution
#'
#' Probability mass function, distribution function, quantile function and random generation
#' for the negative hypergeometric distribution.
#'
#' @param x,q        vector of quantiles representing the number of balls drawn without
#'                   replacement from an urn which contains both black and white balls.
#' @param p	         vector of probabilities.
#' @param nn         number of observations. If \code{length(n) > 1},
#'                   the length is taken to be the number required.
#' @param m          the number of white balls in the urn.
#' @param n          the number of black balls in the urn.
#' @param r          the number of white balls that needs to be drawn for the sampling
#'                   to be stopped.
#' @param log,log.p  logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                   otherwise, \eqn{P[X > x]}.
#'
#'
#' @details
#'
#' Negative hypergeometric distribution describes number of balls \eqn{x} observed
#' until drawing without replacement to obtain \eqn{r} white balls
#' from the urn containing \eqn{m} white balls and \eqn{n} black balls,
#' and is defined as
#'
#' \deqn{
#' f(x) = \frac{{x-1 \choose r-1}{m+n-x \choose m-r}}{{m+n \choose n}}
#' }{
#' f(x) = choose(x-1, r-1)*choose(m+n-x, m-r)/choose(m+n, n)
#' }
#'
#' The algorithm used for calculating probability mass function,
#' cumulative distribution function and quantile function is based
#' on Fortran program NHYPERG created by Berry and Mielke (1996, 1998).
#' Random generation is done by inverse transform sampling.
#'
#' @references
#' Berry, K. J., & Mielke, P. W. (1998).
#' The negative hypergeometric probability distribution:
#' Sampling without replacement from a finite population.
#' Perceptual and motor skills, 86(1), 207-210.
#' \url{https://journals.sagepub.com/doi/10.2466/pms.1998.86.1.207}
#'
#' @references
#' Berry, K. J., & Mielke, P. W. (1996).
#' Exact confidence limits for population proportions based on the negative
#' hypergeometric probability distribution.
#' Perceptual and motor skills, 83(3 suppl), 1216-1218.
#' \url{https://journals.sagepub.com/doi/10.2466/pms.1996.83.3f.1216}
#'
#' @references
#' Schuster, E. F., & Sype, W. R. (1987).
#' On the negative hypergeometric distribution.
#' International Journal of Mathematical Education in Science and Technology, 18(3), 453-459.
#'
#' @references
#' Chae, K. C. (1993).
#' Presenting the negative hypergeometric distribution to the introductory statistics courses.
#' International Journal of Mathematical Education in Science and Technology, 24(4), 523-526.
#'
#' @references
#' Jones, S.N. (2013). A Gaming Application of the Negative Hypergeometric Distribution.
#' UNLV Theses, Dissertations, Professional Papers, and Capstones. Paper 1846.
#' \url{https://oasis.library.unlv.edu/cgi/viewcontent.cgi?referer=&httpsredir=1&article=2847&context=thesesdissertations}
#'
#' @seealso \code{\link[stats]{Hypergeometric}}
#'
#' @examples
#'
#' x <- rnhyper(1e5, 60, 35, 15)
#' xx <- 15:95
#' plot(prop.table(table(x)))
#' lines(xx, dnhyper(xx, 60, 35, 15), col = "red")
#' hist(pnhyper(x, 60, 35, 15))
#'
#' xx <- seq(0, 100, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pnhyper(xx, 60, 35, 15), col = "red", lwd = 2)
#'
#' @name NegHyper
#' @aliases NegHyper
#' @aliases dnhyper
#'
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dnhyper <- function(x, n, m, r, log = FALSE) {
  cpp_dnhyper(x, n, m, r, log[1L])
}


#' @rdname NegHyper
#' @export

pnhyper <- function(q, n, m, r, lower.tail = TRUE, log.p = FALSE) {
  cpp_pnhyper(q, n, m, r, lower.tail[1L], log.p[1L])
}


#' @rdname NegHyper
#' @export

qnhyper <- function(p, n, m, r, lower.tail = TRUE, log.p = FALSE) {
  cpp_qnhyper(p, n, m, r, lower.tail[1L], log.p[1L])
}


#' @rdname NegHyper
#' @export

rnhyper <- function(nn, n, m, r) {
  if (length(nn) > 1) {
    nn <- length(nn)
  }
  cpp_rnhyper(nn, n, m, r)
}

#' Birnbaum-Saunders (fatigue life) distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Birnbaum-Saunders (fatigue life) distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param alpha,beta,mu   shape, scale and location parameters.
#'                        Scale and shape must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'                        
#' @details
#' 
#' Probability density function
#' \deqn{
#' f(x) = \left (\frac{\sqrt{\frac{x-\mu} {\beta}} + \sqrt{\frac{\beta} 
#' {x-\mu}}} {2\alpha (x-\mu)} \right) 
#' \phi \left( \frac{1}{\alpha}\left( \sqrt{\frac{x-\mu}{\beta}} -
#' \sqrt{\frac{\beta}{x-\mu}} \right) \right)
#' }{
#' f(x) = ((sqrt((x-\mu)/\beta) + sqrt(\beta/(x-\mu)))/(2*\alpha*(x-\mu))) *
#' \phi((sqrt((x-\mu)/\beta) - sqrt(\beta/(x-\mu)))/\alpha)
#' }
#' 
#' Cumulative distribution function
#' \deqn{
#' F(x) = \Phi \left(\frac{1}{\alpha}\left( \sqrt{\frac{x-\mu}{\beta}} -
#' \sqrt{\frac{\beta}{x-\mu}} \right) \right)
#' }{
#' F(x) = \Phi(((sqrt((x-\mu)/\beta) - sqrt(\beta/(x-\mu)))/\alpha)
#' }
#' 
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \left[\frac{\alpha}{2} \Phi^{-1}(p) +
#' \sqrt{\left(\frac{\alpha}{2} \Phi^{-1}(p)\right)^{2} + 1}\right]^{2} \beta + \mu
#' }{
#' F^-1(p) = (\alpha/2 * \Phi^-1(p) +
#' sqrt((\alpha/2 * \Phi^-1(p))^2 + 1)^2 * \beta + \mu
#' }
#'
#' @references
#' Birnbaum, Z. W. and Saunders, S. C. (1969).
#' A new family of life distributions.
#' Journal of Applied Probability, 6(2), 637-652.
#' 
#' @references 
#' Desmond, A. (1985) Stochastic models of failure in random environments.
#' Canadian Journal of Statistics, 13, 171-183.
#' 
#' @references 
#' Vilca-Labra, F., and Leiva-Sanchez, V. (2006).
#' A new fatigue life model based on the family of skew-elliptical distributions.
#' Communications in Statistics-Theory and Methods, 35(2), 229-244.
#' 
#' @references 
#' Leiva, V., Sanhueza, A., Sen, P. K., and Paula, G. A. (2008).
#' Random number generators for the generalized Birnbaum-Saunders distribution.
#' Journal of Statistical Computation and Simulation, 78(11), 1105-1118.
#'
#' @examples 
#' 
#' x <- rfatigue(1e5, .5, 2, 5)
#' hist(x, 100, freq = FALSE)
#' curve(dfatigue(x, .5, 2, 5), 2, 20, col = "red", add = TRUE)
#' hist(pfatigue(x, .5, 2, 5))
#' plot(ecdf(x))
#' curve(pfatigue(x, .5, 2, 5), 2, 20, col = "red", lwd = 2, add = TRUE)
#'
#' @name BirnbaumSaunders
#' @aliases BirnbaumSaunders
#' @aliases dfatigue
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dfatigue <- function(x, alpha, beta = 1, mu = 0, log = FALSE) {
  cpp_dfatigue(x, alpha, beta, mu, log[1L])
}


#' @rdname BirnbaumSaunders
#' @export

pfatigue <- function(q, alpha, beta = 1, mu = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_pfatigue(q, alpha, beta, mu, lower.tail[1L], log.p[1L])
}


#' @rdname BirnbaumSaunders
#' @export

qfatigue <- function(p, alpha, beta = 1, mu = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_qfatigue(p, alpha, beta, mu, lower.tail[1L], log.p[1L])
}


#' @rdname BirnbaumSaunders
#' @export

rfatigue <- function(n, alpha, beta = 1, mu = 0) {
  if (length(n) > 1) n <- length(n)
  cpp_rfatigue(n, alpha, beta, mu)
}

#' Kumaraswamy distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Kumaraswamy distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param a,b             positive valued parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = abx^{a-1} (1-x^a)^{b-1}
#' }{
#' f(x) = a*b*x^(a-1)*(1-x^a)^(b-1)
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = 1-(1-x^a)^b
#' }{
#' F(x) = 1-(1-x^a)^b
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = 1-(1-p^{1/b})^{1/a}
#' }{
#' F^-1(p) = 1-(1-p^(1/b))^(1/a)
#' }
#'
#' @references
#' Jones, M. C. (2009). Kumaraswamy's distribution: A beta-type distribution with
#' some tractability advantages. Statistical Methodology, 6, 70-81.
#'
#' @references
#' Cordeiro, G.M. and de Castro, M. (2009). A new family of generalized distributions.
#' Journal of Statistical Computation & Simulation, 1-17.
#' 
#' @examples 
#' 
#' x <- rkumar(1e5, 5, 16)
#' hist(x, 100, freq = FALSE)
#' curve(dkumar(x, 5, 16), 0, 1, col = "red", add = TRUE)
#' hist(pkumar(x, 5, 16))
#' plot(ecdf(x))
#' curve(pkumar(x, 5, 16), 0, 1, col = "red", lwd = 2, add = TRUE)
#'
#' @name Kumaraswamy
#' @aliases Kumaraswamy
#' @aliases dkumar
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dkumar <- function(x, a = 1, b = 1, log = FALSE) {
  cpp_dkumar(x, a, b, log[1L])
}


#' @rdname Kumaraswamy
#' @export

pkumar <- function(q, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pkumar(q, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname Kumaraswamy
#' @export

qkumar <- function(p, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qkumar(p, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname Kumaraswamy
#' @export

rkumar <- function(n, a = 1, b = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rkumar(n, a, b)
}

#' Beta distribution of proportions
#' 
#' Probability mass function, distribution function and random generation
#' for the reparametrized beta distribution.
#' 
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param size            non-negative real number; precision or number of binomial trials.
#' @param mean            mean proportion or probability of success on each trial;
#'                        \code{0 < mean < 1}.
#' @param prior           (see below) with \code{prior = 0} (default)
#'                        the distribution corresponds to re-parametrized beta
#'                        distribution used in beta regression. This parameter needs
#'                        to be non-negative.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'                        
#' @details
#' 
#' Beta can be understood as a distribution of \eqn{x = k/\phi} proportions in
#' \eqn{\phi} trials where the average proportion is denoted as \eqn{\mu},
#' so it's parameters become \eqn{\alpha = \phi\mu} and
#' \eqn{\beta = \phi(1-\mu)} and it's density function becomes
#' 
#' \deqn{
#' f(x) = \frac{x^{\phi\mu+\pi-1} (1-x)^{\phi(1-\mu)+\pi-1}}{\mathrm{B}(\phi\mu+\pi, \phi(1-\mu)+\pi)}
#' }{
#' f(x) = (x^(\phi\mu+\pi-1) * (1-x)^(\phi(1-\mu)+\pi-1))/B(\phi\mu+\pi, \phi(1-\mu)+\pi)
#' }
#' 
#' where \eqn{\pi} is a \emph{prior} parameter, so the distribution is a
#' \emph{posterior} distribution after observing \eqn{\phi\mu} successes and
#' \eqn{\phi(1-\mu)} failures in \eqn{\phi} trials with binomial likelihood
#' and symmetric \eqn{\mathrm{Beta}(\pi, \pi)}{Beta(\pi, \pi)} prior for
#' probability of success. Parameter value \eqn{\pi = 1} corresponds to
#' uniform prior; \eqn{\pi = 1/2} corresponds to Jeffreys prior; \eqn{\pi = 0}
#' corresponds to "uninformative" Haldane prior, this is also the re-parametrized
#' distribution used in beta regression. With \eqn{\pi = 0} the distribution
#' can be understood as a continuous analog to binomial distribution dealing
#' with proportions rather then counts. Alternatively \eqn{\phi} may be
#' understood as precision parameter (as in beta regression).
#' 
#' Notice that in pre-1.8.4 versions of this package, \code{prior} was not settable
#' and by default fixed to one, instead of zero. To obtain the same results as in
#' the previous versions, use \code{prior = 1} in each of the functions. 
#' 
#' @seealso
#' \code{\link{beta}}, \code{\link{binomial}}
#' 
#' @references
#' Ferrari, S., & Cribari-Neto, F. (2004). Beta regression for modelling rates and proportions.
#' Journal of Applied Statistics, 31(7), 799-815.
#' 
#' @references 
#' Smithson, M., & Verkuilen, J. (2006). A better lemon squeezer? Maximum-likelihood regression
#' with beta-distributed dependent variables.
#' Psychological Methods, 11(1), 54-71.
#' 
#' @examples 
#' 
#' x <- rprop(1e5, 100, 0.33)
#' hist(x, 100, freq = FALSE)
#' curve(dprop(x, 100, 0.33), 0, 1, col = "red", add = TRUE)
#' hist(pprop(x, 100, 0.33))
#' plot(ecdf(x))
#' curve(pprop(x, 100, 0.33), 0, 1, col = "red", lwd = 2, add = TRUE)
#' 
#' n <- 500
#' p <- 0.23
#' k <- rbinom(1e5, n, p)
#' hist(k/n, freq = FALSE, 100)
#' curve(dprop(x, n, p), 0, 1, col = "red", add = TRUE, n = 500)
#'                        
#' @name PropBeta
#' @aliases PropBeta
#' @aliases dprop
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dprop <- function(x, size, mean, prior = 0, log = FALSE) {
  cpp_dprop(x, size, mean, prior, log[1L])
}


#' @rdname PropBeta
#' @export

pprop <- function(q, size, mean, prior = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_pprop(q, size, mean, prior, lower.tail[1L], log.p[1L])
}


#' @rdname PropBeta
#' @export

qprop <- function(p, size, mean, prior = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_qprop(p, size, mean, prior, lower.tail[1L], log.p[1L])
}


#' @rdname PropBeta
#' @export

rprop <- function(n, size, mean, prior = 0) {
  if (length(n) > 1) n <- length(n)
  cpp_rprop(n, size, mean, prior)
}

#' Tukey lambda distribution
#'
#' Quantile function, and random generation for the Tukey lambda
#' distribution.
#'
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param lambda	        shape parameter.   
#' @param log.p	          logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#' 
#' Tukey lambda distribution is a continuous probability distribution defined in terms
#' of its quantile function. It is typically used to identify other distributions.
#' 
#' Quantile function:
#' 
#' \deqn{F^{-1}(p) = \left\{\begin{array}{ll}
#' \frac{1}{\lambda} [p^\lambda - (1-p)^\lambda] & \lambda \ne 0 \\
#' \log(\frac{p}{1-p}) & \lambda = 0
#' \end{array}\right.
#' }{
#' F^-1(p) = [if \lambda != 0:] (p^\lambda - (1-p)^\lambda)/\lambda
#' [if \lambda = 0:] log(p/(1-p))
#' }
#'
#' @references
#' 
#' Joiner, B.L., & Rosenblatt, J.R. (1971).
#' Some properties of the range in samples from Tukey's symmetric lambda distributions.
#' Journal of the American Statistical Association, 66(334), 394-399.
#' 
#' @references 
#' 
#' Hastings Jr, C., Mosteller, F., Tukey, J.W., & Winsor, C.P. (1947).
#' Low moments for small samples: a comparative study of order statistics.
#' The Annals of Mathematical Statistics, 413-426.
#' 
#' @examples 
#' 
#' pp = seq(0, 1, by = 0.001)
#' partmp <- par(mfrow = c(2,3))
#' plot(qtlambda(pp, -1), pp, type = "l", main = "lambda = -1 (Cauchy)")
#' plot(qtlambda(pp, 0), pp, type = "l", main = "lambda = 0 (logistic)")
#' plot(qtlambda(pp, 0.14), pp, type = "l", main = "lambda = 0.14 (normal)")
#' plot(qtlambda(pp, 0.5), pp, type = "l", main = "lambda = 0.5 (concave)")
#' plot(qtlambda(pp, 1), pp, type = "l", main = "lambda = 1 (uniform)")
#' plot(qtlambda(pp, 2), pp, type = "l", main = "lambda = 2 (uniform)")
#' 
#' hist(rtlambda(1e5, -1), freq = FALSE, main = "lambda = -1 (Cauchy)")
#' hist(rtlambda(1e5, 0), freq = FALSE, main = "lambda = 0 (logistic)")
#' hist(rtlambda(1e5, 0.14), freq = FALSE, main = "lambda = 0.14 (normal)")
#' hist(rtlambda(1e5, 0.5), freq = FALSE, main = "lambda = 0.5 (concave)")
#' hist(rtlambda(1e5, 1), freq = FALSE, main = "lambda = 1 (uniform)")
#' hist(rtlambda(1e5, 2), freq = FALSE, main = "lambda = 2 (uniform)")
#' par(partmp)
#'
#' @name TukeyLambda
#' @aliases TukeyLambda
#' @aliases qtlambda
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

qtlambda <- function(p, lambda, lower.tail = TRUE, log.p = FALSE) {
  cpp_qtlambda(p, lambda, lower.tail[1L], log.p[1L])
}


#' @rdname TukeyLambda
#' @export

rtlambda <- function(n, lambda) {
  if (length(n) > 1) n <- length(n)
  cpp_rtlambda(n, lambda)
}

#' Slash distribution
#' 
#' Probability mass function, distribution function and random generation
#' for slash distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu              vector of locations
#' @param sigma           vector of positive valued scale parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @details
#' 
#' If \eqn{Z \sim \mathrm{Normal}(0, 1)}{Z ~ Normal(0, 1)} and \eqn{U \sim \mathrm{Uniform}(0, 1)}{U ~ Uniform(0, 1)},
#' then \eqn{Z/U} follows slash distribution.
#' 
#' Probability density function
#' 
#' \deqn{
#' f(x) = \left\{\begin{array}{ll}
#' \frac{\phi(0) - \phi(x)}{x^2} & x \ne 0 \\
#' \frac{1}{2\sqrt{2\pi}} & x = 0
#' \end{array}\right.
#' }{
#' f(x) = [if x != 0:] (\phi(0)-\phi(x))/x^2 [else:] 1/(2*sqrt(2*\pi))
#' }
#' 
#' Cumulative distribution function
#' 
#' \deqn{
#' F(x) = \left\{\begin{array}{ll}
#' \Phi(x) - \frac{\phi(0)-\phi(x)}{x} & x \neq 0 \\
#' \frac{1}{2} & x = 0
#' \end{array}\right.
#' }{
#' F(x) = [if x != 0:] \Phi(x) - [\phi(0)-\phi(x)]/x [else:] 1/2
#' }
#' 
#' @examples 
#' 
#' x <- rslash(1e5, 5, 3)
#' hist(x, 1e5, freq = FALSE, xlim = c(-100, 100))
#' curve(dslash(x, 5, 3), -100, 100, col = "red", n = 500, add = TRUE)
#' hist(pslash(x, 5, 3))
#' plot(ecdf(x), xlim = c(-100, 100))
#' curve(pslash(x, 5, 3), -100, 100, col = "red", lwd = 2, n = 500, add = TRUE)
#' 
#' @name Slash
#' @aliases Slash
#' @aliases dslash
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#' 
#' @export

dslash <- function(x, mu = 0, sigma = 1, log = FALSE) {
  cpp_dslash(x, mu, sigma, log[1L])
}


#' @rdname Slash
#' @export

pslash <- function(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pslash(q, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Slash
#' @export

rslash <- function(n, mu = 0, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rslash(n, mu, sigma)
}

#' Truncated normal distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the truncated normal distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mean,sd         location and scale parameters. Scale must be positive.
#' @param a,b             lower and upper truncation points (\code{a < x <= b},
#'                        with \code{a = -Inf} and \code{b = Inf} by default).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{\phi(\frac{x-\mu}{\sigma})}
#'             {\Phi(\frac{b-\mu}{\sigma}) - \Phi(\frac{a-\mu}{\sigma})}
#' }{
#' f(x) = \phi((x-\mu)/\sigma) / (\Phi((b-\mu)/\sigma) - \Phi((a-\mu)/\sigma))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \frac{\Phi(\frac{x-\mu}{\sigma}) - \Phi(\frac{a-\mu}{\sigma})}
#'             {\Phi(\frac{b-\mu}{\sigma}) - \Phi(\frac{a-\mu}{\sigma})}
#' }{
#' F(x) = (\Phi((x-\mu)/\sigma) - \Phi((a-\mu)/\sigma)) / (\Phi((b-\mu)/\sigma) - \Phi((a-\mu)/\sigma))
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \Phi^{-1}\left(\Phi\left(\frac{a-\mu}{\sigma}\right) + p \times
#'                      \left[\Phi\left(\frac{b-\mu}{\sigma}\right) -
#'                      \Phi\left(\frac{a-\mu}{\sigma}\right)\right]\right)
#' }{
#' F^-1(p) = \Phi^-1(\Phi((a-\mu)/\sigma) + p * (\Phi((b-\mu)/\sigma) - \Phi((a-\mu)/\sigma)))
#' }
#'
#' For random generation algorithm described by Robert (1995) is used.
#'
#' @references
#' Robert, C.P. (1995). Simulation of truncated normal variables.
#' Statistics and Computing 5(2): 121-125. \url{https://arxiv.org/abs/0907.4010}
#'
#' @references
#' Burkardt, J. (17 October 2014). The Truncated Normal Distribution. Florida State University.
#' \url{https://people.sc.fsu.edu/~jburkardt/presentations/truncated_normal.pdf}
#' 
#' @examples 
#' 
#' x <- rtnorm(1e5, 5, 3, b = 7)
#' hist(x, 100, freq = FALSE)
#' curve(dtnorm(x, 5, 3, b = 7), -8, 8, col = "red", add = TRUE)
#' hist(ptnorm(x, 5, 3, b = 7))
#' plot(ecdf(x))
#' curve(ptnorm(x, 5, 3, b = 7), -8, 8, col = "red", lwd = 2, add = TRUE)
#' 
#' R <- 1e5
#' partmp <- par(mfrow = c(2,4), mar = c(2,2,2,2))
#' 
#' hist(rtnorm(R), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x), -5, 5, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, a = 0), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, a = 0), -1, 5, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, b = 0), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, b = 0), -5, 5, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, a = 0, b = 1), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, a = 0, b = 1), -1, 2, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, a = -1, b = 0), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, a = -1, b = 0), -2, 2, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, mean = -6, a = 0), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, mean = -6, a = 0), -2, 1, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, mean = 8, b = 0), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, mean = 8, b = 0), -2, 1, col = "red", add = TRUE)
#' 
#' hist(rtnorm(R, a = 3, b = 5), freq= FALSE, main = "", xlab = "", ylab = "")
#' curve(dtnorm(x, a = 3, b = 5), 2, 5, col = "red", add = TRUE)
#' 
#' par(partmp)
#'
#' @name TruncNormal
#' @aliases TruncNormal
#' @aliases dtnorm
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dtnorm <- function(x, mean = 0, sd = 1, a = -Inf, b = Inf, log = FALSE) {
  cpp_dtnorm(x, mean, sd, a, b, log[1L])
}


#' @rdname TruncNormal
#' @export

ptnorm <- function(q, mean = 0, sd = 1, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE) {
  cpp_ptnorm(q, mean, sd, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname TruncNormal
#' @export

qtnorm <- function(p, mean = 0, sd = 1, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE) {
  cpp_qtnorm(p, mean, sd, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname TruncNormal
#' @export

rtnorm <- function(n, mean = 0, sd = 1, a = -Inf, b = Inf) {
  if (length(n) > 1) n <- length(n)
  cpp_rtnorm(n, mean, sd, a, b)
}

test_that("other tests", {
  
  expect_true(all(rsign(1e5) %in% c(-1, 1)))
  
  expect_silent(qcat(seq(0, 1, by = 0.1), c(1, 1, 1), labels = letters[1:3]))
  expect_warning(qcat(seq(0, 1, by = 0.1), c(1, 1, 1), labels = letters[1:2]))
  
  expect_silent(rcat(10, c(1, 1, 1), labels = letters[1:3]))
  expect_warning(rcat(10, c(1, 1, 1), labels = letters[1:2]))
  
  expect_silent(rcatlp(10, log(c(1, 1, 1)/3), labels = letters[1:3]))
  expect_warning(rcatlp(10, log(c(1, 1, 1)/3), labels = letters[1:2]))

  expect_error(dbvpois(1:10, a = 1, b = 1, c= 1))
  
  expect_silent(!anyNA(rtlambda(5000, -1)))
  expect_silent(!anyNA(rtlambda(5000, 0)))
  expect_silent(!anyNA(rtlambda(5000, 1)))
  
  expect_silent(dbvnorm(mtcars[, 1:2]))
  expect_error(dbvnorm(mtcars))
  
  xx <- seq(-6, 6, by = 0.01)
  expect_identical(dnorm(xx), dtnorm(xx))
  expect_identical(pnorm(xx), ptnorm(xx))
  
  pp <- seq(0, 1, by = 0.01)
  expect_identical(qnorm(pp), qtnorm(pp))
  
  x <- c(-Inf, -100, -10, -5, -1, -0.5, 0, 0.5, 1, 5, 10, 100, Inf)
  
  expect_equal(pmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3), lower.tail = TRUE),
               1 - pmixnorm(x, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3), lower.tail = FALSE))
  expect_equal(suppressWarnings(pmixpois(x, c(1,2,3), c(1/3,1/3,1/3), lower.tail = TRUE)),
               1 - suppressWarnings(pmixpois(x, c(1,2,3), c(1/3,1/3,1/3), lower.tail = FALSE)))
  
})

all_in_bounds <- function(object, lower, upper) {
  all(object >= lower & object <= upper)
}


test_that("Check if values generated using rtnorm stay in bounds", {
  
  N <- 5000
  mu <- 0
  sigma <- 1
  
  a <- -1
  b <- 1
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- 0
  b <- 1
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- -1
  b <- 0
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- 4
  b <- 5
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- -5
  b <- -4
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- -Inf
  b <- 5
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- -5
  b <- Inf
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- -Inf
  b <- -4
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

  a <- 4
  b <- Inf
  X <- rtnorm(N, mu, sigma, a, b)
  
  expect_true(all_in_bounds(X, a, b))

})

#' Mixture of Poisson distributions
#'
#' Density, distribution function and random generation
#' for the mixture of Poisson distributions.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param lambda 	        matrix (or vector) of (non-negative) means.
#' @param alpha           matrix (or vector) of mixing proportions;
#'                        mixing proportions need to sum up to 1.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \alpha_1 f_1(x; \lambda_1) + \dots + \alpha_k f_k(x; \lambda_k)
#' }{
#' f(x) = \alpha[1] * f1(x; \lambda[1]) + \dots + \alpha[k] * fk(x; \lambda[k])
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \alpha_1 F_1(x; \lambda_1) + \dots + \alpha_k F_k(x; \lambda_k)
#' }{
#' F(x) = \alpha[1] * F1(x; \lambda[1]) + \dots + \alpha[k] * Fk(x; \lambda[k])
#' }
#' 
#' where \eqn{\sum_i \alpha_i = 1}{sum(\alpha[i]) == 1}.
#'
#' @examples 
#' 
#' x <- rmixpois(1e5, c(5, 12, 19), c(1/3, 1/3, 1/3))
#' xx <- seq(-1, 50)
#' plot(prop.table(table(x)))
#' lines(xx, dmixpois(xx, c(5, 12, 19), c(1/3, 1/3, 1/3)), col = "red")
#' hist(pmixpois(x, c(5, 12, 19), c(1/3, 1/3, 1/3)))
#' 
#' xx <- seq(0, 50, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pmixpois(xx, c(5, 12, 19), c(1/3, 1/3, 1/3)), col = "red", lwd = 2)
#'
#' @name PoissonMix
#' @aliases PoissonMix
#' @aliases dmixpois
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dmixpois <- function(x, lambda, alpha, log = FALSE) {
  
  if (is.vector(lambda))
    lambda <- matrix(lambda, nrow = 1)
  else if (!is.matrix(lambda))
    lambda <- as.matrix(lambda)
  
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  
  cpp_dmixpois(x, lambda, alpha, log[1L])
}


#' @rdname PoissonMix
#' @export

pmixpois <- function(q, lambda, alpha, lower.tail = TRUE, log.p = FALSE) {
  
  if (is.vector(lambda))
    lambda <- matrix(lambda, nrow = 1)
  else if (!is.matrix(lambda))
    lambda <- as.matrix(lambda)
  
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  
  cpp_pmixpois(q, lambda, alpha, lower.tail[1L], log.p[1L])
}


#' @rdname PoissonMix
#' @export

rmixpois <- function(n, lambda, alpha) {
  if (length(n) > 1) n <- length(n)
  
  if (is.vector(lambda))
    lambda <- matrix(lambda, nrow = 1)
  else if (!is.matrix(lambda))
    lambda <- as.matrix(lambda)
  
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  
  cpp_rmixpois(n, lambda, alpha)
}

#' Non-standard beta distribution
#' 
#' Non-standard form of beta distribution with lower and upper bounds
#' denoted as \code{min} and \code{max}. By default \code{min=0}
#' and \code{max=1} what leads to standard beta distribution.
#' 
#' @param x,q	          vector of quantiles.
#' @param p             vector of probabilities.
#' @param n             number of observations. If \code{length(n) > 1},
#'                      the length is taken to be the number required.
#' @param shape1,shape2 non-negative parameters of the Beta distribution.
#' @param min,max       lower and upper bounds.
#' @param log,log.p	    logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	  logical; if TRUE (default), probabilities are \eqn{P[X \leq x]},
#'                      otherwise, \eqn{P[X > x]}.
#'                      
#' @seealso \code{\link[stats]{Beta}}
#' 
#' @examples 
#' 
#' x <- rnsbeta(1e5, 5, 13, -4, 8)
#' hist(x, 100, freq = FALSE)
#' curve(dnsbeta(x, 5, 13, -4, 8), -4, 6, col = "red", add = TRUE) 
#' hist(pnsbeta(x, 5, 13, -4, 8))
#' plot(ecdf(x))
#' curve(pnsbeta(x, 5, 13, -4, 8), -4, 6, col = "red", lwd = 2, add = TRUE)
#' 
#' @name NSBeta
#' @aliases NSBeta
#' @aliases dgbeta
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'                     
#' @export

dnsbeta <- function(x, shape1, shape2, min = 0, max = 1, log = FALSE) {
  cpp_dnsbeta(x, shape1, shape2, min, max, log[1L])
}


#' @rdname NSBeta
#' @export

pnsbeta <- function(q, shape1, shape2, min = 0, max = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pnsbeta(q, shape1, shape2, min, max, lower.tail[1L], log.p[1L])
}


#' @rdname NSBeta
#' @export

qnsbeta <- function(p, shape1, shape2, min = 0, max = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qnsbeta(p, shape1, shape2, min, max, lower.tail[1L], log.p[1L])
}


#' @rdname NSBeta
#' @export

rnsbeta <- function(n, shape1, shape2, min = 0, max = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rnsbeta(n, shape1, shape2, min, max)
}

#' Bernoulli distribution
#'
#' Probability mass function, distribution function, quantile function and random generation
#' for the Bernoulli distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param prob            probability of success; (\code{0 < prob < 1}).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'                        
#' @seealso \code{\link[stats]{Binomial}}
#' 
#' @examples 
#' 
#' prop.table(table(rbern(1e5, 0.5)))
#'
#' @name Bernoulli
#' @aliases Bernoulli
#' @aliases dbern
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dbern <- function(x, prob = 0.5, log = FALSE) {
  cpp_dbern(x, prob, log[1L])
}


#' @rdname Bernoulli
#' @export

pbern <- function(q, prob = 0.5, lower.tail = TRUE, log.p = FALSE) {
  cpp_pbern(q, prob, lower.tail[1L], log.p[1L])
}


#' @rdname Bernoulli
#' @export

qbern <- function(p, prob = 0.5, lower.tail = TRUE, log.p = FALSE) {
  cpp_qbern(p, prob, lower.tail[1L], log.p[1L])
}


#' @rdname Bernoulli
#' @export

rbern <- function(n, prob = 0.5) {
  if (length(n) > 1) n <- length(n)
  cpp_rbern(n, prob)
}

#' Multivariate hypergeometric distribution
#'
#' Probability mass function and random generation
#' for the multivariate hypergeometric distribution.
#'
#' @param x 	 \eqn{m}-column matrix of quantiles.
#' @param nn   number of observations. If \code{length(n) > 1},
#'             the length is taken to be the number required.
#' @param n    \eqn{m}-length vector or \eqn{m}-column matrix
#'             of numbers of balls in \eqn{m} colors.
#' @param k    the number of balls drawn from the urn.
#' @param log  logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#'
#' Probability mass function
#' \deqn{
#' f(x) = \frac{\prod_{i=1}^m {n_i \choose x_i}}{{N \choose k}}
#' }{
#' f(x) = prod(choose(n, x)) / choose(N, k)
#' }
#' 
#' @details
#' The multivariate hypergeometric distribution is generalization of
#' hypergeometric distribution. It is used for sampling \emph{without} replacement
#' \eqn{k} out of \eqn{N} marbles in \eqn{m} colors, where each of the colors appears
#' \eqn{n_i}{n[i]} times. Where \eqn{k=\sum_{i=1}^m x_i}{k=sum(x)},
#' \eqn{N=\sum_{i=1}^m n_i}{N=sum(n)} and \eqn{k \le N}{k<=N}.
#' 
#' @references 
#' Gentle, J.E. (2006). Random number generation and Monte Carlo methods. Springer.
#'
#' @seealso \code{\link[stats]{Hypergeometric}}
#' 
#' @examples 
#' 
#' # Generating 10 random draws from multivariate hypergeometric
#' # distribution parametrized using a vector
#' 
#' rmvhyper(10, c(10, 12, 5, 8, 11), 33)
#'
#' @name MultiHypergeometric
#' @aliases MultiHypergeometric
#' @aliases dmvhyper
#' 
#' @keywords distribution
#' @concept Multivariate
#' @concept Discrete
#'
#' @export

dmvhyper <- function(x, n, k, log = FALSE) {
  if (is.vector(n))
    n <- matrix(n, nrow = 1)
  else if (!is.matrix(n))
    n <- as.matrix(n)
  
  if (is.vector(x))
    x <- matrix(x, nrow = 1)
  else if (!is.matrix(x))
    x <- as.matrix(x)
  
  cpp_dmvhyper(x, n, k, log[1L])
}


#' @rdname MultiHypergeometric
#' @export

rmvhyper <- function(nn, n, k) {
  if (length(nn) > 1)
    nn <- length(nn)
  
  if (is.vector(n) && length(k) == 1) {
    if (anyNA(n) || is.na(k)) {
      warning("NAs produced")
      return(matrix(rep(NA, nn), nrow = nn, byrow = TRUE))
    }
    if (sum(n) == k)
      return(matrix(rep(n, nn), nrow = nn, byrow = TRUE))
    n <- matrix(n, nrow = 1)
  } else if (!is.matrix(n)) {
    n <- as.matrix(n)
  }
  
  cpp_rmvhyper(nn, n, k)
}

# extraDistr::ddirichlet was compared to MCMCpack::ddirichlet

test_that("Testing multivariate distributions", {
  
  skip_on_cran()

  expect_true(all.equal(rowSums(rmnom(5000, 50, c(2/10, 5/10, 3/10))), rep(50, 5000)))
  expect_true(all.equal(rowSums(rdirichlet(5000, c(2/10, 5/10, 3/10))), rep(1.0, 5000)))

  xx <- expand.grid(0:20, 0:20, 0:20)
  expect_equal(sum(ddirmnom(xx[rowSums(xx) == 20,], 20, c(2, 5, 3))), 1)
  expect_equal(sum(dmnom(xx[rowSums(xx) == 20,], 20, c(2/10, 5/10, 3/10))), 1)
  expect_equal(sum(dmvhyper(xx[rowSums(xx) == 35,], c(20, 20, 20), 35)), 1)

  p <- c(4, 5, 1, 6, 2)

  expect_equal(prop.table(colSums(rmnom(1e5, 100, p/sum(p)))),
               p/sum(p),
               tolerance = 1e-2)

  expect_equal(as.numeric(prop.table(table(rcat(1e5, p/sum(p))))),
               p/sum(p),
               tolerance = 1e-2)

  expect_equal(prop.table(colSums(rdirichlet(1e5, p))),
               p/sum(p),
               tolerance = 1e-2)

  expect_equal(prop.table(colSums(rdirmnom(1e5, 100, p))),
               p/sum(p),
               tolerance = 1e-2)

  n <- c(11, 24, 43, 7, 56)

  expect_equal(prop.table(colSums(rmvhyper(1e5, n, 100))),
               n/sum(n),
               tolerance = 1e-2)

})


test_that("Evaluate wrong parameters first", {

  expect_warning(expect_true(is.nan(dbvpois(-1, -1, -1, 1, 1))))
  expect_warning(expect_true(is.nan(ddirichlet(c(2, 2), c(-1, 0.5)))))
  expect_warning(expect_true(is.nan(ddirmnom(c(-1, 1, 1), 1.5, c(1, 1, 1)))))
  expect_warning(expect_true(is.nan(dmnom(c(-1, 1, 1), 1.5, c(1/3, 1/3, 1/3)))))
  expect_warning(expect_true(is.nan(dmvhyper(c(-1, 2, 2), c(2,3,4), -5))))

})


test_that("Check if rmnom and rdirmnom deal with underflow (#3, #7)", {
  
  skip_on_cran()
  
  expect_false(anyNA(rmnom(5000, 100, c(0.504115095275327, 2.669522645838e-39, 0, 2.58539638831141, 0))))
  expect_false(anyNA(rdirmnom(5000, 100, c(1.480592e+00, 1.394943e-03, 4.529932e-06, 3.263573e+00, 4.554952e-06))))
  
  p <- c(0, 0, 1, 0, 2.77053929981958e-18)
  
  expect_false(anyNA(rmnom(5000, 100, p)))
  expect_false(anyNA(rdirmnom(5000, 100, p + 1e-5)))
  
})

#' Beta prime distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the beta prime distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param shape1,shape2	  non-negative parameters.
#' @param scale           positive valued scale parameter.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details 
#' 
#' If \eqn{X \sim \mathrm{Beta}(\alpha, \beta)}{X ~ Beta(\alpha, \beta)}, then
#' \eqn{\frac{X}{1-X} \sim \mathrm{BetaPrime}(\alpha, \beta)}{X/(1-X) ~ BetaPrime(\alpha, \beta)}.
#' 
#' Probability density function
#' 
#' \deqn{
#' f(x) = \frac{(x/\sigma)^{\alpha-1} (1+x/\sigma)^{-\alpha -\beta}}{\mathrm{B}(\alpha,\beta)\sigma}
#' }{
#' f(x) = ((x/\sigma)^(\alpha-1) * (1 + x/\sigma)^(-\alpha-\beta)) / (B(\alpha,\beta) * \sigma)
#' }
#' 
#' Cumulative distribution function
#' 
#' \deqn{
#' F(x) = I_{\frac{x/\sigma}{1+x/\sigma}}(\alpha, \beta)
#' }{
#' F(x) = pbeta((x/\sigma)/(1+(x/\sigma)), \alpha, \beta)
#' }
#' 
#' @seealso \code{\link[stats]{Beta}}
#'
#' @examples 
#' 
#' x <- rbetapr(1e5, 5, 3, 2)
#' hist(x, 350, freq = FALSE, xlim = c(0, 100))
#' curve(dbetapr(x, 5, 3, 2), 0, 100, col = "red", add = TRUE, n = 500)
#' hist(pbetapr(x, 5, 3, 2))
#' plot(ecdf(x), xlim = c(0, 100))
#' curve(pbetapr(x, 5, 3, 2), 0, 100, col = "red", add = TRUE, n = 500)
#' 
#' @name BetaPrime
#' @aliases BetaPrime
#' @aliases dbetapr
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dbetapr <- function(x, shape1, shape2, scale = 1, log = FALSE) {
  cpp_dbetapr(x, shape1, shape2, scale, log[1L])
}


#' @rdname BetaPrime
#' @export

pbetapr <- function(q, shape1, shape2, scale = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pbetapr(q, shape1, shape2, scale, lower.tail[1L], log.p[1L])
}


#' @rdname BetaPrime
#' @export

qbetapr <- function(p, shape1, shape2, scale = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qbetapr(p, shape1, shape2, scale, lower.tail[1L], log.p[1L])
}


#' @rdname BetaPrime
#' @export

rbetapr <- function(n, shape1, shape2, scale = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rbetapr(n, shape1, shape2, scale)
}

#' Dirichlet distribution
#'
#' Density function and random generation for the Dirichlet distribution.
#'
#' @param x               \eqn{k}-column matrix of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param alpha           \eqn{k}-values vector or \eqn{k}-column matrix;
#'                        concentration parameter. Must be positive.
#' @param log     	      logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{\Gamma(\sum_k \alpha_k)}{\prod_k \Gamma(\alpha_k)} \prod_k x_k^{\alpha_k-1}
#' }{
#' f(x) = \Gamma(sum(\alpha[k])) / prod(\Gamma(\alpha[k])) * prod(x[k]^{\alpha[k]-1})
#' }
#'
#' @references
#' Devroye, L. (1986). Non-Uniform Random Variate Generation. Springer-Verlag.
#' 
#' @examples
#' 
#' # Generating 10 random draws from Dirichlet distribution
#' # parametrized using a vector
#' 
#' rdirichlet(10, c(1, 1, 1, 1))
#' 
#' # or parametrized using a matrix where each row
#' # is a vector of parameters
#' 
#' alpha <- matrix(c(1, 1, 1, 1:3, 7:9), ncol = 3, byrow = TRUE)
#' rdirichlet(10, alpha)
#' 
#' @name Dirichlet
#' @aliases Dirichlet
#' @aliases ddirichlet
#' 
#' @keywords distribution
#' @concept Multivariate
#' @concept Continuous
#' 
#' @export

ddirichlet <- function(x, alpha, log = FALSE) {
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  if (is.data.frame(x))
    x <- as.matrix(x)
  else if (is.vector(x))
    x <- matrix(x, byrow = TRUE, nrow = 1)
  cpp_ddirichlet(x, alpha, log[1L])
}


#' @rdname Dirichlet
#' @export

rdirichlet <- function (n, alpha) {
  if (length(n) > 1) n <- length(n)
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  cpp_rdirichlet(n, alpha)
}

#' Generalized extreme value distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the generalized extreme value distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu,sigma,xi	    location, scale, and shape parameters. Scale must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{ f(x) = \left\{\begin{array}{ll}
#' \frac{1}{\sigma} \left(1+\xi \frac{x-\mu}{\sigma}\right)^{-1/\xi-1} \exp\left(-\left(1+\xi \frac{x-\mu}{\sigma}\right)^{-1/\xi}\right) & \xi \neq 0 \\
#' \frac{1}{\sigma} \exp\left(- \frac{x-\mu}{\sigma}\right) \exp\left(-\exp\left(- \frac{x-\mu}{\sigma}\right)\right)                       & \xi = 0
#' \end{array}\right.
#' }{
#' f(x) = [if \xi != 0:] 1/\sigma * (1+\xi*(x-\mu)/\sigma)^{-1/\xi-1} * exp(-(1+\xi*(x-\mu)/\sigma)^{-1/\xi})
#' [else:] 1/\sigma * exp(-(x-\mu)/\sigma) * exp(-exp(-(x-\mu)/\sigma))
#' }
#'
#' Cumulative distribution function
#' \deqn{ F(x) = \left\{\begin{array}{ll}
#' \exp\left(-\left(1+\xi \frac{x-\mu}{\sigma}\right)^{1/\xi}\right) & \xi \neq 0 \\
#' \exp\left(-\exp\left(- \frac{x-\mu}{\sigma}\right)\right)          & \xi = 0
#' \end{array}\right.
#' }{
#' F(x) = [if \xi != 0:] exp(-(1+\xi*(x-\mu)/\sigma)^{1/\xi})
#' [else:] exp(-exp(-(x-\mu)/\sigma))
#' }
#'
#' Quantile function
#' \deqn{ F^{-1}(p) = \left\{\begin{array}{ll}
#' \mu - \frac{\sigma}{\xi} (1 - (-\log(p))^\xi)  & \xi \neq 0 \\
#' \mu - \sigma \log(-\log(p))                    & \xi = 0
#' \end{array}\right.
#' }{
#' F^-1(p) = [if \xi != 0:] \mu - \sigma/\xi * (1 - (-log(p))^\xi)
#'           [else:] \mu - \sigma * log(-log(p))
#' }
#'
#' @references
#' Coles, S. (2001). An Introduction to Statistical Modeling of Extreme Values.
#' Springer.
#'
#' @examples 
#' 
#' curve(dgev(x, xi = -1/2), -4, 4, col = "green", ylab = "")
#' curve(dgev(x, xi = 0), -4, 4, col = "red", add = TRUE)
#' curve(dgev(x, xi = 1/2), -4, 4, col = "blue", add = TRUE)
#' legend("topleft", col = c("green", "red", "blue"), lty = 1,
#'        legend = expression(xi == -1/2, xi == 0, xi == 1/2), bty = "n")
#' 
#' x <- rgev(1e5, 5, 2, .5)
#' hist(x, 1000, freq = FALSE, xlim = c(0, 50))
#' curve(dgev(x, 5, 2, .5), 0, 50, col = "red", add = TRUE, n = 5000)
#' hist(pgev(x, 5, 2, .5))
#' plot(ecdf(x), xlim = c(0, 50))
#' curve(pgev(x, 5, 2, .5), 0, 50, col = "red", lwd = 2, add = TRUE)
#'
#' @name GEV
#' @aliases GEV
#' @aliases dgev
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dgev <- function(x, mu = 0, sigma = 1, xi = 0, log = FALSE) {
  cpp_dgev(x, mu, sigma, xi, log[1L])
}


#' @rdname GEV
#' @export

pgev <- function(q, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_pgev(q, mu, sigma, xi, lower.tail[1L], log.p[1L])
}


#' @rdname GEV
#' @export

qgev <- function(p, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_qgev(p, mu, sigma, xi, lower.tail[1L], log.p[1L])
}


#' @rdname GEV
#' @export

rgev <- function(n, mu = 0, sigma = 1, xi = 0) {
  if (length(n) > 1) n <- length(n)
  cpp_rgev(n, mu, sigma, xi)
}

#' Bivariate normal distribution
#'
#' Density, distribution function and random generation
#' for the bivariate normal distribution.
#'
#' @param x,y	        vectors of quantiles; alternatively x may be a two-column
#'                    matrix (or data.frame) and y may be omitted.
#' @param n	          number of observations. If \code{length(n) > 1},
#'                    the length is taken to be the number required.
#' @param mean1,mean2 vectors of means.
#' @param sd1,sd2     vectors of standard deviations.
#' @param cor         vector of correlations (\code{-1 < cor < 1}).
#' @param log     	  logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{1}{2\pi\sqrt{1-\rho^2}\sigma_1\sigma_2}
#'        \exp\left\{-\frac{1}{2(1-\rho^2)} \left[\left(\frac{x_1 - \mu_1}{\sigma_1}\right)^2 -
#'        2\rho \left(\frac{x_1 - \mu_1}{\sigma_1}\right) \left(\frac{x_2 - \mu_2}{\sigma_2}\right) +
#'        \left(\frac{x_2 - \mu_2}{\sigma_2}\right)^2\right]\right\}
#' }{
#' f(x) = 1/(2*\pi*sqrt(1-\rho^2)*\sigma1*\sigma2) * exp(-(1/(2*(1-\rho^2)*
#'        (((x1-\mu1)/\sigma1)^2 - 2*\rho*((x1-\mu1)/\sigma2)*((x2-\mu2)/\sigma2) *
#'        ((x2-\mu2)/\sigma2)^2))))
#' }
#'
#' @references
#' Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
#' with Applications. Chapman & Hall/CRC
#' 
#' @references 
#' Mukhopadhyay, N. (2000). Probability and statistical inference.
#' Chapman & Hall/CRC
#' 
#' @examples 
#' 
#' y <- x <- seq(-4, 4, by = 0.25)
#' z <- outer(x, y, function(x, y) dbvnorm(x, y, cor = -0.75))
#' persp(x, y, z)
#'
#' y <- x <- seq(-4, 4, by = 0.25)
#' z <- outer(x, y, function(x, y) dbvnorm(x, y, cor = -0.25))
#' persp(x, y, z)
#'
#' @seealso \code{\link[stats]{Normal}}
#'
#' @name BivNormal
#' @aliases BivNormal
#' @aliases dbvnorm
#' 
#' @keywords distribution
#' @concept Multivariate
#' @concept Continuous
#'
#' @export

dbvnorm <- function(x, y = NULL, mean1 = 0, mean2 = mean1, sd1 = 1, sd2 = sd1, cor = 0, log = FALSE) {
  if (is.null(y)) {
    if ((is.matrix(x) || is.data.frame(x)) && ncol(x) == 2) {
      y <- x[, 2]
      x <- x[, 1]
    } else {
      stop("y is not provided while x is not a two-column matrix")
    }
  }
  cpp_dbnorm(x, y, mean1, mean2, sd1, sd2, cor, log[1L])
}


#' @rdname BivNormal
#' @export

rbvnorm <- function(n, mean1 = 0, mean2 = mean1, sd1 = 1, sd2 = sd1, cor = 0) {
  if (length(n) > 1) n <- length(n)
  cpp_rbnorm(n, mean1, mean2, sd1, sd2, cor)
}

#' Inverse chi-squared and scaled chi-squared distributions
#'
#' Density, distribution function and random generation
#' for the inverse chi-squared distribution and scaled chi-squared distribution.
#' The implementation uses a non-standard linear scale parameterization of \eqn{\tau}; 
#' see Details for the precise definition and its relation to the standard \eqn{\tau^2} formulation.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param nu              positive valued shape parameter.
#' @param tau             positive valued scaling parameter; if provided it
#'                        returns values for scaled chi-squared distributions.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
# @details
#
# If \eqn{X} follows \eqn{\chi^2 (\nu)} distribution, then \eqn{1/X} follows inverse
# chi-squared distribution parametrized by \eqn{\nu}. Inverse chi-squared distribution
# is a special case of inverse gamma distribution with parameters
# \eqn{\alpha=\frac{\nu}{2}}{\alpha=\nu/2} and \eqn{\beta=\frac{1}{2}}{\beta=1/2};
# or \eqn{\alpha=\frac{\nu}{2}}{\alpha=\nu/2} and
# \eqn{\beta=\frac{\nu\tau}{2}}{\beta=(\nu\tau)/2} for scaled inverse
# chi-squared distribution.
# 
#' @details
#' If \eqn{X} follows \eqn{\chi^2 (\nu)} distribution, then \eqn{1/X} follows inverse
#' chi-squared distribution parametrized by \eqn{\nu}. Inverse chi-squared distribution
#' is a special case of inverse gamma distribution with parameters
#' \eqn{\alpha=\frac{\nu}{2}}{\alpha=\nu/2} and \eqn{\beta=\frac{1}{2}}{\beta=1/2};
#' or \eqn{\alpha=\frac{\nu}{2}}{\alpha=\nu/2} and
#' \eqn{\beta=\frac{\nu\tau}{2}}{\beta=(\nu\tau)/2}.
#' 
#' \strong{Note on parameterization:} this implementation uses a linear scaling parameter \eqn{\tau}. 
#' The scaled inverse chi-squared distribution is defined via the equivalent inverse-gamma form
#' 
#' \deqn{X \sim \text{InvGamma}\left(\frac{\nu}{2}, \frac{\nu\tau}{2}\right),}
#' 
#' where \eqn{\tau} enters linearly in the second parameter.
#' 
#' This differs from the most common statistical convention, which uses a squared scale parameter \eqn{\tau^2}:
#'   
#'   \deqn{X \sim \text{InvGamma}\left(\frac{\nu}{2}, \frac{\nu\tau^2}{2}\right),}
#' 
#' The two parameterizations are related by
#' 
#' \deqn{\tau_{this implementation} = \tau_{standard}^2.}
#' 
#' Users familiar with the standard scaled inverse chi-squared distribution should be aware 
#' that \eqn{\tau} here corresponds to a variance-scale parameter rather than a 
#' standard-deviation-type parameter.
#' 
#' @seealso \code{\link[stats]{Chisquare}}, \code{\link[stats]{GammaDist}}
#' 
#' @examples 
#' 
#' x <- rinvchisq(1e5, 20)
#' hist(x, 100, freq = FALSE)
#' curve(dinvchisq(x, 20), 0, 1, n = 501, col = "red", add = TRUE)
#' hist(pinvchisq(x, 20))
#' plot(ecdf(x))
#' curve(pinvchisq(x, 20), 0, 1, n = 501, col = "red", lwd = 2, add = TRUE)
#' 
#' # scaled
#' 
#' x <- rinvchisq(1e5, 10, 5)
#' hist(x, 100, freq = FALSE)
#' curve(dinvchisq(x, 10, 5), 0, 150, n = 501, col = "red", add = TRUE)
#' hist(pinvchisq(x, 10, 5))
#' plot(ecdf(x))
#' curve(pinvchisq(x, 10, 5), 0, 150, n = 501, col = "red", lwd = 2, add = TRUE)
#'
#' @name InvChiSq
#' @aliases InvChiSq
#' @aliases dinvchisq
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dinvchisq <- function(x, nu, tau, log = FALSE) {
  if (missing(tau))
    dinvgamma(x, nu/2, 0.5, log = log[1L])
  else
    dinvgamma(x, nu/2, (nu*tau)/2, log = log[1L])
}


#' @rdname InvChiSq
#' @export

pinvchisq <- function(q, nu, tau, lower.tail = TRUE, log.p = FALSE) {
  if (missing(tau))
    cpp_pinvgamma(q, nu/2, 0.5, lower.tail, log.p[1L])
  else
    cpp_pinvgamma(q, nu/2, (nu*tau)/2, lower.tail, log.p[1L])
}


#' @rdname InvChiSq
#' @export

qinvchisq <- function(p, nu, tau, lower.tail = TRUE, log.p = FALSE) {
  if (missing(tau))
    1/qchisq(p, nu, lower.tail = !lower.tail[1L], log.p = log.p[1L])
  else
    1/qgamma(p, nu/2, (nu*tau)/2, lower.tail = !lower.tail[1L], log.p = log.p[1L])
}


#' @rdname InvChiSq
#' @export

rinvchisq <- function(n, nu, tau) {
  if (length(n) > 1) 
    n <- length(n)
  if (missing(tau))
    1/rchisq(n, nu)
  else
    1/rgamma(n, nu/2, (nu*tau)/2)
}

#' Triangular distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the triangular distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param a,b,c           minimum, maximum and mode of the distribution.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \left\{\begin{array}{ll}
#' \frac{2(x-a)}{(b-a)(c-a)} & x < c \\
#' \frac{2}{b-a}             & x = c \\
#' \frac{2(b-x)}{(b-a)(b-c)} & x > c
#' \end{array}\right.
#' }{
#' f(x) = [if x < c:] (2*(x-a)) / ((b-a)*(c-a))
#'        [if x = c:] 2/(b-a)
#'        [if x >= c:] (2*(b-x)) / ((b-a)*(b-c))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \left\{\begin{array}{ll}
#' \frac{(x-a)^2}{(b-a)(c-a)}     & x \leq c \\
#' 1 - \frac{(b-x)^2}{(b-a)(b-c)} & x > c
#' \end{array}\right.
#' }{
#' F(x) = [if x <= c:] (x-a)^2 / ((b-a)*(c-a))
#'        [if x > c:] 1 - ((b-x)^2 / ((b-a)*(b-c)))
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \left\{\begin{array}{ll}
#' a + \sqrt{p \times (b-a)(c-a)} & p \leq \frac{c-a}{b-a} \\
#' b - \sqrt{(1-p)(b-a)(b-c)}     & p > \frac{c-a}{b-a}
#' \end{array}\right.
#' }{
#' F^-1(p) = [if p < (c-a)/(b-a):] a + sqrt(p*(b-a)*(c-a))
#'           [else:] b - sqrt((1-p)*(b-a)*(b-c))
#' }
#' 
#' For random generation MINMAX method described by
#' Stein and Keblis (2009) is used.
#'
#' @references
#' Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
#' Statistical Distributions. John Wiley & Sons.
#' 
#' @references
#' Stein, W. E., & Keblis, M. F. (2009).
#' A new method to simulate the triangular distribution.
#' Mathematical and computer modelling, 49(5), 1143-1147.
#' 
#' @examples 
#' 
#' x <- rtriang(1e5, 5, 7, 6)
#' hist(x, 100, freq = FALSE)
#' curve(dtriang(x, 5, 7, 6), 3, 10, n = 500, col = "red", add = TRUE)
#' hist(ptriang(x, 5, 7, 6))
#' plot(ecdf(x))
#' curve(ptriang(x, 5, 7, 6), 3, 10, n = 500, col = "red", lwd = 2, add = TRUE)
#'
#' @name Triangular
#' @aliases Triangular
#' @aliases dtriang
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dtriang <- function(x, a = -1, b = 1, c = (a+b)/2, log = FALSE) {
  cpp_dtriang(x, a, b, c, log[1L])
}


#' @rdname Triangular
#' @export

ptriang <- function(q, a = -1, b = 1, c = (a+b)/2, lower.tail = TRUE, log.p = FALSE) {
  cpp_ptriang(q, a, b, c, lower.tail[1L], log.p[1L])
}


#' @rdname Triangular
#' @export

qtriang <- function(p, a = -1, b = 1, c = (a+b)/2, lower.tail = TRUE, log.p = FALSE) {
  cpp_qtriang(p, a, b, c, lower.tail[1L], log.p[1L])
}


#' @rdname Triangular
#' @export

rtriang <- function(n, a = -1, b = 1, c = (a+b)/2) {
  if (length(n) > 1) n <- length(n)
  cpp_rtriang(n, a, b, c)
}

#' Half-t distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the half-t distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param nu,sigma	      positive valued degrees of freedom and scale parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'                        
#' @details
#' If \eqn{X} follows t distribution parametrized by degrees of freedom \eqn{\nu}
#' and scale \eqn{\sigma}, then \eqn{|X|} follows half-t distribution parametrized
#' by degrees of freedom \eqn{\nu} and scale \eqn{\sigma}.
#'                        
#' @references
#' Gelman, A. (2006). Prior distributions for variance parameters in hierarchical
#' models (comment on article by Browne and Draper).
#' Bayesian analysis, 1(3), 515-534.
#' 
#' @references 
#' Jacob, E. and Jayakumar, K. (2012).
#' On Half-Cauchy Distribution and Process.
#' International Journal of Statistika and Mathematika, 3(2), 77-81.
#' 
#' @seealso \code{\link{HalfNormal}}, \code{\link{HalfCauchy}}
#' 
#' @examples 
#' 
#' x <- rht(1e5, 2, 2)
#' hist(x, 500, freq = FALSE, xlim = c(0, 100))
#' curve(dht(x, 2, 2), 0, 100, col = "red", add = TRUE)
#' hist(pht(x, 2, 2))
#' plot(ecdf(x), xlim = c(0, 100))
#' curve(pht(x, 2, 2), 0, 100, col = "red", lwd = 2, add = TRUE)
#'
#' @name HalfT
#' @aliases HalfT
#' @aliases dht
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dht <- function(x, nu, sigma = 1, log = FALSE) {
  cpp_dht(x, nu, sigma, log[1L])
}


#' @rdname HalfT
#' @export

pht <- function(q, nu, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pht(q, nu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname HalfT
#' @export

qht <- function(p, nu, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qht(p, nu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname HalfT
#' @export

rht <- function(n, nu, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rht(n, nu, sigma)
}

#' Categorical distribution
#'
#' Probability mass function, distribution function, quantile function and random generation
#' for the categorical distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param prob,log_prob   vector of length \eqn{m}, or \eqn{m}-column matrix
#'                        of non-negative weights (or their logarithms in \code{log_prob}).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' @param labels          if provided, labeled \code{factor} vector is returned.
#'                        Number of labels needs to be the same as
#'                        number of categories (number of columns in prob).
#'                        
#' @details 
#' Probability mass function
#' 
#' \deqn{
#' \Pr(X = k) = \frac{w_k}{\sum_{j=1}^m w_j}
#' }{
#' Pr(X = k) = w[k]/sum(w)
#' }
#' 
#' Cumulative distribution function
#' \deqn{
#' \Pr(X \le k) = \frac{\sum_{i=1}^k w_i}{\sum_{j=1}^m w_j}
#' }{
#' Pr(X <= k) = sum(w[1:k])/sum(w)
#' }
#' 
#' It is possible to sample from categorical distribution parametrized
#' by vector of unnormalized log-probabilities
#' \eqn{\alpha_1,\dots,\alpha_m}{\alpha[1],...,\alpha[m]}
#' without leaving the log space by employing the Gumbel-max trick (Maddison, Tarlow and Minka, 2014).
#' If \eqn{g_1,\dots,g_m}{g[1],...,g[m]} are samples from Gumbel distribution with
#' cumulative distribution function \eqn{F(g) = \exp(-\exp(-g))}{F(g) = exp(-exp(-g))},
#' then \eqn{k = \mathrm{arg\,max}_i \{g_i + \alpha_i\}}{k = argmax(g[i]+\alpha[i])}
#' is a draw from categorical distribution parametrized by
#' vector of probabilities \eqn{p_1,\dots,p_m}{p[1]....,p[m]}, such that
#' \eqn{p_i = \exp(\alpha_i) / [\sum_{j=1}^m \exp(\alpha_j)]}{p[i] = exp(\alpha[i])/sum(exp(\alpha))}.
#' This is implemented in \code{rcatlp} function parametrized by vector of
#' log-probabilities \code{log_prob}.
#' 
#' @references 
#' Maddison, C. J., Tarlow, D., & Minka, T. (2014). A* sampling.
#' [In:] Advances in Neural Information Processing Systems (pp. 3086-3094).
#' \url{https://arxiv.org/abs/1411.0030}
#'
#' @examples 
#' 
#' # Generating 10 random draws from categorical distribution
#' # with k=3 categories occuring with equal probabilities
#' # parametrized using a vector
#' 
#' rcat(10, c(1/3, 1/3, 1/3))
#' 
#' # or with k=5 categories parametrized using a matrix of probabilities
#' # (generated from Dirichlet distribution)
#' 
#' p <- rdirichlet(10, c(1, 1, 1, 1, 1))
#' rcat(10, p)
#' 
#' x <- rcat(1e5, c(0.2, 0.4, 0.3, 0.1))
#' plot(prop.table(table(x)), type = "h")
#' lines(0:5, dcat(0:5, c(0.2, 0.4, 0.3, 0.1)), col = "red")
#' 
#' p <- rdirichlet(1, rep(1, 20))
#' x <- rcat(1e5, matrix(rep(p, 2), nrow = 2, byrow = TRUE))
#' xx <- 0:21
#' plot(prop.table(table(x)))
#' lines(xx, dcat(xx, p), col = "red")
#' 
#' xx <- seq(0, 21, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pcat(xx, p), col = "red", lwd = 2)
#' 
#' pp <- seq(0, 1, by = 0.001)
#' plot(ecdf(x))
#' lines(qcat(pp, p), pp, col = "red", lwd = 2)
#'
#' @name Categorical
#' @aliases Categorical
#' @aliases dcat
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dcat <- function(x, prob, log = FALSE) {
  if (is.vector(prob))
    prob <- matrix(prob, nrow = 1L)
  else if (!is.matrix(prob))
    prob <- as.matrix(prob)
  cpp_dcat(as.numeric(x), prob, log[1L])
}


#' @rdname Categorical
#' @export

pcat <- function(q, prob, lower.tail = TRUE, log.p = FALSE) {
  if (is.vector(prob))
    prob <- matrix(prob, nrow = 1L)
  else if (!is.matrix(prob))
    prob <- as.matrix(prob)
  cpp_pcat(as.numeric(q), prob, lower.tail[1L], log.p[1L])
}


#' @rdname Categorical
#' @export

qcat <- function(p, prob, lower.tail = TRUE, log.p = FALSE, labels) {
  if (is.vector(prob))
    prob <- matrix(prob, nrow = 1L)
  else if (!is.matrix(prob))
    prob <- as.matrix(prob)
  
  x <- cpp_qcat(p, prob, lower.tail[1L], log.p[1L])
  
  if (!missing(labels)) {
    if (length(labels) != ncol(prob))
      warning("Wrong number of labels.")
    else
      return(factor(x, levels = 1:ncol(prob), labels = labels))
  }
  
  return(x)
}


#' @rdname Categorical
#' @export

rcat <- function(n, prob, labels) {
  if (length(n) > 1) n <- length(n)
  
  if (is.vector(prob)) {
    k <- length(prob)
    if (anyNA(prob) || any(prob < 0) || length(prob) == 0) {
      warning("NAs produced")
      x <- rep(NA, n)
    } else {
      x <- sample.int(k, size = n, replace = TRUE, prob = prob)
    }
  } else {
    k <- ncol(prob)
    x <- cpp_rcat(n, prob)
  }
  
  if (!missing(labels)) {
    if (length(labels) != k)
      warning("Wrong number of labels.")
    else
      return(factor(x, levels = 1:k, labels = labels))
  }
  
  return(x)
}


#' @rdname Categorical
#' @export

rcatlp <- function(n, log_prob, labels) {
  if (length(n) > 1) n <- length(n)
  
  if (is.vector(log_prob))
    log_prob <- matrix(log_prob, nrow = 1L)
  
  x <- cpp_rcatlp(n, log_prob)
  
  k <- ncol(log_prob)
  if (!missing(labels)) {
    if (length(labels) != k)
      warning("Wrong number of labels.")
    else
      return(factor(x, levels = 1:k, labels = labels))
  }
  
  return(x)
}

is_zero_length <- function(x) length(x) == 0L

test_that("Zero-length in PDF and PMF functions", {
  
  expect_true(is_zero_length(dbbinom(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dbbinom(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dbbinom(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dbbinom(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dbern(numeric(0), 0.5)))
  expect_true(is_zero_length(dbern(1, numeric(0))))
  
  expect_true(is_zero_length(dbetapr(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dbetapr(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dbetapr(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dbetapr(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dbhatt(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dbhatt(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dbhatt(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dbhatt(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dbnbinom(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dbnbinom(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dbnbinom(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dbnbinom(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dbvnorm(numeric(0), 1, 1, 1, 1, 1, 0.5)))
  expect_true(is_zero_length(dbvnorm(1, numeric(0), 1, 1, 1, 1, 0.5)))
  expect_true(is_zero_length(dbvnorm(1, 1, numeric(0), 1, 1, 1, 0.5)))
  expect_true(is_zero_length(dbvnorm(1, 1, 1, numeric(0), 1, 1, 0.5)))
  expect_true(is_zero_length(dbvnorm(1, 1, 1, 1, numeric(0), 1, 0.5)))
  expect_true(is_zero_length(dbvnorm(1, 1, 1, 1, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(dbvnorm(1, 1, 1, 1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dbvpois(numeric(0), 1, 1, 1, 1)))
  expect_true(is_zero_length(dbvpois(1, numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dbvpois(1, 1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dbvpois(1, 1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dbvpois(1, 1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dcat(numeric(0), c(0.5, 0.5))))
  expect_true(is_zero_length(dcat(1, numeric(0))))
  expect_true(is_zero_length(dcat(1, matrix(1, 0, 0))))
  
  expect_true(is_zero_length(ddirichlet(numeric(0), c(0.5, 0.5))))
  expect_true(is_zero_length(ddirichlet(c(0.5, 0.5), numeric(0))))
  
  expect_true(is_zero_length(ddlaplace(numeric(0), 1, 1)))
  expect_true(is_zero_length(ddlaplace(1, numeric(0), 1)))
  expect_true(is_zero_length(ddlaplace(1, 1, numeric(0))))
  
  expect_true(is_zero_length(ddnorm(numeric(0), 1, 1)))
  expect_true(is_zero_length(ddnorm(1, numeric(0), 1)))
  expect_true(is_zero_length(ddnorm(1, 1, numeric(0))))
  
  expect_true(is_zero_length(ddgamma(numeric(0), 9, 1)))
  expect_true(is_zero_length(ddgamma(1, numeric(0), 1)))
  expect_true(is_zero_length(ddgamma(1, 9, numeric(0))))
  
  expect_true(is_zero_length(ddunif(numeric(0), 1, 10)))
  expect_true(is_zero_length(ddunif(1, numeric(0), 10)))
  expect_true(is_zero_length(ddunif(1, 1, numeric(0))))
  
  expect_true(is_zero_length(ddweibull(numeric(0), 1, 1)))
  expect_true(is_zero_length(ddweibull(1, numeric(0), 1))) 
  expect_true(is_zero_length(ddweibull(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dfatigue(numeric(0), 1, 1)))
  expect_true(is_zero_length(dfatigue(1, numeric(0), 1)))
  expect_true(is_zero_length(dfatigue(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dfrechet(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dfrechet(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dfrechet(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dfrechet(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dgev(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dgev(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dgev(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dgev(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dgompertz(numeric(0), 1, 1)))
  expect_true(is_zero_length(dgompertz(1, numeric(0), 1)))
  expect_true(is_zero_length(dgompertz(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dgpd(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(dgpd(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dgpd(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(dgpd(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(dgpois(numeric(0), 1, 1)))
  expect_true(is_zero_length(dgpois(1, numeric(0), 1)))
  expect_true(is_zero_length(dgpois(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dgumbel(numeric(0), 1, 1)))
  expect_true(is_zero_length(dgumbel(1, numeric(0), 1)))
  expect_true(is_zero_length(dgumbel(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dhcauchy(numeric(0), 1)))
  expect_true(is_zero_length(dhcauchy(1, numeric(0))))
  
  expect_true(is_zero_length(dhnorm(numeric(0), 1)))
  expect_true(is_zero_length(dhnorm(1, numeric(0))))
  
  expect_true(is_zero_length(dht(numeric(0), 5, 1)))
  expect_true(is_zero_length(dht(1, numeric(0), 1)))
  expect_true(is_zero_length(dht(1, 5, numeric(0))))
  
  expect_true(is_zero_length(dhuber(numeric(0), 0, 1, 1)))
  expect_true(is_zero_length(dhuber(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(dhuber(1, 0, numeric(0), 1)))
  expect_true(is_zero_length(dhuber(1, 0, 1, numeric(0))))
  
  expect_true(is_zero_length(dinvgamma(numeric(0), 1, 1)))
  expect_true(is_zero_length(dinvgamma(1, numeric(0), 1)))
  expect_true(is_zero_length(dinvgamma(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dinvchisq(numeric(0), 1, 1)))
  expect_true(is_zero_length(dinvchisq(1, numeric(0), 1)))
  expect_true(is_zero_length(dinvchisq(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dkumar(numeric(0), 1, 1)))
  expect_true(is_zero_length(dkumar(0.5, numeric(0), 1)))
  expect_true(is_zero_length(dkumar(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(dlaplace(numeric(0), 0, 1)))
  expect_true(is_zero_length(dlaplace(1, numeric(0), 1)))
  expect_true(is_zero_length(dlaplace(1, 0, numeric(0))))
  
  expect_true(is_zero_length(dlgser(numeric(0), 0.5)))
  expect_true(is_zero_length(dlgser(1, numeric(0))))
  
  expect_true(is_zero_length(dlomax(numeric(0), 1, 1)))
  expect_true(is_zero_length(dlomax(1, numeric(0), 1)))
  expect_true(is_zero_length(dlomax(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dmixnorm(numeric(0), c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(dmixnorm(0, numeric(0), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(dmixnorm(0, c(1,2,3), numeric(0), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(dmixnorm(0, c(1,2,3), c(1,2,3), numeric(0))))
  
  expect_true(is_zero_length(dmixpois(numeric(0), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(dmixpois(0, numeric(0), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(dmixpois(0, c(1,2,3), numeric(0))))
  
  expect_true(is_zero_length(dnhyper(numeric(0), 60, 35, 15)))
  expect_true(is_zero_length(dnhyper(1, numeric(0), 35, 15)))
  expect_true(is_zero_length(dnhyper(1, 60, numeric(0), 15)))
  expect_true(is_zero_length(dnhyper(1, 60, 35, numeric(0))))
  
  expect_true(is_zero_length(ddirmnom(numeric(0), 3, c(1, 1, 1))))
  expect_true(is_zero_length(ddirmnom(c(1, 1, 1), numeric(0), c(1, 1, 1))))
  expect_true(is_zero_length(ddirmnom(c(1, 1, 1), 3, numeric(0))))
  
  expect_true(is_zero_length(dmnom(numeric(0), 3, c(1/3, 1/3, 1/3))))
  expect_true(is_zero_length(dmnom(c(1, 1, 1), numeric(0), c(1/3, 1/3, 1/3))))
  expect_true(is_zero_length(dmnom(c(1, 1, 1), 3, numeric(0))))
  
  expect_true(is_zero_length(dmvhyper(numeric(0), c(2,3,4), 5)))
  expect_true(is_zero_length(dmvhyper(c(1, 2, 2), numeric(0), 5)))
  expect_true(is_zero_length(dmvhyper(c(1, 2, 2), c(2,3,4), numeric(0))))
  
  expect_true(is_zero_length(dnsbeta(numeric(0), 1, 1, -2, 2)))
  expect_true(is_zero_length(dnsbeta(0.5, numeric(0), 1, -2, 2)))
  expect_true(is_zero_length(dnsbeta(0.5, 1, numeric(0), -2, 2)))
  expect_true(is_zero_length(dnsbeta(0.5, 1, 1, numeric(0), 2)))
  expect_true(is_zero_length(dnsbeta(0.5, 1, 1, -2, numeric(0))))
  
  expect_true(is_zero_length(dlst(numeric(0), 2, 0, 1)))
  expect_true(is_zero_length(dlst(1, numeric(0), 0, 1)))
  expect_true(is_zero_length(dlst(1, 2, numeric(0), 1)))
  expect_true(is_zero_length(dlst(1, 2, 0, numeric(0))))
  
  expect_true(is_zero_length(dpareto(numeric(0), 1, 1)))
  expect_true(is_zero_length(dpareto(1, numeric(0), 1)))
  expect_true(is_zero_length(dpareto(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dprop(numeric(0), 10, 0.5)))
  expect_true(is_zero_length(dprop(1, numeric(0), 0.5)))
  expect_true(is_zero_length(dprop(1, 10, numeric(0))))
  
  expect_true(is_zero_length(drayleigh(numeric(0), 1)))
  expect_true(is_zero_length(drayleigh(1, numeric(0))))
  
  expect_true(is_zero_length(dskellam(numeric(0), 1, 1)))
  expect_true(is_zero_length(dskellam(1, numeric(0), 1)))
  expect_true(is_zero_length(dskellam(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dsgomp(numeric(0), 0.4, 1)))
  expect_true(is_zero_length(dsgomp(1, numeric(0), 1)))
  expect_true(is_zero_length(dsgomp(1, 0.4, numeric(0))))
  
  expect_true(is_zero_length(dslash(numeric(0), 1, 1)))
  expect_true(is_zero_length(dslash(1, numeric(0), 1)))
  expect_true(is_zero_length(dslash(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dtnorm(numeric(0), 0, 1, -2, 2)))
  expect_true(is_zero_length(dtnorm(1, numeric(0), 1, -2, 2)))
  expect_true(is_zero_length(dtnorm(1, 0, numeric(0), -2, 2)))
  expect_true(is_zero_length(dtnorm(1, 0, 1, numeric(0), 2)))
  expect_true(is_zero_length(dtnorm(1, 0, 1, -2, numeric(0))))
  
  expect_true(is_zero_length(dtpois(numeric(0), 5, 0)))
  expect_true(is_zero_length(dtpois(1, numeric(0), 0)))
  expect_true(is_zero_length(dtpois(1, 5, numeric(0))))
  
  expect_true(is_zero_length(dtriang(numeric(0), 0, 1, 0.5)))
  expect_true(is_zero_length(dtriang(0.5, numeric(0), 1, 0.5)))
  expect_true(is_zero_length(dtriang(0.5, 0, numeric(0), 0.5)))
  expect_true(is_zero_length(dtriang(0.5, 0, 1, numeric(0))))
  
  expect_true(is_zero_length(dwald(numeric(0), 1, 1)))
  expect_true(is_zero_length(dwald(1, numeric(0), 1)))
  expect_true(is_zero_length(dwald(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dzip(numeric(0), 1, 0.5)))
  expect_true(is_zero_length(dzip(1, numeric(0), 0.5)))
  expect_true(is_zero_length(dzip(1, 1, numeric(0))))
  
  expect_true(is_zero_length(dzib(numeric(0), 1, 0.5, 0.5)))
  expect_true(is_zero_length(dzib(1, numeric(0), 0.5, 0.5)))
  expect_true(is_zero_length(dzib(1, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(dzib(1, 1, 0.5, numeric(0))))
  
  expect_true(is_zero_length(dzinb(numeric(0), 1, 0.5, 0.5)))
  expect_true(is_zero_length(dzinb(1, numeric(0), 0.5, 0.5)))
  expect_true(is_zero_length(dzinb(1, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(dzinb(1, 1, 0.5, numeric(0))))
  
})





test_that("Zero-length in CDF functions", {
  
  expect_true(is_zero_length(pbbinom(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pbbinom(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pbbinom(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pbbinom(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pbern(numeric(0), 0.5)))
  expect_true(is_zero_length(pbern(1, numeric(0))))
  
  expect_true(is_zero_length(pbetapr(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pbetapr(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pbetapr(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pbetapr(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pbhatt(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pbhatt(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pbhatt(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pbhatt(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pbnbinom(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pbnbinom(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pbnbinom(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pbnbinom(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pcat(numeric(0), c(0.5, 0.5))))
  expect_true(is_zero_length(pcat(1, numeric(0))))
  expect_true(is_zero_length(pcat(1, matrix(1, 0, 0))))
  
  expect_true(is_zero_length(pdlaplace(numeric(0), 1, 1)))
  expect_true(is_zero_length(pdlaplace(1, numeric(0), 1)))
  expect_true(is_zero_length(pdlaplace(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pdnorm(numeric(0), 1, 1)))
  expect_true(is_zero_length(pdnorm(1, numeric(0), 1)))
  expect_true(is_zero_length(pdnorm(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pdgamma(numeric(0), 9, 1)))
  expect_true(is_zero_length(pdgamma(1, numeric(0), 1)))
  expect_true(is_zero_length(pdgamma(1, 9, numeric(0))))
  
  expect_true(is_zero_length(pdunif(numeric(0), 1, 10)))
  expect_true(is_zero_length(pdunif(1, numeric(0), 10)))
  expect_true(is_zero_length(pdunif(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pdweibull(numeric(0), 1, 1)))
  expect_true(is_zero_length(pdweibull(1, numeric(0), 1))) 
  expect_true(is_zero_length(pdweibull(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pfatigue(numeric(0), 1, 1)))
  expect_true(is_zero_length(pfatigue(1, numeric(0), 1)))
  expect_true(is_zero_length(pfatigue(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pfrechet(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pfrechet(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pfrechet(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pfrechet(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pgev(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pgev(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pgev(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pgev(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pgompertz(numeric(0), 1, 1)))
  expect_true(is_zero_length(pgompertz(1, numeric(0), 1)))
  expect_true(is_zero_length(pgompertz(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pgpd(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(pgpd(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(pgpd(1, 1, numeric(0), 1)))
  expect_true(is_zero_length(pgpd(1, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(pgpois(numeric(0), 1, 1)))
  expect_true(is_zero_length(pgpois(1, numeric(0), 1)))
  expect_true(is_zero_length(pgpois(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pgumbel(numeric(0), 1, 1)))
  expect_true(is_zero_length(pgumbel(1, numeric(0), 1)))
  expect_true(is_zero_length(pgumbel(1, 1, numeric(0))))
  
  expect_true(is_zero_length(phcauchy(numeric(0), 1)))
  expect_true(is_zero_length(phcauchy(1, numeric(0))))
  
  expect_true(is_zero_length(phnorm(numeric(0), 1)))
  expect_true(is_zero_length(phnorm(1, numeric(0))))
  
  expect_true(is_zero_length(pht(numeric(0), 5, 1)))
  expect_true(is_zero_length(pht(1, numeric(0), 1)))
  expect_true(is_zero_length(pht(1, 5, numeric(0))))
  
  expect_true(is_zero_length(phuber(numeric(0), 0, 1, 1)))
  expect_true(is_zero_length(phuber(1, numeric(0), 1, 1)))
  expect_true(is_zero_length(phuber(1, 0, numeric(0), 1)))
  expect_true(is_zero_length(phuber(1, 0, 1, numeric(0))))
  
  expect_true(is_zero_length(pinvgamma(numeric(0), 1, 1)))
  expect_true(is_zero_length(pinvgamma(1, numeric(0), 1)))
  expect_true(is_zero_length(pinvgamma(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pinvchisq(numeric(0), 1, 1)))
  expect_true(is_zero_length(pinvchisq(1, numeric(0), 1)))
  expect_true(is_zero_length(pinvchisq(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pkumar(numeric(0), 1, 1)))
  expect_true(is_zero_length(pkumar(0.5, numeric(0), 1)))
  expect_true(is_zero_length(pkumar(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(plaplace(numeric(0), 0, 1)))
  expect_true(is_zero_length(plaplace(1, numeric(0), 1)))
  expect_true(is_zero_length(plaplace(1, 0, numeric(0))))
  
  expect_true(is_zero_length(plgser(numeric(0), 0.5)))
  expect_true(is_zero_length(plgser(1, numeric(0))))
  
  expect_true(is_zero_length(plomax(numeric(0), 1, 1)))
  expect_true(is_zero_length(plomax(1, numeric(0), 1)))
  expect_true(is_zero_length(plomax(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pmixnorm(numeric(0), c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(pmixnorm(0, numeric(0), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(pmixnorm(0, c(1,2,3), numeric(0), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(pmixnorm(0, c(1,2,3), c(1,2,3), numeric(0))))
  
  expect_true(is_zero_length(pmixpois(numeric(0), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(pmixpois(0, numeric(0), c(1/3,1/3,1/3))))
  expect_true(is_zero_length(pmixpois(0, c(1,2,3), numeric(0))))
  
  expect_true(is_zero_length(pnhyper(numeric(0), 60, 35, 15)))
  expect_true(is_zero_length(pnhyper(1, numeric(0), 35, 15)))
  expect_true(is_zero_length(pnhyper(1, 60, numeric(0), 15)))
  expect_true(is_zero_length(pnhyper(1, 60, 35, numeric(0))))
  
  expect_true(is_zero_length(pnsbeta(numeric(0), 1, 1, -2, 2)))
  expect_true(is_zero_length(pnsbeta(0.5, numeric(0), 1, -2, 2)))
  expect_true(is_zero_length(pnsbeta(0.5, 1, numeric(0), -2, 2)))
  expect_true(is_zero_length(pnsbeta(0.5, 1, 1, numeric(0), 2)))
  expect_true(is_zero_length(pnsbeta(0.5, 1, 1, -2, numeric(0))))
  
  expect_true(is_zero_length(plst(numeric(0), 2, 0, 1)))
  expect_true(is_zero_length(plst(1, numeric(0), 0, 1)))
  expect_true(is_zero_length(plst(1, 2, numeric(0), 1)))
  expect_true(is_zero_length(plst(1, 2, 0, numeric(0))))
  
  expect_true(is_zero_length(ppareto(numeric(0), 1, 1)))
  expect_true(is_zero_length(ppareto(1, numeric(0), 1)))
  expect_true(is_zero_length(ppareto(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pprop(numeric(0), 10, 0.5)))
  expect_true(is_zero_length(pprop(1, numeric(0), 0.5)))
  expect_true(is_zero_length(pprop(1, 10, numeric(0))))
  
  expect_true(is_zero_length(prayleigh(numeric(0), 1)))
  expect_true(is_zero_length(prayleigh(1, numeric(0))))
  
  expect_true(is_zero_length(psgomp(numeric(0), 0.4, 1)))
  expect_true(is_zero_length(psgomp(1, numeric(0), 1)))
  expect_true(is_zero_length(psgomp(1, 0.4, numeric(0))))
  
  expect_true(is_zero_length(pslash(numeric(0), 1, 1)))
  expect_true(is_zero_length(pslash(1, numeric(0), 1)))
  expect_true(is_zero_length(pslash(1, 1, numeric(0))))
  
  expect_true(is_zero_length(ptnorm(numeric(0), 0, 1, -2, 2)))
  expect_true(is_zero_length(ptnorm(1, numeric(0), 1, -2, 2)))
  expect_true(is_zero_length(ptnorm(1, 0, numeric(0), -2, 2)))
  expect_true(is_zero_length(ptnorm(1, 0, 1, numeric(0), 2)))
  expect_true(is_zero_length(ptnorm(1, 0, 1, -2, numeric(0))))
  
  expect_true(is_zero_length(ptpois(numeric(0), 5, 0)))
  expect_true(is_zero_length(ptpois(1, numeric(0), 0)))
  expect_true(is_zero_length(ptpois(1, 5, numeric(0))))
  
  expect_true(is_zero_length(ptriang(numeric(0), 0, 1, 0.5)))
  expect_true(is_zero_length(ptriang(0.5, numeric(0), 1, 0.5)))
  expect_true(is_zero_length(ptriang(0.5, 0, numeric(0), 0.5)))
  expect_true(is_zero_length(ptriang(0.5, 0, 1, numeric(0))))
  
  expect_true(is_zero_length(pwald(numeric(0), 1, 1)))
  expect_true(is_zero_length(pwald(1, numeric(0), 1)))
  expect_true(is_zero_length(pwald(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pzip(numeric(0), 1, 0.5)))
  expect_true(is_zero_length(pzip(1, numeric(0), 0.5)))
  expect_true(is_zero_length(pzip(1, 1, numeric(0))))
  
  expect_true(is_zero_length(pzib(numeric(0), 1, 0.5, 0.5)))
  expect_true(is_zero_length(pzib(1, numeric(0), 0.5, 0.5)))
  expect_true(is_zero_length(pzib(1, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(pzib(1, 1, 0.5, numeric(0))))
  
  expect_true(is_zero_length(pzinb(numeric(0), 1, 0.5, 0.5)))
  expect_true(is_zero_length(pzinb(1, numeric(0), 0.5, 0.5)))
  expect_true(is_zero_length(pzinb(1, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(pzinb(1, 1, 0.5, numeric(0))))
  
})




test_that("Zero-length in inverse CDF functions", {
  
  expect_true(is_zero_length(qbern(numeric(0), 0.5)))
  expect_true(is_zero_length(qbern(0.5, numeric(0))))
  
  expect_true(is_zero_length(qbetapr(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(qbetapr(0.5, numeric(0), 1, 1)))
  expect_true(is_zero_length(qbetapr(0.5, 1, numeric(0), 1)))
  expect_true(is_zero_length(qbetapr(0.5, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(qcat(numeric(0), c(0.5, 0.5))))
  expect_true(is_zero_length(qcat(0.5, numeric(0))))
  expect_true(is_zero_length(qcat(0.5, matrix(1, 0, 0))))
  
  expect_true(is_zero_length(qdunif(numeric(0), 1, 10)))
  expect_true(is_zero_length(qdunif(0.5, numeric(0), 10)))
  expect_true(is_zero_length(qdunif(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qdweibull(numeric(0), 1, 1)))
  expect_true(is_zero_length(qdweibull(0.5, numeric(0), 1))) 
  expect_true(is_zero_length(qdweibull(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qfatigue(numeric(0), 1, 1)))
  expect_true(is_zero_length(qfatigue(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qfatigue(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qfrechet(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(qfrechet(0.5, numeric(0), 1, 1)))
  expect_true(is_zero_length(qfrechet(0.5, 1, numeric(0), 1)))
  expect_true(is_zero_length(qfrechet(0.5, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(qgev(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(qgev(0.5, numeric(0), 1, 1)))
  expect_true(is_zero_length(qgev(0.5, 1, numeric(0), 1)))
  expect_true(is_zero_length(qgev(0.5, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(qgompertz(numeric(0), 1, 1)))
  expect_true(is_zero_length(qgompertz(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qgompertz(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qgpd(numeric(0), 1, 1, 1)))
  expect_true(is_zero_length(qgpd(0.5, numeric(0), 1, 1)))
  expect_true(is_zero_length(qgpd(0.5, 1, numeric(0), 1)))
  expect_true(is_zero_length(qgpd(0.5, 1, 1, numeric(0))))
  
  expect_true(is_zero_length(qgumbel(numeric(0), 1, 1)))
  expect_true(is_zero_length(qgumbel(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qgumbel(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qhcauchy(numeric(0), 1)))
  expect_true(is_zero_length(qhcauchy(0.5, numeric(0))))
  
  expect_true(is_zero_length(qhnorm(numeric(0), 1)))
  expect_true(is_zero_length(qhnorm(0.5, numeric(0))))
  
  expect_true(is_zero_length(qht(numeric(0), 5, 1)))
  expect_true(is_zero_length(qht(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qht(0.5, 5, numeric(0))))
  
  expect_true(is_zero_length(qhuber(numeric(0), 0, 1, 1)))
  expect_true(is_zero_length(qhuber(0.5, numeric(0), 1, 1)))
  expect_true(is_zero_length(qhuber(0.5, 0, numeric(0), 1)))
  expect_true(is_zero_length(qhuber(0.5, 0, 1, numeric(0))))
  
  expect_true(is_zero_length(qinvgamma(numeric(0), 1, 1)))
  expect_true(is_zero_length(qinvgamma(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qinvgamma(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qinvchisq(numeric(0), 1, 1)))
  expect_true(is_zero_length(qinvchisq(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qinvchisq(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qkumar(numeric(0), 1, 1)))
  expect_true(is_zero_length(qkumar(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qkumar(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qlaplace(numeric(0), 0, 1)))
  expect_true(is_zero_length(qlaplace(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qlaplace(0.5, 0, numeric(0))))
  
  expect_true(is_zero_length(qlgser(numeric(0), 0.5)))
  expect_true(is_zero_length(qlgser(0.5, numeric(0))))
  
  expect_true(is_zero_length(qlomax(numeric(0), 1, 1)))
  expect_true(is_zero_length(qlomax(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qlomax(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qnhyper(numeric(0), 60, 35, 15)))
  expect_true(is_zero_length(qnhyper(0.5, numeric(0), 35, 15)))
  expect_true(is_zero_length(qnhyper(0.5, 60, numeric(0), 15)))
  expect_true(is_zero_length(qnhyper(0.5, 60, 35, numeric(0))))
  
  expect_true(is_zero_length(qnsbeta(numeric(0), 1, 1, -2, 2)))
  expect_true(is_zero_length(qnsbeta(0.5, numeric(0), 1, -2, 2)))
  expect_true(is_zero_length(qnsbeta(0.5, 1, numeric(0), -2, 2)))
  expect_true(is_zero_length(qnsbeta(0.5, 1, 1, numeric(0), 2)))
  expect_true(is_zero_length(qnsbeta(0.5, 1, 1, -2, numeric(0))))
  
  expect_true(is_zero_length(qlst(numeric(0), 2, 0, 1)))
  expect_true(is_zero_length(qlst(0.5, numeric(0), 0, 1)))
  expect_true(is_zero_length(qlst(0.5, 2, numeric(0), 1)))
  expect_true(is_zero_length(qlst(0.5, 2, 0, numeric(0))))
  
  expect_true(is_zero_length(qpareto(numeric(0), 1, 1)))
  expect_true(is_zero_length(qpareto(0.5, numeric(0), 1)))
  expect_true(is_zero_length(qpareto(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qprop(numeric(0), 10, 0.5)))
  expect_true(is_zero_length(qprop(0.5, numeric(0), 0.5)))
  expect_true(is_zero_length(qprop(0.5, 10, numeric(0))))
  
  expect_true(is_zero_length(qrayleigh(numeric(0), 1)))
  expect_true(is_zero_length(qrayleigh(0.5, numeric(0))))
  
  expect_true(is_zero_length(qtlambda(numeric(0), 0.5)))
  expect_true(is_zero_length(qtlambda(0, numeric(0))))
  
  expect_true(is_zero_length(qtnorm(numeric(0), 0, 1, -2, 2)))
  expect_true(is_zero_length(qtnorm(0.5, numeric(0), 1, -2, 2)))
  expect_true(is_zero_length(qtnorm(0.5, 0, numeric(0), -2, 2)))
  expect_true(is_zero_length(qtnorm(0.5, 0, 1, numeric(0), 2)))
  expect_true(is_zero_length(qtnorm(0.5, 0, 1, -2, numeric(0))))
  
  expect_true(is_zero_length(qtpois(numeric(0), 5, 0)))
  expect_true(is_zero_length(qtpois(0.5, numeric(0), 0)))
  expect_true(is_zero_length(qtpois(0.5, 5, numeric(0))))
  
  expect_true(is_zero_length(qtriang(numeric(0), 0, 1, 0.5)))
  expect_true(is_zero_length(qtriang(0.5, numeric(0), 1, 0.5)))
  expect_true(is_zero_length(qtriang(0.5, 0, numeric(0), 0.5)))
  expect_true(is_zero_length(qtriang(0.5, 0, 1, numeric(0))))
  
  expect_true(is_zero_length(qzip(numeric(0), 1, 0.5)))
  expect_true(is_zero_length(qzip(0.5, numeric(0), 0.5)))
  expect_true(is_zero_length(qzip(0.5, 1, numeric(0))))
  
  expect_true(is_zero_length(qzib(numeric(0), 1, 0.5, 0.5)))
  expect_true(is_zero_length(qzib(0.5, numeric(0), 0.5, 0.5)))
  expect_true(is_zero_length(qzib(0.5, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(qzib(0.5, 1, 0.5, numeric(0))))
  
  expect_true(is_zero_length(qzinb(numeric(0), 1, 0.5, 0.5)))
  expect_true(is_zero_length(qzinb(0.5, numeric(0), 0.5, 0.5)))
  expect_true(is_zero_length(qzinb(0.5, 1, numeric(0), 0.5)))
  expect_true(is_zero_length(qzinb(0.5, 1, 0.5, numeric(0))))
  
})




test_that("Zero-length in RNG functions", {
  
  expect_warning(expect_true(is.na(rbbinom(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rbbinom(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rbbinom(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rbern(1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rbetapr(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rbetapr(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rbetapr(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rbhatt(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rbhatt(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rbhatt(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rbnbinom(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rbnbinom(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rbnbinom(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(all(is.na(rbvnorm(1, numeric(0), 1, 1, 1, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, numeric(0), 1, 1, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, 1, numeric(0), 1, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, 1, 1, numeric(0), 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, 1, 1, 1, numeric(0))))))
  
  expect_warning(expect_true(all(is.na(rbvpois(1, numeric(0), 1, 1)))))
  expect_warning(expect_true(all(is.na(rbvpois(1, 1, numeric(0), 1)))))
  expect_warning(expect_true(all(is.na(rbvpois(1, 1, 1, numeric(0))))))
  
  expect_warning(expect_true(is.na(rcat(1, numeric(0)))))
  expect_warning(expect_true(is.na(rcat(1, matrix(1, 0, 0)))))
  
  expect_warning(expect_true(is.na(rcatlp(1, numeric(0)))))
  expect_warning(expect_true(is.na(rcatlp(1, matrix(1, 0, 0)))))
  
  expect_warning(expect_true(all(is.na(rdirichlet(1, numeric(0))))))
  
  expect_warning(expect_true(is.na(rdlaplace(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rdlaplace(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rdnorm(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rdnorm(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rdgamma(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rdgamma(1, 9, numeric(0)))))
  
  expect_warning(expect_true(is.na(rdunif(1, numeric(0), 10))))
  expect_warning(expect_true(is.na(rdunif(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rdweibull(1, numeric(0), 1)))) 
  expect_warning(expect_true(is.na(rdweibull(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rfatigue(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rfatigue(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rfrechet(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rfrechet(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rfrechet(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rgev(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rgev(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rgev(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rgompertz(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rgompertz(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rgpd(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rgpd(1, 1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rgpd(1, 1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rgpois(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rgpois(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rgumbel(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rgumbel(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rhcauchy(1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rhnorm(1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rht(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rht(1, 5, numeric(0)))))
  
  expect_warning(expect_true(is.na(rhuber(1, numeric(0), 1, 1))))
  expect_warning(expect_true(is.na(rhuber(1, 0, numeric(0), 1))))
  expect_warning(expect_true(is.na(rhuber(1, 0, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rinvgamma(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rinvgamma(1, 1, numeric(0)))))

  expect_warning(expect_true(is.na(rinvchisq(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rinvchisq(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rkumar(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rkumar(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rlaplace(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rlaplace(1, 0, numeric(0)))))
  
  expect_warning(expect_true(is.na(rlgser(1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rlomax(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rlomax(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rmixnorm(1, numeric(0), c(1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), numeric(0), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), numeric(0)))))
  
  expect_warning(expect_true(is.na(rmixpois(1, numeric(0), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), numeric(0)))))
  
  expect_warning(expect_true(is.na(rnhyper(1, numeric(0), 35, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, numeric(0), 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, 35, numeric(0)))))
  
  expect_warning(expect_true(all(is.na(rdirmnom(1, numeric(0), c(1, 1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, numeric(0))))))
  
  expect_warning(expect_true(all(is.na(rmnom(1, numeric(0), c(1/3, 1/3, 1/3))))))
  expect_warning(expect_true(all(is.na(rmnom(1, 3, numeric(0))))))
  
  expect_warning(expect_true(all(is.na(rmvhyper(1, numeric(0), 5)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,3,4), numeric(0))))))
  
  expect_warning(expect_true(is.na(rnsbeta(1, numeric(0), 1, -2, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, numeric(0), -2, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, 1, numeric(0), 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, 1, -2, numeric(0)))))
  
  expect_warning(expect_true(is.na(rlst(1, numeric(0), 0, 1))))
  expect_warning(expect_true(is.na(rlst(1, 2, numeric(0), 1))))
  expect_warning(expect_true(is.na(rlst(1, 2, 0, numeric(0)))))
  
  expect_warning(expect_true(is.na(rpareto(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rpareto(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rprop(1, numeric(0), 0.5))))
  expect_warning(expect_true(is.na(rprop(1, 10, numeric(0)))))
  
  expect_warning(expect_true(is.na(rrayleigh(1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rtlambda(1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rsgomp(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rsgomp(1, 0.4, numeric(0)))))
  
  expect_warning(expect_true(is.na(rskellam(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rskellam(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rslash(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rslash(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rtnorm(1, numeric(0), 1, -2, 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, numeric(0), -2, 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, 1, numeric(0), 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, 1, -2, numeric(0)))))
  
  expect_warning(expect_true(is.na(rtpois(1, numeric(0), 0))))
  expect_warning(expect_true(is.na(rtpois(1, 5, numeric(0)))))
  
  expect_warning(expect_true(is.na(rtriang(1, numeric(0), 1, 0.5))))
  expect_warning(expect_true(is.na(rtriang(1, 0, numeric(0), 0.5))))
  expect_warning(expect_true(is.na(rtriang(1, 0, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rwald(1, numeric(0), 1))))
  expect_warning(expect_true(is.na(rwald(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rzip(1, numeric(0), 0.5))))
  expect_warning(expect_true(is.na(rzip(1, 1, numeric(0)))))
  
  expect_warning(expect_true(is.na(rzib(1, numeric(0), 0.5, 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, numeric(0), 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, 0.5, numeric(0)))))
  
  expect_warning(expect_true(is.na(rzinb(1, numeric(0), 0.5, 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, numeric(0), 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, 0.5, numeric(0)))))
  
})

test_that("Missing values in PDF and PMF functions", {
  
  expect_true(is.na(dbbinom(NA, 1, 1, 1)))
  expect_true(is.na(dbbinom(1, NA, 1, 1)))
  expect_true(is.na(dbbinom(1, 1, NA, 1)))
  expect_true(is.na(dbbinom(1, 1, 1, NA)))
  
  expect_true(is.na(dbern(NA, 0.5)))
  expect_true(is.na(dbern(1, NA)))
  
  expect_true(is.na(dbetapr(NA, 1, 1, 1)))
  expect_true(is.na(dbetapr(1, NA, 1, 1)))
  expect_true(is.na(dbetapr(1, 1, NA, 1)))
  expect_true(is.na(dbetapr(1, 1, 1, NA)))
  
  expect_true(is.na(dbhatt(NA, 1, 1, 1)))
  expect_true(is.na(dbhatt(1, NA, 1, 1)))
  expect_true(is.na(dbhatt(1, 1, NA, 1)))
  expect_true(is.na(dbhatt(1, 1, 1, NA)))
  
  expect_true(is.na(dbnbinom(NA, 1, 1, 1)))
  expect_true(is.na(dbnbinom(1, NA, 1, 1)))
  expect_true(is.na(dbnbinom(1, 1, NA, 1)))
  expect_true(is.na(dbnbinom(1, 1, 1, NA)))
  
  expect_true(is.na(dbvnorm(NA, 1, 1, 1, 1, 1, 0.5)))
  expect_true(is.na(dbvnorm(1, NA, 1, 1, 1, 1, 0.5)))
  expect_true(is.na(dbvnorm(1, 1, NA, 1, 1, 1, 0.5)))
  expect_true(is.na(dbvnorm(1, 1, 1, NA, 1, 1, 0.5)))
  expect_true(is.na(dbvnorm(1, 1, 1, 1, NA, 1, 0.5)))
  expect_true(is.na(dbvnorm(1, 1, 1, 1, 1, NA, 0.5)))
  expect_true(is.na(dbvnorm(1, 1, 1, 1, 1, 1, NA)))
  
  expect_true(is.na(dbvpois(NA, 1, 1, 1, 1)))
  expect_true(is.na(dbvpois(1, NA, 1, 1, 1)))
  expect_true(is.na(dbvpois(1, 1, NA, 1, 1)))
  expect_true(is.na(dbvpois(1, 1, 1, NA, 1)))
  expect_true(is.na(dbvpois(1, 1, 1, 1, NA)))
  
  expect_true(is.na(dcat(NA, c(0.5, 0.5))))
  expect_true(is.na(dcat(1, c(NA, 0.5))))
  expect_true(is.na(dcat(1, c(0.5, NA))))
  
  expect_true(is.na(ddirichlet(c(NA, 0.5), c(0.5, 0.5))))
  expect_true(is.na(ddirichlet(c(0.5, NA), c(0.5, 0.5))))
  expect_true(is.na(ddirichlet(c(0.5, 0.5), c(NA, 0.5))))
  expect_true(is.na(ddirichlet(c(0.5, 0.5), c(0.5, NA))))
  
  expect_true(is.na(ddlaplace(NA, 1, 1)))
  expect_true(is.na(ddlaplace(1, NA, 1)))
  expect_true(is.na(ddlaplace(1, 1, NA)))
  
  expect_true(is.na(ddnorm(NA, 1, 1)))
  expect_true(is.na(ddnorm(1, NA, 1)))
  expect_true(is.na(ddnorm(1, 1, NA)))
  
  expect_true(is.na(ddgamma(NA, 9, 1)))
  expect_true(is.na(ddgamma(1, NA, 1)))
  expect_true(is.na(ddgamma(1, 9, NA)))
  
  expect_true(is.na(ddunif(NA, 1, 10)))
  expect_true(is.na(ddunif(1, NA, 10)))
  expect_true(is.na(ddunif(1, 1, NA)))
  
  expect_true(is.na(ddweibull(NA, 1, 1)))
  expect_true(is.na(ddweibull(1, NA, 1))) 
  expect_true(is.na(ddweibull(1, 1, NA)))
  
  expect_true(is.na(dfatigue(NA, 1, 1)))
  expect_true(is.na(dfatigue(1, NA, 1)))
  expect_true(is.na(dfatigue(1, 1, NA)))
  
  expect_true(is.na(dfrechet(NA, 1, 1, 1)))
  expect_true(is.na(dfrechet(1, NA, 1, 1)))
  expect_true(is.na(dfrechet(1, 1, NA, 1)))
  expect_true(is.na(dfrechet(1, 1, 1, NA)))

  expect_true(is.na(dgev(NA, 1, 1, 1)))
  expect_true(is.na(dgev(1, NA, 1, 1)))
  expect_true(is.na(dgev(1, 1, NA, 1)))
  expect_true(is.na(dgev(1, 1, 1, NA)))
  
  expect_true(is.na(dgompertz(NA, 1, 1)))
  expect_true(is.na(dgompertz(1, NA, 1)))
  expect_true(is.na(dgompertz(1, 1, NA)))
  
  expect_true(is.na(dgpd(NA, 1, 1, 1)))
  expect_true(is.na(dgpd(1, NA, 1, 1)))
  expect_true(is.na(dgpd(1, 1, NA, 1)))
  expect_true(is.na(dgpd(1, 1, 1, NA)))
  
  expect_true(is.na(dgpois(NA, 1, 1)))
  expect_true(is.na(dgpois(1, NA, 1)))
  expect_true(is.na(dgpois(1, 1, NA)))
  
  expect_true(is.na(dgumbel(NA, 1, 1)))
  expect_true(is.na(dgumbel(1, NA, 1)))
  expect_true(is.na(dgumbel(1, 1, NA)))
  
  expect_true(is.na(dhcauchy(NA, 1)))
  expect_true(is.na(dhcauchy(1, NA)))
  
  expect_true(is.na(dhnorm(NA, 1)))
  expect_true(is.na(dhnorm(1, NA)))
  
  expect_true(is.na(dht(NA, 5, 1)))
  expect_true(is.na(dht(1, NA, 1)))
  expect_true(is.na(dht(1, 5, NA)))
  
  expect_true(is.na(dhuber(NA, 0, 1, 1)))
  expect_true(is.na(dhuber(1, NA, 1, 1)))
  expect_true(is.na(dhuber(1, 0, NA, 1)))
  expect_true(is.na(dhuber(1, 0, 1, NA)))
  
  expect_true(is.na(dinvgamma(NA, 1, 1)))
  expect_true(is.na(dinvgamma(1, NA, 1)))
  expect_true(is.na(dinvgamma(1, 1, NA)))
  
  expect_true(is.na(dinvchisq(NA, 1, 1)))
  expect_true(is.na(dinvchisq(1, NA, 1)))
  expect_true(is.na(dinvchisq(1, 1, NA)))
  
  expect_true(is.na(dkumar(NA, 1, 1)))
  expect_true(is.na(dkumar(0.5, NA, 1)))
  expect_true(is.na(dkumar(0.5, 1, NA)))
  
  expect_true(is.na(dlaplace(NA, 0, 1)))
  expect_true(is.na(dlaplace(1, NA, 1)))
  expect_true(is.na(dlaplace(1, 0, NA)))
  
  expect_true(is.na(dlgser(NA, 0.5)))
  expect_true(is.na(dlgser(1, NA)))

  expect_true(is.na(dlomax(NA, 1, 1)))
  expect_true(is.na(dlomax(1, NA, 1)))
  expect_true(is.na(dlomax(1, 1, NA)))

  expect_true(is.na(dmixnorm(NA, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(NA,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,NA,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,2,NA), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,2,3), c(NA,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,2,3), c(1,NA,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,2,3), c(1,2,3), c(NA,1/3,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,NA,1/3))))
  expect_true(is.na(dmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,1/3,NA))))

  expect_true(is.na(dmixpois(NA, c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixpois(0, c(NA,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixpois(0, c(1,NA,3), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixpois(0, c(1,2,NA), c(1/3,1/3,1/3))))
  expect_true(is.na(dmixpois(0, c(1,2,3), c(NA,1/3,1/3))))
  expect_true(is.na(dmixpois(0, c(1,2,3), c(1/3,NA,1/3))))
  expect_true(is.na(dmixpois(0, c(1,2,3), c(1/3,1/3,NA))))

  expect_true(is.na(dnhyper(NA, 60, 35, 15)))
  expect_true(is.na(dnhyper(1, NA, 35, 15)))
  expect_true(is.na(dnhyper(1, 60, NA, 15)))
  expect_true(is.na(dnhyper(1, 60, 35, NA)))
  
  expect_true(is.na(ddirmnom(c(NA, 1, 1), 3, c(1, 1, 1))))
  expect_true(is.na(ddirmnom(c(1, NA, 1), 3, c(1, 1, 1))))
  expect_true(is.na(ddirmnom(c(1, 1, NA), 3, c(1, 1, 1))))
  expect_true(is.na(ddirmnom(c(1, 1, 1), NA, c(1, 1, 1))))
  expect_true(is.na(ddirmnom(c(1, 1, 1), 3, c(NA, 1, 1))))
  expect_true(is.na(ddirmnom(c(1, 1, 1), 3, c(1, NA, 1))))
  expect_true(is.na(ddirmnom(c(1, 1, 1), 3, c(1, 1, NA))))

  expect_true(is.na(dmnom(c(NA, 1, 1), 3, c(1/3, 1/3, 1/3))))
  expect_true(is.na(dmnom(c(1, NA, 1), 3, c(1/3, 1/3, 1/3))))
  expect_true(is.na(dmnom(c(1, 1, NA), 3, c(1/3, 1/3, 1/3))))
  expect_true(is.na(dmnom(c(1, 1, 1), NA, c(1/3, 1/3, 1/3))))
  expect_true(is.na(dmnom(c(1, 1, 1), 3, c(NA, 1/3, 1/3))))
  expect_true(is.na(dmnom(c(1, 1, 1), 3, c(1/3, NA, 1/3))))
  expect_true(is.na(dmnom(c(1, 1, 1), 3, c(1/3, 1/3, NA))))

  expect_true(is.na(dmvhyper(c(NA, 2, 2), c(2,3,4), 5)))
  expect_true(is.na(dmvhyper(c(1, NA, 2), c(2,3,4), 5)))
  expect_true(is.na(dmvhyper(c(1, 2, NA), c(2,3,4), 5)))
  expect_true(is.na(dmvhyper(c(1, 2, 2), c(NA,3,4), 5)))
  expect_true(is.na(dmvhyper(c(1, 2, 2), c(2,NA,4), 5)))
  expect_true(is.na(dmvhyper(c(1, 2, 2), c(2,3,NA), 5)))
  expect_true(is.na(dmvhyper(c(1, 2, 2), c(2,3,4), NA)))

  expect_true(is.na(dnsbeta(NA, 1, 1, -2, 2)))
  expect_true(is.na(dnsbeta(0.5, NA, 1, -2, 2)))
  expect_true(is.na(dnsbeta(0.5, 1, NA, -2, 2)))
  expect_true(is.na(dnsbeta(0.5, 1, 1, NA, 2)))
  expect_true(is.na(dnsbeta(0.5, 1, 1, -2, NA)))

  expect_true(is.na(dlst(NA, 2, 0, 1)))
  expect_true(is.na(dlst(1, NA, 0, 1)))
  expect_true(is.na(dlst(1, 2, NA, 1)))
  expect_true(is.na(dlst(1, 2, 0, NA)))

  expect_true(is.na(dpareto(NA, 1, 1)))
  expect_true(is.na(dpareto(1, NA, 1)))
  expect_true(is.na(dpareto(1, 1, NA)))
  
  expect_true(is.na(dpower(NA, 1, 1)))
  expect_true(is.na(dpower(1, NA, 1)))
  expect_true(is.na(dpower(1, 1, NA)))
  
  expect_true(is.na(dprop(NA, 10, 0.5)))
  expect_true(is.na(dprop(1, NA, 0.5)))
  expect_true(is.na(dprop(1, 10, NA)))

  expect_true(is.na(drayleigh(NA, 1)))
  expect_true(is.na(drayleigh(1, NA)))

  expect_true(is.na(dskellam(NA, 1, 1)))
  expect_true(is.na(dskellam(1, NA, 1)))
  expect_true(is.na(dskellam(1, 1, NA)))
  
  expect_true(is.na(dsgomp(NA, 0.4, 1)))
  expect_true(is.na(dsgomp(1, NA, 1)))
  expect_true(is.na(dsgomp(1, 0.4, NA)))

  expect_true(is.na(dslash(NA, 1, 1)))
  expect_true(is.na(dslash(1, NA, 1)))
  expect_true(is.na(dslash(1, 1, NA)))

  expect_true(is.na(dtnorm(NA, 0, 1, -2, 2)))
  expect_true(is.na(dtnorm(1, NA, 1, -2, 2)))
  expect_true(is.na(dtnorm(1, 0, NA, -2, 2)))
  expect_true(is.na(dtnorm(1, 0, 1, NA, 2)))
  expect_true(is.na(dtnorm(1, 0, 1, -2, NA)))

  expect_true(is.na(dtpois(NA, 5, 0)))
  expect_true(is.na(dtpois(1, NA, 0)))
  expect_true(is.na(dtpois(1, 5, NA)))

  expect_true(is.na(dtriang(NA, 0, 1, 0.5)))
  expect_true(is.na(dtriang(0.5, NA, 1, 0.5)))
  expect_true(is.na(dtriang(0.5, 0, NA, 0.5)))
  expect_true(is.na(dtriang(0.5, 0, 1, NA)))

  expect_true(is.na(dwald(NA, 1, 1)))
  expect_true(is.na(dwald(1, NA, 1)))
  expect_true(is.na(dwald(1, 1, NA)))

  expect_true(is.na(dzip(NA, 1, 0.5)))
  expect_true(is.na(dzip(1, NA, 0.5)))
  expect_true(is.na(dzip(1, 1, NA)))

  expect_true(is.na(dzib(NA, 1, 0.5, 0.5)))
  expect_true(is.na(dzib(1, NA, 0.5, 0.5)))
  expect_true(is.na(dzib(1, 1, NA, 0.5)))
  expect_true(is.na(dzib(1, 1, 0.5, NA)))

  expect_true(is.na(dzinb(NA, 1, 0.5, 0.5)))
  expect_true(is.na(dzinb(1, NA, 0.5, 0.5)))
  expect_true(is.na(dzinb(1, 1, NA, 0.5)))
  expect_true(is.na(dzinb(1, 1, 0.5, NA)))

})





test_that("Wrong parameter values in CDF functions", {
  
  expect_true(is.na(pbbinom(NA, 1, 1, 1)))
  expect_true(is.na(pbbinom(1, NA, 1, 1)))
  expect_true(is.na(pbbinom(1, 1, NA, 1)))
  expect_true(is.na(pbbinom(1, 1, 1, NA)))
  
  expect_true(is.na(pbern(NA, 0.5)))
  expect_true(is.na(pbern(1, NA)))
  
  expect_true(is.na(pbetapr(NA, 1, 1, 1)))
  expect_true(is.na(pbetapr(1, NA, 1, 1)))
  expect_true(is.na(pbetapr(1, 1, NA, 1)))
  expect_true(is.na(pbetapr(1, 1, 1, NA)))
  
  expect_true(is.na(pbhatt(NA, 1, 1, 1)))
  expect_true(is.na(pbhatt(1, NA, 1, 1)))
  expect_true(is.na(pbhatt(1, 1, NA, 1)))
  expect_true(is.na(pbhatt(1, 1, 1, NA)))
  
  expect_true(is.na(pbnbinom(NA, 1, 1, 1)))
  expect_true(is.na(pbnbinom(1, NA, 1, 1)))
  expect_true(is.na(pbnbinom(1, 1, NA, 1)))
  expect_true(is.na(pbnbinom(1, 1, 1, NA)))
  
  expect_true(is.na(pcat(NA, c(0.5, 0.5))))
  expect_true(is.na(pcat(1, c(NA, 0.5))))
  expect_true(is.na(pcat(1, c(0.5, NA))))

  expect_true(is.na(pdlaplace(NA, 1, 1)))
  expect_true(is.na(pdlaplace(1, NA, 1)))
  expect_true(is.na(pdlaplace(1, 1, NA)))
  
  expect_true(is.na(pdnorm(NA, 1, 1)))
  expect_true(is.na(pdnorm(1, NA, 1)))
  expect_true(is.na(pdnorm(1, 1, NA)))
  
  expect_true(is.na(pdgamma(NA, 9, 1)))
  expect_true(is.na(pdgamma(1, NA, 1)))
  expect_true(is.na(pdgamma(1, 9, NA)))
  
  expect_true(is.na(pdunif(NA, 1, 10)))
  expect_true(is.na(pdunif(1, NA, 10)))
  expect_true(is.na(pdunif(1, 1, NA)))
  
  expect_true(is.na(pdweibull(NA, 1, 1)))
  expect_true(is.na(pdweibull(1, NA, 1))) 
  expect_true(is.na(pdweibull(1, 1, NA)))
  
  expect_true(is.na(pfatigue(NA, 1, 1)))
  expect_true(is.na(pfatigue(1, NA, 1)))
  expect_true(is.na(pfatigue(1, 1, NA)))
  
  expect_true(is.na(pfrechet(NA, 1, 1, 1)))
  expect_true(is.na(pfrechet(1, NA, 1, 1)))
  expect_true(is.na(pfrechet(1, 1, NA, 1)))
  expect_true(is.na(pfrechet(1, 1, 1, NA)))
  
  expect_true(is.na(pgev(NA, 1, 1, 1)))
  expect_true(is.na(pgev(1, NA, 1, 1)))
  expect_true(is.na(pgev(1, 1, NA, 1)))
  expect_true(is.na(pgev(1, 1, 1, NA)))
  
  expect_true(is.na(pgompertz(NA, 1, 1)))
  expect_true(is.na(pgompertz(1, NA, 1)))
  expect_true(is.na(pgompertz(1, 1, NA)))
  
  expect_true(is.na(pgpd(NA, 1, 1, 1)))
  expect_true(is.na(pgpd(1, NA, 1, 1)))
  expect_true(is.na(pgpd(1, 1, NA, 1)))
  expect_true(is.na(pgpd(1, 1, 1, NA)))
  
  expect_true(is.na(pgpois(NA, 1, 1)))
  expect_true(is.na(pgpois(1, NA, 1)))
  expect_true(is.na(pgpois(1, 1, NA)))
  
  expect_true(is.na(pgumbel(NA, 1, 1)))
  expect_true(is.na(pgumbel(1, NA, 1)))
  expect_true(is.na(pgumbel(1, 1, NA)))
  
  expect_true(is.na(phcauchy(NA, 1)))
  expect_true(is.na(phcauchy(1, NA)))
  
  expect_true(is.na(phnorm(NA, 1)))
  expect_true(is.na(phnorm(1, NA)))
  
  expect_true(is.na(pht(NA, 5, 1)))
  expect_true(is.na(pht(1, NA, 1)))
  expect_true(is.na(pht(1, 5, NA)))
  
  expect_true(is.na(phuber(NA, 0, 1, 1)))
  expect_true(is.na(phuber(1, NA, 1, 1)))
  expect_true(is.na(phuber(1, 0, NA, 1)))
  expect_true(is.na(phuber(1, 0, 1, NA)))
  
  expect_true(is.na(pinvgamma(NA, 1, 1)))
  expect_true(is.na(pinvgamma(1, NA, 1)))
  expect_true(is.na(pinvgamma(1, 1, NA)))
  
  expect_true(is.na(pinvchisq(NA, 1, 1)))
  expect_true(is.na(pinvchisq(1, NA, 1)))
  expect_true(is.na(pinvchisq(1, 1, NA)))
  
  expect_true(is.na(pkumar(NA, 1, 1)))
  expect_true(is.na(pkumar(0.5, NA, 1)))
  expect_true(is.na(pkumar(0.5, 1, NA)))
  
  expect_true(is.na(plaplace(NA, 0, 1)))
  expect_true(is.na(plaplace(1, NA, 1)))
  expect_true(is.na(plaplace(1, 0, NA)))
  
  expect_true(is.na(plgser(NA, 0.5)))
  expect_true(is.na(plgser(1, NA)))
  
  expect_true(is.na(plomax(NA, 1, 1)))
  expect_true(is.na(plomax(1, NA, 1)))
  expect_true(is.na(plomax(1, 1, NA)))
  
  expect_true(is.na(pmixnorm(NA, c(1,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(NA,2,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,NA,3), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,2,NA), c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,2,3), c(NA,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,2,3), c(1,NA,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,2,3), c(1,2,3), c(NA,1/3,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,NA,1/3))))
  expect_true(is.na(pmixnorm(0, c(1,2,3), c(1,2,3), c(1/3,1/3,NA))))
  
  expect_true(is.na(pmixpois(NA, c(1,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixpois(0, c(NA,2,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixpois(0, c(1,NA,3), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixpois(0, c(1,2,NA), c(1/3,1/3,1/3))))
  expect_true(is.na(pmixpois(0, c(1,2,3), c(NA,1/3,1/3))))
  expect_true(is.na(pmixpois(0, c(1,2,3), c(1/3,NA,1/3))))
  expect_true(is.na(pmixpois(0, c(1,2,3), c(1/3,1/3,NA))))

  expect_true(is.na(pnhyper(NA, 60, 35, 15)))
  expect_true(is.na(pnhyper(1, NA, 35, 15)))
  expect_true(is.na(pnhyper(1, 60, NA, 15)))
  expect_true(is.na(pnhyper(1, 60, 35, NA)))
  
  expect_true(is.na(pnsbeta(NA, 1, 1, -2, 2)))
  expect_true(is.na(pnsbeta(0.5, NA, 1, -2, 2)))
  expect_true(is.na(pnsbeta(0.5, 1, NA, -2, 2)))
  expect_true(is.na(pnsbeta(0.5, 1, 1, NA, 2)))
  expect_true(is.na(pnsbeta(0.5, 1, 1, -2, NA)))
  
  expect_true(is.na(plst(NA, 2, 0, 1)))
  expect_true(is.na(plst(1, NA, 0, 1)))
  expect_true(is.na(plst(1, 2, NA, 1)))
  expect_true(is.na(plst(1, 2, 0, NA)))
  
  expect_true(is.na(ppareto(NA, 1, 1)))
  expect_true(is.na(ppareto(1, NA, 1)))
  expect_true(is.na(ppareto(1, 1, NA)))
  
  expect_true(is.na(ppower(NA, 1, 1)))
  expect_true(is.na(ppower(1, NA, 1)))
  expect_true(is.na(ppower(1, 1, NA)))
  
  expect_true(is.na(pprop(NA, 10, 0.5)))
  expect_true(is.na(pprop(1, NA, 0.5)))
  expect_true(is.na(pprop(1, 10, NA)))
  
  expect_true(is.na(prayleigh(NA, 1)))
  expect_true(is.na(prayleigh(1, NA)))
  
  expect_true(is.na(psgomp(NA, 0.4, 1)))
  expect_true(is.na(psgomp(1, NA, 1)))
  expect_true(is.na(psgomp(1, 0.4, NA)))

  expect_true(is.na(pslash(NA, 1, 1)))
  expect_true(is.na(pslash(1, NA, 1)))
  expect_true(is.na(pslash(1, 1, NA)))
  
  expect_true(is.na(ptnorm(NA, 0, 1, -2, 2)))
  expect_true(is.na(ptnorm(1, NA, 1, -2, 2)))
  expect_true(is.na(ptnorm(1, 0, NA, -2, 2)))
  expect_true(is.na(ptnorm(1, 0, 1, NA, 2)))
  expect_true(is.na(ptnorm(1, 0, 1, -2, NA)))
  
  expect_true(is.na(ptpois(NA, 5, 0)))
  expect_true(is.na(ptpois(1, NA, 0)))
  expect_true(is.na(ptpois(1, 5, NA)))
  
  expect_true(is.na(ptriang(NA, 0, 1, 0.5)))
  expect_true(is.na(ptriang(0.5, NA, 1, 0.5)))
  expect_true(is.na(ptriang(0.5, 0, NA, 0.5)))
  expect_true(is.na(ptriang(0.5, 0, 1, NA)))
  
  expect_true(is.na(pwald(NA, 1, 1)))
  expect_true(is.na(pwald(1, NA, 1)))
  expect_true(is.na(pwald(1, 1, NA)))
  
  expect_true(is.na(pzip(NA, 1, 0.5)))
  expect_true(is.na(pzip(1, NA, 0.5)))
  expect_true(is.na(pzip(1, 1, NA)))
  
  expect_true(is.na(pzib(NA, 1, 0.5, 0.5)))
  expect_true(is.na(pzib(1, NA, 0.5, 0.5)))
  expect_true(is.na(pzib(1, 1, NA, 0.5)))
  expect_true(is.na(pzib(1, 1, 0.5, NA)))
  
  expect_true(is.na(pzinb(NA, 1, 0.5, 0.5)))
  expect_true(is.na(pzinb(1, NA, 0.5, 0.5)))
  expect_true(is.na(pzinb(1, 1, NA, 0.5)))
  expect_true(is.na(pzinb(1, 1, 0.5, NA)))
  
})




test_that("Wrong parameter values in inverse CDF functions", {

  expect_true(is.na(qbern(NA, 0.5)))
  expect_true(is.na(qbern(0.5, NA)))
  
  expect_true(is.na(qbetapr(NA, 1, 1, 1)))
  expect_true(is.na(qbetapr(0.5, NA, 1, 1)))
  expect_true(is.na(qbetapr(0.5, 1, NA, 1)))
  expect_true(is.na(qbetapr(0.5, 1, 1, NA)))
  
  expect_true(is.na(qcat(NA, c(0.5, 0.5))))
  expect_true(is.na(qcat(0.5, c(NA, 0.5))))
  expect_true(is.na(qcat(0.5, c(0.5, NA))))

  expect_true(is.na(qdunif(NA, 1, 10)))
  expect_true(is.na(qdunif(0.5, NA, 10)))
  expect_true(is.na(qdunif(0.5, 1, NA)))
  
  expect_true(is.na(qdweibull(NA, 1, 1)))
  expect_true(is.na(qdweibull(0.5, NA, 1))) 
  expect_true(is.na(qdweibull(0.5, 1, NA)))
  
  expect_true(is.na(qfatigue(NA, 1, 1)))
  expect_true(is.na(qfatigue(0.5, NA, 1)))
  expect_true(is.na(qfatigue(0.5, 1, NA)))
  
  expect_true(is.na(qfrechet(NA, 1, 1, 1)))
  expect_true(is.na(qfrechet(0.5, NA, 1, 1)))
  expect_true(is.na(qfrechet(0.5, 1, NA, 1)))
  expect_true(is.na(qfrechet(0.5, 1, 1, NA)))
  
  expect_true(is.na(qgev(NA, 1, 1, 1)))
  expect_true(is.na(qgev(0.5, NA, 1, 1)))
  expect_true(is.na(qgev(0.5, 1, NA, 1)))
  expect_true(is.na(qgev(0.5, 1, 1, NA)))
  
  expect_true(is.na(qgompertz(NA, 1, 1)))
  expect_true(is.na(qgompertz(0.5, NA, 1)))
  expect_true(is.na(qgompertz(0.5, 1, NA)))
  
  expect_true(is.na(qgpd(NA, 1, 1, 1)))
  expect_true(is.na(qgpd(0.5, NA, 1, 1)))
  expect_true(is.na(qgpd(0.5, 1, NA, 1)))
  expect_true(is.na(qgpd(0.5, 1, 1, NA)))
  
  expect_true(is.na(qgumbel(NA, 1, 1)))
  expect_true(is.na(qgumbel(0.5, NA, 1)))
  expect_true(is.na(qgumbel(0.5, 1, NA)))
  
  expect_true(is.na(qhcauchy(NA, 1)))
  expect_true(is.na(qhcauchy(0.5, NA)))
  
  expect_true(is.na(qhnorm(NA, 1)))
  expect_true(is.na(qhnorm(0.5, NA)))
  
  expect_true(is.na(qht(NA, 5, 1)))
  expect_true(is.na(qht(0.5, NA, 1)))
  expect_true(is.na(qht(0.5, 5, NA)))
  
  expect_true(is.na(qhuber(NA, 0, 1, 1)))
  expect_true(is.na(qhuber(0.5, NA, 1, 1)))
  expect_true(is.na(qhuber(0.5, 0, NA, 1)))
  expect_true(is.na(qhuber(0.5, 0, 1, NA)))
  
  expect_true(is.na(qinvgamma(NA, 1, 1)))
  expect_true(is.na(qinvgamma(0.5, NA, 1)))
  expect_true(is.na(qinvgamma(0.5, 1, NA)))
  
  expect_true(is.na(qinvchisq(NA, 1, 1)))
  expect_true(is.na(qinvchisq(0.5, NA, 1)))
  expect_true(is.na(qinvchisq(0.5, 1, NA)))
  
  expect_true(is.na(qkumar(NA, 1, 1)))
  expect_true(is.na(qkumar(0.5, NA, 1)))
  expect_true(is.na(qkumar(0.5, 1, NA)))
  
  expect_true(is.na(qlaplace(NA, 0, 1)))
  expect_true(is.na(qlaplace(0.5, NA, 1)))
  expect_true(is.na(qlaplace(0.5, 0, NA)))
  
  expect_true(is.na(qlgser(NA, 0.5)))
  expect_true(is.na(qlgser(0.5, NA)))
  
  expect_true(is.na(qlomax(NA, 1, 1)))
  expect_true(is.na(qlomax(0.5, NA, 1)))
  expect_true(is.na(qlomax(0.5, 1, NA)))

  expect_true(is.na(qnhyper(NA, 60, 35, 15)))
  expect_true(is.na(qnhyper(0.5, NA, 35, 15)))
  expect_true(is.na(qnhyper(0.5, 60, NA, 15)))
  expect_true(is.na(qnhyper(0.5, 60, 35, NA)))
  
  expect_true(is.na(qnsbeta(NA, 1, 1, -2, 2)))
  expect_true(is.na(qnsbeta(0.5, NA, 1, -2, 2)))
  expect_true(is.na(qnsbeta(0.5, 1, NA, -2, 2)))
  expect_true(is.na(qnsbeta(0.5, 1, 1, NA, 2)))
  expect_true(is.na(qnsbeta(0.5, 1, 1, -2, NA)))
  
  expect_true(is.na(qlst(NA, 2, 0, 1)))
  expect_true(is.na(qlst(0.5, NA, 0, 1)))
  expect_true(is.na(qlst(0.5, 2, NA, 1)))
  expect_true(is.na(qlst(0.5, 2, 0, NA)))
  
  expect_true(is.na(qpareto(NA, 1, 1)))
  expect_true(is.na(qpareto(0.5, NA, 1)))
  expect_true(is.na(qpareto(0.5, 1, NA)))
  
  expect_true(is.na(dpower(NA, 1, 1)))
  expect_true(is.na(dpower(0.5, NA, 1)))
  expect_true(is.na(dpower(0.5, 1, NA)))
  
  expect_true(is.na(qprop(NA, 10, 0.5)))
  expect_true(is.na(qprop(0.5, NA, 0.5)))
  expect_true(is.na(qprop(0.5, 10, NA)))
  
  expect_true(is.na(qrayleigh(NA, 1)))
  expect_true(is.na(qrayleigh(0.5, NA)))
  
  expect_true(is.na(qtlambda(NA, 0.5)))
  expect_true(is.na(qtlambda(0, NA)))

  expect_true(is.na(qtnorm(NA, 0, 1, -2, 2)))
  expect_true(is.na(qtnorm(0.5, NA, 1, -2, 2)))
  expect_true(is.na(qtnorm(0.5, 0, NA, -2, 2)))
  expect_true(is.na(qtnorm(0.5, 0, 1, NA, 2)))
  expect_true(is.na(qtnorm(0.5, 0, 1, -2, NA)))
  
  expect_true(is.na(qtpois(NA, 5, 0)))
  expect_true(is.na(qtpois(0.5, NA, 0)))
  expect_true(is.na(qtpois(0.5, 5, NA)))
  
  expect_true(is.na(qtriang(NA, 0, 1, 0.5)))
  expect_true(is.na(qtriang(0.5, NA, 1, 0.5)))
  expect_true(is.na(qtriang(0.5, 0, NA, 0.5)))
  expect_true(is.na(qtriang(0.5, 0, 1, NA)))
  
  expect_true(is.na(qzip(NA, 1, 0.5)))
  expect_true(is.na(qzip(0.5, NA, 0.5)))
  expect_true(is.na(qzip(0.5, 1, NA)))
  
  expect_true(is.na(qzib(NA, 1, 0.5, 0.5)))
  expect_true(is.na(qzib(0.5, NA, 0.5, 0.5)))
  expect_true(is.na(qzib(0.5, 1, NA, 0.5)))
  expect_true(is.na(qzib(0.5, 1, 0.5, NA)))
  
  expect_true(is.na(qzinb(NA, 1, 0.5, 0.5)))
  expect_true(is.na(qzinb(0.5, NA, 0.5, 0.5)))
  expect_true(is.na(qzinb(0.5, 1, NA, 0.5)))
  expect_true(is.na(qzinb(0.5, 1, 0.5, NA)))

})

 
 

test_that("Wrong parameter values in RNG functions", {

  expect_warning(expect_true(is.na(rbbinom(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rbbinom(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rbbinom(1, 1, 1, NA))))

  expect_warning(expect_true(is.na(rbern(1, NA))))
  
  expect_warning(expect_true(is.na(rbetapr(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rbetapr(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rbetapr(1, 1, 1, NA))))

  expect_warning(expect_true(is.na(rbhatt(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rbhatt(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rbhatt(1, 1, 1, NA))))

  expect_warning(expect_true(is.na(rbnbinom(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rbnbinom(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rbnbinom(1, 1, 1, NA))))

  expect_warning(expect_true(all(is.na(rbvnorm(1, NA, 1, 1, 1, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, NA, 1, 1, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, 1, NA, 1, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, 1, 1, NA, 0.5)))))
  expect_warning(expect_true(all(is.na(rbvnorm(1, 1, 1, 1, 1, NA)))))
  
  expect_warning(expect_true(all(is.na(rbvpois(1, NA, 1, 1)))))
  expect_warning(expect_true(all(is.na(rbvpois(1, 1, NA, 1)))))
  expect_warning(expect_true(all(is.na(rbvpois(1, 1, 1, NA)))))

  expect_warning(expect_true(is.na(rcat(1, c(NA, 0.5)))))
  expect_warning(expect_true(is.na(rcat(1, c(0.5, NA)))))
  
  expect_warning(expect_true(is.na(rcatlp(1, c(NA, 0.5)))))
  expect_warning(expect_true(is.na(rcatlp(1, c(0.5, NA)))))

  expect_warning(expect_true(all(is.na(rdirichlet(1, c(NA, 0.5))))))
  expect_warning(expect_true(all(is.na(rdirichlet(1, c(0.5, NA))))))
  
  expect_warning(expect_true(is.na(rdlaplace(1, NA, 1))))
  expect_warning(expect_true(is.na(rdlaplace(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rdnorm(1, NA, 1))))
  expect_warning(expect_true(is.na(rdnorm(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rdgamma(1, NA, 1))))
  expect_warning(expect_true(is.na(rdgamma(1, 9, NA))))
  
  expect_warning(expect_true(is.na(rdunif(1, NA, 10))))
  expect_warning(expect_true(is.na(rdunif(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rdweibull(1, NA, 1)))) 
  expect_warning(expect_true(is.na(rdweibull(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rfatigue(1, NA, 1))))
  expect_warning(expect_true(is.na(rfatigue(1, 1, NA))))

  expect_warning(expect_true(is.na(rfrechet(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rfrechet(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rfrechet(1, 1, 1, NA))))
  
  expect_warning(expect_true(is.na(rgev(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rgev(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rgev(1, 1, 1, NA))))

  expect_warning(expect_true(is.na(rgompertz(1, NA, 1))))
  expect_warning(expect_true(is.na(rgompertz(1, 1, NA))))

  expect_warning(expect_true(is.na(rgpd(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rgpd(1, 1, NA, 1))))
  expect_warning(expect_true(is.na(rgpd(1, 1, 1, NA))))

  expect_warning(expect_true(is.na(rgpois(1, NA, 1))))
  expect_warning(expect_true(is.na(rgpois(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rgumbel(1, NA, 1))))
  expect_warning(expect_true(is.na(rgumbel(1, 1, NA))))

  expect_warning(expect_true(is.na(rhcauchy(1, NA))))

  expect_warning(expect_true(is.na(rhnorm(1, NA))))
  
  expect_warning(expect_true(is.na(rht(1, NA, 1))))
  expect_warning(expect_true(is.na(rht(1, 5, NA))))
  
  expect_warning(expect_true(is.na(rhuber(1, NA, 1, 1))))
  expect_warning(expect_true(is.na(rhuber(1, 0, NA, 1))))
  expect_warning(expect_true(is.na(rhuber(1, 0, 1, NA))))
  
  # expect_warning(expect_true(is.na(rinvgamma(1, NA, 1))))
  # expect_warning(expect_true(is.na(rinvgamma(1, 1, NA))))
  # 
  # expect_warning(expect_true(is.na(rinvchisq(1, NA, 1))))
  # expect_warning(expect_true(is.na(rinvchisq(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rkumar(1, NA, 1))))
  expect_warning(expect_true(is.na(rkumar(1, 1, NA))))

  expect_warning(expect_true(is.na(rlaplace(1, NA, 1))))
  expect_warning(expect_true(is.na(rlaplace(1, 0, NA))))
  
  expect_warning(expect_true(is.na(rlgser(1, NA))))
  
  expect_warning(expect_true(is.na(rlomax(1, NA, 1))))
  expect_warning(expect_true(is.na(rlomax(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rmixnorm(1, c(NA,2,3), c(1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,NA,3), c(1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,NA), c(1,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(NA,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,NA,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), c(NA,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), c(1/3,NA,1/3)))))
  expect_warning(expect_true(is.na(rmixnorm(1, c(1,2,3), c(1,2,3), c(1/3,1/3,NA)))))

  expect_warning(expect_true(is.na(rmixpois(1, c(NA,2,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,NA,3), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,NA), c(1/3,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), c(NA,1/3,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), c(1/3,NA,1/3)))))
  expect_warning(expect_true(is.na(rmixpois(1, c(1,2,3), c(1/3,1/3,NA)))))
  
  expect_warning(expect_true(is.na(rnhyper(1, NA, 35, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, NA, 15))))
  expect_warning(expect_true(is.na(rnhyper(1, 60, 35, NA))))
  
  expect_warning(expect_true(all(is.na(rdirmnom(1, NA, c(1, 1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, c(NA, 1, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, c(1, NA, 1))))))
  expect_warning(expect_true(all(is.na(rdirmnom(1, 3, c(1, 1, NA))))))
  
  expect_warning(expect_true(all(is.na(rmnom(1, NA, c(1/3, 1/3, 1/3))))))
  expect_warning(expect_true(all(is.na(rmnom(1, 3, c(NA, 1/3, 1/3))))))
  expect_warning(expect_true(all(is.na(rmnom(1, 3, c(1/3, NA, 1/3))))))
  expect_warning(expect_true(all(is.na(rmnom(1, 3, c(1/3, 1/3, NA))))))
  
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(NA,3,4), 5)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,NA,4), 5)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,3,NA), 5)))))
  expect_warning(expect_true(all(is.na(rmvhyper(1, c(2,3,4), NA)))))
  
  expect_warning(expect_true(is.na(rnsbeta(1, NA, 1, -2, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, NA, -2, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, 1, NA, 2))))
  expect_warning(expect_true(is.na(rnsbeta(1, 1, 1, -2, NA))))

  expect_warning(expect_true(is.na(rlst(1, NA, 0, 1))))
  expect_warning(expect_true(is.na(rlst(1, 2, NA, 1))))
  expect_warning(expect_true(is.na(rlst(1, 2, 0, NA))))
  
  expect_warning(expect_true(is.na(rpareto(1, NA, 1))))
  expect_warning(expect_true(is.na(rpareto(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rpower(1, NA, 1))))
  expect_warning(expect_true(is.na(rpower(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rprop(1, NA, 0.5))))
  expect_warning(expect_true(is.na(rprop(1, 10, NA))))
  
  expect_warning(expect_true(is.na(rrayleigh(1, NA))))
  
  expect_warning(expect_true(is.na(rtlambda(1, NA))))
  
  expect_warning(expect_true(is.na(rsgomp(1, NA, 1))))
  expect_warning(expect_true(is.na(rsgomp(1, 0.4, NA))))
  
  expect_warning(expect_true(is.na(rskellam(1, NA, 1))))
  expect_warning(expect_true(is.na(rskellam(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rslash(1, NA, 1))))
  expect_warning(expect_true(is.na(rslash(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rtnorm(1, NA, 1, -2, 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, NA, -2, 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, 1, NA, 2))))
  expect_warning(expect_true(is.na(rtnorm(1, 0, 1, -2, NA))))

  expect_warning(expect_true(is.na(rtpois(1, NA, 0))))
  expect_warning(expect_true(is.na(rtpois(1, 5, NA))))
  
  expect_warning(expect_true(is.na(rtriang(1, NA, 1, 0.5))))
  expect_warning(expect_true(is.na(rtriang(1, 0, NA, 0.5))))
  expect_warning(expect_true(is.na(rtriang(1, 0, 1, NA))))
  
  expect_warning(expect_true(is.na(rwald(1, NA, 1))))
  expect_warning(expect_true(is.na(rwald(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rzip(1, NA, 0.5))))
  expect_warning(expect_true(is.na(rzip(1, 1, NA))))
  
  expect_warning(expect_true(is.na(rzib(1, NA, 0.5, 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, NA, 0.5))))
  expect_warning(expect_true(is.na(rzib(1, 1, 0.5, NA))))
  
  expect_warning(expect_true(is.na(rzinb(1, NA, 0.5, 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, NA, 0.5))))
  expect_warning(expect_true(is.na(rzinb(1, 1, 0.5, NA))))

})

#' Mixture of normal distributions
#'
#' Density, distribution function and random generation
#' for the mixture of normal distributions.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mean 	          matrix (or vector) of means.
#' @param sd              matrix (or vector) of standard deviations.
#' @param alpha           matrix (or vector) of mixing proportions;
#'                        mixing proportions need to sum up to 1.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \alpha_1 f_1(x; \mu_1, \sigma_1) + \dots + \alpha_k f_k(x; \mu_k, \sigma_k)
#' }{
#' f(x) = \alpha[1] * f1(x; \mu[1], \sigma[1]) + \dots + \alpha[k] * fk(x; \mu[k], \sigma[k])
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \alpha_1 F_1(x; \mu_1, \sigma_1) + \dots + \alpha_k F_k(x; \mu_k, \sigma_k)
#' }{
#' F(x) = \alpha[1] * F1(x; \mu[1], \sigma[1]) + \dots + \alpha[k] * Fk(x; \mu[k], \sigma[k])
#' }
#' 
#' where \eqn{\sum_i \alpha_i = 1}{sum(\alpha[i]) == 1}.
#'
#' @examples 
#' 
#' x <- rmixnorm(1e5, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3))
#' hist(x, 100, freq = FALSE)
#' curve(dmixnorm(x, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3)),
#'       -20, 20, n = 500, col = "red", add = TRUE)
#' hist(pmixnorm(x, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3)))
#' plot(ecdf(x))
#' curve(pmixnorm(x, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3)),
#'       -20, 20, n = 500, col = "red", lwd = 2, add = TRUE)
#'
#' @name NormalMix
#' @aliases NormalMix
#' @aliases dmixnorm
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dmixnorm <- function(x, mean, sd, alpha, log = FALSE) {
  
  if (is.vector(mean))
    mean <- matrix(mean, nrow = 1)
  else if (!is.matrix(mean))
    mean <- as.matrix(mean)
  
  if (is.vector(sd))
    sd <- matrix(sd, nrow = 1)
  else if (!is.matrix(sd))
    sd <- as.matrix(sd)
  
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  
  cpp_dmixnorm(x, mean, sd, alpha, log[1L])
}


#' @rdname NormalMix
#' @export

pmixnorm <- function(q, mean, sd, alpha, lower.tail = TRUE, log.p = FALSE) {
  
  if (is.vector(mean))
    mean <- matrix(mean, nrow = 1)
  else if (!is.matrix(mean))
    mean <- as.matrix(mean)
  
  if (is.vector(sd))
    sd <- matrix(sd, nrow = 1)
  else if (!is.matrix(sd))
    sd <- as.matrix(sd)
  
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  
  cpp_pmixnorm(q, mean, sd, alpha, lower.tail[1L], log.p[1L])
}


#' @rdname NormalMix
#' @export

rmixnorm <- function(n, mean, sd, alpha) {
  if (length(n) > 1) n <- length(n)
  
  if (is.vector(mean))
    mean <- matrix(mean, nrow = 1)
  else if (!is.matrix(mean))
    mean <- as.matrix(mean)
  
  if (is.vector(sd))
    sd <- matrix(sd, nrow = 1)
  else if (!is.matrix(sd))
    sd <- as.matrix(sd)
  
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  
  cpp_rmixnorm(n, mean, sd, alpha)
}

#' Discrete normal distribution
#' 
#' Probability mass function, distribution function and random generation
#' for discrete normal distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mean            vector of means.
#' @param sd              vector of standard deviations.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' @details
#' 
#' Probability mass function
#' 
#' \deqn{
#' f(x) = \Phi\left(\frac{x-\mu+1}{\sigma}\right) - \Phi\left(\frac{x-\mu}{\sigma}\right)
#' }{
#' f(x) = \Phi((x-\mu+1)/\sigma) - \Phi((x-\mu)/\sigma)
#' }
#' 
#' Cumulative distribution function
#' 
#' \deqn{
#' F(x) = \Phi\left(\frac{\lfloor x \rfloor + 1 - \mu}{\sigma}\right)
#' }{
#' F(x) = \Phi((floor(x)+1-\mu)/\sigma)
#' }
#' 
#' @references 
#' Roy, D. (2003). The discrete normal distribution.
#' Communications in Statistics-Theory and Methods, 32, 1871-1883.
#' 
#' @seealso \code{\link[stats]{Normal}}
#' 
#' @examples 
#' 
#' x <- rdnorm(1e5, 0, 3)
#' xx <- -15:15
#' plot(prop.table(table(x)))
#' lines(xx, ddnorm(xx, 0, 3), col = "red")
#' hist(pdnorm(x, 0, 3))
#' plot(ecdf(x))
#' xx <- seq(-15, 15, 0.1)
#' lines(xx, pdnorm(xx, 0, 3), col = "red", lwd = 2, type = "s")
#' 
#' @name DiscreteNormal
#' @aliases DiscreteNormal
#' @aliases ddnorm
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#' 
#' @export

ddnorm <- function(x, mean = 0, sd = 1, log = FALSE) {
  cpp_ddnorm(x, mean, sd, log[1L])
}


#' @rdname DiscreteNormal
#' @export

pdnorm <- function(q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE) {
  pnorm(floor(q)+1, mean, sd, lower.tail[1L], log.p[1L])
}


#' @rdname DiscreteNormal
#' @export

rdnorm <- function(n, mean = 0, sd = 1) {
  floor(rnorm(n, mean, sd))
}

#' Discrete Weibull distribution (type I)
#'
#' Density, distribution function, quantile function and random generation
#' for the discrete Weibull (type I) distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param shape1,shape2   parameters (named q, \eqn{\beta}). Values of \code{shape2}
#'                        need to be positive and \code{0 < shape1 < 1}.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability mass function
#' \deqn{
#' f(x) = q^{x^\beta} - q^{(x+1)^\beta}
#' }{
#' f(x) = q^x^\beta - q^(x+1)^\beta
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = 1-q^{(x+1)^\beta}
#' }{
#' F(x) = 1-q^(x+1)^\beta
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \left \lceil{\left(\frac{\log(1-p)}{\log(q)}\right)^{1/\beta} - 1}\right \rceil
#' }{
#' F^-1(p) = ceiling((log(1-p)/log(q))^(1/\beta) - 1)
#' }
#'
#' @references
#' Nakagawa, T. and Osaki, S. (1975). The Discrete Weibull Distribution.
#' IEEE Transactions on Reliability, R-24, 300-301.
#' 
#' @references 
#' Kulasekera, K.B. (1994).
#' Approximate MLE's of the parameters of a discrete Weibull distribution
#' with type I censored data.
#' Microelectronics Reliability, 34(7), 1185-1188.
#' 
#' @references 
#' Khan, M.A., Khalique, A. and Abouammoh, A.M. (1989).
#' On estimating parameters in a discrete Weibull distribution.
#' IEEE Transactions on Reliability, 38(3), 348-350.
#' 
#' @seealso \code{\link[stats]{Weibull}}
#' 
#' @examples 
#' 
#' x <- rdweibull(1e5, 0.32, 1)
#' xx <- seq(-2, 100, by = 1)
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, ddweibull(xx, .32, 1), col = "red")
#' 
#' # Notice: distribution of F(X) is far from uniform:
#' hist(pdweibull(x, .32, 1), 50)
#' 
#' plot(ecdf(x))
#' lines(xx, pdweibull(xx, .32, 1), col = "red", lwd = 2, type = "s")
#'
#' @name DiscreteWeibull
#' @aliases DiscreteWeibull
#' @aliases ddweibull
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

ddweibull <- function(x, shape1, shape2, log = FALSE) {
  cpp_ddweibull(x, shape1, shape2, log[1L])
}


#' @rdname DiscreteWeibull
#' @export

pdweibull <- function(q, shape1, shape2, lower.tail = TRUE, log.p = FALSE) {
  cpp_pdweibull(q, shape1, shape2, lower.tail[1L], log.p[1L])
}


#' @rdname DiscreteWeibull
#' @export

qdweibull <- function(p, shape1, shape2, lower.tail = TRUE, log.p = FALSE) {
  cpp_qdweibull(p, shape1, shape2, lower.tail[1L], log.p[1L])
}


#' @rdname DiscreteWeibull
#' @export

rdweibull <- function(n, shape1, shape2) {
  if (length(n) > 1) n <- length(n)
  cpp_rdweibull(n, shape1, shape2)
}

#' Bhattacharjee distribution
#'
#' Density, distribution function, and random generation for the Bhattacharjee
#' distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu,sigma,a	    location, scale and shape parameters.
#'                        Scale and shape must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#' 
#' If \eqn{Z \sim \mathrm{Normal}(0, 1)}{Z ~ Normal(0, 1)} and
#' \eqn{U \sim \mathrm{Uniform}(0, 1)}{U ~ Uniform(0, 1)}, then
#' \eqn{Z+U} follows Bhattacharjee distribution.
#' 
#' Probability density function
#' 
#' \deqn{
#' f(z) = \frac{1}{2a} \left[\Phi\left(\frac{x-\mu+a}{\sigma}\right) - \Phi\left(\frac{x-\mu-a}{\sigma}\right)\right]
#' }{
#' f(z) = 1/(2*a) * (\Phi((x-\mu+a)/\sigma) - \Phi((x-\mu+a)/\sigma))
#' }
#' 
#' Cumulative distribution function
#' 
#' \deqn{
#' F(z) = \frac{\sigma}{2a} \left[(x-\mu)\Phi\left(\frac{x-\mu+a}{\sigma}\right) -
#'                                (x-\mu)\Phi\left(\frac{x-\mu-a}{\sigma}\right) +
#'                                \phi\left(\frac{x-\mu+a}{\sigma}\right) -
#'                                \phi\left(\frac{x-\mu-a}{\sigma}\right)\right]
#' }{
#' F(z) = \sigma/(2*a) * ((x-\mu)*\Phi((x-\mu+a)/\sigma) - (x-\mu)*\Phi((x-\mu-a)/\sigma) +
#'                        \phi((x-\mu+a)/\sigma) - \phi((x-\mu-a)/\sigma))
#' }
#'
#' @references
#' Bhattacharjee, G.P., Pandit, S.N.N., and Mohan, R. (1963).
#' Dimensional chains involving rectangular and normal error-distributions.
#' Technometrics, 5, 404-406.
#' 
#' @examples 
#' 
#' x <- rbhatt(1e5, 5, 3, 5)
#' hist(x, 100, freq = FALSE)
#' curve(dbhatt(x, 5, 3, 5), -20, 20, col = "red", add = TRUE)
#' hist(pbhatt(x, 5, 3, 5))
#' plot(ecdf(x))
#' curve(pbhatt(x, 5, 3, 5), -20, 20, col = "red", lwd = 2, add = TRUE)
#'
#' @name Bhattacharjee
#' @aliases Bhattacharjee
#' @aliases dbhatt
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dbhatt <- function(x, mu = 0, sigma = 1, a = sigma, log = FALSE) {
  cpp_dbhatt(x, mu, sigma, a, log[1L])
}


#' @rdname Bhattacharjee
#' @export

pbhatt <- function(q, mu = 0, sigma = 1, a = sigma, lower.tail = TRUE, log.p = FALSE) {
  cpp_pbhatt(q, mu, sigma, a, lower.tail[1L], log.p[1L])
}


#' @rdname Bhattacharjee
#' @export

rbhatt <- function(n, mu = 0, sigma = 1, a = sigma) {
  if (length(n) > 1) n <- length(n)
  cpp_rbhatt(n, mu, sigma, a)
}

#' Zero-inflated negative binomial distribution
#'
#' Probability mass function and random generation
#' for the zero-inflated negative binomial distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param size	          target for number of successful trials, or dispersion
#'                        parameter (the shape parameter of the gamma mixing
#'                        distribution). Must be strictly positive, need not be
#'                        integer.
#' @param prob            probability of success in each trial. \code{0 < prob <= 1}.
#' @param pi              probability of extra zeros.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \left\{\begin{array}{ll}
#' \pi + (1 - \pi) p^r & x = 0 \\
#' (1 - \pi) {x+r-1 \choose x} p^r (1-p)^x & x > 0 \\
#' \end{array}\right.
#' }{
#' f(x) = [if x = 0:] (1-\pi)+\pi * p^r [else:] (1-\pi) * dnbinom(x, size, prob)
#' }
#' 
#' @seealso \code{\link[stats]{NegBinomial}}
#' 
#' @examples 
#' 
#' x <- rzinb(1e5, 100, 0.6, 0.33)
#' xx <- -2:200
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, dzinb(xx, 100, 0.6, 0.33), col = "red")
#' 
#' xx <- seq(0, 200, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pzinb(xx, 100, 0.6, 0.33), col = "red")
#'
#' @name ZINB
#' @aliases ZINB
#' @aliases dzinb
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dzinb <- function(x, size, prob, pi, log = FALSE) {
  cpp_dzinb(x, size, prob, pi, log[1L])
}


#' @rdname ZINB
#' @export

pzinb <- function(q, size, prob, pi, lower.tail = TRUE, log.p = FALSE) {
  cpp_pzinb(q, size, prob, pi, lower.tail[1L], log.p[1L])
}


#' @rdname ZINB
#' @export

qzinb <- function(p, size, prob, pi, lower.tail = TRUE, log.p = FALSE) {
  cpp_qzinb(p, size, prob, pi, lower.tail[1L], log.p[1L])
}


#' @rdname ZINB
#' @export

rzinb <- function(n, size, prob, pi) {
  if (length(n) > 1) n <- length(n)
  cpp_rzinb(n, size, prob, pi)
}

#' Bivariate Poisson distribution
#'
#' Probability mass function and random generation for the bivariate Poisson distribution.
#'
#' @param x,y	  vectors of quantiles; alternatively x may be a two-column
#'              matrix (or data.frame) and y may be omitted.
#' @param n	    number of observations. If \code{length(n) > 1},
#'              the length is taken to be the number required.
#' @param a,b,c positive valued parameters.
#' @param log   logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#'
#' Probability mass function
#' \deqn{
#' f(x,y) = \exp \{-(a+b+c)\} \frac{a^x}{x!} \frac{b^y}{y!} \sum_{k=0}^{\min(x,y)}
#' {x \choose k} {y \choose k} k! \left( \frac{c}{ab} \right)^k
#' }{
#' f(x,y) = exp(-(a+b+c)) * (a^x)/x! * (b^y)/y! *
#' sum(choose(x,k)*choose(y,k)*k!*(c/(a*b))^k)
#' }
#'
#' @references
#' Karlis, D. and Ntzoufras, I. (2003). Analysis of sports data by using bivariate Poisson models.
#' Journal of the Royal Statistical Society: Series D (The Statistician), 52(3), 381-393.
#'
#' @references
#' Kocherlakota, S. and Kocherlakota, K. (1992) Bivariate Discrete Distributions.
#' New York: Dekker.
#'
#' @references
#' Johnson, N., Kotz, S. and Balakrishnan, N. (1997). Discrete Multivariate Distributions.
#' New York: Wiley.
#'
#' @references
#' Holgate, P. (1964). Estimation for the bivariate Poisson distribution.
#' Biometrika, 51(1-2), 241-287.
#'
#' @references
#' Kawamura, K. (1984). Direct calculation of maximum likelihood estimator for the bivariate
#' Poisson distribution. Kodai mathematical journal, 7(2), 211-221.
#'
#' @examples
#'
#' x <- rbvpois(5000, 7, 8, 5)
#' image(prop.table(table(x[,1], x[,2])))
#' colMeans(x)
#'
#' @seealso \code{\link[stats]{Poisson}}
#'
#' @name BivPoiss
#' @aliases BivPoiss
#' @aliases dbvpois
#'
#' @keywords distribution
#' @concept Multivariate
#' @concept Discrete
#'
#' @export

dbvpois <- function(x, y = NULL, a, b, c, log = FALSE) {
  if (is.null(y)) {
    if ((is.matrix(x) || is.data.frame(x)) && ncol(x) == 2) {
      y <- x[, 2]
      x <- x[, 1]
    } else {
      stop("y is not provided while x is not a two-column matrix")
    }
  }
  cpp_dbpois(x, y, a, b, c, log[1L])
}


#' @rdname BivPoiss
#' @export

rbvpois <- function(n, a, b, c) {
  if (length(n) > 1) n <- length(n)
  cpp_rbpois(n, a, b, c)
}

#' "Huber density" distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the "Huber density" distribution.
#'
#' @param x,q	             vector of quantiles.
#' @param p	               vector of probabilities.
#' @param n	               number of observations. If \code{length(n) > 1},
#'                         the length is taken to be the number required.
#' @param mu,sigma,epsilon location, and scale, and shape parameters.
#'                         Scale and shape must be positive.
#' @param log,log.p	       logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	     logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                         otherwise, \eqn{P[X > x]}.
#'
#' @details
#' 
#' Huber density is connected to Huber loss and can be defined as:
#'
#' \deqn{
#' f(x) = \frac{1}{2 \sqrt{2\pi} \left( \Phi(k) + \phi(k)/k - \frac{1}{2} \right)} e^{-\rho_k(x)}
#' }{
#' f(x) = 1/(2 * sqrt(2\pi) * (\Phi(k) + \phi(k)/k - 1/2)) * exp(-\rho(x, k))
#' }
#'
#' where
#'
#' \deqn{
#' \rho_k(x) =
#' \left\{\begin{array}{ll}
#' \frac{1}{2} x^2       & |x|\le k \\
#' k|x|- \frac{1}{2} k^2 & |x|>k
#' \end{array}\right.
#' }{
#' \rho(x, k) = [if abs(x) <= k:] (x^2)/2 [else:] k*abs(x) - (k^2)/2
#' }
#'
#' 
#' @references
#' Huber, P.J. (1964). Robust Estimation of a Location Parameter.
#' Annals of Statistics, 53(1), 73-101.
#' 
#' @references
#' Huber, P.J. (1981). Robust Statistics. Wiley.
#' 
#' @references
#' Schumann, D. (2009). Robust Variable Selection. ProQuest.
#' 
#' @examples 
#' 
#' x <- rhuber(1e5, 5, 2, 3)
#' hist(x, 100, freq = FALSE)
#' curve(dhuber(x, 5, 2, 3), -20, 20, col = "red", add = TRUE, n = 5000)
#' hist(phuber(x, 5, 2, 3))
#' plot(ecdf(x))
#' curve(phuber(x, 5, 2, 3), -20, 20, col = "red", lwd = 2, add = TRUE)
#'
#' @name Huber
#' @aliases Huber
#' @aliases dhuber
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dhuber <- function(x, mu = 0, sigma = 1, epsilon = 1.345, log = FALSE) {
  cpp_dhuber(x, mu, sigma, epsilon, log[1L])
}


#' @rdname Huber
#' @export

phuber <- function(q, mu = 0, sigma = 1, epsilon = 1.345, lower.tail = TRUE, log.p = FALSE) {
  cpp_phuber(q, mu, sigma, epsilon, lower.tail[1L], log.p[1L])
}


#' @rdname Huber
#' @export

qhuber <- function(p, mu = 0, sigma = 1, epsilon = 1.345, lower.tail = TRUE, log.p = FALSE) {
  cpp_qhuber(p, mu, sigma, epsilon, lower.tail[1L], log.p[1L])
}


#' @rdname Huber
#' @export

rhuber <- function(n, mu = 0, sigma = 1, epsilon = 1.345) {
  if (length(n) > 1) n <- length(n)
  cpp_rhuber(n, mu, sigma, epsilon)
}

#' @keywords internal
"_PACKAGE"

#' Additional univariate and multivariate distributions
#'
#' Density, distribution function, quantile function and random
#' generation for a number of univariate and multivariate distributions.
#'
#' @details
#'
#' This package follows naming convention that is consistent with base R,
#' where density (or probability mass) functions, distribution functions,
#' quantile functions and random generation functions names are followed by
#' \code{d}*, \code{p}*, \code{q}*, and \code{r}* prefixes.
#'
#' Behaviour of the functions is consistent with base R, where for
#' not valid parameters values \code{NaN}'s are returned, while
#' for values beyond function support \code{0}'s are returned
#' (e.g. for non-integers in discrete distributions, or for
#' negative values in functions with non-negative support).
#'
#' All the functions vectorized and coded in C++ using \pkg{Rcpp}.
#'
#' @name extraDistr-package
#'
#' @useDynLib extraDistr, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#'
#' @importFrom stats pgamma qgamma rgamma
#' @importFrom stats pnorm qnorm rnorm
#' @importFrom stats rpois
#' @importFrom stats qchisq rchisq
NULL

#' Dirichlet-multinomial (multivariate Polya) distribution
#'
#' Density function, cumulative distribution function and random generation
#' for the Dirichlet-multinomial (multivariate Polya) distribution.
#'
#' @param x               \eqn{k}-column matrix of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param size            numeric vector; number of trials (zero or more).
#' @param alpha           \eqn{k}-values vector or \eqn{k}-column matrix;
#'                        concentration parameter. Must be positive.
#' @param log     	      logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#'
#' If \eqn{(p_1,\dots,p_k) \sim \mathrm{Dirichlet}(\alpha_1,\dots,\alpha_k)}{(p[1],\dots,p[k]) ~ Dirichlet(\alpha[1],\dots,\alpha[k])} and
#' \eqn{(x_1,\dots,x_k) \sim \mathrm{Multinomial}(n, p_1,\dots,p_k)}{(x[1],\dots,x[k]) ~ Multinomial(n, p[1],\dots,p[k])}, then 
#' \eqn{(x_1,\dots,x_k) \sim \mathrm{DirichletMultinomial(n, \alpha_1,\dots,\alpha_k)}}{(x[1],\dots,x[k]) ~ DirichletMultinomial(n, \alpha[1],\dots,\alpha[k])}.
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{\left(n!\right)\Gamma\left(\sum \alpha_k\right)}{\Gamma\left(n+\sum \alpha_k\right)}\prod_{k=1}^K\frac{\Gamma(x_{k}+\alpha_{k})}{\left(x_{k}!\right)\Gamma(\alpha_{k})}
#' }{
#' f(x) = (n! * \Gamma(sum(\alpha[k]))) / (\Gamma(n + sum(\alpha[k]))) * prod((\Gamma(x[k] + \alpha[k])) / (x[k]! * \Gamma(\alpha[k]))
#' }
#' 
#' @seealso \code{\link{Dirichlet}}, \code{\link{Multinomial}}
#'
#' @references 
#' Gentle, J.E. (2006). Random number generation and Monte Carlo methods. Springer.
#'
#' @references
#' Kvam, P. and Day, D. (2001) The multivariate Polya distribution in combat modeling.
#' Naval Research Logistics, 48, 1-17.
#' 
#' @name DirMnom
#' @aliases DirMnom
#' @aliases ddirmnom
#' 
#' @keywords distribution
#' @concept Multivariate
#' @concept Discrete
#' 
#' @export

ddirmnom <- function(x, size, alpha, log = FALSE) {
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  if (is.data.frame(x))
    x <- as.matrix(x)
  else if (is.vector(x))
    x <- matrix(x, byrow = TRUE, nrow = 1)
  cpp_ddirmnom(x, size, alpha, log[1L])
}


#' @rdname DirMnom
#' @export

rdirmnom <- function (n, size, alpha) {
  if (length(n) > 1) n <- length(n)
  if (is.vector(alpha))
    alpha <- matrix(alpha, nrow = 1)
  else if (!is.matrix(alpha))
    alpha <- as.matrix(alpha)
  cpp_rdirmnom(n, size, alpha)
}

#' Wald (inverse Gaussian) distribution
#'
#' Density, distribution function and random generation
#' for the Wald distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu,lambda	      location and shape parameters. Scale must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \sqrt{\frac{\lambda}{2\pi x^3}} \exp\left( \frac{-\lambda(x-\mu)^2}{2\mu^2 x} \right)
#' }{
#' f(x) = sqrt(\lambda/(2*\pi*x^3)) * exp((-\lambda*(x-\mu)^2)/(2*\mu^2*x))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \Phi\left(\sqrt{\frac{\lambda}{x}} \left(\frac{x}{\mu}-1 \right) \right) +
#' \exp\left(\frac{2\lambda}{\mu} \right) \Phi\left(\sqrt{\frac{\lambda}{x}}
#' \left(\frac{x}{\mu}+1 \right) \right)
#' }{
#' F(x) = \Phi(sqrt(\lambda/\mu)*(x/\mu-1)) - exp((2*\lambda)/\mu) *
#' \Phi(sqrt(\lambda/\mu)*(x/\mu+1))
#' }
#' 
#' Random generation is done using the algorithm described by Michael, Schucany and Haas (1976).
#' 
#' @references 
#' 
#' Michael, J.R., Schucany, W.R., and Haas, R.W. (1976).
#' Generating Random Variates Using Transformations with Multiple Roots.
#' The American Statistician, 30(2): 88-90.
#' 
#' @examples 
#' 
#' x <- rwald(1e5, 5, 16)
#' hist(x, 100, freq = FALSE)
#' curve(dwald(x, 5, 16), 0, 50, col = "red", add = TRUE)
#' hist(pwald(x, 5, 16))
#' plot(ecdf(x))
#' curve(pwald(x, 5, 16), 0, 50, col = "red", lwd = 2, add = TRUE)
#'
#' @name Wald
#' @aliases Wald
#' @aliases dwald
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dwald <- function(x, mu, lambda, log = FALSE) {
  cpp_dwald(x, mu, lambda, log[1L])
}


#' @rdname Wald
#' @export

pwald <- function(q, mu, lambda, lower.tail = TRUE, log.p = FALSE) {
  cpp_pwald(q, mu, lambda, lower.tail[1L], log.p[1L])
}


#' @rdname Wald
#' @export

rwald <- function(n, mu, lambda) {
  if (length(n) > 1) n <- length(n)
  cpp_rwald(n, mu, lambda)
}

#' Truncated binomial distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the truncated binomial distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param size	          number of trials (zero or more).
#' @param prob            probability of success on each trial.
#' @param a,b             lower and upper truncation points (\code{a < x <= b}).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @examples 
#' 
#' x <- rtbinom(1e5, 100, 0.83, 76, 86)
#' xx <- seq(0, 100)
#' plot(prop.table(table(x)))
#' lines(xx, dtbinom(xx, 100, 0.83, 76, 86), col = "red")
#' hist(ptbinom(x, 100, 0.83, 76, 86))
#' 
#' xx <- seq(0, 100, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, ptbinom(xx, 100, 0.83, 76, 86), col = "red", lwd = 2)
#' uu <- seq(0, 1, by = 0.001)
#' lines(qtbinom(uu, 100, 0.83, 76, 86), uu, col = "blue", lty = 2)
#'
#' @name TruncBinom
#' @aliases TruncBinom
#' @aliases dtbinom
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dtbinom <- function(x, size, prob, a = -Inf, b = Inf, log = FALSE) {
  cpp_dtbinom(x, size, prob, a, b, log[1L])
}


#' @rdname TruncBinom
#' @export

ptbinom <- function(q, size, prob, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE) {
  cpp_ptbinom(q, size, prob, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname TruncBinom
#' @export

qtbinom <- function(p, size, prob, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE) {
  cpp_qtbinom(p, size, prob, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname TruncBinom
#' @export

rtbinom <- function(n, size, prob, a = -Inf, b = Inf) {
  if (length(n) > 1) n <- length(n)
  cpp_rtbinom(n, size, prob, a, b)
}

#' Shifted Gompertz distribution
#'
#' Density, distribution function, and random generation
#' for the shifted Gompertz distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param b,eta           positive valued scale and shape parameters;
#'                        both need to be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#' 
#' If \eqn{X} follows exponential distribution parametrized by scale \eqn{b} and
#' \eqn{Y} follows reparametrized Gumbel distribution with cumulative distribution function
#' \eqn{F(x) = \exp(-\eta  e^{-bx})}{F(x) = exp(-\eta*exp(-b*x))} parametrized by 
#' scale \eqn{b} and shape \eqn{\eta}, then \eqn{\max(X,Y)}{max(X,Y)} follows shifted
#' Gompertz distribution parametrized by scale \eqn{b>0} and shape \eqn{\eta>0}.
#' The above relation is used by \code{rsgomp} function for random generation from
#' shifted Gompertz distribution.
#'
#' Probability density function
#' \deqn{
#' f(x) = b e^{-bx} \exp(-\eta e^{-bx}) \left[1 + \eta(1 - e^{-bx})\right]
#' }{
#' f(x) = b*exp(-b*x) * exp(-\eta*exp(-b*x)) * (1 + \eta*(1 - exp(-b*x)))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = (1-e^{-bx}) \exp(-\eta e^{-bx})
#' }{
#' F(x) = (1-exp(-b*x)) * exp(-\eta*exp(-b*x))
#' }
#' 
#' @references 
#' Bemmaor, A.C. (1994).
#' Modeling the Diffusion of New Durable Goods: Word-of-Mouth Effect Versus Consumer Heterogeneity.
#' [In:] G. Laurent, G.L. Lilien & B. Pras. Research Traditions in Marketing.
#' Boston: Kluwer Academic Publishers. pp. 201-223.
#' 
#' @references 
#' Jimenez, T.F. and Jodra, P. (2009).
#' A Note on the Moments and Computer Generation of the Shifted Gompertz Distribution.
#' Communications in Statistics - Theory and Methods, 38(1), 78-89.
#' 
#' @references
#' Jimenez T.F. (2014).
#' Estimation of the Parameters of the Shifted Gompertz Distribution,
#' Using Least Squares, Maximum Likelihood and Moments Methods.
#' Journal of Computational and Applied Mathematics, 255(1), 867-877.
#' 
#' @examples 
#' 
#' x <- rsgomp(1e5, 0.4, 1)
#' hist(x, 50, freq = FALSE)
#' curve(dsgomp(x, 0.4, 1), 0, 30, col = "red", add = TRUE)
#' hist(psgomp(x, 0.4, 1))
#' plot(ecdf(x))
#' curve(psgomp(x, 0.4, 1), 0, 30, col = "red", lwd = 2, add = TRUE)
#'
#' @name ShiftGomp
#' @aliases ShiftGomp
#' @aliases dsgomp
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dsgomp <- function(x, b, eta, log = FALSE) {
  cpp_dsgomp(x, b, eta, log[1L])
}


#' @rdname ShiftGomp
#' @export

psgomp <- function(q, b, eta, lower.tail = TRUE, log.p = FALSE) {
  cpp_psgomp(q, b, eta, lower.tail[1L], log.p[1L])
}


#' @rdname ShiftGomp
#' @export

rsgomp <- function(n, b, eta) {
  if (length(n) > 1) n <- length(n)
  cpp_rsgomp(n, b, eta)
}

#' Frechet distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Frechet distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param lambda,sigma,mu shape, scale, and location parameters.
#'                        Scale and shape must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{\lambda}{\sigma} \left(\frac{x-\mu}{\sigma}\right)^{-1-\lambda} \exp\left(-\left(\frac{x-\mu}{\sigma}\right)^{-\lambda}\right)
#' }{
#' f(x) = \lambda/\sigma * ((x-\mu)/\sigma)^(-1-\lambda) * exp(-((x-\mu)/\sigma)^-\lambda)
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \exp\left(-\left(\frac{x-\mu}{\sigma}\right)^{-\lambda}\right)
#' }{
#' F(x) = exp(-((x-\mu)/\sigma)^-\lambda)
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \mu + \sigma -\log(p)^{-1/\lambda}
#' }{
#' F^-1(p) = \mu + \sigma * -log(p)^{-1/\lambda}
#' }
#'
#' @references
#' Bury, K. (1999). Statistical Distributions in Engineering.
#' Cambridge University Press.
#' 
#' @examples 
#' 
#' x <- rfrechet(1e5, 5, 2, 1.5)
#' xx <- seq(0, 1000, by = 0.1)
#' hist(x, 200, freq = FALSE)
#' lines(xx, dfrechet(xx, 5, 2, 1.5), col = "red") 
#' hist(pfrechet(x, 5, 2, 1.5))
#' plot(ecdf(x))
#' lines(xx, pfrechet(xx, 5, 2, 1.5), col = "red", lwd = 2)
#'
#' @name Frechet
#' @aliases Frechet
#' @aliases dfrechet
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dfrechet <- function(x, lambda = 1, mu = 0, sigma = 1, log = FALSE) {
  cpp_dfrechet(x, lambda, mu, sigma, log[1L])
}


#' @rdname Frechet
#' @export

pfrechet <- function(q, lambda = 1, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pfrechet(q, lambda, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Frechet
#' @export

qfrechet <- function(p, lambda = 1, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qfrechet(p, lambda, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Frechet
#' @export

rfrechet <- function(n, lambda = 1, mu = 0, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rfrechet(n, lambda, mu, sigma)
}

#' Gumbel distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Gumbel distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu,sigma        location and scale parameters. Scale must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{1}{\sigma} \exp\left(-\left(\frac{x-\mu}{\sigma} + \exp\left(-\frac{x-\mu}{\sigma}\right)\right)\right)
#' }{
#' f(x) = 1/\sigma * exp(-((x-\mu)/\sigma + exp(-(x-\mu)/\sigma)))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \exp\left(-\exp\left(-\frac{x-\mu}{\sigma}\right)\right)
#' }{
#' F(x) = exp(-exp(-(x-\mu)/\sigma))
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \mu - \sigma \log(-\log(p))
#' }{
#' F^-1(p) = \mu - \sigma * log(-log(p))
#' }
#'
#' @references
#' Bury, K. (1999). Statistical Distributions in Engineering.
#' Cambridge University Press.
#' 
#' @examples 
#' 
#' x <- rgumbel(1e5, 5, 2)
#' hist(x, 100, freq = FALSE)
#' curve(dgumbel(x, 5, 2), 0, 25, col = "red", add = TRUE)
#' hist(pgumbel(x, 5, 2))
#' plot(ecdf(x))
#' curve(pgumbel(x, 5, 2), 0, 25, col = "red", lwd = 2, add = TRUE)
#'
#' @name Gumbel
#' @aliases Gumbel
#' @aliases dgumbel
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dgumbel <- function(x, mu = 0, sigma = 1, log = FALSE) {
  cpp_dgumbel(x, mu, sigma, log[1L])
}


#' @rdname Gumbel
#' @export

pgumbel <- function(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pgumbel(q, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Gumbel
#' @export

qgumbel <- function(p, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qgumbel(p, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Gumbel
#' @export

rgumbel <- function(n, mu = 0, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rgumbel(n, mu, sigma)
}

#' Laplace distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Laplace distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu,sigma	      location and scale parameters. Scale must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{1}{2\sigma} \exp\left(-\left|\frac{x-\mu}{\sigma}\right|\right)
#' }{
#' f(x) = 1/(2*\sigma) * exp(-|(x-\mu)/\sigma|)
#' }
#'
#' Cumulative distribution function
#' \deqn{ F(x) = \left\{\begin{array}{ll}
#' \frac{1}{2} \exp\left(\frac{x-\mu}{\sigma}\right)     & x < \mu \\
#' 1 - \frac{1}{2} \exp\left(\frac{x-\mu}{\sigma}\right) & x \geq \mu
#' \end{array}\right.
#' }{
#' F(x) = [if x < mu:] 1/2 * exp((x-\mu)/\sigma)
#'        [else:] 1 - 1/2 * exp((x-\mu)/\sigma)
#' }
#'
#' Quantile function
#' \deqn{ F^{-1}(p) = \left\{\begin{array}{ll}
#' \mu + \sigma \log(2p)     & p < 0.5 \\
#' \mu - \sigma \log(2(1-p)) & p \geq 0.5
#' \end{array}\right.
#' }{
#' F^-1(p) = [if p < 0.5:] \mu + \sigma * log(2*p)
#'           [else:] \mu - \sigma * log(2*(1-p))
#' }
#'
#' @references
#' Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
#' with Applications. Chapman & Hall/CRC
#'
#' @references
#' Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
#' Statistical Distributions. John Wiley & Sons.
#' 
#' @examples 
#' 
#' x <- rlaplace(1e5, 5, 16)
#' hist(x, 100, freq = FALSE)
#' curve(dlaplace(x, 5, 16), -200, 200, n = 500, col = "red", add = TRUE)
#' hist(plaplace(x, 5, 16))
#' plot(ecdf(x))
#' curve(plaplace(x, 5, 16), -200, 200, n = 500, col = "red", lwd = 2, add = TRUE)
#'
#' @name Laplace
#' @aliases Laplace
#' @aliases dlaplace
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dlaplace <- function(x, mu = 0, sigma = 1, log = FALSE) {
  cpp_dlaplace(x, mu, sigma, log[1L])
}


#' @rdname Laplace
#' @export

plaplace <- function(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_plaplace(q, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Laplace
#' @export

qlaplace <- function(p, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qlaplace(p, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Laplace
#' @export

rlaplace <- function(n, mu = 0, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rlaplace(n, mu, sigma)
}

#' Truncated Poisson distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the truncated Poisson distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param lambda	        vector of (non-negative) means.
#' @param a,b             lower and upper truncation points (\code{a < x <= b}).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @references
#' Plackett, R.L. (1953). The truncated Poisson distribution.
#' Biometrics, 9(4), 485-488.
#' 
#' @references 
#' Singh, J. (1978). A characterization of positive Poisson distribution and
#' its statistical application.
#' SIAM Journal on Applied Mathematics, 34(3), 545-548.
#' 
#' @references 
#' Dalgaard, P. (May 1, 2005). [R] simulate zero-truncated Poisson distribution.
#' R-help mailing list.
#' \url{https://stat.ethz.ch/pipermail/r-help/2005-May/070680.html}
#' 
#' @examples 
#' 
#' x <- rtpois(1e5, 14, 16)
#' xx <- seq(-1, 50)
#' plot(prop.table(table(x)))
#' lines(xx, dtpois(xx, 14, 16), col = "red")
#' hist(ptpois(x, 14, 16))
#' 
#' xx <- seq(0, 50, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, ptpois(xx, 14, 16), col = "red", lwd = 2)
#' 
#' uu <- seq(0, 1, by = 0.001)
#' lines(qtpois(uu, 14, 16), uu, col = "blue", lty = 2)
#' 
#' # Zero-truncated Poisson
#' 
#' x <- rtpois(1e5, 5, 0)
#' xx <- seq(-1, 50)
#' plot(prop.table(table(x)))
#' lines(xx, dtpois(xx, 5, 0), col = "red")
#' hist(ptpois(x, 5, 0))
#' 
#' xx <- seq(0, 50, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, ptpois(xx, 5, 0), col = "red", lwd = 2)
#' lines(qtpois(uu, 5, 0), uu, col = "blue", lty = 2)
#'
#' @name TruncPoisson
#' @aliases TruncPoisson
#' @aliases dtpois
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dtpois <- function(x, lambda, a = -Inf, b = Inf, log = FALSE) {
  cpp_dtpois(x, lambda, a, b, log)
}


#' @rdname TruncPoisson
#' @export

ptpois <- function(q, lambda, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE) {
  cpp_ptpois(q, lambda, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname TruncPoisson
#' @export

qtpois <- function(p, lambda, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE) {
  cpp_qtpois(p, lambda, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname TruncPoisson
#' @export

rtpois <- function(n, lambda, a = -Inf, b = Inf) {
  if (length(n) > 1) n <- length(n)
  cpp_rtpois(n, lambda, a, b)
}

#' Zero-inflated Poisson distribution
#'
#' Probability mass function and random generation
#' for the zero-inflated Poisson distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param lambda	        vector of (non-negative) means.
#' @param pi              probability of extra zeros.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \left\{\begin{array}{ll}
#' \pi + (1 - \pi) e^{-\lambda} & x = 0 \\
#' (1 - \pi) \frac{\lambda^{x} e^{-\lambda}} {x!} & x > 0 \\
#' \end{array}\right.
#' }{
#' f(x) = [if x = 0:] \pi + (1-\pi) * exp(-\lambda) [else:] (1-\pi) * dpois(x, lambda)
#' }
#' 
#' @seealso \code{\link[stats]{Poisson}}
#' 
#' @examples 
#' 
#' x <- rzip(1e5, 6, 0.33)
#' xx <- -2:20
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, dzip(xx, 6, 0.33), col = "red")
#' 
#' xx <- seq(0, 20, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pzip(xx, 6, 0.33), col = "red")
#'
#' @name ZIP
#' @aliases ZIP
#' @aliases dzip
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dzip <- function(x, lambda, pi, log = FALSE) {
  cpp_dzip(x, lambda, pi, log[1L])
}


#' @rdname ZIP
#' @export

pzip <- function(q, lambda, pi, lower.tail = TRUE, log.p = FALSE) {
  cpp_pzip(q, lambda, pi, lower.tail[1L], log.p[1L])
}


#' @rdname ZIP
#' @export

qzip <- function(p, lambda, pi, lower.tail = TRUE, log.p = FALSE) {
  cpp_qzip(p, lambda, pi, lower.tail[1L], log.p[1L])
}


#' @rdname ZIP
#' @export

rzip <- function(n, lambda, pi) {
  if (length(n) > 1) n <- length(n)
  cpp_rzip(n, lambda, pi)
}

#' Zero-inflated binomial distribution
#'
#' Probability mass function and random generation
#' for the zero-inflated binomial distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param size	          number of trials (zero or more).
#' @param prob            probability of success in each trial. \code{0 < prob <= 1}.
#' @param pi              probability of extra zeros.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \left\{\begin{array}{ll}
#' \pi + (1 - \pi) (1-p)^n & x = 0 \\
#' (1 - \pi) {n \choose x} p^x (1-p)^{n-x} & x > 0 \\
#' \end{array}\right.
#' }{
#' f(x) = [if x = 0:] (1-\pi)+\pi * p^r [else:] (1-\pi) * dnbinom(x, size, prob)
#' }
#' 
#' @seealso \code{\link[stats]{Binomial}}
#' 
#' @examples 
#' 
#' x <- rzib(1e5, 10, 0.6, 0.33)
#' xx <- -2:20
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, dzib(xx, 10, 0.6, 0.33), col = "red")
#' 
#' xx <- seq(0, 20, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pzib(xx, 10, 0.6, 0.33), col = "red")
#'
#' @name ZIB
#' @aliases ZIB
#' @aliases dzib
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dzib <- function(x, size, prob, pi, log = FALSE) {
  cpp_dzib(x, size, prob, pi, log[1L])
}


#' @rdname ZIB
#' @export

pzib <- function(q, size, prob, pi, lower.tail = TRUE, log.p = FALSE) {
  cpp_pzib(q, size, prob, pi, lower.tail[1L], log.p[1L])
}


#' @rdname ZIB
#' @export

qzib <- function(p, size, prob, pi, lower.tail = TRUE, log.p = FALSE) {
  cpp_qzib(p, size, prob, pi, lower.tail[1L], log.p[1L])
}


#' @rdname ZIB
#' @export

rzib <- function(n, size, prob, pi) {
  if (length(n) > 1) n <- length(n)
  cpp_rzib(n, size, prob, pi)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_dbern <- function(x, prob, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbern`, x, prob, log_prob)
}

cpp_pbern <- function(x, prob, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pbern`, x, prob, lower_tail, log_prob)
}

cpp_qbern <- function(p, prob, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qbern`, p, prob, lower_tail, log_prob)
}

cpp_rbern <- function(n, prob) {
    .Call(`_extraDistr_cpp_rbern`, n, prob)
}

cpp_dbbinom <- function(x, size, alpha, beta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbbinom`, x, size, alpha, beta, log_prob)
}

cpp_pbbinom <- function(x, size, alpha, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pbbinom`, x, size, alpha, beta, lower_tail, log_prob)
}

cpp_rbbinom <- function(n, size, alpha, beta) {
    .Call(`_extraDistr_cpp_rbbinom`, n, size, alpha, beta)
}

cpp_dbnbinom <- function(x, size, alpha, beta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbnbinom`, x, size, alpha, beta, log_prob)
}

cpp_pbnbinom <- function(x, size, alpha, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pbnbinom`, x, size, alpha, beta, lower_tail, log_prob)
}

cpp_rbnbinom <- function(n, size, alpha, beta) {
    .Call(`_extraDistr_cpp_rbnbinom`, n, size, alpha, beta)
}

cpp_dbetapr <- function(x, alpha, beta, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbetapr`, x, alpha, beta, sigma, log_prob)
}

cpp_pbetapr <- function(x, alpha, beta, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pbetapr`, x, alpha, beta, sigma, lower_tail, log_prob)
}

cpp_qbetapr <- function(p, alpha, beta, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qbetapr`, p, alpha, beta, sigma, lower_tail, log_prob)
}

cpp_rbetapr <- function(n, alpha, beta, sigma) {
    .Call(`_extraDistr_cpp_rbetapr`, n, alpha, beta, sigma)
}

cpp_dbhatt <- function(x, mu, sigma, a, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbhatt`, x, mu, sigma, a, log_prob)
}

cpp_pbhatt <- function(x, mu, sigma, a, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pbhatt`, x, mu, sigma, a, lower_tail, log_prob)
}

cpp_rbhatt <- function(n, mu, sigma, a) {
    .Call(`_extraDistr_cpp_rbhatt`, n, mu, sigma, a)
}

cpp_dfatigue <- function(x, alpha, beta, mu, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dfatigue`, x, alpha, beta, mu, log_prob)
}

cpp_pfatigue <- function(x, alpha, beta, mu, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pfatigue`, x, alpha, beta, mu, lower_tail, log_prob)
}

cpp_qfatigue <- function(p, alpha, beta, mu, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qfatigue`, p, alpha, beta, mu, lower_tail, log_prob)
}

cpp_rfatigue <- function(n, alpha, beta, mu) {
    .Call(`_extraDistr_cpp_rfatigue`, n, alpha, beta, mu)
}

cpp_dbnorm <- function(x, y, mu1, mu2, sigma1, sigma2, rho, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbnorm`, x, y, mu1, mu2, sigma1, sigma2, rho, log_prob)
}

cpp_rbnorm <- function(n, mu1, mu2, sigma1, sigma2, rho) {
    .Call(`_extraDistr_cpp_rbnorm`, n, mu1, mu2, sigma1, sigma2, rho)
}

cpp_dbpois <- function(x, y, a, b, c, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dbpois`, x, y, a, b, c, log_prob)
}

cpp_rbpois <- function(n, a, b, c) {
    .Call(`_extraDistr_cpp_rbpois`, n, a, b, c)
}

cpp_rcatlp <- function(n, log_prob) {
    .Call(`_extraDistr_cpp_rcatlp`, n, log_prob)
}

cpp_dcat <- function(x, prob, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dcat`, x, prob, log_prob)
}

cpp_pcat <- function(x, prob, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pcat`, x, prob, lower_tail, log_prob)
}

cpp_qcat <- function(p, prob, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qcat`, p, prob, lower_tail, log_prob)
}

cpp_rcat <- function(n, prob) {
    .Call(`_extraDistr_cpp_rcat`, n, prob)
}

cpp_ddirichlet <- function(x, alpha, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddirichlet`, x, alpha, log_prob)
}

cpp_rdirichlet <- function(n, alpha) {
    .Call(`_extraDistr_cpp_rdirichlet`, n, alpha)
}

cpp_ddirmnom <- function(x, size, alpha, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddirmnom`, x, size, alpha, log_prob)
}

cpp_rdirmnom <- function(n, size, alpha) {
    .Call(`_extraDistr_cpp_rdirmnom`, n, size, alpha)
}

cpp_ddgamma <- function(x, shape, scale, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddgamma`, x, shape, scale, log_prob)
}

cpp_ddlaplace <- function(x, location, scale, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddlaplace`, x, location, scale, log_prob)
}

cpp_pdlaplace <- function(x, location, scale, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pdlaplace`, x, location, scale, lower_tail, log_prob)
}

cpp_rdlaplace <- function(n, location, scale) {
    .Call(`_extraDistr_cpp_rdlaplace`, n, location, scale)
}

cpp_ddnorm <- function(x, mu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddnorm`, x, mu, sigma, log_prob)
}

cpp_ddunif <- function(x, min, max, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddunif`, x, min, max, log_prob)
}

cpp_pdunif <- function(x, min, max, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pdunif`, x, min, max, lower_tail, log_prob)
}

cpp_qdunif <- function(p, min, max, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qdunif`, p, min, max, lower_tail, log_prob)
}

cpp_rdunif <- function(n, min, max) {
    .Call(`_extraDistr_cpp_rdunif`, n, min, max)
}

cpp_ddweibull <- function(x, q, beta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ddweibull`, x, q, beta, log_prob)
}

cpp_pdweibull <- function(x, q, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pdweibull`, x, q, beta, lower_tail, log_prob)
}

cpp_qdweibull <- function(p, q, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qdweibull`, p, q, beta, lower_tail, log_prob)
}

cpp_rdweibull <- function(n, q, beta) {
    .Call(`_extraDistr_cpp_rdweibull`, n, q, beta)
}

cpp_dfrechet <- function(x, lambda, mu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dfrechet`, x, lambda, mu, sigma, log_prob)
}

cpp_pfrechet <- function(x, lambda, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pfrechet`, x, lambda, mu, sigma, lower_tail, log_prob)
}

cpp_qfrechet <- function(p, lambda, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qfrechet`, p, lambda, mu, sigma, lower_tail, log_prob)
}

cpp_rfrechet <- function(n, lambda, mu, sigma) {
    .Call(`_extraDistr_cpp_rfrechet`, n, lambda, mu, sigma)
}

cpp_dgpois <- function(x, alpha, beta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dgpois`, x, alpha, beta, log_prob)
}

cpp_pgpois <- function(x, alpha, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pgpois`, x, alpha, beta, lower_tail, log_prob)
}

cpp_rgpois <- function(n, alpha, beta) {
    .Call(`_extraDistr_cpp_rgpois`, n, alpha, beta)
}

cpp_dgev <- function(x, mu, sigma, xi, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dgev`, x, mu, sigma, xi, log_prob)
}

cpp_pgev <- function(x, mu, sigma, xi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pgev`, x, mu, sigma, xi, lower_tail, log_prob)
}

cpp_qgev <- function(p, mu, sigma, xi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qgev`, p, mu, sigma, xi, lower_tail, log_prob)
}

cpp_rgev <- function(n, mu, sigma, xi) {
    .Call(`_extraDistr_cpp_rgev`, n, mu, sigma, xi)
}

cpp_dgompertz <- function(x, a, b, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dgompertz`, x, a, b, log_prob)
}

cpp_pgompertz <- function(x, a, b, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pgompertz`, x, a, b, lower_tail, log_prob)
}

cpp_qgompertz <- function(p, a, b, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qgompertz`, p, a, b, lower_tail, log_prob)
}

cpp_rgompertz <- function(n, a, b) {
    .Call(`_extraDistr_cpp_rgompertz`, n, a, b)
}

cpp_dgpd <- function(x, mu, sigma, xi, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dgpd`, x, mu, sigma, xi, log_prob)
}

cpp_pgpd <- function(x, mu, sigma, xi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pgpd`, x, mu, sigma, xi, lower_tail, log_prob)
}

cpp_qgpd <- function(p, mu, sigma, xi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qgpd`, p, mu, sigma, xi, lower_tail, log_prob)
}

cpp_rgpd <- function(n, mu, sigma, xi) {
    .Call(`_extraDistr_cpp_rgpd`, n, mu, sigma, xi)
}

cpp_dgumbel <- function(x, mu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dgumbel`, x, mu, sigma, log_prob)
}

cpp_pgumbel <- function(x, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pgumbel`, x, mu, sigma, lower_tail, log_prob)
}

cpp_qgumbel <- function(p, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qgumbel`, p, mu, sigma, lower_tail, log_prob)
}

cpp_rgumbel <- function(n, mu, sigma) {
    .Call(`_extraDistr_cpp_rgumbel`, n, mu, sigma)
}

cpp_dhcauchy <- function(x, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dhcauchy`, x, sigma, log_prob)
}

cpp_phcauchy <- function(x, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_phcauchy`, x, sigma, lower_tail, log_prob)
}

cpp_qhcauchy <- function(p, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qhcauchy`, p, sigma, lower_tail, log_prob)
}

cpp_rhcauchy <- function(n, sigma) {
    .Call(`_extraDistr_cpp_rhcauchy`, n, sigma)
}

cpp_dhnorm <- function(x, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dhnorm`, x, sigma, log_prob)
}

cpp_phnorm <- function(x, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_phnorm`, x, sigma, lower_tail, log_prob)
}

cpp_qhnorm <- function(p, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qhnorm`, p, sigma, lower_tail, log_prob)
}

cpp_rhnorm <- function(n, sigma) {
    .Call(`_extraDistr_cpp_rhnorm`, n, sigma)
}

cpp_dht <- function(x, nu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dht`, x, nu, sigma, log_prob)
}

cpp_pht <- function(x, nu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pht`, x, nu, sigma, lower_tail, log_prob)
}

cpp_qht <- function(p, nu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qht`, p, nu, sigma, lower_tail, log_prob)
}

cpp_rht <- function(n, nu, sigma) {
    .Call(`_extraDistr_cpp_rht`, n, nu, sigma)
}

cpp_dhuber <- function(x, mu, sigma, epsilon, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dhuber`, x, mu, sigma, epsilon, log_prob)
}

cpp_phuber <- function(x, mu, sigma, epsilon, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_phuber`, x, mu, sigma, epsilon, lower_tail, log_prob)
}

cpp_qhuber <- function(p, mu, sigma, epsilon, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qhuber`, p, mu, sigma, epsilon, lower_tail, log_prob)
}

cpp_rhuber <- function(n, mu, sigma, epsilon) {
    .Call(`_extraDistr_cpp_rhuber`, n, mu, sigma, epsilon)
}

cpp_dinvgamma <- function(x, alpha, beta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dinvgamma`, x, alpha, beta, log_prob)
}

cpp_pinvgamma <- function(x, alpha, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pinvgamma`, x, alpha, beta, lower_tail, log_prob)
}

cpp_dkumar <- function(x, a, b, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dkumar`, x, a, b, log_prob)
}

cpp_pkumar <- function(x, a, b, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pkumar`, x, a, b, lower_tail, log_prob)
}

cpp_qkumar <- function(p, a, b, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qkumar`, p, a, b, lower_tail, log_prob)
}

cpp_rkumar <- function(n, a, b) {
    .Call(`_extraDistr_cpp_rkumar`, n, a, b)
}

cpp_dlaplace <- function(x, mu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dlaplace`, x, mu, sigma, log_prob)
}

cpp_plaplace <- function(x, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_plaplace`, x, mu, sigma, lower_tail, log_prob)
}

cpp_qlaplace <- function(p, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qlaplace`, p, mu, sigma, lower_tail, log_prob)
}

cpp_rlaplace <- function(n, mu, sigma) {
    .Call(`_extraDistr_cpp_rlaplace`, n, mu, sigma)
}

cpp_dlst <- function(x, nu, mu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dlst`, x, nu, mu, sigma, log_prob)
}

cpp_plst <- function(x, nu, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_plst`, x, nu, mu, sigma, lower_tail, log_prob)
}

cpp_qlst <- function(p, nu, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qlst`, p, nu, mu, sigma, lower_tail, log_prob)
}

cpp_rlst <- function(n, nu, mu, sigma) {
    .Call(`_extraDistr_cpp_rlst`, n, nu, mu, sigma)
}

cpp_dlgser <- function(x, theta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dlgser`, x, theta, log_prob)
}

cpp_plgser <- function(x, theta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_plgser`, x, theta, lower_tail, log_prob)
}

cpp_qlgser <- function(p, theta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qlgser`, p, theta, lower_tail, log_prob)
}

cpp_rlgser <- function(n, theta) {
    .Call(`_extraDistr_cpp_rlgser`, n, theta)
}

cpp_dlomax <- function(x, lambda, kappa, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dlomax`, x, lambda, kappa, log_prob)
}

cpp_plomax <- function(x, lambda, kappa, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_plomax`, x, lambda, kappa, lower_tail, log_prob)
}

cpp_qlomax <- function(p, lambda, kappa, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qlomax`, p, lambda, kappa, lower_tail, log_prob)
}

cpp_rlomax <- function(n, lambda, kappa) {
    .Call(`_extraDistr_cpp_rlomax`, n, lambda, kappa)
}

cpp_dmixnorm <- function(x, mu, sigma, alpha, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dmixnorm`, x, mu, sigma, alpha, log_prob)
}

cpp_pmixnorm <- function(x, mu, sigma, alpha, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pmixnorm`, x, mu, sigma, alpha, lower_tail, log_prob)
}

cpp_rmixnorm <- function(n, mu, sigma, alpha) {
    .Call(`_extraDistr_cpp_rmixnorm`, n, mu, sigma, alpha)
}

cpp_dmixpois <- function(x, lambda, alpha, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dmixpois`, x, lambda, alpha, log_prob)
}

cpp_pmixpois <- function(x, lambda, alpha, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pmixpois`, x, lambda, alpha, lower_tail, log_prob)
}

cpp_rmixpois <- function(n, lambda, alpha) {
    .Call(`_extraDistr_cpp_rmixpois`, n, lambda, alpha)
}

cpp_dmnom <- function(x, size, prob, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dmnom`, x, size, prob, log_prob)
}

cpp_rmnom <- function(n, size, prob) {
    .Call(`_extraDistr_cpp_rmnom`, n, size, prob)
}

cpp_dmvhyper <- function(x, n, k, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dmvhyper`, x, n, k, log_prob)
}

cpp_rmvhyper <- function(nn, n, k) {
    .Call(`_extraDistr_cpp_rmvhyper`, nn, n, k)
}

cpp_dnhyper <- function(x, n, m, r, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dnhyper`, x, n, m, r, log_prob)
}

cpp_pnhyper <- function(x, n, m, r, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pnhyper`, x, n, m, r, lower_tail, log_prob)
}

cpp_qnhyper <- function(p, n, m, r, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qnhyper`, p, n, m, r, lower_tail, log_prob)
}

cpp_rnhyper <- function(nn, n, m, r) {
    .Call(`_extraDistr_cpp_rnhyper`, nn, n, m, r)
}

cpp_dnsbeta <- function(x, alpha, beta, lower, upper, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dnsbeta`, x, alpha, beta, lower, upper, log_prob)
}

cpp_pnsbeta <- function(x, alpha, beta, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pnsbeta`, x, alpha, beta, lower, upper, lower_tail, log_prob)
}

cpp_qnsbeta <- function(p, alpha, beta, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qnsbeta`, p, alpha, beta, lower, upper, lower_tail, log_prob)
}

cpp_rnsbeta <- function(n, alpha, beta, lower, upper) {
    .Call(`_extraDistr_cpp_rnsbeta`, n, alpha, beta, lower, upper)
}

cpp_dpareto <- function(x, a, b, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dpareto`, x, a, b, log_prob)
}

cpp_ppareto <- function(x, a, b, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ppareto`, x, a, b, lower_tail, log_prob)
}

cpp_qpareto <- function(p, a, b, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qpareto`, p, a, b, lower_tail, log_prob)
}

cpp_rpareto <- function(n, a, b) {
    .Call(`_extraDistr_cpp_rpareto`, n, a, b)
}

cpp_dpower <- function(x, alpha, beta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dpower`, x, alpha, beta, log_prob)
}

cpp_ppower <- function(x, alpha, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ppower`, x, alpha, beta, lower_tail, log_prob)
}

cpp_qpower <- function(p, alpha, beta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qpower`, p, alpha, beta, lower_tail, log_prob)
}

cpp_rpower <- function(n, alpha, beta) {
    .Call(`_extraDistr_cpp_rpower`, n, alpha, beta)
}

cpp_dprop <- function(x, size, mean, prior, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dprop`, x, size, mean, prior, log_prob)
}

cpp_pprop <- function(x, size, mean, prior, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pprop`, x, size, mean, prior, lower_tail, log_prob)
}

cpp_qprop <- function(p, size, mean, prior, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qprop`, p, size, mean, prior, lower_tail, log_prob)
}

cpp_rprop <- function(n, size, mean, prior) {
    .Call(`_extraDistr_cpp_rprop`, n, size, mean, prior)
}

cpp_rsign <- function(n) {
    .Call(`_extraDistr_cpp_rsign`, n)
}

cpp_drayleigh <- function(x, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_drayleigh`, x, sigma, log_prob)
}

cpp_prayleigh <- function(x, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_prayleigh`, x, sigma, lower_tail, log_prob)
}

cpp_qrayleigh <- function(p, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qrayleigh`, p, sigma, lower_tail, log_prob)
}

cpp_rrayleigh <- function(n, sigma) {
    .Call(`_extraDistr_cpp_rrayleigh`, n, sigma)
}

cpp_dsgomp <- function(x, b, eta, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dsgomp`, x, b, eta, log_prob)
}

cpp_psgomp <- function(x, b, eta, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_psgomp`, x, b, eta, lower_tail, log_prob)
}

cpp_rsgomp <- function(n, b, eta) {
    .Call(`_extraDistr_cpp_rsgomp`, n, b, eta)
}

cpp_dskellam <- function(x, mu1, mu2, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dskellam`, x, mu1, mu2, log_prob)
}

cpp_rskellam <- function(n, mu1, mu2) {
    .Call(`_extraDistr_cpp_rskellam`, n, mu1, mu2)
}

cpp_dslash <- function(x, mu, sigma, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dslash`, x, mu, sigma, log_prob)
}

cpp_pslash <- function(x, mu, sigma, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pslash`, x, mu, sigma, lower_tail, log_prob)
}

cpp_rslash <- function(n, mu, sigma) {
    .Call(`_extraDistr_cpp_rslash`, n, mu, sigma)
}

cpp_dtriang <- function(x, a, b, c, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dtriang`, x, a, b, c, log_prob)
}

cpp_ptriang <- function(x, a, b, c, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ptriang`, x, a, b, c, lower_tail, log_prob)
}

cpp_qtriang <- function(p, a, b, c, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qtriang`, p, a, b, c, lower_tail, log_prob)
}

cpp_rtriang <- function(n, a, b, c) {
    .Call(`_extraDistr_cpp_rtriang`, n, a, b, c)
}

cpp_dtbinom <- function(x, size, prob, lower, upper, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dtbinom`, x, size, prob, lower, upper, log_prob)
}

cpp_ptbinom <- function(x, size, prob, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ptbinom`, x, size, prob, lower, upper, lower_tail, log_prob)
}

cpp_qtbinom <- function(p, size, prob, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qtbinom`, p, size, prob, lower, upper, lower_tail, log_prob)
}

cpp_rtbinom <- function(n, size, prob, lower, upper) {
    .Call(`_extraDistr_cpp_rtbinom`, n, size, prob, lower, upper)
}

cpp_dtnorm <- function(x, mu, sigma, lower, upper, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dtnorm`, x, mu, sigma, lower, upper, log_prob)
}

cpp_ptnorm <- function(x, mu, sigma, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ptnorm`, x, mu, sigma, lower, upper, lower_tail, log_prob)
}

cpp_qtnorm <- function(p, mu, sigma, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qtnorm`, p, mu, sigma, lower, upper, lower_tail, log_prob)
}

cpp_rtnorm <- function(n, mu, sigma, lower, upper) {
    .Call(`_extraDistr_cpp_rtnorm`, n, mu, sigma, lower, upper)
}

cpp_dtpois <- function(x, lambda, lower, upper, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dtpois`, x, lambda, lower, upper, log_prob)
}

cpp_ptpois <- function(x, lambda, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_ptpois`, x, lambda, lower, upper, lower_tail, log_prob)
}

cpp_qtpois <- function(p, lambda, lower, upper, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qtpois`, p, lambda, lower, upper, lower_tail, log_prob)
}

cpp_rtpois <- function(n, lambda, lower, upper) {
    .Call(`_extraDistr_cpp_rtpois`, n, lambda, lower, upper)
}

cpp_qtlambda <- function(p, lambda, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qtlambda`, p, lambda, lower_tail, log_prob)
}

cpp_rtlambda <- function(n, lambda) {
    .Call(`_extraDistr_cpp_rtlambda`, n, lambda)
}

cpp_dwald <- function(x, mu, lambda, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dwald`, x, mu, lambda, log_prob)
}

cpp_pwald <- function(x, mu, lambda, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pwald`, x, mu, lambda, lower_tail, log_prob)
}

cpp_rwald <- function(n, mu, lambda) {
    .Call(`_extraDistr_cpp_rwald`, n, mu, lambda)
}

cpp_dzib <- function(x, size, prob, pi, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dzib`, x, size, prob, pi, log_prob)
}

cpp_pzib <- function(x, size, prob, pi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pzib`, x, size, prob, pi, lower_tail, log_prob)
}

cpp_qzib <- function(p, size, prob, pi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qzib`, p, size, prob, pi, lower_tail, log_prob)
}

cpp_rzib <- function(n, size, prob, pi) {
    .Call(`_extraDistr_cpp_rzib`, n, size, prob, pi)
}

cpp_dzinb <- function(x, size, prob, pi, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dzinb`, x, size, prob, pi, log_prob)
}

cpp_pzinb <- function(x, size, prob, pi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pzinb`, x, size, prob, pi, lower_tail, log_prob)
}

cpp_qzinb <- function(p, size, prob, pi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qzinb`, p, size, prob, pi, lower_tail, log_prob)
}

cpp_rzinb <- function(n, size, prob, pi) {
    .Call(`_extraDistr_cpp_rzinb`, n, size, prob, pi)
}

cpp_dzip <- function(x, lambda, pi, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_dzip`, x, lambda, pi, log_prob)
}

cpp_pzip <- function(x, lambda, pi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_pzip`, x, lambda, pi, lower_tail, log_prob)
}

cpp_qzip <- function(p, lambda, pi, lower_tail = TRUE, log_prob = FALSE) {
    .Call(`_extraDistr_cpp_qzip`, p, lambda, pi, lower_tail, log_prob)
}

cpp_rzip <- function(n, lambda, pi) {
    .Call(`_extraDistr_cpp_rzip`, n, lambda, pi)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_extraDistr_RcppExport_registerCCallable`)
})

#' Logarithmic series distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the logarithmic series distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param theta           vector; concentration parameter; (\code{0 < theta < 1}).
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability mass function
#' \deqn{
#' f(x) = \frac{-1}{\log(1-\theta)} \frac{\theta^x}{x}
#' }{
#' f(x) = (-1/log(1-\theta)*\theta^x) / x
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \frac{-1}{\log(1-\theta)} \sum_{k=1}^x \frac{\theta^x}{x}
#' }{
#' F(x) = -1/log(1-\theta) * sum((\theta^x)/x)
#' }
#'
#' Quantile function and random generation are computed using
#' algorithm described in Krishnamoorthy (2006).
#'
#' @references
#' Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
#' with Applications. Chapman & Hall/CRC
#'
#' @references
#' Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
#' Statistical Distributions. John Wiley & Sons.
#' 
#' @examples 
#' 
#' x <- rlgser(1e5, 0.66)
#' xx <- seq(0, 100, by = 1)
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, dlgser(xx, 0.66), col = "red")
#' 
#' # Notice: distribution of F(X) is far from uniform:
#' hist(plgser(x, 0.66), 50)
#' 
#' xx <- seq(0, 100, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, plgser(xx, 0.66), col = "red", lwd = 2)
#'
#' @name LogSeries
#' @aliases LogSeries
#' @aliases dlgser
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#' 
#' @export

dlgser <- function(x, theta, log = FALSE) {
  cpp_dlgser(x, theta, log[1L])
}


#' @rdname LogSeries
#' @export

plgser <- function(q, theta, lower.tail = TRUE, log.p = FALSE) {
  cpp_plgser(q, theta, lower.tail[1L], log.p[1L])
}


#' @rdname LogSeries
#' @export

qlgser <- function(p, theta, lower.tail = TRUE, log.p = FALSE) {
  cpp_qlgser(p, theta, lower.tail[1L], log.p[1L])
}


#' @rdname LogSeries
#' @export

rlgser <- function (n, theta) {
  if (length(n) > 1) n <- length(n)
  cpp_rlgser(n, theta)
}

#' Pareto distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Pareto distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param a,b             positive valued scale and location parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{ab^a}{x^{a+1}}
#' }{
#' f(x) = (a*b^a) / x^(a+1)
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = 1 - \left(\frac{b}{x}\right)^a
#' }{
#' F(x) = 1 - (b/x)^a
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \frac{b}{(1-p)^{1-a}}
#' }{
#' F^-1(p) = b/(1-p)^(1-a)
#' }
#'
#' @references
#' Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
#' with Applications. Chapman & Hall/CRC
#' 
#' @examples 
#' 
#' x <- rpareto(1e5, 5, 16)
#' hist(x, 100, freq = FALSE)
#' curve(dpareto(x, 5, 16), 0, 200, col = "red", add = TRUE)
#' hist(ppareto(x, 5, 16))
#' plot(ecdf(x))
#' curve(ppareto(x, 5, 16), 0, 200, col = "red", lwd = 2, add = TRUE)
#'
#' @name Pareto
#' @aliases Pareto
#' @aliases dpareto
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dpareto <- function(x, a = 1, b = 1, log = FALSE) {
  cpp_dpareto(x, a, b, log[1L])
}


#' @rdname Pareto
#' @export

ppareto <- function(q, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_ppareto(q, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname Pareto
#' @export

qpareto <- function(p, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qpareto(p, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname Pareto
#' @export

rpareto <- function(n, a = 1, b = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rpareto(n, a, b)
}

is_int <- function(x) x %% 1 == 0

dbbinomR <- function(k, n, alpha, beta) {
  ifelse(k<0 | k>n | !is_int(k), 0,
         choose(n, k) * (beta(k+alpha, n-k+beta)) / (beta(alpha, beta)))
}

dbnbinomR <- function(k, r, alpha, beta) {
  ifelse(k<0 | !is_int(k), 0,
         gamma(r+k)/(factorial(k) * gamma(r)) * beta(alpha+r, beta+k)/beta(alpha, beta))
}

dbetaprR <- function(x, alpha, beta, sigma) {
  z <- x/sigma
  ifelse(x<=0, 0,
         z^(alpha-1.0) * (z+1.0)^(-alpha-beta) / beta(alpha, beta) / sigma)
}

dfatigueR <- function(x, alpha, beta, mu) {
  z <- x-mu
  zb <- suppressWarnings(sqrt(z/beta)) # warns when x <= mu
  bz <- suppressWarnings(sqrt(beta/z)) # warns when x <= mu
  ifelse(x<=mu, 0, 
         (zb+bz)/(2.0*alpha*z) * dnorm((zb-bz)/alpha))
}

ddlaplaceR <- function(x, mu, p) {
  ifelse(!is_int(x), 0,
         (1.0-p)/(1.0+p) * p^abs(x-mu))
}

ddweibullR <- function(x, q, beta) {
  ifelse(x<0 | !is_int(x), 0, 
         q^x^beta - q^(x+1)^beta)
}

dfrechetR <- function(x, lambda, mu, sigma) {
  z <- (x-mu)/sigma
  ifelse(x<=mu, 0,
         lambda/sigma * z^(-1-lambda) * exp(-z^(-lambda)))
}

dgpoisR <- function(x, alpha, beta) {
  ifelse(x<0 | !is_int(x), 0,
         (gamma(x+beta)*alpha^x)/(gamma(beta)*(1+alpha)^(beta+x)*factorial(x)))
}

dgevR <- function(x, mu, sigma, xi) {
  z <- (x-mu)/sigma
  if (xi != 0) {
    ifelse((1.0+xi*z) <= 0.0, 0, 
           1/sigma * (1+xi*z)^(-1/xi-1) * exp(-(1+xi*z)^(-1/xi)))
  } else {
    ifelse((1.0+xi*z) <= 0.0, 0, 
           1/sigma * exp(-z) * exp(-exp(-z)))
  }
}

dgompertzR <- function(x, a, b) {
  ifelse(x<0, 0,
         a*exp(b*x - a/b * (exp(b*x)-1)))
}

dgpdR <- function(x, mu, sigma, xi) {
  z <- (x-mu)/sigma
  if (xi != 0) {
    ifelse((1.0+xi*z) <= 0.0 | z <= 0, 0,
           (1+xi*z)^(-(xi+1)/xi)/sigma)
  } else {
    ifelse((1.0+xi*z) <= 0.0 | z <= 0, 0,
           exp(-z)/sigma)
  }
}

dgumbelR <- function(x, mu, sigma) {
  z <- (x-mu)/sigma
  1/sigma * exp(-(z+exp(-z)))
}

dgumbelR_log <- function(x, mu, sigma) {
  z <- (x-mu)/sigma
  exp(-(z+exp(-z)) - log(sigma))
}

dinvgammaR <- function(x, alpha, beta) {
  ifelse(x<=0, 0, 
         (beta^alpha * x^(-alpha-1) * exp(-beta/x)) / gamma(alpha))
}

dlaplaceR <- function(x, mu, sigma) {
  z <- (x-mu)/sigma
  1/(2*sigma) * exp(-abs(z))
}

dlaplaceR_log <- function(x, mu, sigma) {
  LOG_2F <- 0.6931471805599452862268
  z <- abs(x-mu)/sigma
  exp(-z - LOG_2F - log(sigma))
}

dlgserR <- function(x, theta) {
  ifelse(x<1 | !is_int(x), 0,
         (-1/log(1-theta)*theta^x) / x)
}

dlgserR_log <- function(x, theta) {
  a <- suppressWarnings(-1.0/log1p(-theta))
  ifelse(x<1 | !is_int(x), 0,
         exp(log(a) + (log(theta) * x) - log(x)))
}

dlomaxR <- function(x, lambda, kappa) {
  ifelse(x<=0, 0, 
         lambda*kappa / (1+lambda*x)^(kappa+1))
}

dparetoR <- function(x, a, b) {
  ifelse(x<b, 0,
         (a*b^a) / x^(a+1))
}

dpowerR <- function(x, alpha, beta) {
  ifelse(x<=0 | x>=alpha, 0,
         (beta*x^(beta-1)) / (alpha^beta))
}

drayleighR <- function(x, sigma) {
  ifelse(x<=0, 0,
         x/sigma^2 * exp(-(x^2 / 2*sigma^2)))
}

dsgompR <- function(x, b, eta) {
  ebx <- exp(-b*x)
  ifelse(x<0, 0,
         b*ebx * exp(-eta*ebx) * (1 + eta*(1 - ebx)))
}

# Below code was taken from bivpois package ver 0.50-3 (CRAN archive)

pbivpois <- function(x, y=NULL, a, b, c, log=FALSE) {
  # ------------------------------------------------------------------------------
  # Karlis and Ntzoufras (2003, 2004)
  # EM algorithms for Bivariate Poisson Models
  # ------------------------------------------------------------------------------
  # x      : matrix or vector of length n
  # y      : vector of length n. If x is matrix then it is not used
  # lambda : parameters of the bivariate poisson distribution
  # log    : argument controlling the calculation of the log-probability or the 
  #          probability function. 
  # ------------------------------------------------------------------------------
  #	
  
  lambda <- c(a, b, c) # edited here
  
  if ( is.matrix(x) ) {
    var1<-x[,1]
    var2<-x[,2]
  }
  else if (is.vector(x)&is.vector(y)){
    if (length(x)==length(y)){
      var1<-x
      var2<-y
    }
    else{
      stop('lengths of x and y are not equal')
    }	
  }
  else{
    stop('x is not a matrix or x and y are not vectors')
  }
  n <- length(var1)
  logbp<-vector(length=n)
  #
  for (k in 1:n){
    x0<-var1[k]
    y0<-var2[k]
    xymin<-min( x0,y0 )
    lambdaratio<-lambda[3]/(lambda[1]*lambda[2])
    #	
    i<-0:xymin
    sums<- -lgamma(var1[k]-i+1)-lgamma(i+1)-lgamma(var2[k]-i+1)+i*log(lambdaratio)
    maxsums <- max(sums)
    sums<- sums - maxsums
    logsummation<- log( sum(exp(sums)) ) + maxsums 
    logbp[k]<- -sum(lambda) + var1[k] * log( lambda[1] ) + var2[k] * log( lambda[2] ) + logsummation 
  }
  if (log) { result<-    logbp }
  else     { result<-exp(logbp)  }
  result
  #	end of function bivpois
}

test_that("Compare PDF's/PMF's to pure-R benchmarks", {
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  n <- length(x)
  
  expect_warning(expect_equal(dbbinom(x, 100, 1, 10), dbbinomR(x, 100, 1, 10)))
  expect_warning(expect_equal(dbbinom(x, 100, 1000, 0.001), dbbinomR(x, 100, 1000, 0.001)))
  expect_warning(expect_equal(dbnbinom(x[1:(n-2)], 100, 1, 10),
                              dbnbinomR(x[1:(n-2)], 100, 1, 10))) # numerical precission in R version
  expect_equal(dbetapr(x, 1, 1, 1), dbetaprR(x, 1, 1, 1))
  expect_equal(dbetapr(x, 0.0001, 0.0001, 0.0001), dbetaprR(x, 0.0001, 0.0001, 0.0001))
  expect_equal(dbetapr(x, 100, 100, 100), dbetaprR(x, 100, 100, 100))
  expect_equal(dfatigue(x, 1, 1, 0), dfatigueR(x, 1, 1, 0))
  expect_equal(dfatigue(x, 0.0001, 0.0001, -100), dfatigueR(x, 0.0001, 0.0001, -100))
  expect_warning(expect_equal(ddlaplace(x, 0, 0.5), ddlaplaceR(x, 0, 0.5)))
  expect_warning(expect_equal(ddlaplace(x, 0, 0.999), ddlaplaceR(x, 0, 0.999)))
  expect_warning(expect_equal(ddlaplace(x, 0, 0.0001), ddlaplaceR(x, 0, 0.0001)))
  expect_warning(expect_equal(ddweibull(x, 0.5, 1), ddweibullR(x, 0.5, 1)))
  expect_warning(expect_equal(ddweibull(x, 0.0001, 0.0001), ddweibullR(x, 0.0001, 0.0001)))
  expect_warning(expect_equal(ddweibull(x, 0.9999, 1000), ddweibullR(x, 0.9999, 1000)))
  expect_equal(dfrechet(x, 1, 1, 1), dfrechetR(x, 1, 1, 1))
  expect_warning(expect_equal(dgpois(x[1:(n-1)], 1, 1),
                              dgpoisR(x[1:(n-1)], 1, 1))) # numerical precission in R version
  expect_equal(dgev(x, 1, 1, 1), dgevR(x, 1, 1, 1))
  expect_equal(dgev(x[2:n], 1, 1, 0), dgevR(x[2:n], 1, 1, 0)) # numerical precission in R version
  expect_equal(dgev(x, 1, 1, -1), dgevR(x, 1, 1, -1))
  expect_equal(dgompertz(x, 1, 1), dgompertzR(x, 1, 1))
  expect_equal(dgpd(x, 1, 1, 1), dgpdR(x, 1, 1, 1))
  expect_equal(dgpd(x, 1, 1, 0), dgpdR(x, 1, 1, 0))
  expect_equal(dgpd(x, 1, 1, -1), dgpdR(x, 1, 1, -1))
  expect_equal(dgumbel(x, 1, 1), dgumbelR(x, 1, 1))
  expect_equal(dinvgamma(x, 1, 1), dinvgammaR(x, 1, 1))
  expect_equal(dinvgamma(x, 1.2, 0.9), dinvgammaR(x, 1.2, 0.9))
  expect_equal(dlaplace(x, -1, 5), dlaplaceR(x, -1, 5))
  expect_equal(dlaplace(x, 9999, 0.000001), dlaplaceR(x, 9999, 0.000001))
  expect_warning(expect_equal(dlgser(x, 0.5), dlgserR(x, 0.5)))
  expect_warning(expect_equal(dlgser(x, 0.00001), dlgserR(x, 0.00001)))
  expect_warning(expect_equal(dlgser(x, 0.9999), dlgserR(x, 0.9999)))
  expect_equal(dlomax(x, 1, 0.5), dlomaxR(x, 1, 0.5))
  expect_equal(dpareto(x, 1, 1), dparetoR(x, 1, 1))
  expect_equal(dpower(x, 1, 1), dpowerR(x, 1, 1))
  expect_equal(drayleigh(x, 1), drayleighR(x, 1))
  expect_equal(dsgomp(x, 0.5, 1), dsgompR(x, 0.5, 1))
  
})


test_that("Compare dinvgamma to actuar implementation", {
  
  skip_on_cran()
  skip_if_not_installed("actuar")
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  
  expect_equal(dinvgamma(x, 1.2, 0.9), actuar::dinvgamma(x, 1.2, scale=0.9))
  expect_equal(pinvgamma(x, 1.2, 0.9), actuar::pinvgamma(x, 1.2, scale=0.9))
  
})


test_that("Compare dhuber to hoa implementation", {
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  n <- length(x)
  
  skip_on_cran()
  skip_if_not_installed("hoa")

  expect_equal(dhuber(x), hoa::dHuber(x))
  expect_equal(phuber(x), hoa::pHuber(x))
  
})


test_that("Compare triangular to triangle implementation", {
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  n <- length(x)
  
  skip_on_cran()
  skip_if_not_installed("triangle")
  
  expect_equal(dtriang(x, -1, 1), triangle::dtriangle(x, -1, 1))
  expect_equal(dtriang(x, 0, 1, 0.8), triangle::dtriangle(x, 0, 1, 0.8))
  
  expect_equal(ptriang(x, -1, 1), triangle::ptriangle(x, -1, 1))
  expect_equal(ptriang(x, 0, 1, 0.8), triangle::ptriangle(x, 0, 1, 0.8))
  
})


test_that("Compare GEV and GPD to evd implementation", {
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  n <- length(x)
  
  skip_on_cran()
  skip_if_not_installed("evd")

  expect_equal(dgev(x, 0, 1, 0), evd::dgev(x, 0, 1, 0))
  expect_equal(dgev(x, 0, 1, 1), evd::dgev(x, 0, 1, 1))
  expect_equal(dgev(x, 0, 1, -1), evd::dgev(x, 0, 1, -1))
  
  expect_equal(pgev(x, 0, 1, 0), evd::pgev(x, 0, 1, 0))
  expect_equal(pgev(x, 0, 1, 1), evd::pgev(x, 0, 1, 1))
  expect_equal(pgev(x, 0, 1, -1), evd::pgev(x, 0, 1, -1))
  
  expect_equal(dgpd(x, 0, 1, 0), evd::dgpd(x, 0, 1, 0))
  expect_equal(dgpd(x, 0, 1, 1), evd::dgpd(x, 0, 1, 1))
  expect_equal(dgpd(x, 0, 1, -1), evd::dgpd(x, 0, 1, -1))
  
  expect_equal(pgpd(x, 0, 1, 0), evd::pgpd(x, 0, 1, 0))
  expect_equal(pgpd(x, 0, 1, 1), evd::pgpd(x, 0, 1, 1))
  expect_equal(pgpd(x, 0, 1, -1), evd::pgpd(x, 0, 1, -1))
  
})


test_that("Compare to distributions from VGAM package", {
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  n <- length(x)
  
  skip_on_cran()
  skip_if_not_installed("VGAM")
  
  expect_warning(expect_equal(dzib(x, 45, 0.7, 0.2), VGAM::dzibinom(x, 45, 0.7, 0.2)))
  expect_warning(expect_equal(dzib(x, 45, 0.7, 0.0001), VGAM::dzibinom(x, 45, 0.7, 0.0001)))
  expect_warning(expect_equal(dzib(x, 45, 0.7, 0.9999), VGAM::dzibinom(x, 45, 0.7, 0.9999)))
  expect_equal(pzib(x, 45, 0.7, 0.2), VGAM::pzibinom(x, 45, 0.7, 0.2))
  expect_equal(pzib(x, 45, 0.7, 0.0001), VGAM::pzibinom(x, 45, 0.7, 0.0001))
  expect_equal(pzib(x, 45, 0.7, 0.9999), VGAM::pzibinom(x, 45, 0.7, 0.9999))
  
  expect_warning(expect_equal(dzinb(x, 45, 0.7, 0.2), VGAM::dzinegbin(x, 45, 0.7, NULL, 0.2)))
  expect_warning(expect_equal(dzinb(x, 45, 0.7, 0.0001), VGAM::dzinegbin(x, 45, 0.7, NULL, 0.0001)))
  expect_warning(expect_equal(dzinb(x, 45, 0.7, 0.9999), VGAM::dzinegbin(x, 45, 0.7, NULL, 0.9999)))
  expect_equal(pzinb(x, 45, 0.7, 0.2), VGAM::pzinegbin(x, 45, 0.7, NULL, 0.2))
  expect_equal(pzinb(x, 45, 0.7, 0.0001), VGAM::pzinegbin(x, 45, 0.7, NULL, 0.0001))
  expect_equal(pzinb(x, 45, 0.7, 0.9999), VGAM::pzinegbin(x, 45, 0.7, NULL, 0.9999))
  
  expect_warning(expect_equal(dzip(x, 7, 0.2), VGAM::dzipois(x, 7, 0.2)))
  expect_warning(expect_equal(dzip(x, 7, 0.0001), VGAM::dzipois(x, 7, 0.0001)))
  expect_warning(expect_equal(dzip(x, 7, 0.9999), VGAM::dzipois(x, 7, 0.9999)))
  expect_equal(pzip(x, 7, 0.2), VGAM::pzipois(x, 7, 0.2))
  expect_equal(pzip(x, 7, 0.0001), VGAM::pzipois(x, 7, 0.0001))
  expect_equal(pzip(x, 7, 0.9999), VGAM::pzipois(x, 7, 0.9999))
  
  expect_equal(dlaplace(x), VGAM::dlaplace(x))
  expect_equal(plaplace(x), VGAM::plaplace(x))
  
  expect_equal(dpareto(x, 1, 1), VGAM::dpareto(x, 1, 1))
  expect_equal(ppareto(x, 1, 1), suppressWarnings(VGAM::ppareto(x, 1, 1)))
  
  expect_warning(expect_equal(dbbinom(x, 100, 2, 7), VGAM::dbetabinom.ab(x, 100, 2, 7)))
  expect_equal(pbbinom(x, 100, 2, 7), VGAM::pbetabinom.ab(x, 100, 2, 7))
  
  expect_equal(dfrechet(x, 1, 0, 1), VGAM::dfrechet(x, 0, 1, 1))
  expect_equal(pfrechet(x, 1, 0, 1), VGAM::pfrechet(x, 0, 1, 1))
  
  expect_equal(dgumbel(x, 0, 1), VGAM::dgumbel(x, 0, 1))
  expect_equal(pgumbel(x, 0, 1), VGAM::pgumbel(x, 0, 1))
  
  expect_equal(dgompertz(x, 1, 1), VGAM::dgompertz(x, 1, 1))
  expect_equal(pgompertz(x, 1, 1), VGAM::pgompertz(x, 1, 1))  
  
  expect_equal(dkumar(x, 3, 7), VGAM::dkumar(x, 3, 7))
  expect_equal(pkumar(x, 3, 7), suppressWarnings(VGAM::pkumar(x, 3, 7)))
  
  expect_equal(dslash(x), VGAM::dslash(x))
  # expect_equal(pslash(x), VGAM::pslash(x))
  
  expect_equal(dhuber(x), VGAM::dhuber(x, 1.345))
  expect_equal(phuber(x), VGAM::phuber(x, 1.345))
  
  expect_warning(expect_equal(dlgser(x, 0.5), VGAM::dlog(x, 0.5)))
  expect_warning(expect_equal(dlgser(x, 0.0001), VGAM::dlog(x, 0.0001)))
  expect_warning(expect_equal(dlgser(x, 0.9999), VGAM::dlog(x, 0.9999)))
  
  expect_equal(drayleigh(x), VGAM::drayleigh(x))
  expect_equal(prayleigh(x), VGAM::prayleigh(x))
  
  expect_warning(expect_equal(
    extraDistr::dskellam(x, 7, 8),
    VGAM::dskellam(x, 7, 8)
  ))

})


test_that("Check against the parameter values tested in VGAM", {
  
  x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
         0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  n <- length(x)
  
  skip_on_cran()
  skip_if_not_installed("VGAM")

  expect_warning(expect_equal(dbbinom(x, 10, 0.8, 1.2),
                              VGAM::dbetabinom.ab(x, 10, 0.8, 1.2)))
  
  expect_equal(dpareto(x, 1.9, 2.3), dparetoR(x, 1.9, 2.3))
  expect_equal(dpareto(x, 2.3, 1.9), VGAM::dpareto(x, 1.9, 2.3)) # reverse order of params!
  
  expect_equal(dlst(x, 3, -0.9, 2), dt((x+0.9)/2, df = 3)/2)
  
  expect_equal(dlaplace(x, -0.9, 2), VGAM::dlaplace(x, -0.9, 2))
  
  expect_warning(expect_equal(dbern(x, 0.3), dbinom(x, 1, 0.3)))

})


test_that("Compare with LaplacesDemon package implementations", {
  
  skip_on_cran()
  skip_if_not_installed("LaplacesDemon")
  
  alpha <- runif(5, 0, 3)
  x <- rdirichlet(5000, alpha)
  
  expect_equal(ddirichlet(x, alpha),
               LaplacesDemon::ddirichlet(x, alpha))
  
  alpha <- c(0.0001, 0.0001, 0.0001, 0.0001, 0.0001)
  
  expect_equal(ddirichlet(x, alpha),
               LaplacesDemon::ddirichlet(x, alpha))
  
  alpha <- c(1000, 1000, 1000, 1000, 1000)
  
  expect_equal(ddirichlet(x, alpha),
               LaplacesDemon::ddirichlet(x, alpha))
  
  alpha <- c(1e-4, 10000, 100, 1e-5, 1000)
  
  expect_equal(ddirichlet(x, alpha),
               LaplacesDemon::ddirichlet(x, alpha))
  
  # x <- c(-1e5, -100, -10, -5.5, -5, -1.01, -1, -0.5, 0.001, 0,
  #        0.001, 0.5, 1, 1.01, 5, 5.5, 10, 100, 1e5)
  # 
  # expect_equal(dbern(x, 0.32), LaplacesDemon::dbern(x, 0.32))
  # expect_equal(dgpd(x, 0.2, 3.4, 1.4), LaplacesDemon::dgpd(x, 0.2, 3.4, 1.4))
  # expect_equal(dhcauchy(x, 2.6), LaplacesDemon::dhalfcauchy(x, 2.6))
  # expect_equal(dhnorm(x, 8.3), LaplacesDemon::dhalfnorm(x, 8.3))
  # expect_equal(dht(x, sigma=3.4, nu=7), LaplacesDemon::dhalft(x, scale=3.4, nu=7))
  # expect_equal(dinvchisq(x, 2.3, 5), LaplacesDemon::dinvchisq(x, 2.3, 5))
  # expect_equal(dinvgamma(x, 7.6, 3), LaplacesDemon::dinvgamma(x, 7.6, 3))
  # expect_equal(dwald(x, -2, 6), LaplacesDemon::dinvgaussian(x, -2, 6))
  # expect_equal(dpareto(x, 5, 6), LaplacesDemon::dpareto(x, 5, 6))
  # 
  # expect_equal(dcat(), LaplacesDemon::dcat())
  
})


test_that("Compare dmnom to dmultinom from base R", {

  n <- 100
  p <- runif(5)
  p <- p/sum(p)
  
  x <- rmnom(5000, n, p)
  
  expect_equal(dmnom(x, n, p), apply(x, 1, dmultinom, n, p))
  
  p <- c(1e-4, 10000, 100, 1e-5, 1000)
  p <- p/sum(p)
  
  expect_equal(dmnom(x, n, p), apply(x, 1, dmultinom, n, p))
  
})


test_that("Bivariate Poisson distribution agrees with code by Karlis and Ntzoufras", {
  
  x <- rbvpois(1000, 7, 8, 5)

  expect_equal(
    dbvpois(x[,1], x[,2], 7, 8, 5),
    pbivpois(x[,1], x[,2], 7, 8, 5)
  )
  
})


test_that("Skellam distribution agrees with the implementation in skellam package", {
  
  skip_on_cran()
  skip_if_not_installed("skellam")
  
  x <- extraDistr::rskellam(5000, 7, 8)
  
  expect_equal(
    extraDistr::dskellam(x, 7, 8),
    skellam::dskellam(x, 7, 8)
  )
  
})

#' Gamma-Poisson distribution
#'
#' Probability mass function and random generation
#' for the gamma-Poisson distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param rate	          an alternative way to specify the scale.
#' @param shape,scale	    shape and scale parameters. Must be positive,
#'                        scale strictly.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#' Gamma-Poisson distribution arises as a continuous mixture of
#' Poisson distributions, where the mixing distribution
#' of the Poisson rate \eqn{\lambda} is a gamma distribution.
#' When \eqn{X \sim \mathrm{Poisson}(\lambda)}{X ~ Poisson(\lambda)}
#' and \eqn{\lambda \sim \mathrm{Gamma}(\alpha, \beta)}{\lambda ~ Gamma(\alpha, \beta)}, then \eqn{X \sim \mathrm{GammaPoisson}(\alpha, \beta)}{X ~ Gamma-Poisson(\alpha, \beta)}.
#'
#' Probability mass function
#' \deqn{
#' f(x) = \frac{\Gamma(\alpha+x)}{x! \, \Gamma(\alpha)} \left(\frac{\beta}{1+\beta}\right)^x \left(1-\frac{\beta}{1+\beta}\right)^\alpha
#' }{
#' f(x) = \Gamma(\alpha+x) / (x!*\Gamma(\alpha)) * (\beta/(1+\beta))^x * (1-\beta/(1+\beta))^\alpha
#' }
#' 
#' Cumulative distribution function is calculated using recursive algorithm that employs the fact that
#' \eqn{\Gamma(x) = (x - 1)!}. This enables re-writing probability mass function as
#' 
#' \deqn{
#' f(x) = \frac{(\alpha+x-1)!}{x! \, \Gamma(\alpha)} \left( \frac{\beta}{1+\beta} \right)^x \left( 1- \frac{\beta}{1+\beta} \right)^\alpha
#' }{
#' f(x) = ((\alpha+x-1)!)/(x!*\Gamma(\alpha))*(\beta/(1+\beta))^x*(1-\beta/(1+\beta))^\alpha
#' }
#' 
#' what makes recursive updating from \eqn{x} to \eqn{x+1} easy using the properties of factorials
#' 
#' \deqn{
#' f(x+1) = \frac{(\alpha+x-1)! \, (\alpha+x)}{x! \,(x+1) \, \Gamma(\alpha)} \left( \frac{\beta}{1+\beta} \right)^x \left( \frac{\beta}{1+\beta} \right) \left( 1- \frac{\beta}{1+\beta} \right)^\alpha
#' }{
#' f(x+1) = ((\alpha+x-1)!*(\alpha+x))/(x!*(x+1)*\Gamma(\alpha))*(\beta/(1+\beta))^x*(\beta/(1+\beta))*(1-\beta/(1+\beta))^\alpha
#' }
#' 
#' and let's us efficiently calculate cumulative distribution function as a sum of probability mass functions
#' 
#' \deqn{F(x) = \sum_{k=0}^x f(k)}{F(x) = f(0)+...+f(x)}
#' 
#'
#' @seealso \code{\link[stats]{Gamma}}, \code{\link[stats]{Poisson}}
#' 
#' @examples 
#' 
#' x <- rgpois(1e5, 7, 0.002)
#' xx <- seq(0, 12000, by = 1)
#' hist(x, 100, freq = FALSE)
#' lines(xx, dgpois(xx, 7, 0.002), col = "red")
#' hist(pgpois(x, 7, 0.002))
#' xx <- seq(0, 12000, by = 0.1)
#' plot(ecdf(x))
#' lines(xx, pgpois(xx, 7, 0.002), col = "red", lwd = 2)
#'
#' @name GammaPoiss
#' @aliases GammaPoiss
#' @aliases dgpois
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dgpois <- function(x, shape, rate, scale = 1/rate, log = FALSE) {
  cpp_dgpois(x, shape, scale, log[1L])
}


#' @rdname GammaPoiss
#' @export

pgpois <- function(q, shape, rate, scale = 1/rate, lower.tail = TRUE, log.p = FALSE) {
  cpp_pgpois(q, shape, scale, lower.tail[1L], log.p[1L])
}


#' @rdname GammaPoiss
#' @export

rgpois <- function(n, shape, rate, scale = 1/rate) {
  cpp_rgpois(n, shape, scale)
}

#' Skellam distribution
#'
#' Probability mass function and random generation
#' for the Skellam distribution.
#'
#' @param x 	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu1,mu2         positive valued parameters.
#' @param log     	      logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#' 
#' If \eqn{X} and \eqn{Y} follow Poisson distributions with means
#' \eqn{\mu_1}{\mu[1]} and \eqn{\mu_2}{\mu[2]}, than \eqn{X-Y} follows
#' Skellam distribution parametrized by \eqn{\mu_1}{\mu[1]} and \eqn{\mu_2}{\mu[2]}.
#'
#' Probability mass function
#' \deqn{
#' f(x) = e^{-(\mu_1\!+\!\mu_2)} \left(\frac{\mu_1}{\mu_2}\right)^{k/2}\!\!I_{k}(2\sqrt{\mu_1\mu_2})
#' }{
#' f(x) = exp(-(\mu1+\mu2)) * (\mu1/\mu2)^(x/2) * besselI(2*sqrt(\mu1*\mu2), x)
#' }
#' 
#' @references
#' Karlis, D., & Ntzoufras, I. (2006). Bayesian analysis of the differences of count data.
#' Statistics in medicine, 25(11), 1885-1905.
#' 
#' @references
#' Skellam, J.G. (1946). The frequency distribution of the difference between
#' two Poisson variates belonging to different populations.
#' Journal of the Royal Statistical Society, series A, 109(3), 26.
#' 
#' @examples 
#' 
#' x <- rskellam(1e5, 5, 13)
#' xx <- -40:40
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, dskellam(xx, 5, 13), col = "red")
#'
#' @name Skellam
#' @aliases Skellam
#' @aliases dskellam
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dskellam <- function(x, mu1, mu2, log = FALSE) {
  cpp_dskellam(x, mu1, mu2, log[1L])
}


#' @rdname Skellam
#' @export

rskellam <- function(n, mu1, mu2) {
  if (length(n) > 1) n <- length(n)
  cpp_rskellam(n, mu1, mu2)
}

#' Gompertz distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Gompertz distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param a,b             positive valued scale and location parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = a \exp\left(bx - \frac{a}{b} (\exp(bx)-1)\right)
#' }{
#' f(x) = a*exp(b*x - a/b * (exp(b*x)-1))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = 1-\exp\left(-\frac{a}{b} (\exp(bx)-1)\right)
#' }{
#' F(x) = 1-exp(-a/b * (exp(b*x)-1))
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \frac{1}{b} \log\left(1-\frac{b}{a}\log(1-p)\right)
#' }{
#' F^-1(p) = 1/b * log(1 - b/a * log(1-p))
#' }
#'
#' @references
#' Lenart, A. (2012). The Gompertz distribution and Maximum Likelihood Estimation
#' of its parameters - a revision. MPIDR WORKING PAPER WP 2012-008.
#' \url{https://www.demogr.mpg.de/papers/working/wp-2012-008.pdf}
#'
#' @examples
#'
#' x <- rgompertz(1e5, 5, 2)
#' hist(x, 100, freq = FALSE)
#' curve(dgompertz(x, 5, 2), 0, 1, col = "red", add = TRUE)
#' hist(pgompertz(x, 5, 2))
#' plot(ecdf(x))
#' curve(pgompertz(x, 5, 2), 0, 1, col = "red", lwd = 2, add = TRUE)
#'
#' @name Gompertz
#' @aliases Gompertz
#' @aliases dgompertz
#'
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dgompertz <- function(x, a = 1, b = 1, log = FALSE) {
  cpp_dgompertz(x, a, b, log[1L])
}


#' @rdname Gompertz
#' @export

pgompertz <- function(q, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pgompertz(q, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname Gompertz
#' @export

qgompertz <- function(p, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qgompertz(p, a, b, lower.tail[1L], log.p[1L])
}


#' @rdname Gompertz
#' @export

rgompertz <- function(n, a = 1, b = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rgompertz(n, a, b)
}

#' Inverse-gamma distribution
#'
#' Density, distribution function and random generation
#' for the inverse-gamma distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param alpha,beta      positive valued shape and scale parameters.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability mass function
#' \deqn{
#' f(x) = \frac{\beta^\alpha x^{-\alpha-1} \exp(-\frac{\beta}{x})}{\Gamma(\alpha)}
#' }{
#' f(x) = (\beta^\alpha * x^(-\alpha-1) * exp(-\beta/x)) / \Gamma(\alpha)
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = \frac{\gamma(\alpha, \frac{\beta}{x})}{\Gamma(\alpha)}
#' }{
#' F(x) = \gamma(\alpha, \beta/x) / \Gamma(\alpha)
#' }
#'
#' @references
#' Witkovsky, V. (2001). Computing the distribution of a linear
#' combination of inverted gamma variables. Kybernetika 37(1), 79-90.
#'
#' @references
#' Leemis, L.M. and McQueston, L.T. (2008). Univariate Distribution
#' Relationships. American Statistician 62(1): 45-53.
#'
#' @seealso \code{\link[stats]{GammaDist}}
#' 
#' @examples 
#' 
#' x <- rinvgamma(1e5, 20, 3)
#' hist(x, 100, freq = FALSE)
#' curve(dinvgamma(x, 20, 3), 0, 1, col = "red", add = TRUE, n = 5000)
#' hist(pinvgamma(x, 20, 3))
#' plot(ecdf(x))
#' curve(pinvgamma(x, 20, 3), 0, 1, col = "red", lwd = 2, add = TRUE, n = 5000) 
#'
#' @name InvGamma
#' @aliases InvGamma
#' @aliases dinvgamma
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dinvgamma <- function(x, alpha, beta = 1, log = FALSE) {
  cpp_dinvgamma(x, alpha, 1/beta, log[1L])
}


#' @rdname InvGamma
#' @export

pinvgamma <- function(q, alpha, beta = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pinvgamma(q, alpha, beta, lower.tail, log.p[1L])
}


#' @rdname InvGamma
#' @export

qinvgamma <- function(p, alpha, beta = 1, lower.tail = TRUE, log.p = FALSE) {
  1/qgamma(p, alpha, beta, lower.tail = !lower.tail[1L], log.p = log.p[1L])
}


#' @rdname InvGamma
#' @export

rinvgamma <- function(n, alpha, beta = 1) {
  1/rgamma(n, alpha, beta)
}

#' Location-scale version of the t-distribution
#' 
#' Probability mass function, distribution function and random generation
#' for location-scale version of the t-distribution. Location-scale version
#' of the t-distribution besides degrees of freedom \eqn{\nu}, is parametrized
#' using additional parameters \eqn{\mu} for location and \eqn{\sigma} for
#' scale (\eqn{\mu = 0} and \eqn{\sigma = 1} for standard t-distribution).
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu              vector of locations
#' @param sigma           vector of positive valued scale parameters.
#' @param df              degrees of freedom (> 0, maybe non-integer). \code{df = Inf} is allowed.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @seealso \code{\link[stats]{TDist}}
#' 
#' @examples 
#' 
#' x <- rlst(1e5, 1000, 5, 13)
#' hist(x, 100, freq = FALSE)
#' curve(dlst(x, 1000, 5, 13), -60, 60, col = "red", add = TRUE)
#' hist(plst(x, 1000, 5, 13))
#' plot(ecdf(x))
#' curve(plst(x, 1000, 5, 13), -60, 60, col = "red", lwd = 2, add = TRUE)
#' 
#' @name LocationScaleT
#' @aliases LocationScaleT
#' @aliases dlst
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#' 
#' @export

dlst <- function(x, df, mu = 0, sigma = 1, log = FALSE) {
  cpp_dlst(x, df, mu, sigma, log[1L])
}


#' @rdname LocationScaleT
#' @export

plst <- function(q, df, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_plst(q, df, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname LocationScaleT
#' @export

qlst <- function(p, df, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qlst(p, df, mu, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname LocationScaleT
#' @export

rlst <- function(n, df, mu = 0, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rlst(n, df, mu, sigma)
}

#' Rayleigh distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Rayleigh distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param sigma           positive valued parameter.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{
#' f(x) = \frac{x}{\sigma^2} \exp\left(-\frac{x^2}{2\sigma^2}\right)
#' }{
#' f(x) = x/\sigma^2 * exp(-(x^2 / 2*\sigma^2))
#' }
#'
#' Cumulative distribution function
#' \deqn{
#' F(x) = 1 - \exp\left(-\frac{x^2}{2\sigma^2}\right)
#' }{
#' F(x) = 1 - exp(-x^2 / 2*\sigma^2)
#' }
#'
#' Quantile function
#' \deqn{
#' F^{-1}(p) = \sqrt{-2\sigma^2 \log(1-p)}
#' }{
#' F^-1(p) = sqrt(-2*\sigma^2 * log(1-p))
#' }
#'
#' @references
#' Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
#' with Applications. Chapman & Hall/CRC.
#'
#' @references
#' Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
#' Statistical Distributions. John Wiley & Sons.
#' 
#' @examples 
#' 
#' x <- rrayleigh(1e5, 13)
#' hist(x, 100, freq = FALSE)
#' curve(drayleigh(x, 13), 0, 60, col = "red", add = TRUE)
#' hist(prayleigh(x, 13)) 
#' plot(ecdf(x))
#' curve(prayleigh(x, 13), 0, 60, col = "red", lwd = 2, add = TRUE) 
#'
#' @name Rayleigh
#' @aliases Rayleigh
#' @aliases drayleigh
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

drayleigh <- function(x, sigma = 1, log = FALSE) {
  cpp_drayleigh(x, sigma, log[1L])
}


#' @rdname Rayleigh
#' @export

prayleigh <- function(q, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_prayleigh(q, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Rayleigh
#' @export

qrayleigh <- function(p, sigma = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_qrayleigh(p, sigma, lower.tail[1L], log.p[1L])
}


#' @rdname Rayleigh
#' @export

rrayleigh <- function(n, sigma = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rrayleigh(n, sigma)
}

#' Discrete gamma distribution
#' 
#' Probability mass function, distribution function and random generation
#' for discrete gamma distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param rate	          an alternative way to specify the scale.
#' @param shape,scale	    shape and scale parameters. Must be positive, scale strictly.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'                
#' @details 
#'                                 
#' Probability mass function of discrete gamma distribution \eqn{f_Y(y)}{f}
#' is defined by discretization of continuous gamma distribution
#' \eqn{f_Y(y) = S_X(y) - S_X(y+1)}{f(y) = S(x) - S(x+1)}
#' where \eqn{S_X}{S} is a survival function of continuous gamma distribution.
#' 
#' @references 
#' Chakraborty, S. and Chakravarty, D. (2012).
#' Discrete Gamma distributions: Properties and parameter estimations.
#' Communications in Statistics-Theory and Methods, 41(18), 3301-3324.
#' 
#' @seealso \code{\link[stats]{GammaDist}}, \code{\link{DiscreteNormal}}
#' 
#' @examples 
#' 
#' x <- rdgamma(1e5, 9, 1)
#' xx <- 0:50
#' plot(prop.table(table(x)))
#' lines(xx, ddgamma(xx, 9, 1), col = "red")
#' hist(pdgamma(x, 9, 1))
#' plot(ecdf(x))
#' xx <- seq(0, 50, 0.1)
#' lines(xx, pdgamma(xx, 9, 1), col = "red", lwd = 2, type = "s")
#' 
#' @name DiscreteGamma
#' @aliases DiscreteGamma
#' @aliases ddgamma
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#' 
#' @export

ddgamma <- function(x, shape, rate = 1, scale = 1/rate, log = FALSE) {
  if (!missing(rate) && !missing(scale)) {
    if (abs(rate * scale - 1) < 1e-15)
      warning("specify 'rate' or 'scale' but not both")
    else stop("specify 'rate' or 'scale' but not both")
  }
  cpp_ddgamma(x, shape, scale, log[1L])
}


#' @rdname DiscreteGamma
#' @export

pdgamma <- function(q, shape, rate = 1, scale = 1/rate, lower.tail = TRUE, log.p = FALSE) {
  pgamma(floor(q)+1, shape, scale = scale, lower.tail = lower.tail[1L], log.p = log.p[1L])
}


#' @rdname DiscreteGamma
#' @export

rdgamma <- function(n, shape, rate = 1, scale = 1/rate) {
  floor(rgamma(n, shape, scale = scale))
}

#' Multinomial distribution
#'
#' Probability mass function and random generation
#' for the multinomial distribution.
#'
#' @param x 	 \eqn{k}-column matrix of quantiles.
#' @param n   number of observations. If \code{length(n) > 1},
#'             the length is taken to be the number required.
#' @param size numeric vector; number of trials (zero or more).
#' @param prob \eqn{k}-column numeric matrix; probability of success on each trial.
#' @param log  logical; if TRUE, probabilities p are given as log(p).
#'
#' @details
#'
#' Probability mass function
#' \deqn{
#' f(x) = \frac{n!}{\prod_{i=1}^k x_i} \prod_{i=1}^k p_i^{x_i}
#' }{
#' f(x) = n!/prod(x[i]!) * prod(p[i]^x[i])
#' }
#' 
#' @references 
#' Gentle, J.E. (2006). Random number generation and Monte Carlo methods. Springer.
#'
#' @seealso \code{\link[stats]{Binomial}}, \code{\link[stats]{Multinomial}}
#' 
#' @examples 
#' 
#' # Generating 10 random draws from multinomial distribution
#' # parametrized using a vector
#' 
#' (x <- rmnom(10, 3, c(1/3, 1/3, 1/3)))
#' 
#' # Results are consistent with dmultinom() from stats:
#' 
#' all.equal(dmultinom(x[1,], 3, c(1/3, 1/3, 1/3)),
#'           dmnom(x[1, , drop = FALSE], 3, c(1/3, 1/3, 1/3)))
#'
#' @name Multinomial
#' @aliases Multinomial
#' @aliases dmnom
#' 
#' @keywords distribution
#' @concept Multivariate
#' @concept Discrete
#'
#' @export

dmnom <- function(x, size, prob, log = FALSE) {
  if (is.vector(prob))
    prob <- matrix(prob, nrow = 1)
  else if (!is.matrix(prob))
    prob <- as.matrix(prob)
  
  if (is.vector(x))
    x <- matrix(x, nrow = 1)
  else if (!is.matrix(x))
    x <- as.matrix(x)
  
  cpp_dmnom(x, size, prob, log[1L])
}


#' @rdname Multinomial
#' @export

rmnom <- function(n, size, prob) {
  if (length(n) > 1)
    n <- length(n)
  
  if (is.vector(prob))
    prob <- matrix(prob, nrow = 1)
  else if (!is.matrix(prob))
    prob <- as.matrix(prob)
  
  cpp_rmnom(n, size, prob)
}

#' Discrete Laplace distribution
#' 
#' Probability mass, distribution function and random generation
#' for the discrete Laplace distribution parametrized by location and scale.
#' 
#' @param x,q             vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param scale           scale parameter; \code{0 < scale < 1}.
#' @param location        location parameter.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @details 
#' 
#' If \eqn{U \sim \mathrm{Geometric}(1-p)}{U ~ Geometric(1-p)} and
#' \eqn{V \sim \mathrm{Geometric}(1-p)}{V ~ Geometric(1-p)},
#' then \eqn{U-V \sim \mathrm{DiscreteLaplace}(p)}{U-V ~ DiscreteLaplace(p)},
#' where geometric distribution is related to discrete Laplace distribution
#' in similar way as exponential distribution is related to Laplace distribution.
#' 
#' Probability mass function
#' 
#' \deqn{
#' f(x) = \frac{1-p}{1+p} p^{|x-\mu|}
#' }{
#' f(x) = (1-p)/(1+p) * p^(|x-\mu|)
#' }
#' 
#' Cumulative distribution function
#' 
#' \deqn{
#' F(x) = \left\{\begin{array}{ll}
#' \frac{p^{-|x-\mu|}}{1+p} & x < 0 \\
#' 1 - \frac{p^{|x-\mu|+1}}{1+p} & x \ge 0
#' \end{array}\right.
#' }{
#' F(x) = [if x < 0:] p^-floor(x-\mu))/(1+p) [else:] 1-(p^(floor(x-\mu)+1))/(1+p)
#' }
#' 
#' @references 
#' Inusah, S., & Kozubowski, T.J. (2006). A discrete analogue of the Laplace distribution.
#' Journal of statistical planning and inference, 136(3), 1090-1102.
#' 
#' @references 
#' Kotz, S., Kozubowski, T., & Podgorski, K. (2012).
#' The Laplace distribution and generalizations: a revisit with applications
#' to communications, economics, engineering, and finance.
#' Springer Science & Business Media.
#' 
#' @examples 
#' 
#' p <- 0.45
#' x <- rdlaplace(1e5, 0, p)
#' xx <- seq(-200, 200, by = 1)
#' plot(prop.table(table(x)))
#' lines(xx, ddlaplace(xx, 0, p), col = "red")
#' hist(pdlaplace(x, 0, p))
#' plot(ecdf(x))
#' lines(xx, pdlaplace(xx, 0, p), col = "red", type = "s")
#' 
#' @name DiscreteLaplace
#' @aliases DiscreteLaplace
#' @aliases ddlaplace
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#' 
#' @export

ddlaplace <- function(x, location, scale, log = FALSE) {
  cpp_ddlaplace(x, location, scale, log[1L])
}


#' @rdname DiscreteLaplace
#' @export

pdlaplace <- function(q, location, scale, lower.tail = TRUE, log.p = FALSE) {
  cpp_pdlaplace(q, location, scale, lower.tail[1L], log.p[1L])
}


#' @rdname DiscreteLaplace
#' @export

rdlaplace <- function(n, location, scale) {
  if (length(n) > 1) n <- length(n)
  cpp_rdlaplace(n, location, scale)
}

#' Beta-negative binomial distribution
#'
#' Probability mass function and random generation
#' for the beta-negative binomial distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param alpha,beta      non-negative parameters of the beta distribution.
#' @param size            number of trials (zero or more). Must be strictly positive, need not be integer.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' If \eqn{p \sim \mathrm{Beta}(\alpha, \beta)}{p ~ Beta(\alpha, \beta)} and
#' \eqn{X \sim \mathrm{NegBinomial}(r, p)}{X ~ NegBinomial(r, p)}, then 
#' \eqn{X \sim \mathrm{BetaNegBinomial}(r, \alpha, \beta)}{X ~ BetaNegBinomial(r, \alpha, \beta)}.
#'
#' Probability mass function
#' \deqn{
#' f(x) = \frac{\Gamma(r+x)}{x! \,\Gamma(r)}
#'        \frac{\mathrm{B}(\alpha+r, \beta+x)}{\mathrm{B}(\alpha, \beta)}
#' }{
#' f(x) = \Gamma(r+x)/(x! \Gamma(r)) * B(\alpha+r, \beta+x) / B(\alpha, \beta)
#' }
#'
#' Cumulative distribution function is calculated using recursive algorithm that employs the fact that
#' \eqn{\Gamma(x) = (x - 1)!} and
#' \eqn{
#' \mathrm{B}(x, y) = \frac{\Gamma(x)\Gamma(y)}{\Gamma(x+y)}
#' }{
#' B(x, y) = (\Gamma(x)\Gamma(y))/\Gamma(x+y)
#' }. This enables re-writing probability mass function as
#' 
#' \deqn{
#' f(x) = \frac{(r+x-1)!}{x! \, \Gamma(r)} \frac{\frac{(\alpha+r-1)!\,(\beta+x-1)!}{(\alpha+\beta+r+x-1)!}}{\mathrm{B}(\alpha,\beta)}
#' }{
#' f(x) = ((r+x-1)!)/(x!*\Gamma(r))*(((\alpha+r-1)!*(\beta+x-1)!)/((\alpha+\beta+r+x-1)!))/B(\alpha,\beta)
#' }
#' 
#' what makes recursive updating from \eqn{x} to \eqn{x+1} easy using the properties of factorials
#' 
#' \deqn{
#' f(x+1) = \frac{(r+x-1)!\,(r+x)}{x!\,(x+1) \, \Gamma(r)} \frac{\frac{(\alpha+r-1)!\,(\beta+x-1)!\,(\beta+x)}{(\alpha+\beta+r+x-1)!\,(\alpha+\beta+r+x)}}{\mathrm{B}(\alpha,\beta)}
#' }{
#' f(x+1) = ((r+x-1)!*(r+x))/(x!*(x+1)*\Gamma(r))*(((\alpha+r-1)!*(\beta+x-1)!*(\beta+x))/((\alpha+\beta+r+x-1)!*(\alpha+\beta+r+x)))/B(\alpha,\beta)
#' }
#' 
#' and let's us efficiently calculate cumulative distribution function as a sum of probability mass functions
#' 
#' \deqn{F(x) = \sum_{k=0}^x f(k)}{F(x) = f(0)+...+f(x)}
#' 
#'
#' @seealso \code{\link[stats]{Beta}}, \code{\link[stats]{NegBinomial}}
#' 
#' @examples 
#' 
#' x <- rbnbinom(1e5, 1000, 5, 13)
#' xx <- 0:1e5
#' hist(x, 100, freq = FALSE)
#' lines(xx-0.5, dbnbinom(xx, 1000, 5, 13), col = "red")
#' hist(pbnbinom(x, 1000, 5, 13))
#' xx <- seq(0, 1e5, by = 0.1)
#' plot(ecdf(x))
#' lines(xx, pbnbinom(xx, 1000, 5, 13), col = "red", lwd = 2)
#'
#' @name BetaNegBinom
#' @aliases BetaNegBinom
#' @aliases dbnbinom
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

dbnbinom <- function(x, size, alpha = 1, beta = 1, log = FALSE) {
  cpp_dbnbinom(x, size, alpha, beta, log[1L])
}


#' @rdname BetaNegBinom
#' @export

pbnbinom <- function(q, size, alpha = 1, beta = 1, lower.tail = TRUE, log.p = FALSE) {
  cpp_pbnbinom(q, size, alpha, beta, lower.tail[1L], log.p[1L])
}


#' @rdname BetaNegBinom
#' @export

rbnbinom <- function(n, size, alpha = 1, beta = 1) {
  if (length(n) > 1) n <- length(n)
  cpp_rbnbinom(n, size, alpha, beta)
}

#' Random generation from Rademacher distribution
#'
#' Random generation for the Rademacher distribution (values -1 and +1
#' with equal probability).
#'
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#'
#' @name Rademacher
#' @aliases Rademacher
#' @aliases rsign
#'
#' @keywords distribution
#' 
#' @export

rsign <- function(n) {
  if (length(n) > 1) n <- length(n)
  cpp_rsign(n)
}

#' Generalized Pareto distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the generalized Pareto distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param mu,sigma,xi	    location, scale, and shape parameters. Scale must be positive.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#'
#' @details
#'
#' Probability density function
#' \deqn{ f(x) = \left\{\begin{array}{ll}
#' \frac{1}{\sigma} \left(1+\xi \frac{x-\mu}{\sigma}\right)^{-(\xi+1)/\xi} & \xi \neq 0 \\
#' \frac{1}{\sigma} \exp\left(-\frac{x-\mu}{\sigma}\right)                 & \xi = 0
#' \end{array}\right.
#' }{
#' f(x) = [if \xi != 0:] (1+\xi*(x-\mu)/\sigma)^{-(\xi+1)/\xi}/\sigma
#'        [else:] exp(-(x-\mu)/\sigma)/\sigma
#' }
#'
#' Cumulative distribution function
#' \deqn{ F(x) = \left\{\begin{array}{ll}
#' 1-\left(1+\xi \frac{x-\mu}{\sigma}\right)^{-1/\xi} & \xi \neq 0 \\
#' 1-\exp\left(-\frac{x-\mu}{\sigma}\right)           & \xi = 0
#' \end{array}\right.
#' }{
#' F(x) = [if \xi != 0:] 1-(1+\xi*(x-\mu)/\sigma)^{-1/\xi}
#'        [else:] 1-exp(-(x-\mu)/\sigma)
#' }
#'
#' Quantile function
#' \deqn{ F^{-1}(x) = \left\{\begin{array}{ll}
#' \mu + \sigma \frac{(1-p)^{-\xi}-1}{\xi} & \xi \neq 0 \\
#' \mu - \sigma \log(1-p)                  & \xi = 0
#' \end{array}\right.
#' }{
#' F^-1(x) = [if \xi != 0:] \mu + \sigma * ((1-p)^{-\xi}-1)/\xi
#'           [else:] \mu - \sigma * log(1-p)
#' }
#'
#' @references
#' Coles, S. (2001). An Introduction to Statistical Modeling of Extreme Values.
#' Springer.
#' 
#' @examples 
#' 
#' x <- rgpd(1e5, 5, 2, .1)
#' hist(x, 100, freq = FALSE, xlim = c(0, 50))
#' curve(dgpd(x, 5, 2, .1), 0, 50, col = "red", add = TRUE, n = 5000)
#' hist(pgpd(x, 5, 2, .1))
#' plot(ecdf(x))
#' curve(pgpd(x, 5, 2, .1), 0, 50, col = "red", lwd = 2, add = TRUE)
#'
#' @name GPD
#' @aliases GPD
#' @aliases dgpd
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Continuous
#'
#' @export

dgpd <- function(x, mu = 0, sigma = 1, xi = 0, log = FALSE) {
  cpp_dgpd(x, mu, sigma, xi, log[1L])
}


#' @rdname GPD
#' @export

pgpd <- function(q, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_pgpd(q, mu, sigma, xi, lower.tail[1L], log.p[1L])
}


#' @rdname GPD
#' @export

qgpd <- function(p, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE) {
  cpp_qgpd(p, mu, sigma, xi, lower.tail[1L], log.p[1L])
}


#' @rdname GPD
#' @export

rgpd <- function(n, mu = 0, sigma = 1, xi = 0) {
  if (length(n) > 1) n <- length(n)
  cpp_rgpd(n, mu, sigma, xi)
}

test_that("Zero probabilities for values <0", {
  expect_equal(c(0, 0), dbetapr(c(-1, 0), 1, 1, 1))
  expect_equal(0, ddweibull(-1, 0.5, 1))
  expect_equal(0, ddunif(-1, 0, 5))
  expect_equal(0, dcat(-1, c(0.5, 0.5)))
  expect_equal(0, dmnom(c(-1, 1), 5, c(0.5, 0.5)))
  expect_warning(expect_equal(0, dbern(-1)))
  expect_equal(0, dbbinom(-1, 1, 1, 1))
  expect_equal(0, dbnbinom(-1, 1, 1, 1))
  expect_equal(0, dgpois(-1, 1, 1))
  expect_equal(0, dmvhyper(c(1, 1, -1), c(2, 2, 2), 3))
  expect_equal(0, dzip(-1, 1, 0.5))
  expect_equal(0, dzib(-1, 1, 0.5, 0.5))
  expect_equal(0, dzinb(-1, 1, 0.5, 0.5))

  expect_equal(0, dbvpois(-1, 1, 1, 1, 1))
  expect_equal(0, dbvpois(1, -1, 1, 1, 1))

  expect_equal(0, dinvchisq(-1, 1))
  expect_equal(0, dinvchisq(-1, 1, 1))
  expect_equal(0, dinvgamma(-1, 1, 1))
  expect_equal(0, dgompertz(-1, 1, 1))
  expect_equal(0, dgpois(-1, 1, 1))
  expect_equal(0, dlomax(-1, 1, 1))
  expect_equal(0, dpower(-1, 1, 0.5))
  expect_equal(0, dsgomp(-1, 0.4, 1))
  expect_equal(0, drayleigh(-1, 1))
  expect_equal(0, dwald(-1, 1, 1))

  expect_equal(0, dhcauchy(-1, 1))
  expect_equal(0, dhnorm(-1, 1))
  expect_equal(0, dht(-1, 5, 1))

  expect_equal(0, dmixpois(-1, c(1, 2, 3), c(1 / 3, 1 / 3, 1 / 3)))
  expect_equal(0, dtpois(-1, lambda = 5, a = 6))

  expect_equal(0, dnhyper(-1, 60, 35, 15))

  expect_equal(0, ddgamma(-1, 9, 1))
})

test_that("Zero probabilities for values x < mean", {
  expect_equal(0, dfatigue(-1, 1, 1, 0))
  expect_equal(0, dfrechet(-1, 1, 0, 1))
  expect_equal(0, dgpd(-1, 0, 1, 1))
})

test_that("Zero probabilities for values < 1", {
  expect_equal(c(0, 0), dlgser(c(-1, 0), 0.5))
  expect_equal(c(0, 0), dpareto(c(-1, 0), 1, 1))
  expect_equal(c(0, 0), dtpois(c(-1, 0), lambda = 5, a = 0))
})

test_that("No negative samples for distributions with positive support", {
  set.seed(42)
  expect_true(all(rgpd(1000, xi = 0) > 0))
  expect_true(all(rgpd(1000, xi = 0.1) > 0))
  expect_true(all(rgpd(1000, xi = -0.1) > 0))
})

#' Discrete uniform distribution
#'
#' Probability mass function, distribution function, quantile function and random generation
#' for the discrete uniform distribution.
#'
#' @param x,q	            vector of quantiles.
#' @param p	              vector of probabilities.
#' @param n	              number of observations. If \code{length(n) > 1},
#'                        the length is taken to be the number required.
#' @param min,max         lower and upper limits of the distribution. Must be finite.
#' @param log,log.p	      logical; if TRUE, probabilities p are given as log(p).
#' @param lower.tail	    logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
#'                        otherwise, \eqn{P[X > x]}.
#' 
#' @details 
#' 
#' If \code{min == max}, then discrete uniform distribution is a degenerate distribution.
#'                                           
#' @examples 
#' 
#' x <- rdunif(1e5, 1, 10) 
#' xx <- -1:11
#' plot(prop.table(table(x)), type = "h")
#' lines(xx, ddunif(xx, 1, 10), col = "red")
#' hist(pdunif(x, 1, 10))
#' xx <- seq(-1, 11, by = 0.01)
#' plot(ecdf(x))
#' lines(xx, pdunif(xx, 1, 10), col = "red")
#'
#' @name DiscreteUniform
#' @aliases DiscreteUniform
#' @aliases ddunif
#' 
#' @keywords distribution
#' @concept Univariate
#' @concept Discrete
#'
#' @export

ddunif <- function(x, min, max, log = FALSE) {
  cpp_ddunif(x, min, max, log[1L])
}


#' @rdname DiscreteUniform
#' @export

pdunif <- function(q, min, max, lower.tail = TRUE, log.p = FALSE) {
  cpp_pdunif(q, min, max, lower.tail[1L], log.p[1L])
}


#' @rdname DiscreteUniform
#' @export

qdunif <- function(p, min, max, lower.tail = TRUE, log.p = FALSE) {
  cpp_qdunif(p, min, max, lower.tail[1L], log.p[1L])
}


#' @rdname DiscreteUniform
#' @export

rdunif <- function(n, min, max) {
  if (length(n) > 1) n <- length(n)
  cpp_rdunif(n, min, max)
}

# Tests from 'tests/p-r-random-tests.R' from base R

superror <- function(rfoo,pfoo,sample.size,...) {
  x <- rfoo(sample.size,...)
  tx <- table(signif(x, 12)) # such that xi will be sort(unique(x))
  xi <- as.numeric(names(tx))
  f <- pfoo(xi,...)
  fhat <- cumsum(tx)/sample.size
  max(abs(fhat-f))
}

pdkwbound <- function(n,t) 2*exp(-2*n*t*t)

qdkwbound <- function(n,p) sqrt(log(p/2)/(-2*n))

dkwtest <- function(stub = "norm", ...,
                    sample.size = 10000, pthreshold = 0.001,
                    print.result = FALSE, print.detail = FALSE, # don't print by default
                    stop.on.failure = FALSE)                    # don't stop by default
{
  rfoo <- eval(as.name(paste0("r", stub)))
  pfoo <- eval(as.name(paste0("p", stub)))
  s <- superror(rfoo, pfoo, sample.size, ...)
  if (print.result || print.detail) {
    printargs <- substitute(list(...))
    printargs[[1]] <- as.name(stub)
    cat(deparse(printargs))
    if (print.detail)
      cat("\nsupremum error = ",signif(s,2),
          " with p-value=",min(1,round(pdkwbound(sample.size,s),4)),"\n")
  }
  rval <- (s < qdkwbound(sample.size,pthreshold))
  if (print.result)
    cat(c(" FAILED\n"," PASSED\n")[rval+1])
  if (stop.on.failure && !rval)
    stop("dkwtest failed")
  rval
}


test_that("p-r random tests", {
  
  skip_on_cran()
  
  expect_true(dkwtest("bbinom", 1, 1, 1))
  expect_true(dkwtest("bbinom", 10, 1, 1))
  expect_true(dkwtest("bbinom", 10, 100, 1))
  expect_true(dkwtest("bbinom", 10, 1, 100))
  expect_true(dkwtest("bbinom", 100, 1, 1))
  expect_true(dkwtest("bbinom", 100, 100, 1))
  expect_true(dkwtest("bbinom", 100, 1, 100))
  
  expect_true(dkwtest("bern", 0.5))
  
  expect_true(dkwtest("betapr", 1, 1, 1))
  
  expect_true(dkwtest("bhatt", 1, 1, 1))
  
  expect_true(dkwtest("bnbinom", 1, 1, 1))
  expect_true(dkwtest("bnbinom", 10, 1, 1))
  expect_true(dkwtest("bnbinom", 10, 100, 1))
  
  expect_true(dkwtest("cat", c(0.5, 0.5)))
  expect_true(dkwtest("cat", c(1e-6, 1, 2.5, 100)))
  
  expect_true(dkwtest("dlaplace", 1, 0.001))
  expect_true(dkwtest("dlaplace", 0, 0.5))
  expect_true(dkwtest("dlaplace", 0, 0.999))
  
  expect_true(dkwtest("dnorm", 1, 1))
  
  expect_true(dkwtest("dgamma", 9, 1))
  
  expect_true(dkwtest("dunif", 1, 10))
  
  expect_true(dkwtest("dweibull", 0.5, 1))
  expect_true(dkwtest("dweibull", 0.001, 1))
  expect_true(dkwtest("dweibull", 0.999, 1))
  
  expect_true(dkwtest("fatigue", 1, 1))
  
  expect_true(dkwtest("frechet", 1, 1, 1))
  
  expect_true(dkwtest("gev", 1, 1, 1))
  
  expect_true(dkwtest("gompertz", 1, 1))
  
  expect_true(dkwtest("gpd", 1, 1, 1))
  
  expect_true(dkwtest("gpois", 1, 1))
  
  expect_true(dkwtest("gumbel", 1, 1))
  
  expect_true(dkwtest("hcauchy", 1))
  
  expect_true(dkwtest("hnorm", 1))
  
  expect_true(dkwtest("ht", 5, 1))
  
  expect_true(dkwtest("huber", 0, 1, 1))
  
  expect_true(dkwtest("invgamma", 1, 1))
  
  expect_true(dkwtest("invchisq", 1, 1))
  
  expect_true(dkwtest("kumar", 1, 1))
  expect_true(dkwtest("kumar", 100, 1))
  expect_true(dkwtest("kumar", 1, 100))
  expect_true(dkwtest("kumar", 100, 100))
  
  expect_true(dkwtest("laplace", 0, 1))
  expect_true(dkwtest("laplace", 0, 1000))
  
  expect_true(dkwtest("lgser", 0.001))
  expect_true(dkwtest("lgser", 0.5))
  expect_true(dkwtest("lgser", 0.999))
  
  expect_true(dkwtest("lomax", 1, 0.001))
  expect_true(dkwtest("lomax", 1, 0.5))
  expect_true(dkwtest("lomax", 1, 0.999))
  
  expect_true(dkwtest("mixnorm", c(1,2,3), c(1,2,3), c(1/3,1/3,1/3)))
  
  expect_true(dkwtest("mixpois", c(1,2,3), c(1/3,1/3,1/3)))
  
  expect_true(dkwtest("nhyper", 60, 35, 15))
  expect_true(dkwtest("nhyper", 1, 100, 15))
  expect_true(dkwtest("nhyper", 5, 5, 4))
  expect_true(dkwtest("nhyper", 1000, 5, 4))
  
  expect_true(dkwtest("nsbeta", 1, 1, -2, 2))
  
  expect_true(dkwtest("lst", 2, 0, 1))
  
  expect_true(dkwtest("pareto", 1, 1))
  
  expect_true(dkwtest("power", 1, 1))
  expect_true(dkwtest("power", 5, 16))
  
  expect_true(dkwtest("prop", 10, 0.5))
  expect_true(dkwtest("prop", 100, 0.5))
  expect_true(dkwtest("prop", 1000, 0.5))
  expect_true(dkwtest("prop", 10, 0.01))
  expect_true(dkwtest("prop", 10, 0.5))
  expect_true(dkwtest("prop", 10, 0.99))
  
  expect_true(dkwtest("rayleigh", 1))
  
  expect_true(dkwtest("sgomp", 0.4, 1))
  
  expect_true(dkwtest("slash", 1, 1))

  expect_true(dkwtest("tbinom", 200, 0.5, a = 100))
  expect_true(dkwtest("tbinom", 200, 0.5, b = 100))
  
  expect_true(dkwtest("tnorm", 0, 1, -1, 1))
  expect_true(dkwtest("tnorm", 0, 1, -2, 2))
  expect_true(dkwtest("tnorm", 0, 1, 2, Inf))
  expect_true(dkwtest("tnorm", 0, 1, 4, Inf))
  expect_true(dkwtest("tnorm", 0, 1, -Inf, -2))
  expect_true(dkwtest("tnorm", 0, 1, -Inf, -4))
  expect_true(dkwtest("tnorm", 0, 1, -6, -4))
  expect_true(dkwtest("tnorm", 0, 1, 4, 6))
  
  expect_true(dkwtest("tpois", 5, 0))
  expect_true(dkwtest("tpois", 50, 45, 55))
  
  expect_true(dkwtest("triang"))
  expect_true(dkwtest("triang", 0, 1, 0.5))
  
  expect_true(dkwtest("wald", 1, 1))
  
  expect_true(dkwtest("zip", 1, 0.5))
  
  expect_true(dkwtest("zib", 1, 0.5, 0.5))
  
  expect_true(dkwtest("zinb", 1, 0.5, 0.5))
  
})

probCoverage <- function(stub, ..., n = 5000L) {
  rfoo <- eval(as.name(paste0("r", stub)))
  pfoo <- eval(as.name(paste0("p", stub)))
  diff(pfoo(range(rfoo(n, ...)), ...))
}


test_that("Coverage of RNG's", {
  
  skip_on_cran()

  expect_gte(probCoverage("betapr", 1, 1, 1), 0.99)
  
  expect_gte(probCoverage("bhatt", 1, 1, 1), 0.99)
  
  expect_gte(probCoverage("fatigue", 1, 1), 0.99)
  
  expect_gte(probCoverage("frechet", 1, 1, 1), 0.99)
  
  expect_gte(probCoverage("gev", 1, 1, 1), 0.99)
  
  expect_gte(probCoverage("gompertz", 1, 1), 0.99)
  
  expect_gte(probCoverage("gpd", 1, 1, 1), 0.99)
  
  expect_gte(probCoverage("gumbel", 1, 1), 0.99)
  
  expect_gte(probCoverage("hcauchy", 1), 0.99)
  
  expect_gte(probCoverage("hnorm", 1), 0.99)
  
  expect_gte(probCoverage("ht", 5, 1), 0.99)
  
  expect_gte(probCoverage("huber", 0, 1, 1), 0.99)
  
  expect_gte(probCoverage("invgamma", 1, 1), 0.99)
  
  expect_gte(probCoverage("invchisq", 1, 1), 0.99)
  
  expect_gte(probCoverage("kumar", 1, 1), 0.99)
  expect_gte(probCoverage("kumar", 100, 1), 0.99)
  expect_gte(probCoverage("kumar", 1, 100), 0.99)
  expect_gte(probCoverage("kumar", 100, 100), 0.99)
  
  expect_gte(probCoverage("laplace", 0, 1), 0.99)
  expect_gte(probCoverage("laplace", 0, 1000), 0.99)
  
  expect_gte(probCoverage("lomax", 1, 0.001), 0.99)
  expect_gte(probCoverage("lomax", 1, 0.5), 0.99)
  expect_gte(probCoverage("lomax", 1, 0.999), 0.99)
  
  expect_gte(probCoverage("mixnorm", c(1,2,3), c(1,2,3), c(1/3,1/3,1/3)), 0.99)
  
  expect_gte(probCoverage("nsbeta", 1, 1, -2, 2), 0.99)
  
  expect_gte(probCoverage("lst", 2, 0, 1), 0.99)
  
  expect_gte(probCoverage("pareto", 1, 1), 0.99)
  
  expect_gte(probCoverage("prop", 10, 0.5), 0.99)
  expect_gte(probCoverage("prop", 100, 0.5), 0.99)
  expect_gte(probCoverage("prop", 1000, 0.5), 0.99)
  expect_gte(probCoverage("prop", 10, 0.01), 0.99)
  expect_gte(probCoverage("prop", 10, 0.5), 0.99)
  expect_gte(probCoverage("prop", 10, 0.99), 0.99)
  
  expect_gte(probCoverage("rayleigh", 1), 0.99)
  
  expect_gte(probCoverage("sgomp", 0.4, 1), 0.99)
  
  expect_gte(probCoverage("slash", 1, 1), 0.99)
  
  expect_gte(probCoverage("tnorm", 0, 1, -1, 1), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, -2, 2), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, 2, Inf), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, 4, Inf), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, -Inf, -2), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, -Inf, -4), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, -6, -4), 0.99)
  expect_gte(probCoverage("tnorm", 0, 1, 4, 6), 0.99)
  
  expect_gte(probCoverage("triang"), 0.99)
  expect_gte(probCoverage("triang", 0, 1, 0.5), 0.99)
  
  expect_gte(probCoverage("wald", 1, 1), 0.99)
  
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lomax-distribution.R
\name{Lomax}
\alias{Lomax}
\alias{dlomax}
\alias{plomax}
\alias{qlomax}
\alias{rlomax}
\title{Lomax distribution}
\usage{
dlomax(x, lambda, kappa, log = FALSE)

plomax(q, lambda, kappa, lower.tail = TRUE, log.p = FALSE)

qlomax(p, lambda, kappa, lower.tail = TRUE, log.p = FALSE)

rlomax(n, lambda, kappa)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{lambda, kappa}{positive valued parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Lomax distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{\lambda \kappa}{(1+\lambda x)^{\kappa+1}}
}{
f(x) = \lambda*\kappa / (1+\lambda*x)^(\kappa+1)
}

Cumulative distribution function
\deqn{
F(x) = 1-(1+\lambda x)^{-\kappa}
}{
F(x) = 1-(1+\lambda*x)^-\kappa
}

Quantile function
\deqn{
F^{-1}(p) = \frac{(1-p)^{-1/\kappa} -1}{\lambda}
}{
F^-1(p) = ((1-p)^(-1/\kappa)-1) / \lambda
}
}
\examples{

x <- rlomax(1e5, 5, 16)
hist(x, 100, freq = FALSE)
curve(dlomax(x, 5, 16), 0, 1, col = "red", add = TRUE, n = 5000)
hist(plomax(x, 5, 16))
plot(ecdf(x))
curve(plomax(x, 5, 16), 0, 1, col = "red", lwd = 2, add = TRUE)

}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kumaraswamy-distribution.R
\name{Kumaraswamy}
\alias{Kumaraswamy}
\alias{dkumar}
\alias{pkumar}
\alias{qkumar}
\alias{rkumar}
\title{Kumaraswamy distribution}
\usage{
dkumar(x, a = 1, b = 1, log = FALSE)

pkumar(q, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE)

qkumar(p, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE)

rkumar(n, a = 1, b = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{a, b}{positive valued parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Kumaraswamy distribution.
}
\details{
Probability density function
\deqn{
f(x) = abx^{a-1} (1-x^a)^{b-1}
}{
f(x) = a*b*x^(a-1)*(1-x^a)^(b-1)
}

Cumulative distribution function
\deqn{
F(x) = 1-(1-x^a)^b
}{
F(x) = 1-(1-x^a)^b
}

Quantile function
\deqn{
F^{-1}(p) = 1-(1-p^{1/b})^{1/a}
}{
F^-1(p) = 1-(1-p^(1/b))^(1/a)
}
}
\examples{

x <- rkumar(1e5, 5, 16)
hist(x, 100, freq = FALSE)
curve(dkumar(x, 5, 16), 0, 1, col = "red", add = TRUE)
hist(pkumar(x, 5, 16))
plot(ecdf(x))
curve(pkumar(x, 5, 16), 0, 1, col = "red", lwd = 2, add = TRUE)

}
\references{
Jones, M. C. (2009). Kumaraswamy's distribution: A beta-type distribution with
some tractability advantages. Statistical Methodology, 6, 70-81.

Cordeiro, G.M. and de Castro, M. (2009). A new family of generalized distributions.
Journal of Statistical Computation & Simulation, 1-17.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inverse-chi-squared-distribution.R
\name{InvChiSq}
\alias{InvChiSq}
\alias{dinvchisq}
\alias{pinvchisq}
\alias{qinvchisq}
\alias{rinvchisq}
\title{Inverse chi-squared and scaled chi-squared distributions}
\usage{
dinvchisq(x, nu, tau, log = FALSE)

pinvchisq(q, nu, tau, lower.tail = TRUE, log.p = FALSE)

qinvchisq(p, nu, tau, lower.tail = TRUE, log.p = FALSE)

rinvchisq(n, nu, tau)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{nu}{positive valued shape parameter.}

\item{tau}{positive valued scaling parameter; if provided it
returns values for scaled chi-squared distributions.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function and random generation
for the inverse chi-squared distribution and scaled chi-squared distribution.
The implementation uses a non-standard linear scale parameterization of \eqn{\tau}; 
see Details for the precise definition and its relation to the standard \eqn{\tau^2} formulation.
}
\details{
If \eqn{X} follows \eqn{\chi^2 (\nu)} distribution, then \eqn{1/X} follows inverse
chi-squared distribution parametrized by \eqn{\nu}. Inverse chi-squared distribution
is a special case of inverse gamma distribution with parameters
\eqn{\alpha=\frac{\nu}{2}}{\alpha=\nu/2} and \eqn{\beta=\frac{1}{2}}{\beta=1/2};
or \eqn{\alpha=\frac{\nu}{2}}{\alpha=\nu/2} and
\eqn{\beta=\frac{\nu\tau}{2}}{\beta=(\nu\tau)/2}.

\strong{Note on parameterization:} this implementation uses a linear scaling parameter \eqn{\tau}. 
The scaled inverse chi-squared distribution is defined via the equivalent inverse-gamma form

\deqn{X \sim \text{InvGamma}\left(\frac{\nu}{2}, \frac{\nu\tau}{2}\right),}

where \eqn{\tau} enters linearly in the second parameter.

This differs from the most common statistical convention, which uses a squared scale parameter \eqn{\tau^2}:
  
  \deqn{X \sim \text{InvGamma}\left(\frac{\nu}{2}, \frac{\nu\tau^2}{2}\right),}

The two parameterizations are related by

\deqn{\tau_{this implementation} = \tau_{standard}^2.}

Users familiar with the standard scaled inverse chi-squared distribution should be aware 
that \eqn{\tau} here corresponds to a variance-scale parameter rather than a 
standard-deviation-type parameter.
}
\examples{

x <- rinvchisq(1e5, 20)
hist(x, 100, freq = FALSE)
curve(dinvchisq(x, 20), 0, 1, n = 501, col = "red", add = TRUE)
hist(pinvchisq(x, 20))
plot(ecdf(x))
curve(pinvchisq(x, 20), 0, 1, n = 501, col = "red", lwd = 2, add = TRUE)

# scaled

x <- rinvchisq(1e5, 10, 5)
hist(x, 100, freq = FALSE)
curve(dinvchisq(x, 10, 5), 0, 150, n = 501, col = "red", add = TRUE)
hist(pinvchisq(x, 10, 5))
plot(ecdf(x))
curve(pinvchisq(x, 10, 5), 0, 150, n = 501, col = "red", lwd = 2, add = TRUE)

}
\seealso{
\code{\link[stats]{Chisquare}}, \code{\link[stats]{GammaDist}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dirichlet-distribution.R
\name{Dirichlet}
\alias{Dirichlet}
\alias{ddirichlet}
\alias{rdirichlet}
\title{Dirichlet distribution}
\usage{
ddirichlet(x, alpha, log = FALSE)

rdirichlet(n, alpha)
}
\arguments{
\item{x}{\eqn{k}-column matrix of quantiles.}

\item{alpha}{\eqn{k}-values vector or \eqn{k}-column matrix;
concentration parameter. Must be positive.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density function and random generation for the Dirichlet distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{\Gamma(\sum_k \alpha_k)}{\prod_k \Gamma(\alpha_k)} \prod_k x_k^{\alpha_k-1}
}{
f(x) = \Gamma(sum(\alpha[k])) / prod(\Gamma(\alpha[k])) * prod(x[k]^{\alpha[k]-1})
}
}
\examples{

# Generating 10 random draws from Dirichlet distribution
# parametrized using a vector

rdirichlet(10, c(1, 1, 1, 1))

# or parametrized using a matrix where each row
# is a vector of parameters

alpha <- matrix(c(1, 1, 1, 1:3, 7:9), ncol = 3, byrow = TRUE)
rdirichlet(10, alpha)

}
\references{
Devroye, L. (1986). Non-Uniform Random Variate Generation. Springer-Verlag.
}
\concept{Continuous}
\concept{Multivariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixture-of-normal-distributions.R
\name{NormalMix}
\alias{NormalMix}
\alias{dmixnorm}
\alias{pmixnorm}
\alias{rmixnorm}
\title{Mixture of normal distributions}
\usage{
dmixnorm(x, mean, sd, alpha, log = FALSE)

pmixnorm(q, mean, sd, alpha, lower.tail = TRUE, log.p = FALSE)

rmixnorm(n, mean, sd, alpha)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mean}{matrix (or vector) of means.}

\item{sd}{matrix (or vector) of standard deviations.}

\item{alpha}{matrix (or vector) of mixing proportions;
mixing proportions need to sum up to 1.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}

\item{p}{vector of probabilities.}
}
\description{
Density, distribution function and random generation
for the mixture of normal distributions.
}
\details{
Probability density function
\deqn{
f(x) = \alpha_1 f_1(x; \mu_1, \sigma_1) + \dots + \alpha_k f_k(x; \mu_k, \sigma_k)
}{
f(x) = \alpha[1] * f1(x; \mu[1], \sigma[1]) + \dots + \alpha[k] * fk(x; \mu[k], \sigma[k])
}

Cumulative distribution function
\deqn{
F(x) = \alpha_1 F_1(x; \mu_1, \sigma_1) + \dots + \alpha_k F_k(x; \mu_k, \sigma_k)
}{
F(x) = \alpha[1] * F1(x; \mu[1], \sigma[1]) + \dots + \alpha[k] * Fk(x; \mu[k], \sigma[k])
}

where \eqn{\sum_i \alpha_i = 1}{sum(\alpha[i]) == 1}.
}
\examples{

x <- rmixnorm(1e5, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3))
hist(x, 100, freq = FALSE)
curve(dmixnorm(x, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3)),
      -20, 20, n = 500, col = "red", add = TRUE)
hist(pmixnorm(x, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3)))
plot(ecdf(x))
curve(pmixnorm(x, c(0.5, 3, 6), c(3, 1, 1), c(1/3, 1/3, 1/3)),
      -20, 20, n = 500, col = "red", lwd = 2, add = TRUE)

}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tukey-lambda-distribution.R
\name{TukeyLambda}
\alias{TukeyLambda}
\alias{qtlambda}
\alias{rtlambda}
\title{Tukey lambda distribution}
\usage{
qtlambda(p, lambda, lower.tail = TRUE, log.p = FALSE)

rtlambda(n, lambda)
}
\arguments{
\item{p}{vector of probabilities.}

\item{lambda}{shape parameter.}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Quantile function, and random generation for the Tukey lambda
distribution.
}
\details{
Tukey lambda distribution is a continuous probability distribution defined in terms
of its quantile function. It is typically used to identify other distributions.

Quantile function:

\deqn{F^{-1}(p) = \left\{\begin{array}{ll}
\frac{1}{\lambda} [p^\lambda - (1-p)^\lambda] & \lambda \ne 0 \\
\log(\frac{p}{1-p}) & \lambda = 0
\end{array}\right.
}{
F^-1(p) = [if \lambda != 0:] (p^\lambda - (1-p)^\lambda)/\lambda
[if \lambda = 0:] log(p/(1-p))
}
}
\examples{

pp = seq(0, 1, by = 0.001)
partmp <- par(mfrow = c(2,3))
plot(qtlambda(pp, -1), pp, type = "l", main = "lambda = -1 (Cauchy)")
plot(qtlambda(pp, 0), pp, type = "l", main = "lambda = 0 (logistic)")
plot(qtlambda(pp, 0.14), pp, type = "l", main = "lambda = 0.14 (normal)")
plot(qtlambda(pp, 0.5), pp, type = "l", main = "lambda = 0.5 (concave)")
plot(qtlambda(pp, 1), pp, type = "l", main = "lambda = 1 (uniform)")
plot(qtlambda(pp, 2), pp, type = "l", main = "lambda = 2 (uniform)")

hist(rtlambda(1e5, -1), freq = FALSE, main = "lambda = -1 (Cauchy)")
hist(rtlambda(1e5, 0), freq = FALSE, main = "lambda = 0 (logistic)")
hist(rtlambda(1e5, 0.14), freq = FALSE, main = "lambda = 0.14 (normal)")
hist(rtlambda(1e5, 0.5), freq = FALSE, main = "lambda = 0.5 (concave)")
hist(rtlambda(1e5, 1), freq = FALSE, main = "lambda = 1 (uniform)")
hist(rtlambda(1e5, 2), freq = FALSE, main = "lambda = 2 (uniform)")
par(partmp)

}
\references{
Joiner, B.L., & Rosenblatt, J.R. (1971).
Some properties of the range in samples from Tukey's symmetric lambda distributions.
Journal of the American Statistical Association, 66(334), 394-399.

Hastings Jr, C., Mosteller, F., Tukey, J.W., & Winsor, C.P. (1947).
Low moments for small samples: a comparative study of order statistics.
The Annals of Mathematical Statistics, 413-426.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/non-standard-beta-distribution.R
\name{NSBeta}
\alias{NSBeta}
\alias{dnsbeta}
\alias{dgbeta}
\alias{pnsbeta}
\alias{qnsbeta}
\alias{rnsbeta}
\title{Non-standard beta distribution}
\usage{
dnsbeta(x, shape1, shape2, min = 0, max = 1, log = FALSE)

pnsbeta(q, shape1, shape2, min = 0, max = 1, lower.tail = TRUE, log.p = FALSE)

qnsbeta(p, shape1, shape2, min = 0, max = 1, lower.tail = TRUE, log.p = FALSE)

rnsbeta(n, shape1, shape2, min = 0, max = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{shape1, shape2}{non-negative parameters of the Beta distribution.}

\item{min, max}{lower and upper bounds.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \leq x]},
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Non-standard form of beta distribution with lower and upper bounds
denoted as \code{min} and \code{max}. By default \code{min=0}
and \code{max=1} what leads to standard beta distribution.
}
\examples{

x <- rnsbeta(1e5, 5, 13, -4, 8)
hist(x, 100, freq = FALSE)
curve(dnsbeta(x, 5, 13, -4, 8), -4, 6, col = "red", add = TRUE) 
hist(pnsbeta(x, 5, 13, -4, 8))
plot(ecdf(x))
curve(pnsbeta(x, 5, 13, -4, 8), -4, 6, col = "red", lwd = 2, add = TRUE)

}
\seealso{
\code{\link[stats]{Beta}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixture-of-poisson-distributions.R
\name{PoissonMix}
\alias{PoissonMix}
\alias{dmixpois}
\alias{pmixpois}
\alias{rmixpois}
\title{Mixture of Poisson distributions}
\usage{
dmixpois(x, lambda, alpha, log = FALSE)

pmixpois(q, lambda, alpha, lower.tail = TRUE, log.p = FALSE)

rmixpois(n, lambda, alpha)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{lambda}{matrix (or vector) of (non-negative) means.}

\item{alpha}{matrix (or vector) of mixing proportions;
mixing proportions need to sum up to 1.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}

\item{p}{vector of probabilities.}
}
\description{
Density, distribution function and random generation
for the mixture of Poisson distributions.
}
\details{
Probability density function
\deqn{
f(x) = \alpha_1 f_1(x; \lambda_1) + \dots + \alpha_k f_k(x; \lambda_k)
}{
f(x) = \alpha[1] * f1(x; \lambda[1]) + \dots + \alpha[k] * fk(x; \lambda[k])
}

Cumulative distribution function
\deqn{
F(x) = \alpha_1 F_1(x; \lambda_1) + \dots + \alpha_k F_k(x; \lambda_k)
}{
F(x) = \alpha[1] * F1(x; \lambda[1]) + \dots + \alpha[k] * Fk(x; \lambda[k])
}

where \eqn{\sum_i \alpha_i = 1}{sum(\alpha[i]) == 1}.
}
\examples{

x <- rmixpois(1e5, c(5, 12, 19), c(1/3, 1/3, 1/3))
xx <- seq(-1, 50)
plot(prop.table(table(x)))
lines(xx, dmixpois(xx, c(5, 12, 19), c(1/3, 1/3, 1/3)), col = "red")
hist(pmixpois(x, c(5, 12, 19), c(1/3, 1/3, 1/3)))

xx <- seq(0, 50, by = 0.01)
plot(ecdf(x))
lines(xx, pmixpois(xx, c(5, 12, 19), c(1/3, 1/3, 1/3)), col = "red", lwd = 2)

}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zero-inflated-negative-binomial-distribution.R
\name{ZINB}
\alias{ZINB}
\alias{dzinb}
\alias{pzinb}
\alias{qzinb}
\alias{rzinb}
\title{Zero-inflated negative binomial distribution}
\usage{
dzinb(x, size, prob, pi, log = FALSE)

pzinb(q, size, prob, pi, lower.tail = TRUE, log.p = FALSE)

qzinb(p, size, prob, pi, lower.tail = TRUE, log.p = FALSE)

rzinb(n, size, prob, pi)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{size}{target for number of successful trials, or dispersion
parameter (the shape parameter of the gamma mixing
distribution). Must be strictly positive, need not be
integer.}

\item{prob}{probability of success in each trial. \code{0 < prob <= 1}.}

\item{pi}{probability of extra zeros.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the zero-inflated negative binomial distribution.
}
\details{
Probability density function
\deqn{
f(x) = \left\{\begin{array}{ll}
\pi + (1 - \pi) p^r & x = 0 \\
(1 - \pi) {x+r-1 \choose x} p^r (1-p)^x & x > 0 \\
\end{array}\right.
}{
f(x) = [if x = 0:] (1-\pi)+\pi * p^r [else:] (1-\pi) * dnbinom(x, size, prob)
}
}
\examples{

x <- rzinb(1e5, 100, 0.6, 0.33)
xx <- -2:200
plot(prop.table(table(x)), type = "h")
lines(xx, dzinb(xx, 100, 0.6, 0.33), col = "red")

xx <- seq(0, 200, by = 0.01)
plot(ecdf(x))
lines(xx, pzinb(xx, 100, 0.6, 0.33), col = "red")

}
\seealso{
\code{\link[stats]{NegBinomial}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-weibull-distribution.R
\name{DiscreteWeibull}
\alias{DiscreteWeibull}
\alias{ddweibull}
\alias{pdweibull}
\alias{qdweibull}
\alias{rdweibull}
\title{Discrete Weibull distribution (type I)}
\usage{
ddweibull(x, shape1, shape2, log = FALSE)

pdweibull(q, shape1, shape2, lower.tail = TRUE, log.p = FALSE)

qdweibull(p, shape1, shape2, lower.tail = TRUE, log.p = FALSE)

rdweibull(n, shape1, shape2)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{shape1, shape2}{parameters (named q, \eqn{\beta}). Values of \code{shape2}
need to be positive and \code{0 < shape1 < 1}.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the discrete Weibull (type I) distribution.
}
\details{
Probability mass function
\deqn{
f(x) = q^{x^\beta} - q^{(x+1)^\beta}
}{
f(x) = q^x^\beta - q^(x+1)^\beta
}

Cumulative distribution function
\deqn{
F(x) = 1-q^{(x+1)^\beta}
}{
F(x) = 1-q^(x+1)^\beta
}

Quantile function
\deqn{
F^{-1}(p) = \left \lceil{\left(\frac{\log(1-p)}{\log(q)}\right)^{1/\beta} - 1}\right \rceil
}{
F^-1(p) = ceiling((log(1-p)/log(q))^(1/\beta) - 1)
}
}
\examples{

x <- rdweibull(1e5, 0.32, 1)
xx <- seq(-2, 100, by = 1)
plot(prop.table(table(x)), type = "h")
lines(xx, ddweibull(xx, .32, 1), col = "red")

# Notice: distribution of F(X) is far from uniform:
hist(pdweibull(x, .32, 1), 50)

plot(ecdf(x))
lines(xx, pdweibull(xx, .32, 1), col = "red", lwd = 2, type = "s")

}
\references{
Nakagawa, T. and Osaki, S. (1975). The Discrete Weibull Distribution.
IEEE Transactions on Reliability, R-24, 300-301.

Kulasekera, K.B. (1994).
Approximate MLE's of the parameters of a discrete Weibull distribution
with type I censored data.
Microelectronics Reliability, 34(7), 1185-1188.

Khan, M.A., Khalique, A. and Abouammoh, A.M. (1989).
On estimating parameters in a discrete Weibull distribution.
IEEE Transactions on Reliability, 38(3), 348-350.
}
\seealso{
\code{\link[stats]{Weibull}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rademacher-distribution.R
\name{Rademacher}
\alias{Rademacher}
\alias{rsign}
\title{Random generation from Rademacher distribution}
\usage{
rsign(n)
}
\arguments{
\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Random generation for the Rademacher distribution (values -1 and +1
with equal probability).
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pareto-distribution.R
\name{Pareto}
\alias{Pareto}
\alias{dpareto}
\alias{ppareto}
\alias{qpareto}
\alias{rpareto}
\title{Pareto distribution}
\usage{
dpareto(x, a = 1, b = 1, log = FALSE)

ppareto(q, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE)

qpareto(p, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE)

rpareto(n, a = 1, b = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{a, b}{positive valued scale and location parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Pareto distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{ab^a}{x^{a+1}}
}{
f(x) = (a*b^a) / x^(a+1)
}

Cumulative distribution function
\deqn{
F(x) = 1 - \left(\frac{b}{x}\right)^a
}{
F(x) = 1 - (b/x)^a
}

Quantile function
\deqn{
F^{-1}(p) = \frac{b}{(1-p)^{1-a}}
}{
F^-1(p) = b/(1-p)^(1-a)
}
}
\examples{

x <- rpareto(1e5, 5, 16)
hist(x, 100, freq = FALSE)
curve(dpareto(x, 5, 16), 0, 200, col = "red", add = TRUE)
hist(ppareto(x, 5, 16))
plot(ecdf(x))
curve(ppareto(x, 5, 16), 0, 200, col = "red", lwd = 2, add = TRUE)

}
\references{
Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
with Applications. Chapman & Hall/CRC
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-normal-distribution.R
\name{DiscreteNormal}
\alias{DiscreteNormal}
\alias{ddnorm}
\alias{pdnorm}
\alias{rdnorm}
\title{Discrete normal distribution}
\usage{
ddnorm(x, mean = 0, sd = 1, log = FALSE)

pdnorm(q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)

rdnorm(n, mean = 0, sd = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mean}{vector of means.}

\item{sd}{vector of standard deviations.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function and random generation
for discrete normal distribution.
}
\details{
Probability mass function

\deqn{
f(x) = \Phi\left(\frac{x-\mu+1}{\sigma}\right) - \Phi\left(\frac{x-\mu}{\sigma}\right)
}{
f(x) = \Phi((x-\mu+1)/\sigma) - \Phi((x-\mu)/\sigma)
}

Cumulative distribution function

\deqn{
F(x) = \Phi\left(\frac{\lfloor x \rfloor + 1 - \mu}{\sigma}\right)
}{
F(x) = \Phi((floor(x)+1-\mu)/\sigma)
}
}
\examples{

x <- rdnorm(1e5, 0, 3)
xx <- -15:15
plot(prop.table(table(x)))
lines(xx, ddnorm(xx, 0, 3), col = "red")
hist(pdnorm(x, 0, 3))
plot(ecdf(x))
xx <- seq(-15, 15, 0.1)
lines(xx, pdnorm(xx, 0, 3), col = "red", lwd = 2, type = "s")

}
\references{
Roy, D. (2003). The discrete normal distribution.
Communications in Statistics-Theory and Methods, 32, 1871-1883.
}
\seealso{
\code{\link[stats]{Normal}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinomial-distribution.R
\name{Multinomial}
\alias{Multinomial}
\alias{dmnom}
\alias{rmnom}
\title{Multinomial distribution}
\usage{
dmnom(x, size, prob, log = FALSE)

rmnom(n, size, prob)
}
\arguments{
\item{x}{\eqn{k}-column matrix of quantiles.}

\item{size}{numeric vector; number of trials (zero or more).}

\item{prob}{\eqn{k}-column numeric matrix; probability of success on each trial.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the multinomial distribution.
}
\details{
Probability mass function
\deqn{
f(x) = \frac{n!}{\prod_{i=1}^k x_i} \prod_{i=1}^k p_i^{x_i}
}{
f(x) = n!/prod(x[i]!) * prod(p[i]^x[i])
}
}
\examples{

# Generating 10 random draws from multinomial distribution
# parametrized using a vector

(x <- rmnom(10, 3, c(1/3, 1/3, 1/3)))

# Results are consistent with dmultinom() from stats:

all.equal(dmultinom(x[1,], 3, c(1/3, 1/3, 1/3)),
          dmnom(x[1, , drop = FALSE], 3, c(1/3, 1/3, 1/3)))

}
\references{
Gentle, J.E. (2006). Random number generation and Monte Carlo methods. Springer.
}
\seealso{
\code{\link[stats]{Binomial}}, \code{\link[stats]{Multinomial}}
}
\concept{Discrete}
\concept{Multivariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/truncated-normal-distribution.R
\name{TruncNormal}
\alias{TruncNormal}
\alias{dtnorm}
\alias{ptnorm}
\alias{qtnorm}
\alias{rtnorm}
\title{Truncated normal distribution}
\usage{
dtnorm(x, mean = 0, sd = 1, a = -Inf, b = Inf, log = FALSE)

ptnorm(
  q,
  mean = 0,
  sd = 1,
  a = -Inf,
  b = Inf,
  lower.tail = TRUE,
  log.p = FALSE
)

qtnorm(
  p,
  mean = 0,
  sd = 1,
  a = -Inf,
  b = Inf,
  lower.tail = TRUE,
  log.p = FALSE
)

rtnorm(n, mean = 0, sd = 1, a = -Inf, b = Inf)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mean, sd}{location and scale parameters. Scale must be positive.}

\item{a, b}{lower and upper truncation points (\code{a < x <= b},
with \code{a = -Inf} and \code{b = Inf} by default).}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the truncated normal distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{\phi(\frac{x-\mu}{\sigma})}
            {\Phi(\frac{b-\mu}{\sigma}) - \Phi(\frac{a-\mu}{\sigma})}
}{
f(x) = \phi((x-\mu)/\sigma) / (\Phi((b-\mu)/\sigma) - \Phi((a-\mu)/\sigma))
}

Cumulative distribution function
\deqn{
F(x) = \frac{\Phi(\frac{x-\mu}{\sigma}) - \Phi(\frac{a-\mu}{\sigma})}
            {\Phi(\frac{b-\mu}{\sigma}) - \Phi(\frac{a-\mu}{\sigma})}
}{
F(x) = (\Phi((x-\mu)/\sigma) - \Phi((a-\mu)/\sigma)) / (\Phi((b-\mu)/\sigma) - \Phi((a-\mu)/\sigma))
}

Quantile function
\deqn{
F^{-1}(p) = \Phi^{-1}\left(\Phi\left(\frac{a-\mu}{\sigma}\right) + p \times
                     \left[\Phi\left(\frac{b-\mu}{\sigma}\right) -
                     \Phi\left(\frac{a-\mu}{\sigma}\right)\right]\right)
}{
F^-1(p) = \Phi^-1(\Phi((a-\mu)/\sigma) + p * (\Phi((b-\mu)/\sigma) - \Phi((a-\mu)/\sigma)))
}

For random generation algorithm described by Robert (1995) is used.
}
\examples{

x <- rtnorm(1e5, 5, 3, b = 7)
hist(x, 100, freq = FALSE)
curve(dtnorm(x, 5, 3, b = 7), -8, 8, col = "red", add = TRUE)
hist(ptnorm(x, 5, 3, b = 7))
plot(ecdf(x))
curve(ptnorm(x, 5, 3, b = 7), -8, 8, col = "red", lwd = 2, add = TRUE)

R <- 1e5
partmp <- par(mfrow = c(2,4), mar = c(2,2,2,2))

hist(rtnorm(R), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x), -5, 5, col = "red", add = TRUE)

hist(rtnorm(R, a = 0), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, a = 0), -1, 5, col = "red", add = TRUE)

hist(rtnorm(R, b = 0), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, b = 0), -5, 5, col = "red", add = TRUE)

hist(rtnorm(R, a = 0, b = 1), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, a = 0, b = 1), -1, 2, col = "red", add = TRUE)

hist(rtnorm(R, a = -1, b = 0), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, a = -1, b = 0), -2, 2, col = "red", add = TRUE)

hist(rtnorm(R, mean = -6, a = 0), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, mean = -6, a = 0), -2, 1, col = "red", add = TRUE)

hist(rtnorm(R, mean = 8, b = 0), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, mean = 8, b = 0), -2, 1, col = "red", add = TRUE)

hist(rtnorm(R, a = 3, b = 5), freq= FALSE, main = "", xlab = "", ylab = "")
curve(dtnorm(x, a = 3, b = 5), 2, 5, col = "red", add = TRUE)

par(partmp)

}
\references{
Robert, C.P. (1995). Simulation of truncated normal variables.
Statistics and Computing 5(2): 121-125. \url{https://arxiv.org/abs/0907.4010}

Burkardt, J. (17 October 2014). The Truncated Normal Distribution. Florida State University.
\url{https://people.sc.fsu.edu/~jburkardt/presentations/truncated_normal.pdf}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dirichlet-multinomial-distribution.R
\name{DirMnom}
\alias{DirMnom}
\alias{ddirmnom}
\alias{rdirmnom}
\title{Dirichlet-multinomial (multivariate Polya) distribution}
\usage{
ddirmnom(x, size, alpha, log = FALSE)

rdirmnom(n, size, alpha)
}
\arguments{
\item{x}{\eqn{k}-column matrix of quantiles.}

\item{size}{numeric vector; number of trials (zero or more).}

\item{alpha}{\eqn{k}-values vector or \eqn{k}-column matrix;
concentration parameter. Must be positive.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density function, cumulative distribution function and random generation
for the Dirichlet-multinomial (multivariate Polya) distribution.
}
\details{
If \eqn{(p_1,\dots,p_k) \sim \mathrm{Dirichlet}(\alpha_1,\dots,\alpha_k)}{(p[1],\dots,p[k]) ~ Dirichlet(\alpha[1],\dots,\alpha[k])} and
\eqn{(x_1,\dots,x_k) \sim \mathrm{Multinomial}(n, p_1,\dots,p_k)}{(x[1],\dots,x[k]) ~ Multinomial(n, p[1],\dots,p[k])}, then 
\eqn{(x_1,\dots,x_k) \sim \mathrm{DirichletMultinomial(n, \alpha_1,\dots,\alpha_k)}}{(x[1],\dots,x[k]) ~ DirichletMultinomial(n, \alpha[1],\dots,\alpha[k])}.

Probability density function
\deqn{
f(x) = \frac{\left(n!\right)\Gamma\left(\sum \alpha_k\right)}{\Gamma\left(n+\sum \alpha_k\right)}\prod_{k=1}^K\frac{\Gamma(x_{k}+\alpha_{k})}{\left(x_{k}!\right)\Gamma(\alpha_{k})}
}{
f(x) = (n! * \Gamma(sum(\alpha[k]))) / (\Gamma(n + sum(\alpha[k]))) * prod((\Gamma(x[k] + \alpha[k])) / (x[k]! * \Gamma(\alpha[k]))
}
}
\references{
Gentle, J.E. (2006). Random number generation and Monte Carlo methods. Springer.

Kvam, P. and Day, D. (2001) The multivariate Polya distribution in combat modeling.
Naval Research Logistics, 48, 1-17.
}
\seealso{
\code{\link{Dirichlet}}, \code{\link{Multinomial}}
}
\concept{Discrete}
\concept{Multivariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/skellam-distribution.R
\name{Skellam}
\alias{Skellam}
\alias{dskellam}
\alias{rskellam}
\title{Skellam distribution}
\usage{
dskellam(x, mu1, mu2, log = FALSE)

rskellam(n, mu1, mu2)
}
\arguments{
\item{x}{vector of quantiles.}

\item{mu1, mu2}{positive valued parameters.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the Skellam distribution.
}
\details{
If \eqn{X} and \eqn{Y} follow Poisson distributions with means
\eqn{\mu_1}{\mu[1]} and \eqn{\mu_2}{\mu[2]}, than \eqn{X-Y} follows
Skellam distribution parametrized by \eqn{\mu_1}{\mu[1]} and \eqn{\mu_2}{\mu[2]}.

Probability mass function
\deqn{
f(x) = e^{-(\mu_1\!+\!\mu_2)} \left(\frac{\mu_1}{\mu_2}\right)^{k/2}\!\!I_{k}(2\sqrt{\mu_1\mu_2})
}{
f(x) = exp(-(\mu1+\mu2)) * (\mu1/\mu2)^(x/2) * besselI(2*sqrt(\mu1*\mu2), x)
}
}
\examples{

x <- rskellam(1e5, 5, 13)
xx <- -40:40
plot(prop.table(table(x)), type = "h")
lines(xx, dskellam(xx, 5, 13), col = "red")

}
\references{
Karlis, D., & Ntzoufras, I. (2006). Bayesian analysis of the differences of count data.
Statistics in medicine, 25(11), 1885-1905.

Skellam, J.G. (1946). The frequency distribution of the difference between
two Poisson variates belonging to different populations.
Journal of the Royal Statistical Society, series A, 109(3), 26.
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangular-distribution.R
\name{Triangular}
\alias{Triangular}
\alias{dtriang}
\alias{ptriang}
\alias{qtriang}
\alias{rtriang}
\title{Triangular distribution}
\usage{
dtriang(x, a = -1, b = 1, c = (a + b)/2, log = FALSE)

ptriang(q, a = -1, b = 1, c = (a + b)/2, lower.tail = TRUE, log.p = FALSE)

qtriang(p, a = -1, b = 1, c = (a + b)/2, lower.tail = TRUE, log.p = FALSE)

rtriang(n, a = -1, b = 1, c = (a + b)/2)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{a, b, c}{minimum, maximum and mode of the distribution.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the triangular distribution.
}
\details{
Probability density function
\deqn{
f(x) = \left\{\begin{array}{ll}
\frac{2(x-a)}{(b-a)(c-a)} & x < c \\
\frac{2}{b-a}             & x = c \\
\frac{2(b-x)}{(b-a)(b-c)} & x > c
\end{array}\right.
}{
f(x) = [if x < c:] (2*(x-a)) / ((b-a)*(c-a))
       [if x = c:] 2/(b-a)
       [if x >= c:] (2*(b-x)) / ((b-a)*(b-c))
}

Cumulative distribution function
\deqn{
F(x) = \left\{\begin{array}{ll}
\frac{(x-a)^2}{(b-a)(c-a)}     & x \leq c \\
1 - \frac{(b-x)^2}{(b-a)(b-c)} & x > c
\end{array}\right.
}{
F(x) = [if x <= c:] (x-a)^2 / ((b-a)*(c-a))
       [if x > c:] 1 - ((b-x)^2 / ((b-a)*(b-c)))
}

Quantile function
\deqn{
F^{-1}(p) = \left\{\begin{array}{ll}
a + \sqrt{p \times (b-a)(c-a)} & p \leq \frac{c-a}{b-a} \\
b - \sqrt{(1-p)(b-a)(b-c)}     & p > \frac{c-a}{b-a}
\end{array}\right.
}{
F^-1(p) = [if p < (c-a)/(b-a):] a + sqrt(p*(b-a)*(c-a))
          [else:] b - sqrt((1-p)*(b-a)*(b-c))
}

For random generation MINMAX method described by
Stein and Keblis (2009) is used.
}
\examples{

x <- rtriang(1e5, 5, 7, 6)
hist(x, 100, freq = FALSE)
curve(dtriang(x, 5, 7, 6), 3, 10, n = 500, col = "red", add = TRUE)
hist(ptriang(x, 5, 7, 6))
plot(ecdf(x))
curve(ptriang(x, 5, 7, 6), 3, 10, n = 500, col = "red", lwd = 2, add = TRUE)

}
\references{
Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
Statistical Distributions. John Wiley & Sons.

Stein, W. E., & Keblis, M. F. (2009).
A new method to simulate the triangular distribution.
Mathematical and computer modelling, 49(5), 1143-1147.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gev-distribution.R
\name{GEV}
\alias{GEV}
\alias{dgev}
\alias{pgev}
\alias{qgev}
\alias{rgev}
\title{Generalized extreme value distribution}
\usage{
dgev(x, mu = 0, sigma = 1, xi = 0, log = FALSE)

pgev(q, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE)

qgev(p, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE)

rgev(n, mu = 0, sigma = 1, xi = 0)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, sigma, xi}{location, scale, and shape parameters. Scale must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the generalized extreme value distribution.
}
\details{
Probability density function
\deqn{ f(x) = \left\{\begin{array}{ll}
\frac{1}{\sigma} \left(1+\xi \frac{x-\mu}{\sigma}\right)^{-1/\xi-1} \exp\left(-\left(1+\xi \frac{x-\mu}{\sigma}\right)^{-1/\xi}\right) & \xi \neq 0 \\
\frac{1}{\sigma} \exp\left(- \frac{x-\mu}{\sigma}\right) \exp\left(-\exp\left(- \frac{x-\mu}{\sigma}\right)\right)                       & \xi = 0
\end{array}\right.
}{
f(x) = [if \xi != 0:] 1/\sigma * (1+\xi*(x-\mu)/\sigma)^{-1/\xi-1} * exp(-(1+\xi*(x-\mu)/\sigma)^{-1/\xi})
[else:] 1/\sigma * exp(-(x-\mu)/\sigma) * exp(-exp(-(x-\mu)/\sigma))
}

Cumulative distribution function
\deqn{ F(x) = \left\{\begin{array}{ll}
\exp\left(-\left(1+\xi \frac{x-\mu}{\sigma}\right)^{1/\xi}\right) & \xi \neq 0 \\
\exp\left(-\exp\left(- \frac{x-\mu}{\sigma}\right)\right)          & \xi = 0
\end{array}\right.
}{
F(x) = [if \xi != 0:] exp(-(1+\xi*(x-\mu)/\sigma)^{1/\xi})
[else:] exp(-exp(-(x-\mu)/\sigma))
}

Quantile function
\deqn{ F^{-1}(p) = \left\{\begin{array}{ll}
\mu - \frac{\sigma}{\xi} (1 - (-\log(p))^\xi)  & \xi \neq 0 \\
\mu - \sigma \log(-\log(p))                    & \xi = 0
\end{array}\right.
}{
F^-1(p) = [if \xi != 0:] \mu - \sigma/\xi * (1 - (-log(p))^\xi)
          [else:] \mu - \sigma * log(-log(p))
}
}
\examples{

curve(dgev(x, xi = -1/2), -4, 4, col = "green", ylab = "")
curve(dgev(x, xi = 0), -4, 4, col = "red", add = TRUE)
curve(dgev(x, xi = 1/2), -4, 4, col = "blue", add = TRUE)
legend("topleft", col = c("green", "red", "blue"), lty = 1,
       legend = expression(xi == -1/2, xi == 0, xi == 1/2), bty = "n")

x <- rgev(1e5, 5, 2, .5)
hist(x, 1000, freq = FALSE, xlim = c(0, 50))
curve(dgev(x, 5, 2, .5), 0, 50, col = "red", add = TRUE, n = 5000)
hist(pgev(x, 5, 2, .5))
plot(ecdf(x), xlim = c(0, 50))
curve(pgev(x, 5, 2, .5), 0, 50, col = "red", lwd = 2, add = TRUE)

}
\references{
Coles, S. (2001). An Introduction to Statistical Modeling of Extreme Values.
Springer.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shifted-gompertz-distribution.R
\name{ShiftGomp}
\alias{ShiftGomp}
\alias{dsgomp}
\alias{psgomp}
\alias{rsgomp}
\title{Shifted Gompertz distribution}
\usage{
dsgomp(x, b, eta, log = FALSE)

psgomp(q, b, eta, lower.tail = TRUE, log.p = FALSE)

rsgomp(n, b, eta)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{b, eta}{positive valued scale and shape parameters;
both need to be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, and random generation
for the shifted Gompertz distribution.
}
\details{
If \eqn{X} follows exponential distribution parametrized by scale \eqn{b} and
\eqn{Y} follows reparametrized Gumbel distribution with cumulative distribution function
\eqn{F(x) = \exp(-\eta  e^{-bx})}{F(x) = exp(-\eta*exp(-b*x))} parametrized by 
scale \eqn{b} and shape \eqn{\eta}, then \eqn{\max(X,Y)}{max(X,Y)} follows shifted
Gompertz distribution parametrized by scale \eqn{b>0} and shape \eqn{\eta>0}.
The above relation is used by \code{rsgomp} function for random generation from
shifted Gompertz distribution.

Probability density function
\deqn{
f(x) = b e^{-bx} \exp(-\eta e^{-bx}) \left[1 + \eta(1 - e^{-bx})\right]
}{
f(x) = b*exp(-b*x) * exp(-\eta*exp(-b*x)) * (1 + \eta*(1 - exp(-b*x)))
}

Cumulative distribution function
\deqn{
F(x) = (1-e^{-bx}) \exp(-\eta e^{-bx})
}{
F(x) = (1-exp(-b*x)) * exp(-\eta*exp(-b*x))
}
}
\examples{

x <- rsgomp(1e5, 0.4, 1)
hist(x, 50, freq = FALSE)
curve(dsgomp(x, 0.4, 1), 0, 30, col = "red", add = TRUE)
hist(psgomp(x, 0.4, 1))
plot(ecdf(x))
curve(psgomp(x, 0.4, 1), 0, 30, col = "red", lwd = 2, add = TRUE)

}
\references{
Bemmaor, A.C. (1994).
Modeling the Diffusion of New Durable Goods: Word-of-Mouth Effect Versus Consumer Heterogeneity.
[In:] G. Laurent, G.L. Lilien & B. Pras. Research Traditions in Marketing.
Boston: Kluwer Academic Publishers. pp. 201-223.

Jimenez, T.F. and Jodra, P. (2009).
A Note on the Moments and Computer Generation of the Shifted Gompertz Distribution.
Communications in Statistics - Theory and Methods, 38(1), 78-89.

Jimenez T.F. (2014).
Estimation of the Parameters of the Shifted Gompertz Distribution,
Using Least Squares, Maximum Likelihood and Moments Methods.
Journal of Computational and Applied Mathematics, 255(1), 867-877.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/location-scale-t-distribution.R
\name{LocationScaleT}
\alias{LocationScaleT}
\alias{dlst}
\alias{plst}
\alias{qlst}
\alias{rlst}
\title{Location-scale version of the t-distribution}
\usage{
dlst(x, df, mu = 0, sigma = 1, log = FALSE)

plst(q, df, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qlst(p, df, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rlst(n, df, mu = 0, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{df}{degrees of freedom (> 0, maybe non-integer). \code{df = Inf} is allowed.}

\item{mu}{vector of locations}

\item{sigma}{vector of positive valued scale parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function and random generation
for location-scale version of the t-distribution. Location-scale version
of the t-distribution besides degrees of freedom \eqn{\nu}, is parametrized
using additional parameters \eqn{\mu} for location and \eqn{\sigma} for
scale (\eqn{\mu = 0} and \eqn{\sigma = 1} for standard t-distribution).
}
\examples{

x <- rlst(1e5, 1000, 5, 13)
hist(x, 100, freq = FALSE)
curve(dlst(x, 1000, 5, 13), -60, 60, col = "red", add = TRUE)
hist(plst(x, 1000, 5, 13))
plot(ecdf(x))
curve(plst(x, 1000, 5, 13), -60, 60, col = "red", lwd = 2, add = TRUE)

}
\seealso{
\code{\link[stats]{TDist}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/half-normal-distribution.R
\name{HalfNormal}
\alias{HalfNormal}
\alias{dhnorm}
\alias{phnorm}
\alias{qhnorm}
\alias{rhnorm}
\title{Half-normal distribution}
\usage{
dhnorm(x, sigma = 1, log = FALSE)

phnorm(q, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qhnorm(p, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rhnorm(n, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{sigma}{positive valued scale parameter.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the half-normal distribution.
}
\details{
If \eqn{X} follows normal distribution centered at 0 and parametrized
by scale \eqn{\sigma}, then \eqn{|X|} follows half-normal distribution
parametrized by scale \eqn{\sigma}. Half-t distribution with \eqn{\nu=\infty}
degrees of freedom converges to half-normal distribution.
}
\examples{

x <- rhnorm(1e5, 2)
hist(x, 100, freq = FALSE)
curve(dhnorm(x, 2), 0, 8, col = "red", add = TRUE)
hist(phnorm(x, 2))
plot(ecdf(x))
curve(phnorm(x, 2), 0, 8, col = "red", lwd = 2, add = TRUE)

}
\references{
Gelman, A. (2006). Prior distributions for variance parameters in hierarchical
models (comment on article by Browne and Draper).
Bayesian analysis, 1(3), 515-534.

Jacob, E. and Jayakumar, K. (2012).
On Half-Cauchy Distribution and Process.
International Journal of Statistika and Mathematika, 3(2), 77-81.
}
\seealso{
\code{\link{HalfT}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logarithmic-series-distribution.R
\name{LogSeries}
\alias{LogSeries}
\alias{dlgser}
\alias{plgser}
\alias{qlgser}
\alias{rlgser}
\title{Logarithmic series distribution}
\usage{
dlgser(x, theta, log = FALSE)

plgser(q, theta, lower.tail = TRUE, log.p = FALSE)

qlgser(p, theta, lower.tail = TRUE, log.p = FALSE)

rlgser(n, theta)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{theta}{vector; concentration parameter; (\code{0 < theta < 1}).}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the logarithmic series distribution.
}
\details{
Probability mass function
\deqn{
f(x) = \frac{-1}{\log(1-\theta)} \frac{\theta^x}{x}
}{
f(x) = (-1/log(1-\theta)*\theta^x) / x
}

Cumulative distribution function
\deqn{
F(x) = \frac{-1}{\log(1-\theta)} \sum_{k=1}^x \frac{\theta^x}{x}
}{
F(x) = -1/log(1-\theta) * sum((\theta^x)/x)
}

Quantile function and random generation are computed using
algorithm described in Krishnamoorthy (2006).
}
\examples{

x <- rlgser(1e5, 0.66)
xx <- seq(0, 100, by = 1)
plot(prop.table(table(x)), type = "h")
lines(xx, dlgser(xx, 0.66), col = "red")

# Notice: distribution of F(X) is far from uniform:
hist(plgser(x, 0.66), 50)

xx <- seq(0, 100, by = 0.01)
plot(ecdf(x))
lines(xx, plgser(xx, 0.66), col = "red", lwd = 2)

}
\references{
Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
with Applications. Chapman & Hall/CRC

Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
Statistical Distributions. John Wiley & Sons.
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gamma-poisson-distribution.R
\name{GammaPoiss}
\alias{GammaPoiss}
\alias{dgpois}
\alias{pgpois}
\alias{rgpois}
\title{Gamma-Poisson distribution}
\usage{
dgpois(x, shape, rate, scale = 1/rate, log = FALSE)

pgpois(q, shape, rate, scale = 1/rate, lower.tail = TRUE, log.p = FALSE)

rgpois(n, shape, rate, scale = 1/rate)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{shape, scale}{shape and scale parameters. Must be positive,
scale strictly.}

\item{rate}{an alternative way to specify the scale.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the gamma-Poisson distribution.
}
\details{
Gamma-Poisson distribution arises as a continuous mixture of
Poisson distributions, where the mixing distribution
of the Poisson rate \eqn{\lambda} is a gamma distribution.
When \eqn{X \sim \mathrm{Poisson}(\lambda)}{X ~ Poisson(\lambda)}
and \eqn{\lambda \sim \mathrm{Gamma}(\alpha, \beta)}{\lambda ~ Gamma(\alpha, \beta)}, then \eqn{X \sim \mathrm{GammaPoisson}(\alpha, \beta)}{X ~ Gamma-Poisson(\alpha, \beta)}.

Probability mass function
\deqn{
f(x) = \frac{\Gamma(\alpha+x)}{x! \, \Gamma(\alpha)} \left(\frac{\beta}{1+\beta}\right)^x \left(1-\frac{\beta}{1+\beta}\right)^\alpha
}{
f(x) = \Gamma(\alpha+x) / (x!*\Gamma(\alpha)) * (\beta/(1+\beta))^x * (1-\beta/(1+\beta))^\alpha
}

Cumulative distribution function is calculated using recursive algorithm that employs the fact that
\eqn{\Gamma(x) = (x - 1)!}. This enables re-writing probability mass function as

\deqn{
f(x) = \frac{(\alpha+x-1)!}{x! \, \Gamma(\alpha)} \left( \frac{\beta}{1+\beta} \right)^x \left( 1- \frac{\beta}{1+\beta} \right)^\alpha
}{
f(x) = ((\alpha+x-1)!)/(x!*\Gamma(\alpha))*(\beta/(1+\beta))^x*(1-\beta/(1+\beta))^\alpha
}

what makes recursive updating from \eqn{x} to \eqn{x+1} easy using the properties of factorials

\deqn{
f(x+1) = \frac{(\alpha+x-1)! \, (\alpha+x)}{x! \,(x+1) \, \Gamma(\alpha)} \left( \frac{\beta}{1+\beta} \right)^x \left( \frac{\beta}{1+\beta} \right) \left( 1- \frac{\beta}{1+\beta} \right)^\alpha
}{
f(x+1) = ((\alpha+x-1)!*(\alpha+x))/(x!*(x+1)*\Gamma(\alpha))*(\beta/(1+\beta))^x*(\beta/(1+\beta))*(1-\beta/(1+\beta))^\alpha
}

and let's us efficiently calculate cumulative distribution function as a sum of probability mass functions

\deqn{F(x) = \sum_{k=0}^x f(k)}{F(x) = f(0)+...+f(x)}
}
\examples{

x <- rgpois(1e5, 7, 0.002)
xx <- seq(0, 12000, by = 1)
hist(x, 100, freq = FALSE)
lines(xx, dgpois(xx, 7, 0.002), col = "red")
hist(pgpois(x, 7, 0.002))
xx <- seq(0, 12000, by = 0.1)
plot(ecdf(x))
lines(xx, pgpois(xx, 7, 0.002), col = "red", lwd = 2)

}
\seealso{
\code{\link[stats]{Gamma}}, \code{\link[stats]{Poisson}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inverse-gamma-distribution.R
\name{InvGamma}
\alias{InvGamma}
\alias{dinvgamma}
\alias{pinvgamma}
\alias{qinvgamma}
\alias{rinvgamma}
\title{Inverse-gamma distribution}
\usage{
dinvgamma(x, alpha, beta = 1, log = FALSE)

pinvgamma(q, alpha, beta = 1, lower.tail = TRUE, log.p = FALSE)

qinvgamma(p, alpha, beta = 1, lower.tail = TRUE, log.p = FALSE)

rinvgamma(n, alpha, beta = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{alpha, beta}{positive valued shape and scale parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function and random generation
for the inverse-gamma distribution.
}
\details{
Probability mass function
\deqn{
f(x) = \frac{\beta^\alpha x^{-\alpha-1} \exp(-\frac{\beta}{x})}{\Gamma(\alpha)}
}{
f(x) = (\beta^\alpha * x^(-\alpha-1) * exp(-\beta/x)) / \Gamma(\alpha)
}

Cumulative distribution function
\deqn{
F(x) = \frac{\gamma(\alpha, \frac{\beta}{x})}{\Gamma(\alpha)}
}{
F(x) = \gamma(\alpha, \beta/x) / \Gamma(\alpha)
}
}
\examples{

x <- rinvgamma(1e5, 20, 3)
hist(x, 100, freq = FALSE)
curve(dinvgamma(x, 20, 3), 0, 1, col = "red", add = TRUE, n = 5000)
hist(pinvgamma(x, 20, 3))
plot(ecdf(x))
curve(pinvgamma(x, 20, 3), 0, 1, col = "red", lwd = 2, add = TRUE, n = 5000) 

}
\references{
Witkovsky, V. (2001). Computing the distribution of a linear
combination of inverted gamma variables. Kybernetika 37(1), 79-90.

Leemis, L.M. and McQueston, L.T. (2008). Univariate Distribution
Relationships. American Statistician 62(1): 45-53.
}
\seealso{
\code{\link[stats]{GammaDist}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/half-t-distribution.R
\name{HalfT}
\alias{HalfT}
\alias{dht}
\alias{pht}
\alias{qht}
\alias{rht}
\title{Half-t distribution}
\usage{
dht(x, nu, sigma = 1, log = FALSE)

pht(q, nu, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qht(p, nu, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rht(n, nu, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{nu, sigma}{positive valued degrees of freedom and scale parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the half-t distribution.
}
\details{
If \eqn{X} follows t distribution parametrized by degrees of freedom \eqn{\nu}
and scale \eqn{\sigma}, then \eqn{|X|} follows half-t distribution parametrized
by degrees of freedom \eqn{\nu} and scale \eqn{\sigma}.
}
\examples{

x <- rht(1e5, 2, 2)
hist(x, 500, freq = FALSE, xlim = c(0, 100))
curve(dht(x, 2, 2), 0, 100, col = "red", add = TRUE)
hist(pht(x, 2, 2))
plot(ecdf(x), xlim = c(0, 100))
curve(pht(x, 2, 2), 0, 100, col = "red", lwd = 2, add = TRUE)

}
\references{
Gelman, A. (2006). Prior distributions for variance parameters in hierarchical
models (comment on article by Browne and Draper).
Bayesian analysis, 1(3), 515-534.

Jacob, E. and Jayakumar, K. (2012).
On Half-Cauchy Distribution and Process.
International Journal of Statistika and Mathematika, 3(2), 77-81.
}
\seealso{
\code{\link{HalfNormal}}, \code{\link{HalfCauchy}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate-normal-distribution.R
\name{BivNormal}
\alias{BivNormal}
\alias{dbvnorm}
\alias{rbvnorm}
\title{Bivariate normal distribution}
\usage{
dbvnorm(
  x,
  y = NULL,
  mean1 = 0,
  mean2 = mean1,
  sd1 = 1,
  sd2 = sd1,
  cor = 0,
  log = FALSE
)

rbvnorm(n, mean1 = 0, mean2 = mean1, sd1 = 1, sd2 = sd1, cor = 0)
}
\arguments{
\item{x, y}{vectors of quantiles; alternatively x may be a two-column
matrix (or data.frame) and y may be omitted.}

\item{mean1, mean2}{vectors of means.}

\item{sd1, sd2}{vectors of standard deviations.}

\item{cor}{vector of correlations (\code{-1 < cor < 1}).}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function and random generation
for the bivariate normal distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{1}{2\pi\sqrt{1-\rho^2}\sigma_1\sigma_2}
       \exp\left\{-\frac{1}{2(1-\rho^2)} \left[\left(\frac{x_1 - \mu_1}{\sigma_1}\right)^2 -
       2\rho \left(\frac{x_1 - \mu_1}{\sigma_1}\right) \left(\frac{x_2 - \mu_2}{\sigma_2}\right) +
       \left(\frac{x_2 - \mu_2}{\sigma_2}\right)^2\right]\right\}
}{
f(x) = 1/(2*\pi*sqrt(1-\rho^2)*\sigma1*\sigma2) * exp(-(1/(2*(1-\rho^2)*
       (((x1-\mu1)/\sigma1)^2 - 2*\rho*((x1-\mu1)/\sigma2)*((x2-\mu2)/\sigma2) *
       ((x2-\mu2)/\sigma2)^2))))
}
}
\examples{

y <- x <- seq(-4, 4, by = 0.25)
z <- outer(x, y, function(x, y) dbvnorm(x, y, cor = -0.75))
persp(x, y, z)

y <- x <- seq(-4, 4, by = 0.25)
z <- outer(x, y, function(x, y) dbvnorm(x, y, cor = -0.25))
persp(x, y, z)

}
\references{
Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
with Applications. Chapman & Hall/CRC

Mukhopadhyay, N. (2000). Probability and statistical inference.
Chapman & Hall/CRC
}
\seealso{
\code{\link[stats]{Normal}}
}
\concept{Continuous}
\concept{Multivariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/categorical-distribution.R
\name{Categorical}
\alias{Categorical}
\alias{dcat}
\alias{pcat}
\alias{qcat}
\alias{rcat}
\alias{rcatlp}
\title{Categorical distribution}
\usage{
dcat(x, prob, log = FALSE)

pcat(q, prob, lower.tail = TRUE, log.p = FALSE)

qcat(p, prob, lower.tail = TRUE, log.p = FALSE, labels)

rcat(n, prob, labels)

rcatlp(n, log_prob, labels)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{prob, log_prob}{vector of length \eqn{m}, or \eqn{m}-column matrix
of non-negative weights (or their logarithms in \code{log_prob}).}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{labels}{if provided, labeled \code{factor} vector is returned.
Number of labels needs to be the same as
number of categories (number of columns in prob).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function, quantile function and random generation
for the categorical distribution.
}
\details{
Probability mass function

\deqn{
\Pr(X = k) = \frac{w_k}{\sum_{j=1}^m w_j}
}{
Pr(X = k) = w[k]/sum(w)
}

Cumulative distribution function
\deqn{
\Pr(X \le k) = \frac{\sum_{i=1}^k w_i}{\sum_{j=1}^m w_j}
}{
Pr(X <= k) = sum(w[1:k])/sum(w)
}

It is possible to sample from categorical distribution parametrized
by vector of unnormalized log-probabilities
\eqn{\alpha_1,\dots,\alpha_m}{\alpha[1],...,\alpha[m]}
without leaving the log space by employing the Gumbel-max trick (Maddison, Tarlow and Minka, 2014).
If \eqn{g_1,\dots,g_m}{g[1],...,g[m]} are samples from Gumbel distribution with
cumulative distribution function \eqn{F(g) = \exp(-\exp(-g))}{F(g) = exp(-exp(-g))},
then \eqn{k = \mathrm{arg\,max}_i \{g_i + \alpha_i\}}{k = argmax(g[i]+\alpha[i])}
is a draw from categorical distribution parametrized by
vector of probabilities \eqn{p_1,\dots,p_m}{p[1]....,p[m]}, such that
\eqn{p_i = \exp(\alpha_i) / [\sum_{j=1}^m \exp(\alpha_j)]}{p[i] = exp(\alpha[i])/sum(exp(\alpha))}.
This is implemented in \code{rcatlp} function parametrized by vector of
log-probabilities \code{log_prob}.
}
\examples{

# Generating 10 random draws from categorical distribution
# with k=3 categories occuring with equal probabilities
# parametrized using a vector

rcat(10, c(1/3, 1/3, 1/3))

# or with k=5 categories parametrized using a matrix of probabilities
# (generated from Dirichlet distribution)

p <- rdirichlet(10, c(1, 1, 1, 1, 1))
rcat(10, p)

x <- rcat(1e5, c(0.2, 0.4, 0.3, 0.1))
plot(prop.table(table(x)), type = "h")
lines(0:5, dcat(0:5, c(0.2, 0.4, 0.3, 0.1)), col = "red")

p <- rdirichlet(1, rep(1, 20))
x <- rcat(1e5, matrix(rep(p, 2), nrow = 2, byrow = TRUE))
xx <- 0:21
plot(prop.table(table(x)))
lines(xx, dcat(xx, p), col = "red")

xx <- seq(0, 21, by = 0.01)
plot(ecdf(x))
lines(xx, pcat(xx, p), col = "red", lwd = 2)

pp <- seq(0, 1, by = 0.001)
plot(ecdf(x))
lines(qcat(pp, p), pp, col = "red", lwd = 2)

}
\references{
Maddison, C. J., Tarlow, D., & Minka, T. (2014). A* sampling.
[In:] Advances in Neural Information Processing Systems (pp. 3086-3094).
\url{https://arxiv.org/abs/1411.0030}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpd-distribution.R
\name{GPD}
\alias{GPD}
\alias{dgpd}
\alias{pgpd}
\alias{qgpd}
\alias{rgpd}
\title{Generalized Pareto distribution}
\usage{
dgpd(x, mu = 0, sigma = 1, xi = 0, log = FALSE)

pgpd(q, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE)

qgpd(p, mu = 0, sigma = 1, xi = 0, lower.tail = TRUE, log.p = FALSE)

rgpd(n, mu = 0, sigma = 1, xi = 0)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, sigma, xi}{location, scale, and shape parameters. Scale must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the generalized Pareto distribution.
}
\details{
Probability density function
\deqn{ f(x) = \left\{\begin{array}{ll}
\frac{1}{\sigma} \left(1+\xi \frac{x-\mu}{\sigma}\right)^{-(\xi+1)/\xi} & \xi \neq 0 \\
\frac{1}{\sigma} \exp\left(-\frac{x-\mu}{\sigma}\right)                 & \xi = 0
\end{array}\right.
}{
f(x) = [if \xi != 0:] (1+\xi*(x-\mu)/\sigma)^{-(\xi+1)/\xi}/\sigma
       [else:] exp(-(x-\mu)/\sigma)/\sigma
}

Cumulative distribution function
\deqn{ F(x) = \left\{\begin{array}{ll}
1-\left(1+\xi \frac{x-\mu}{\sigma}\right)^{-1/\xi} & \xi \neq 0 \\
1-\exp\left(-\frac{x-\mu}{\sigma}\right)           & \xi = 0
\end{array}\right.
}{
F(x) = [if \xi != 0:] 1-(1+\xi*(x-\mu)/\sigma)^{-1/\xi}
       [else:] 1-exp(-(x-\mu)/\sigma)
}

Quantile function
\deqn{ F^{-1}(x) = \left\{\begin{array}{ll}
\mu + \sigma \frac{(1-p)^{-\xi}-1}{\xi} & \xi \neq 0 \\
\mu - \sigma \log(1-p)                  & \xi = 0
\end{array}\right.
}{
F^-1(x) = [if \xi != 0:] \mu + \sigma * ((1-p)^{-\xi}-1)/\xi
          [else:] \mu - \sigma * log(1-p)
}
}
\examples{

x <- rgpd(1e5, 5, 2, .1)
hist(x, 100, freq = FALSE, xlim = c(0, 50))
curve(dgpd(x, 5, 2, .1), 0, 50, col = "red", add = TRUE, n = 5000)
hist(pgpd(x, 5, 2, .1))
plot(ecdf(x))
curve(pgpd(x, 5, 2, .1), 0, 50, col = "red", lwd = 2, add = TRUE)

}
\references{
Coles, S. (2001). An Introduction to Statistical Modeling of Extreme Values.
Springer.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frechet-distribution.R
\name{Frechet}
\alias{Frechet}
\alias{dfrechet}
\alias{pfrechet}
\alias{qfrechet}
\alias{rfrechet}
\title{Frechet distribution}
\usage{
dfrechet(x, lambda = 1, mu = 0, sigma = 1, log = FALSE)

pfrechet(q, lambda = 1, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qfrechet(p, lambda = 1, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rfrechet(n, lambda = 1, mu = 0, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{lambda, sigma, mu}{shape, scale, and location parameters.
Scale and shape must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Frechet distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{\lambda}{\sigma} \left(\frac{x-\mu}{\sigma}\right)^{-1-\lambda} \exp\left(-\left(\frac{x-\mu}{\sigma}\right)^{-\lambda}\right)
}{
f(x) = \lambda/\sigma * ((x-\mu)/\sigma)^(-1-\lambda) * exp(-((x-\mu)/\sigma)^-\lambda)
}

Cumulative distribution function
\deqn{
F(x) = \exp\left(-\left(\frac{x-\mu}{\sigma}\right)^{-\lambda}\right)
}{
F(x) = exp(-((x-\mu)/\sigma)^-\lambda)
}

Quantile function
\deqn{
F^{-1}(p) = \mu + \sigma -\log(p)^{-1/\lambda}
}{
F^-1(p) = \mu + \sigma * -log(p)^{-1/\lambda}
}
}
\examples{

x <- rfrechet(1e5, 5, 2, 1.5)
xx <- seq(0, 1000, by = 0.1)
hist(x, 200, freq = FALSE)
lines(xx, dfrechet(xx, 5, 2, 1.5), col = "red") 
hist(pfrechet(x, 5, 2, 1.5))
plot(ecdf(x))
lines(xx, pfrechet(xx, 5, 2, 1.5), col = "red", lwd = 2)

}
\references{
Bury, K. (1999). Statistical Distributions in Engineering.
Cambridge University Press.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative-hypergeometric-distribution.R
\name{NegHyper}
\alias{NegHyper}
\alias{dnhyper}
\alias{pnhyper}
\alias{qnhyper}
\alias{rnhyper}
\title{Negative hypergeometric distribution}
\usage{
dnhyper(x, n, m, r, log = FALSE)

pnhyper(q, n, m, r, lower.tail = TRUE, log.p = FALSE)

qnhyper(p, n, m, r, lower.tail = TRUE, log.p = FALSE)

rnhyper(nn, n, m, r)
}
\arguments{
\item{x, q}{vector of quantiles representing the number of balls drawn without
replacement from an urn which contains both black and white balls.}

\item{n}{the number of black balls in the urn.}

\item{m}{the number of white balls in the urn.}

\item{r}{the number of white balls that needs to be drawn for the sampling
to be stopped.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{nn}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function, quantile function and random generation
for the negative hypergeometric distribution.
}
\details{
Negative hypergeometric distribution describes number of balls \eqn{x} observed
until drawing without replacement to obtain \eqn{r} white balls
from the urn containing \eqn{m} white balls and \eqn{n} black balls,
and is defined as

\deqn{
f(x) = \frac{{x-1 \choose r-1}{m+n-x \choose m-r}}{{m+n \choose n}}
}{
f(x) = choose(x-1, r-1)*choose(m+n-x, m-r)/choose(m+n, n)
}

The algorithm used for calculating probability mass function,
cumulative distribution function and quantile function is based
on Fortran program NHYPERG created by Berry and Mielke (1996, 1998).
Random generation is done by inverse transform sampling.
}
\examples{

x <- rnhyper(1e5, 60, 35, 15)
xx <- 15:95
plot(prop.table(table(x)))
lines(xx, dnhyper(xx, 60, 35, 15), col = "red")
hist(pnhyper(x, 60, 35, 15))

xx <- seq(0, 100, by = 0.01)
plot(ecdf(x))
lines(xx, pnhyper(xx, 60, 35, 15), col = "red", lwd = 2)

}
\references{
Berry, K. J., & Mielke, P. W. (1998).
The negative hypergeometric probability distribution:
Sampling without replacement from a finite population.
Perceptual and motor skills, 86(1), 207-210.
\url{https://journals.sagepub.com/doi/10.2466/pms.1998.86.1.207}

Berry, K. J., & Mielke, P. W. (1996).
Exact confidence limits for population proportions based on the negative
hypergeometric probability distribution.
Perceptual and motor skills, 83(3 suppl), 1216-1218.
\url{https://journals.sagepub.com/doi/10.2466/pms.1996.83.3f.1216}

Schuster, E. F., & Sype, W. R. (1987).
On the negative hypergeometric distribution.
International Journal of Mathematical Education in Science and Technology, 18(3), 453-459.

Chae, K. C. (1993).
Presenting the negative hypergeometric distribution to the introductory statistics courses.
International Journal of Mathematical Education in Science and Technology, 24(4), 523-526.

Jones, S.N. (2013). A Gaming Application of the Negative Hypergeometric Distribution.
UNLV Theses, Dissertations, Professional Papers, and Capstones. Paper 1846.
\url{https://oasis.library.unlv.edu/cgi/viewcontent.cgi?referer=&httpsredir=1&article=2847&context=thesesdissertations}
}
\seealso{
\code{\link[stats]{Hypergeometric}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rayleigh-distribution.R
\name{Rayleigh}
\alias{Rayleigh}
\alias{drayleigh}
\alias{prayleigh}
\alias{qrayleigh}
\alias{rrayleigh}
\title{Rayleigh distribution}
\usage{
drayleigh(x, sigma = 1, log = FALSE)

prayleigh(q, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qrayleigh(p, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rrayleigh(n, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{sigma}{positive valued parameter.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Rayleigh distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{x}{\sigma^2} \exp\left(-\frac{x^2}{2\sigma^2}\right)
}{
f(x) = x/\sigma^2 * exp(-(x^2 / 2*\sigma^2))
}

Cumulative distribution function
\deqn{
F(x) = 1 - \exp\left(-\frac{x^2}{2\sigma^2}\right)
}{
F(x) = 1 - exp(-x^2 / 2*\sigma^2)
}

Quantile function
\deqn{
F^{-1}(p) = \sqrt{-2\sigma^2 \log(1-p)}
}{
F^-1(p) = sqrt(-2*\sigma^2 * log(1-p))
}
}
\examples{

x <- rrayleigh(1e5, 13)
hist(x, 100, freq = FALSE)
curve(drayleigh(x, 13), 0, 60, col = "red", add = TRUE)
hist(prayleigh(x, 13)) 
plot(ecdf(x))
curve(prayleigh(x, 13), 0, 60, col = "red", lwd = 2, add = TRUE) 

}
\references{
Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
with Applications. Chapman & Hall/CRC.

Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
Statistical Distributions. John Wiley & Sons.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate-poisson-distribution.R
\name{BivPoiss}
\alias{BivPoiss}
\alias{dbvpois}
\alias{rbvpois}
\title{Bivariate Poisson distribution}
\usage{
dbvpois(x, y = NULL, a, b, c, log = FALSE)

rbvpois(n, a, b, c)
}
\arguments{
\item{x, y}{vectors of quantiles; alternatively x may be a two-column
matrix (or data.frame) and y may be omitted.}

\item{a, b, c}{positive valued parameters.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation for the bivariate Poisson distribution.
}
\details{
Probability mass function
\deqn{
f(x,y) = \exp \{-(a+b+c)\} \frac{a^x}{x!} \frac{b^y}{y!} \sum_{k=0}^{\min(x,y)}
{x \choose k} {y \choose k} k! \left( \frac{c}{ab} \right)^k
}{
f(x,y) = exp(-(a+b+c)) * (a^x)/x! * (b^y)/y! *
sum(choose(x,k)*choose(y,k)*k!*(c/(a*b))^k)
}
}
\examples{

x <- rbvpois(5000, 7, 8, 5)
image(prop.table(table(x[,1], x[,2])))
colMeans(x)

}
\references{
Karlis, D. and Ntzoufras, I. (2003). Analysis of sports data by using bivariate Poisson models.
Journal of the Royal Statistical Society: Series D (The Statistician), 52(3), 381-393.

Kocherlakota, S. and Kocherlakota, K. (1992) Bivariate Discrete Distributions.
New York: Dekker.

Johnson, N., Kotz, S. and Balakrishnan, N. (1997). Discrete Multivariate Distributions.
New York: Wiley.

Holgate, P. (1964). Estimation for the bivariate Poisson distribution.
Biometrika, 51(1-2), 241-287.

Kawamura, K. (1984). Direct calculation of maximum likelihood estimator for the bivariate
Poisson distribution. Kodai mathematical journal, 7(2), 211-221.
}
\seealso{
\code{\link[stats]{Poisson}}
}
\concept{Discrete}
\concept{Multivariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-uniform-distribution.R
\name{DiscreteUniform}
\alias{DiscreteUniform}
\alias{ddunif}
\alias{pdunif}
\alias{qdunif}
\alias{rdunif}
\title{Discrete uniform distribution}
\usage{
ddunif(x, min, max, log = FALSE)

pdunif(q, min, max, lower.tail = TRUE, log.p = FALSE)

qdunif(p, min, max, lower.tail = TRUE, log.p = FALSE)

rdunif(n, min, max)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{min, max}{lower and upper limits of the distribution. Must be finite.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function, quantile function and random generation
for the discrete uniform distribution.
}
\details{
If \code{min == max}, then discrete uniform distribution is a degenerate distribution.
}
\examples{

x <- rdunif(1e5, 1, 10) 
xx <- -1:11
plot(prop.table(table(x)), type = "h")
lines(xx, ddunif(xx, 1, 10), col = "red")
hist(pdunif(x, 1, 10))
xx <- seq(-1, 11, by = 0.01)
plot(ecdf(x))
lines(xx, pdunif(xx, 1, 10), col = "red")

}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multivariate-hypergeometric-distribution.R
\name{MultiHypergeometric}
\alias{MultiHypergeometric}
\alias{dmvhyper}
\alias{rmvhyper}
\title{Multivariate hypergeometric distribution}
\usage{
dmvhyper(x, n, k, log = FALSE)

rmvhyper(nn, n, k)
}
\arguments{
\item{x}{\eqn{m}-column matrix of quantiles.}

\item{n}{\eqn{m}-length vector or \eqn{m}-column matrix
of numbers of balls in \eqn{m} colors.}

\item{k}{the number of balls drawn from the urn.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{nn}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the multivariate hypergeometric distribution.
}
\details{
Probability mass function
\deqn{
f(x) = \frac{\prod_{i=1}^m {n_i \choose x_i}}{{N \choose k}}
}{
f(x) = prod(choose(n, x)) / choose(N, k)
}

The multivariate hypergeometric distribution is generalization of
hypergeometric distribution. It is used for sampling \emph{without} replacement
\eqn{k} out of \eqn{N} marbles in \eqn{m} colors, where each of the colors appears
\eqn{n_i}{n[i]} times. Where \eqn{k=\sum_{i=1}^m x_i}{k=sum(x)},
\eqn{N=\sum_{i=1}^m n_i}{N=sum(n)} and \eqn{k \le N}{k<=N}.
}
\examples{

# Generating 10 random draws from multivariate hypergeometric
# distribution parametrized using a vector

rmvhyper(10, c(10, 12, 5, 8, 11), 33)

}
\references{
Gentle, J.E. (2006). Random number generation and Monte Carlo methods. Springer.
}
\seealso{
\code{\link[stats]{Hypergeometric}}
}
\concept{Discrete}
\concept{Multivariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slash-distribution.R
\name{Slash}
\alias{Slash}
\alias{dslash}
\alias{pslash}
\alias{rslash}
\title{Slash distribution}
\usage{
dslash(x, mu = 0, sigma = 1, log = FALSE)

pslash(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rslash(n, mu = 0, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu}{vector of locations}

\item{sigma}{vector of positive valued scale parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function and random generation
for slash distribution.
}
\details{
If \eqn{Z \sim \mathrm{Normal}(0, 1)}{Z ~ Normal(0, 1)} and \eqn{U \sim \mathrm{Uniform}(0, 1)}{U ~ Uniform(0, 1)},
then \eqn{Z/U} follows slash distribution.

Probability density function

\deqn{
f(x) = \left\{\begin{array}{ll}
\frac{\phi(0) - \phi(x)}{x^2} & x \ne 0 \\
\frac{1}{2\sqrt{2\pi}} & x = 0
\end{array}\right.
}{
f(x) = [if x != 0:] (\phi(0)-\phi(x))/x^2 [else:] 1/(2*sqrt(2*\pi))
}

Cumulative distribution function

\deqn{
F(x) = \left\{\begin{array}{ll}
\Phi(x) - \frac{\phi(0)-\phi(x)}{x} & x \neq 0 \\
\frac{1}{2} & x = 0
\end{array}\right.
}{
F(x) = [if x != 0:] \Phi(x) - [\phi(0)-\phi(x)]/x [else:] 1/2
}
}
\examples{

x <- rslash(1e5, 5, 3)
hist(x, 1e5, freq = FALSE, xlim = c(-100, 100))
curve(dslash(x, 5, 3), -100, 100, col = "red", n = 500, add = TRUE)
hist(pslash(x, 5, 3))
plot(ecdf(x), xlim = c(-100, 100))
curve(pslash(x, 5, 3), -100, 100, col = "red", lwd = 2, n = 500, add = TRUE)

}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/laplace-distribution.R
\name{Laplace}
\alias{Laplace}
\alias{dlaplace}
\alias{plaplace}
\alias{qlaplace}
\alias{rlaplace}
\title{Laplace distribution}
\usage{
dlaplace(x, mu = 0, sigma = 1, log = FALSE)

plaplace(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qlaplace(p, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rlaplace(n, mu = 0, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, sigma}{location and scale parameters. Scale must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Laplace distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{1}{2\sigma} \exp\left(-\left|\frac{x-\mu}{\sigma}\right|\right)
}{
f(x) = 1/(2*\sigma) * exp(-|(x-\mu)/\sigma|)
}

Cumulative distribution function
\deqn{ F(x) = \left\{\begin{array}{ll}
\frac{1}{2} \exp\left(\frac{x-\mu}{\sigma}\right)     & x < \mu \\
1 - \frac{1}{2} \exp\left(\frac{x-\mu}{\sigma}\right) & x \geq \mu
\end{array}\right.
}{
F(x) = [if x < mu:] 1/2 * exp((x-\mu)/\sigma)
       [else:] 1 - 1/2 * exp((x-\mu)/\sigma)
}

Quantile function
\deqn{ F^{-1}(p) = \left\{\begin{array}{ll}
\mu + \sigma \log(2p)     & p < 0.5 \\
\mu - \sigma \log(2(1-p)) & p \geq 0.5
\end{array}\right.
}{
F^-1(p) = [if p < 0.5:] \mu + \sigma * log(2*p)
          [else:] \mu - \sigma * log(2*(1-p))
}
}
\examples{

x <- rlaplace(1e5, 5, 16)
hist(x, 100, freq = FALSE)
curve(dlaplace(x, 5, 16), -200, 200, n = 500, col = "red", add = TRUE)
hist(plaplace(x, 5, 16))
plot(ecdf(x))
curve(plaplace(x, 5, 16), -200, 200, n = 500, col = "red", lwd = 2, add = TRUE)

}
\references{
Krishnamoorthy, K. (2006). Handbook of Statistical Distributions
with Applications. Chapman & Hall/CRC

Forbes, C., Evans, M. Hastings, N., & Peacock, B. (2011).
Statistical Distributions. John Wiley & Sons.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gumbel-distribution.R
\name{Gumbel}
\alias{Gumbel}
\alias{dgumbel}
\alias{pgumbel}
\alias{qgumbel}
\alias{rgumbel}
\title{Gumbel distribution}
\usage{
dgumbel(x, mu = 0, sigma = 1, log = FALSE)

pgumbel(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

qgumbel(p, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)

rgumbel(n, mu = 0, sigma = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, sigma}{location and scale parameters. Scale must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Gumbel distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{1}{\sigma} \exp\left(-\left(\frac{x-\mu}{\sigma} + \exp\left(-\frac{x-\mu}{\sigma}\right)\right)\right)
}{
f(x) = 1/\sigma * exp(-((x-\mu)/\sigma + exp(-(x-\mu)/\sigma)))
}

Cumulative distribution function
\deqn{
F(x) = \exp\left(-\exp\left(-\frac{x-\mu}{\sigma}\right)\right)
}{
F(x) = exp(-exp(-(x-\mu)/\sigma))
}

Quantile function
\deqn{
F^{-1}(p) = \mu - \sigma \log(-\log(p))
}{
F^-1(p) = \mu - \sigma * log(-log(p))
}
}
\examples{

x <- rgumbel(1e5, 5, 2)
hist(x, 100, freq = FALSE)
curve(dgumbel(x, 5, 2), 0, 25, col = "red", add = TRUE)
hist(pgumbel(x, 5, 2))
plot(ecdf(x))
curve(pgumbel(x, 5, 2), 0, 25, col = "red", lwd = 2, add = TRUE)

}
\references{
Bury, K. (1999). Statistical Distributions in Engineering.
Cambridge University Press.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/beta-negative-binomial-distribution.R
\name{BetaNegBinom}
\alias{BetaNegBinom}
\alias{dbnbinom}
\alias{pbnbinom}
\alias{rbnbinom}
\title{Beta-negative binomial distribution}
\usage{
dbnbinom(x, size, alpha = 1, beta = 1, log = FALSE)

pbnbinom(q, size, alpha = 1, beta = 1, lower.tail = TRUE, log.p = FALSE)

rbnbinom(n, size, alpha = 1, beta = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{size}{number of trials (zero or more). Must be strictly positive, need not be integer.}

\item{alpha, beta}{non-negative parameters of the beta distribution.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the beta-negative binomial distribution.
}
\details{
If \eqn{p \sim \mathrm{Beta}(\alpha, \beta)}{p ~ Beta(\alpha, \beta)} and
\eqn{X \sim \mathrm{NegBinomial}(r, p)}{X ~ NegBinomial(r, p)}, then 
\eqn{X \sim \mathrm{BetaNegBinomial}(r, \alpha, \beta)}{X ~ BetaNegBinomial(r, \alpha, \beta)}.

Probability mass function
\deqn{
f(x) = \frac{\Gamma(r+x)}{x! \,\Gamma(r)}
       \frac{\mathrm{B}(\alpha+r, \beta+x)}{\mathrm{B}(\alpha, \beta)}
}{
f(x) = \Gamma(r+x)/(x! \Gamma(r)) * B(\alpha+r, \beta+x) / B(\alpha, \beta)
}

Cumulative distribution function is calculated using recursive algorithm that employs the fact that
\eqn{\Gamma(x) = (x - 1)!} and
\eqn{
\mathrm{B}(x, y) = \frac{\Gamma(x)\Gamma(y)}{\Gamma(x+y)}
}{
B(x, y) = (\Gamma(x)\Gamma(y))/\Gamma(x+y)
}. This enables re-writing probability mass function as

\deqn{
f(x) = \frac{(r+x-1)!}{x! \, \Gamma(r)} \frac{\frac{(\alpha+r-1)!\,(\beta+x-1)!}{(\alpha+\beta+r+x-1)!}}{\mathrm{B}(\alpha,\beta)}
}{
f(x) = ((r+x-1)!)/(x!*\Gamma(r))*(((\alpha+r-1)!*(\beta+x-1)!)/((\alpha+\beta+r+x-1)!))/B(\alpha,\beta)
}

what makes recursive updating from \eqn{x} to \eqn{x+1} easy using the properties of factorials

\deqn{
f(x+1) = \frac{(r+x-1)!\,(r+x)}{x!\,(x+1) \, \Gamma(r)} \frac{\frac{(\alpha+r-1)!\,(\beta+x-1)!\,(\beta+x)}{(\alpha+\beta+r+x-1)!\,(\alpha+\beta+r+x)}}{\mathrm{B}(\alpha,\beta)}
}{
f(x+1) = ((r+x-1)!*(r+x))/(x!*(x+1)*\Gamma(r))*(((\alpha+r-1)!*(\beta+x-1)!*(\beta+x))/((\alpha+\beta+r+x-1)!*(\alpha+\beta+r+x)))/B(\alpha,\beta)
}

and let's us efficiently calculate cumulative distribution function as a sum of probability mass functions

\deqn{F(x) = \sum_{k=0}^x f(k)}{F(x) = f(0)+...+f(x)}
}
\examples{

x <- rbnbinom(1e5, 1000, 5, 13)
xx <- 0:1e5
hist(x, 100, freq = FALSE)
lines(xx-0.5, dbnbinom(xx, 1000, 5, 13), col = "red")
hist(pbnbinom(x, 1000, 5, 13))
xx <- seq(0, 1e5, by = 0.1)
plot(ecdf(x))
lines(xx, pbnbinom(xx, 1000, 5, 13), col = "red", lwd = 2)

}
\seealso{
\code{\link[stats]{Beta}}, \code{\link[stats]{NegBinomial}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proportion-distribution.R
\name{PropBeta}
\alias{PropBeta}
\alias{dprop}
\alias{pprop}
\alias{qprop}
\alias{rprop}
\title{Beta distribution of proportions}
\usage{
dprop(x, size, mean, prior = 0, log = FALSE)

pprop(q, size, mean, prior = 0, lower.tail = TRUE, log.p = FALSE)

qprop(p, size, mean, prior = 0, lower.tail = TRUE, log.p = FALSE)

rprop(n, size, mean, prior = 0)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{size}{non-negative real number; precision or number of binomial trials.}

\item{mean}{mean proportion or probability of success on each trial;
\code{0 < mean < 1}.}

\item{prior}{(see below) with \code{prior = 0} (default)
the distribution corresponds to re-parametrized beta
distribution used in beta regression. This parameter needs
to be non-negative.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function and random generation
for the reparametrized beta distribution.
}
\details{
Beta can be understood as a distribution of \eqn{x = k/\phi} proportions in
\eqn{\phi} trials where the average proportion is denoted as \eqn{\mu},
so it's parameters become \eqn{\alpha = \phi\mu} and
\eqn{\beta = \phi(1-\mu)} and it's density function becomes

\deqn{
f(x) = \frac{x^{\phi\mu+\pi-1} (1-x)^{\phi(1-\mu)+\pi-1}}{\mathrm{B}(\phi\mu+\pi, \phi(1-\mu)+\pi)}
}{
f(x) = (x^(\phi\mu+\pi-1) * (1-x)^(\phi(1-\mu)+\pi-1))/B(\phi\mu+\pi, \phi(1-\mu)+\pi)
}

where \eqn{\pi} is a \emph{prior} parameter, so the distribution is a
\emph{posterior} distribution after observing \eqn{\phi\mu} successes and
\eqn{\phi(1-\mu)} failures in \eqn{\phi} trials with binomial likelihood
and symmetric \eqn{\mathrm{Beta}(\pi, \pi)}{Beta(\pi, \pi)} prior for
probability of success. Parameter value \eqn{\pi = 1} corresponds to
uniform prior; \eqn{\pi = 1/2} corresponds to Jeffreys prior; \eqn{\pi = 0}
corresponds to "uninformative" Haldane prior, this is also the re-parametrized
distribution used in beta regression. With \eqn{\pi = 0} the distribution
can be understood as a continuous analog to binomial distribution dealing
with proportions rather then counts. Alternatively \eqn{\phi} may be
understood as precision parameter (as in beta regression).

Notice that in pre-1.8.4 versions of this package, \code{prior} was not settable
and by default fixed to one, instead of zero. To obtain the same results as in
the previous versions, use \code{prior = 1} in each of the functions.
}
\examples{

x <- rprop(1e5, 100, 0.33)
hist(x, 100, freq = FALSE)
curve(dprop(x, 100, 0.33), 0, 1, col = "red", add = TRUE)
hist(pprop(x, 100, 0.33))
plot(ecdf(x))
curve(pprop(x, 100, 0.33), 0, 1, col = "red", lwd = 2, add = TRUE)

n <- 500
p <- 0.23
k <- rbinom(1e5, n, p)
hist(k/n, freq = FALSE, 100)
curve(dprop(x, n, p), 0, 1, col = "red", add = TRUE, n = 500)
                       
}
\references{
Ferrari, S., & Cribari-Neto, F. (2004). Beta regression for modelling rates and proportions.
Journal of Applied Statistics, 31(7), 799-815.

Smithson, M., & Verkuilen, J. (2006). A better lemon squeezer? Maximum-likelihood regression
with beta-distributed dependent variables.
Psychological Methods, 11(1), 54-71.
}
\seealso{
\code{\link{beta}}, \code{\link{binomial}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-laplace-distribution.R
\name{DiscreteLaplace}
\alias{DiscreteLaplace}
\alias{ddlaplace}
\alias{pdlaplace}
\alias{rdlaplace}
\title{Discrete Laplace distribution}
\usage{
ddlaplace(x, location, scale, log = FALSE)

pdlaplace(q, location, scale, lower.tail = TRUE, log.p = FALSE)

rdlaplace(n, location, scale)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{location}{location parameter.}

\item{scale}{scale parameter; \code{0 < scale < 1}.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass, distribution function and random generation
for the discrete Laplace distribution parametrized by location and scale.
}
\details{
If \eqn{U \sim \mathrm{Geometric}(1-p)}{U ~ Geometric(1-p)} and
\eqn{V \sim \mathrm{Geometric}(1-p)}{V ~ Geometric(1-p)},
then \eqn{U-V \sim \mathrm{DiscreteLaplace}(p)}{U-V ~ DiscreteLaplace(p)},
where geometric distribution is related to discrete Laplace distribution
in similar way as exponential distribution is related to Laplace distribution.

Probability mass function

\deqn{
f(x) = \frac{1-p}{1+p} p^{|x-\mu|}
}{
f(x) = (1-p)/(1+p) * p^(|x-\mu|)
}

Cumulative distribution function

\deqn{
F(x) = \left\{\begin{array}{ll}
\frac{p^{-|x-\mu|}}{1+p} & x < 0 \\
1 - \frac{p^{|x-\mu|+1}}{1+p} & x \ge 0
\end{array}\right.
}{
F(x) = [if x < 0:] p^-floor(x-\mu))/(1+p) [else:] 1-(p^(floor(x-\mu)+1))/(1+p)
}
}
\examples{

p <- 0.45
x <- rdlaplace(1e5, 0, p)
xx <- seq(-200, 200, by = 1)
plot(prop.table(table(x)))
lines(xx, ddlaplace(xx, 0, p), col = "red")
hist(pdlaplace(x, 0, p))
plot(ecdf(x))
lines(xx, pdlaplace(xx, 0, p), col = "red", type = "s")

}
\references{
Inusah, S., & Kozubowski, T.J. (2006). A discrete analogue of the Laplace distribution.
Journal of statistical planning and inference, 136(3), 1090-1102.

Kotz, S., Kozubowski, T., & Podgorski, K. (2012).
The Laplace distribution and generalizations: a revisit with applications
to communications, economics, engineering, and finance.
Springer Science & Business Media.
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/truncated-binomial-distribution.R
\name{TruncBinom}
\alias{TruncBinom}
\alias{dtbinom}
\alias{ptbinom}
\alias{qtbinom}
\alias{rtbinom}
\title{Truncated binomial distribution}
\usage{
dtbinom(x, size, prob, a = -Inf, b = Inf, log = FALSE)

ptbinom(q, size, prob, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE)

qtbinom(p, size, prob, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE)

rtbinom(n, size, prob, a = -Inf, b = Inf)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{size}{number of trials (zero or more).}

\item{prob}{probability of success on each trial.}

\item{a, b}{lower and upper truncation points (\code{a < x <= b}).}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the truncated binomial distribution.
}
\examples{

x <- rtbinom(1e5, 100, 0.83, 76, 86)
xx <- seq(0, 100)
plot(prop.table(table(x)))
lines(xx, dtbinom(xx, 100, 0.83, 76, 86), col = "red")
hist(ptbinom(x, 100, 0.83, 76, 86))

xx <- seq(0, 100, by = 0.01)
plot(ecdf(x))
lines(xx, ptbinom(xx, 100, 0.83, 76, 86), col = "red", lwd = 2)
uu <- seq(0, 1, by = 0.001)
lines(qtbinom(uu, 100, 0.83, 76, 86), uu, col = "blue", lty = 2)

}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wald-distribution.R
\name{Wald}
\alias{Wald}
\alias{dwald}
\alias{pwald}
\alias{rwald}
\title{Wald (inverse Gaussian) distribution}
\usage{
dwald(x, mu, lambda, log = FALSE)

pwald(q, mu, lambda, lower.tail = TRUE, log.p = FALSE)

rwald(n, mu, lambda)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, lambda}{location and shape parameters. Scale must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}

\item{p}{vector of probabilities.}
}
\description{
Density, distribution function and random generation
for the Wald distribution.
}
\details{
Probability density function
\deqn{
f(x) = \sqrt{\frac{\lambda}{2\pi x^3}} \exp\left( \frac{-\lambda(x-\mu)^2}{2\mu^2 x} \right)
}{
f(x) = sqrt(\lambda/(2*\pi*x^3)) * exp((-\lambda*(x-\mu)^2)/(2*\mu^2*x))
}

Cumulative distribution function
\deqn{
F(x) = \Phi\left(\sqrt{\frac{\lambda}{x}} \left(\frac{x}{\mu}-1 \right) \right) +
\exp\left(\frac{2\lambda}{\mu} \right) \Phi\left(\sqrt{\frac{\lambda}{x}}
\left(\frac{x}{\mu}+1 \right) \right)
}{
F(x) = \Phi(sqrt(\lambda/\mu)*(x/\mu-1)) - exp((2*\lambda)/\mu) *
\Phi(sqrt(\lambda/\mu)*(x/\mu+1))
}

Random generation is done using the algorithm described by Michael, Schucany and Haas (1976).
}
\examples{

x <- rwald(1e5, 5, 16)
hist(x, 100, freq = FALSE)
curve(dwald(x, 5, 16), 0, 50, col = "red", add = TRUE)
hist(pwald(x, 5, 16))
plot(ecdf(x))
curve(pwald(x, 5, 16), 0, 50, col = "red", lwd = 2, add = TRUE)

}
\references{
Michael, J.R., Schucany, W.R., and Haas, R.W. (1976).
Generating Random Variates Using Transformations with Multiple Roots.
The American Statistician, 30(2): 88-90.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/beta-binomial-distribution.R
\name{BetaBinom}
\alias{BetaBinom}
\alias{dbbinom}
\alias{pbbinom}
\alias{rbbinom}
\title{Beta-binomial distribution}
\usage{
dbbinom(x, size, alpha = 1, beta = 1, log = FALSE)

pbbinom(q, size, alpha = 1, beta = 1, lower.tail = TRUE, log.p = FALSE)

rbbinom(n, size, alpha = 1, beta = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{size}{number of trials (zero or more).}

\item{alpha, beta}{non-negative parameters of the beta distribution.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the beta-binomial distribution.
}
\details{
If \eqn{p \sim \mathrm{Beta}(\alpha, \beta)}{p ~ Beta(\alpha, \beta)} and
\eqn{X \sim \mathrm{Binomial}(n, p)}{X ~ Binomial(n, p)}, then 
\eqn{X \sim \mathrm{BetaBinomial}(n, \alpha, \beta)}{X ~ BetaBinomial(n, \alpha, \beta)}.

Probability mass function
\deqn{
f(x) = {n \choose x} \frac{\mathrm{B}(x+\alpha, n-x+\beta)}{\mathrm{B}(\alpha, \beta)}
}{
f(x) = choose(n, x) * B(x+\alpha, n-x+\beta) / B(\alpha, \beta)
}

Cumulative distribution function is calculated using recursive algorithm that employs the fact that
\eqn{\Gamma(x) = (x - 1)!}, and
\eqn{
\mathrm{B}(x, y) = \frac{\Gamma(x)\Gamma(y)}{\Gamma(x+y)}
}{
B(x, y) = (\Gamma(x)\Gamma(y))/\Gamma(x+y)
}, and that
\eqn{
{n \choose k} = \prod_{i=1}^k \frac{n+1-i}{i}
}{
choose(n,k) = prod((n+1-(1:k))/(1:k))
}. This enables re-writing probability mass function as

\deqn{
f(x) = \left( \prod_{i=1}^x \frac{n+1-i}{i} \right) \frac{\frac{(\alpha+x-1)!\,(\beta+n-x-1)!}{(\alpha+\beta+n-1)!}}{\mathrm{B}(\alpha,\beta)}
}{
f(x) = prod((n+1-(1:x))/(1:x)) * (((\alpha+x-1)!*(\beta+n-x-1)!)/((\alpha+\beta+n+1)!)) / B(\alpha, \beta)
}

what makes recursive updating from \eqn{x} to \eqn{x+1} easy using the properties of factorials

\deqn{
f(x+1) = \left( \prod_{i=1}^x \frac{n+1-i}{i} \right) \frac{n+1-x+1}{x+1} \frac{\frac{(\alpha+x-1)! \,(\alpha+x)\,(\beta+n-x-1)! \, (\beta+n-x)^{-1}}{(\alpha+\beta+n-1)!\,(\alpha+\beta+n)}}{\mathrm{B}(\alpha,\beta)}
}{
f(x+1) = prod((n+1-(1:x))/(1:x)) * ((n+1-x+1)/(x+1)) * (((\alpha+x-1)!*(\alpha+x)*(\beta+n-x-1)!/(\beta+n-x))/((\alpha+\beta+n+1)!*(\alpha+\beta+n))) / B(\alpha, \beta)
}

and let's us efficiently calculate cumulative distribution function as a sum of probability mass functions

\deqn{F(x) = \sum_{k=0}^x f(k)}{F(x) = f(0)+...+f(x)}
}
\examples{

x <- rbbinom(1e5, 1000, 5, 13)
xx <- 0:1000
hist(x, 100, freq = FALSE)
lines(xx-0.5, dbbinom(xx, 1000, 5, 13), col = "red")
hist(pbbinom(x, 1000, 5, 13))
xx <- seq(0, 1000, by = 0.1)
plot(ecdf(x))
lines(xx, pbbinom(xx, 1000, 5, 13), col = "red", lwd = 2)

}
\seealso{
\code{\link[stats]{Beta}}, \code{\link[stats]{Binomial}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bhattacharjee-distribution.R
\name{Bhattacharjee}
\alias{Bhattacharjee}
\alias{dbhatt}
\alias{pbhatt}
\alias{rbhatt}
\title{Bhattacharjee distribution}
\usage{
dbhatt(x, mu = 0, sigma = 1, a = sigma, log = FALSE)

pbhatt(q, mu = 0, sigma = 1, a = sigma, lower.tail = TRUE, log.p = FALSE)

rbhatt(n, mu = 0, sigma = 1, a = sigma)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, sigma, a}{location, scale and shape parameters.
Scale and shape must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, and random generation for the Bhattacharjee
distribution.
}
\details{
If \eqn{Z \sim \mathrm{Normal}(0, 1)}{Z ~ Normal(0, 1)} and
\eqn{U \sim \mathrm{Uniform}(0, 1)}{U ~ Uniform(0, 1)}, then
\eqn{Z+U} follows Bhattacharjee distribution.

Probability density function

\deqn{
f(z) = \frac{1}{2a} \left[\Phi\left(\frac{x-\mu+a}{\sigma}\right) - \Phi\left(\frac{x-\mu-a}{\sigma}\right)\right]
}{
f(z) = 1/(2*a) * (\Phi((x-\mu+a)/\sigma) - \Phi((x-\mu+a)/\sigma))
}

Cumulative distribution function

\deqn{
F(z) = \frac{\sigma}{2a} \left[(x-\mu)\Phi\left(\frac{x-\mu+a}{\sigma}\right) -
                               (x-\mu)\Phi\left(\frac{x-\mu-a}{\sigma}\right) +
                               \phi\left(\frac{x-\mu+a}{\sigma}\right) -
                               \phi\left(\frac{x-\mu-a}{\sigma}\right)\right]
}{
F(z) = \sigma/(2*a) * ((x-\mu)*\Phi((x-\mu+a)/\sigma) - (x-\mu)*\Phi((x-\mu-a)/\sigma) +
                       \phi((x-\mu+a)/\sigma) - \phi((x-\mu-a)/\sigma))
}
}
\examples{

x <- rbhatt(1e5, 5, 3, 5)
hist(x, 100, freq = FALSE)
curve(dbhatt(x, 5, 3, 5), -20, 20, col = "red", add = TRUE)
hist(pbhatt(x, 5, 3, 5))
plot(ecdf(x))
curve(pbhatt(x, 5, 3, 5), -20, 20, col = "red", lwd = 2, add = TRUE)

}
\references{
Bhattacharjee, G.P., Pandit, S.N.N., and Mohan, R. (1963).
Dimensional chains involving rectangular and normal error-distributions.
Technometrics, 5, 404-406.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gompertz-distribution.R
\name{Gompertz}
\alias{Gompertz}
\alias{dgompertz}
\alias{pgompertz}
\alias{qgompertz}
\alias{rgompertz}
\title{Gompertz distribution}
\usage{
dgompertz(x, a = 1, b = 1, log = FALSE)

pgompertz(q, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE)

qgompertz(p, a = 1, b = 1, lower.tail = TRUE, log.p = FALSE)

rgompertz(n, a = 1, b = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{a, b}{positive valued scale and location parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Gompertz distribution.
}
\details{
Probability density function
\deqn{
f(x) = a \exp\left(bx - \frac{a}{b} (\exp(bx)-1)\right)
}{
f(x) = a*exp(b*x - a/b * (exp(b*x)-1))
}

Cumulative distribution function
\deqn{
F(x) = 1-\exp\left(-\frac{a}{b} (\exp(bx)-1)\right)
}{
F(x) = 1-exp(-a/b * (exp(b*x)-1))
}

Quantile function
\deqn{
F^{-1}(p) = \frac{1}{b} \log\left(1-\frac{b}{a}\log(1-p)\right)
}{
F^-1(p) = 1/b * log(1 - b/a * log(1-p))
}
}
\examples{

x <- rgompertz(1e5, 5, 2)
hist(x, 100, freq = FALSE)
curve(dgompertz(x, 5, 2), 0, 1, col = "red", add = TRUE)
hist(pgompertz(x, 5, 2))
plot(ecdf(x))
curve(pgompertz(x, 5, 2), 0, 1, col = "red", lwd = 2, add = TRUE)

}
\references{
Lenart, A. (2012). The Gompertz distribution and Maximum Likelihood Estimation
of its parameters - a revision. MPIDR WORKING PAPER WP 2012-008.
\url{https://www.demogr.mpg.de/papers/working/wp-2012-008.pdf}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/truncated-poisson-distributtion.R
\name{TruncPoisson}
\alias{TruncPoisson}
\alias{dtpois}
\alias{ptpois}
\alias{qtpois}
\alias{rtpois}
\title{Truncated Poisson distribution}
\usage{
dtpois(x, lambda, a = -Inf, b = Inf, log = FALSE)

ptpois(q, lambda, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE)

qtpois(p, lambda, a = -Inf, b = Inf, lower.tail = TRUE, log.p = FALSE)

rtpois(n, lambda, a = -Inf, b = Inf)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{lambda}{vector of (non-negative) means.}

\item{a, b}{lower and upper truncation points (\code{a < x <= b}).}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the truncated Poisson distribution.
}
\examples{

x <- rtpois(1e5, 14, 16)
xx <- seq(-1, 50)
plot(prop.table(table(x)))
lines(xx, dtpois(xx, 14, 16), col = "red")
hist(ptpois(x, 14, 16))

xx <- seq(0, 50, by = 0.01)
plot(ecdf(x))
lines(xx, ptpois(xx, 14, 16), col = "red", lwd = 2)

uu <- seq(0, 1, by = 0.001)
lines(qtpois(uu, 14, 16), uu, col = "blue", lty = 2)

# Zero-truncated Poisson

x <- rtpois(1e5, 5, 0)
xx <- seq(-1, 50)
plot(prop.table(table(x)))
lines(xx, dtpois(xx, 5, 0), col = "red")
hist(ptpois(x, 5, 0))

xx <- seq(0, 50, by = 0.01)
plot(ecdf(x))
lines(xx, ptpois(xx, 5, 0), col = "red", lwd = 2)
lines(qtpois(uu, 5, 0), uu, col = "blue", lty = 2)

}
\references{
Plackett, R.L. (1953). The truncated Poisson distribution.
Biometrics, 9(4), 485-488.

Singh, J. (1978). A characterization of positive Poisson distribution and
its statistical application.
SIAM Journal on Applied Mathematics, 34(3), 545-548.

Dalgaard, P. (May 1, 2005). [R] simulate zero-truncated Poisson distribution.
R-help mailing list.
\url{https://stat.ethz.ch/pipermail/r-help/2005-May/070680.html}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zero-inflated-poisson-distribution.R
\name{ZIP}
\alias{ZIP}
\alias{dzip}
\alias{pzip}
\alias{qzip}
\alias{rzip}
\title{Zero-inflated Poisson distribution}
\usage{
dzip(x, lambda, pi, log = FALSE)

pzip(q, lambda, pi, lower.tail = TRUE, log.p = FALSE)

qzip(p, lambda, pi, lower.tail = TRUE, log.p = FALSE)

rzip(n, lambda, pi)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{lambda}{vector of (non-negative) means.}

\item{pi}{probability of extra zeros.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the zero-inflated Poisson distribution.
}
\details{
Probability density function
\deqn{
f(x) = \left\{\begin{array}{ll}
\pi + (1 - \pi) e^{-\lambda} & x = 0 \\
(1 - \pi) \frac{\lambda^{x} e^{-\lambda}} {x!} & x > 0 \\
\end{array}\right.
}{
f(x) = [if x = 0:] \pi + (1-\pi) * exp(-\lambda) [else:] (1-\pi) * dpois(x, lambda)
}
}
\examples{

x <- rzip(1e5, 6, 0.33)
xx <- -2:20
plot(prop.table(table(x)), type = "h")
lines(xx, dzip(xx, 6, 0.33), col = "red")

xx <- seq(0, 20, by = 0.01)
plot(ecdf(x))
lines(xx, pzip(xx, 6, 0.33), col = "red")

}
\seealso{
\code{\link[stats]{Poisson}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bernoulli-distribution.R
\name{Bernoulli}
\alias{Bernoulli}
\alias{dbern}
\alias{pbern}
\alias{qbern}
\alias{rbern}
\title{Bernoulli distribution}
\usage{
dbern(x, prob = 0.5, log = FALSE)

pbern(q, prob = 0.5, lower.tail = TRUE, log.p = FALSE)

qbern(p, prob = 0.5, lower.tail = TRUE, log.p = FALSE)

rbern(n, prob = 0.5)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{prob}{probability of success; (\code{0 < prob < 1}).}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function, quantile function and random generation
for the Bernoulli distribution.
}
\examples{

prop.table(table(rbern(1e5, 0.5)))

}
\seealso{
\code{\link[stats]{Binomial}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-gamma-distribution.R
\name{DiscreteGamma}
\alias{DiscreteGamma}
\alias{ddgamma}
\alias{pdgamma}
\alias{rdgamma}
\title{Discrete gamma distribution}
\usage{
ddgamma(x, shape, rate = 1, scale = 1/rate, log = FALSE)

pdgamma(q, shape, rate = 1, scale = 1/rate, lower.tail = TRUE, log.p = FALSE)

rdgamma(n, shape, rate = 1, scale = 1/rate)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{shape, scale}{shape and scale parameters. Must be positive, scale strictly.}

\item{rate}{an alternative way to specify the scale.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function, distribution function and random generation
for discrete gamma distribution.
}
\details{
Probability mass function of discrete gamma distribution \eqn{f_Y(y)}{f}
is defined by discretization of continuous gamma distribution
\eqn{f_Y(y) = S_X(y) - S_X(y+1)}{f(y) = S(x) - S(x+1)}
where \eqn{S_X}{S} is a survival function of continuous gamma distribution.
}
\examples{

x <- rdgamma(1e5, 9, 1)
xx <- 0:50
plot(prop.table(table(x)))
lines(xx, ddgamma(xx, 9, 1), col = "red")
hist(pdgamma(x, 9, 1))
plot(ecdf(x))
xx <- seq(0, 50, 0.1)
lines(xx, pdgamma(xx, 9, 1), col = "red", lwd = 2, type = "s")

}
\references{
Chakraborty, S. and Chakravarty, D. (2012).
Discrete Gamma distributions: Properties and parameter estimations.
Communications in Statistics-Theory and Methods, 41(18), 3301-3324.
}
\seealso{
\code{\link[stats]{GammaDist}}, \code{\link{DiscreteNormal}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power-distribution.R
\name{PowerDist}
\alias{PowerDist}
\alias{dpower}
\alias{ppower}
\alias{qpower}
\alias{rpower}
\title{Power distribution}
\usage{
dpower(x, alpha, beta, log = FALSE)

ppower(q, alpha, beta, lower.tail = TRUE, log.p = FALSE)

qpower(p, alpha, beta, lower.tail = TRUE, log.p = FALSE)

rpower(n, alpha, beta)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{alpha, beta}{parameters.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the power distribution.
}
\details{
Probability density function
\deqn{
f(x) = \frac{\beta x^{\beta-1}}{\alpha^\beta}
}{
f(x) = (\beta*x^(\beta-1)) / (\alpha^\beta)
}

Cumulative distribution function
\deqn{
F(x) = \frac{x^\beta}{\alpha^\beta}
}{
F(x) = x^\beta / \alpha^\beta
}

Quantile function
\deqn{
F^{-1}(p) = \alpha p^{1/\beta}
}{
F^-1(p) = \alpha * p^(1/\beta)
}
}
\examples{

x <- rpower(1e5, 5, 16)
hist(x, 100, freq = FALSE)
curve(dpower(x, 5, 16), 2, 6, col = "red", add = TRUE, n = 5000)
hist(ppower(x, 5, 16))
plot(ecdf(x))
curve(ppower(x, 5, 16), 2, 6, col = "red", lwd = 2, add = TRUE)

}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/beta-prime-distribution.R
\name{BetaPrime}
\alias{BetaPrime}
\alias{dbetapr}
\alias{pbetapr}
\alias{qbetapr}
\alias{rbetapr}
\title{Beta prime distribution}
\usage{
dbetapr(x, shape1, shape2, scale = 1, log = FALSE)

pbetapr(q, shape1, shape2, scale = 1, lower.tail = TRUE, log.p = FALSE)

qbetapr(p, shape1, shape2, scale = 1, lower.tail = TRUE, log.p = FALSE)

rbetapr(n, shape1, shape2, scale = 1)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{shape1, shape2}{non-negative parameters.}

\item{scale}{positive valued scale parameter.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the beta prime distribution.
}
\details{
If \eqn{X \sim \mathrm{Beta}(\alpha, \beta)}{X ~ Beta(\alpha, \beta)}, then
\eqn{\frac{X}{1-X} \sim \mathrm{BetaPrime}(\alpha, \beta)}{X/(1-X) ~ BetaPrime(\alpha, \beta)}.

Probability density function

\deqn{
f(x) = \frac{(x/\sigma)^{\alpha-1} (1+x/\sigma)^{-\alpha -\beta}}{\mathrm{B}(\alpha,\beta)\sigma}
}{
f(x) = ((x/\sigma)^(\alpha-1) * (1 + x/\sigma)^(-\alpha-\beta)) / (B(\alpha,\beta) * \sigma)
}

Cumulative distribution function

\deqn{
F(x) = I_{\frac{x/\sigma}{1+x/\sigma}}(\alpha, \beta)
}{
F(x) = pbeta((x/\sigma)/(1+(x/\sigma)), \alpha, \beta)
}
}
\examples{

x <- rbetapr(1e5, 5, 3, 2)
hist(x, 350, freq = FALSE, xlim = c(0, 100))
curve(dbetapr(x, 5, 3, 2), 0, 100, col = "red", add = TRUE, n = 500)
hist(pbetapr(x, 5, 3, 2))
plot(ecdf(x), xlim = c(0, 100))
curve(pbetapr(x, 5, 3, 2), 0, 100, col = "red", add = TRUE, n = 500)

}
\seealso{
\code{\link[stats]{Beta}}
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zero-inflated-binomial-distribution.R
\name{ZIB}
\alias{ZIB}
\alias{dzib}
\alias{pzib}
\alias{qzib}
\alias{rzib}
\title{Zero-inflated binomial distribution}
\usage{
dzib(x, size, prob, pi, log = FALSE)

pzib(q, size, prob, pi, lower.tail = TRUE, log.p = FALSE)

qzib(p, size, prob, pi, lower.tail = TRUE, log.p = FALSE)

rzib(n, size, prob, pi)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{size}{number of trials (zero or more).}

\item{prob}{probability of success in each trial. \code{0 < prob <= 1}.}

\item{pi}{probability of extra zeros.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Probability mass function and random generation
for the zero-inflated binomial distribution.
}
\details{
Probability density function
\deqn{
f(x) = \left\{\begin{array}{ll}
\pi + (1 - \pi) (1-p)^n & x = 0 \\
(1 - \pi) {n \choose x} p^x (1-p)^{n-x} & x > 0 \\
\end{array}\right.
}{
f(x) = [if x = 0:] (1-\pi)+\pi * p^r [else:] (1-\pi) * dnbinom(x, size, prob)
}
}
\examples{

x <- rzib(1e5, 10, 0.6, 0.33)
xx <- -2:20
plot(prop.table(table(x)), type = "h")
lines(xx, dzib(xx, 10, 0.6, 0.33), col = "red")

xx <- seq(0, 20, by = 0.01)
plot(ecdf(x))
lines(xx, pzib(xx, 10, 0.6, 0.33), col = "red")

}
\seealso{
\code{\link[stats]{Binomial}}
}
\concept{Discrete}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraDistr-package.R
\docType{package}
\name{extraDistr-package}
\alias{extraDistr}
\alias{extraDistr-package}
\title{extraDistr: Additional Univariate and Multivariate Distributions}
\description{
Density, distribution function, quantile function and random generation for a number of univariate and multivariate distributions. This package implements the following distributions: Bernoulli, beta-binomial, beta-negative binomial, beta prime, Bhattacharjee, Birnbaum-Saunders, bivariate normal, bivariate Poisson, categorical, Dirichlet, Dirichlet-multinomial, discrete gamma, discrete Laplace, discrete normal, discrete uniform, discrete Weibull, Frechet, gamma-Poisson, generalized extreme value, Gompertz, generalized Pareto, Gumbel, half-Cauchy, half-normal, half-t, Huber density, inverse chi-squared, inverse-gamma, Kumaraswamy, Laplace, location-scale t, logarithmic, Lomax, multivariate hypergeometric, multinomial, negative hypergeometric, non-standard beta, normal mixture, Poisson mixture, Pareto, power, reparametrized beta, Rayleigh, shifted Gompertz, Skellam, slash, triangular, truncated binomial, truncated normal, truncated Poisson, Tukey lambda, Wald, zero-inflated binomial, zero-inflated negative binomial, zero-inflated Poisson.

Density, distribution function, quantile function and random
generation for a number of univariate and multivariate distributions.
}
\details{
This package follows naming convention that is consistent with base R,
where density (or probability mass) functions, distribution functions,
quantile functions and random generation functions names are followed by
\code{d}*, \code{p}*, \code{q}*, and \code{r}* prefixes.

Behaviour of the functions is consistent with base R, where for
not valid parameters values \code{NaN}'s are returned, while
for values beyond function support \code{0}'s are returned
(e.g. for non-integers in discrete distributions, or for
negative values in functions with non-negative support).

All the functions vectorized and coded in C++ using \pkg{Rcpp}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/sigbertklinke/extraDistr}
}

}
\author{
\strong{Maintainer}: Sigbert Klinke \email{sigbert@wiwi.hu-berlin.de}

Authors:
\itemize{
  \item Tymoteusz Wolodzko \email{twolodzko+extradistr@gmail.com}
}

Other contributors:
\itemize{
  \item Thomas Farrar \email{FarrarT@cput.ac.za} [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/birnbaum-saunders-distribution.R
\name{BirnbaumSaunders}
\alias{BirnbaumSaunders}
\alias{dfatigue}
\alias{pfatigue}
\alias{qfatigue}
\alias{rfatigue}
\title{Birnbaum-Saunders (fatigue life) distribution}
\usage{
dfatigue(x, alpha, beta = 1, mu = 0, log = FALSE)

pfatigue(q, alpha, beta = 1, mu = 0, lower.tail = TRUE, log.p = FALSE)

qfatigue(p, alpha, beta = 1, mu = 0, lower.tail = TRUE, log.p = FALSE)

rfatigue(n, alpha, beta = 1, mu = 0)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{alpha, beta, mu}{shape, scale and location parameters.
Scale and shape must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the Birnbaum-Saunders (fatigue life) distribution.
}
\details{
Probability density function
\deqn{
f(x) = \left (\frac{\sqrt{\frac{x-\mu} {\beta}} + \sqrt{\frac{\beta} 
{x-\mu}}} {2\alpha (x-\mu)} \right) 
\phi \left( \frac{1}{\alpha}\left( \sqrt{\frac{x-\mu}{\beta}} -
\sqrt{\frac{\beta}{x-\mu}} \right) \right)
}{
f(x) = ((sqrt((x-\mu)/\beta) + sqrt(\beta/(x-\mu)))/(2*\alpha*(x-\mu))) *
\phi((sqrt((x-\mu)/\beta) - sqrt(\beta/(x-\mu)))/\alpha)
}

Cumulative distribution function
\deqn{
F(x) = \Phi \left(\frac{1}{\alpha}\left( \sqrt{\frac{x-\mu}{\beta}} -
\sqrt{\frac{\beta}{x-\mu}} \right) \right)
}{
F(x) = \Phi(((sqrt((x-\mu)/\beta) - sqrt(\beta/(x-\mu)))/\alpha)
}

Quantile function
\deqn{
F^{-1}(p) = \left[\frac{\alpha}{2} \Phi^{-1}(p) +
\sqrt{\left(\frac{\alpha}{2} \Phi^{-1}(p)\right)^{2} + 1}\right]^{2} \beta + \mu
}{
F^-1(p) = (\alpha/2 * \Phi^-1(p) +
sqrt((\alpha/2 * \Phi^-1(p))^2 + 1)^2 * \beta + \mu
}
}
\examples{

x <- rfatigue(1e5, .5, 2, 5)
hist(x, 100, freq = FALSE)
curve(dfatigue(x, .5, 2, 5), 2, 20, col = "red", add = TRUE)
hist(pfatigue(x, .5, 2, 5))
plot(ecdf(x))
curve(pfatigue(x, .5, 2, 5), 2, 20, col = "red", lwd = 2, add = TRUE)

}
\references{
Birnbaum, Z. W. and Saunders, S. C. (1969).
A new family of life distributions.
Journal of Applied Probability, 6(2), 637-652.

Desmond, A. (1985) Stochastic models of failure in random environments.
Canadian Journal of Statistics, 13, 171-183.

Vilca-Labra, F., and Leiva-Sanchez, V. (2006).
A new fatigue life model based on the family of skew-elliptical distributions.
Communications in Statistics-Theory and Methods, 35(2), 229-244.

Leiva, V., Sanhueza, A., Sen, P. K., and Paula, G. A. (2008).
Random number generators for the generalized Birnbaum-Saunders distribution.
Journal of Statistical Computation and Simulation, 78(11), 1105-1118.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/huber-distribution.R
\name{Huber}
\alias{Huber}
\alias{dhuber}
\alias{phuber}
\alias{qhuber}
\alias{rhuber}
\title{"Huber density" distribution}
\usage{
dhuber(x, mu = 0, sigma = 1, epsilon = 1.345, log = FALSE)

phuber(q, mu = 0, sigma = 1, epsilon = 1.345, lower.tail = TRUE, log.p = FALSE)

qhuber(p, mu = 0, sigma = 1, epsilon = 1.345, lower.tail = TRUE, log.p = FALSE)

rhuber(n, mu = 0, sigma = 1, epsilon = 1.345)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{mu, sigma, epsilon}{location, and scale, and shape parameters.
Scale and shape must be positive.}

\item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}

\item{lower.tail}{logical; if TRUE (default), probabilities are \eqn{P[X \le x]}
otherwise, \eqn{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1},
the length is taken to be the number required.}
}
\description{
Density, distribution function, quantile function and random generation
for the "Huber density" distribution.
}
\details{
Huber density is connected to Huber loss and can be defined as:

\deqn{
f(x) = \frac{1}{2 \sqrt{2\pi} \left( \Phi(k) + \phi(k)/k - \frac{1}{2} \right)} e^{-\rho_k(x)}
}{
f(x) = 1/(2 * sqrt(2\pi) * (\Phi(k) + \phi(k)/k - 1/2)) * exp(-\rho(x, k))
}

where

\deqn{
\rho_k(x) =
\left\{\begin{array}{ll}
\frac{1}{2} x^2       & |x|\le k \\
k|x|- \frac{1}{2} k^2 & |x|>k
\end{array}\right.
}{
\rho(x, k) = [if abs(x) <= k:] (x^2)/2 [else:] k*abs(x) - (k^2)/2
}
}
\examples{

x <- rhuber(1e5, 5, 2, 3)
hist(x, 100, freq = FALSE)
curve(dhuber(x, 5, 2, 3), -20, 20, col = "red", add = TRUE, n = 5000)
hist(phuber(x, 5, 2, 3))
plot(ecdf(x))
curve(phuber(x, 5, 2, 3), -20, 20, col = "red", lwd = 2, add = TRUE)

}
\references{
Huber, P.J. (1964). Robust Estimation of a Location Parameter.
Annals of Statistics, 53(1), 73-101.

Huber, P.J. (1981). Robust Statistics. Wiley.

Schumann, D. (2009). Robust Variable Selection. ProQuest.
}
\concept{Continuous}
\concept{Univariate}
\keyword{distribution}
